module(...,package.seeall)
 
local config,messages

config = require (tweety_modules_prefix .. "config")
messages = require (tweety_modules_prefix .. "messages")
datasource = require (tweety_modules_prefix .. "datasource")
local excel = require (tweety_modules_prefix .. "excelWriter")
local biReport = require ( tweety_modules_prefix .. "biReport" )
local rpwidget = require ( tweety_modules_prefix .. "rpwidget" )
local tweetyio = require ( tweety_modules_prefix .. "tweetyio" )
require "LuaOrm.Date"
require "uuid"
require (tweety_modules_prefix .. "json");
require "socket"
require "base64"

--const
local MAX_SEARCHABLE_FIELDS = 5

function new(self)
	local _appName
	local _app={
 		catalog={
			views={},
			resources={},
			datasources={}
		},
		data={
			views={},
			datasources={},
			images={},
			resources={}
		},
	}
	self = self or {}
	self.vccName = self.vccName or "system"
	-- LogDebug ( "application vcc %s ",self.vccName); 
	function self:getAppPath(appName)
		local vccName;
		local fh;
		if not self.hostVCC then
			-- a ver si esta app es de system
			fh = tweetyio.open(config.usersPath .. config.pathSeparator .. "system" .. config.pathSeparator .. appName .. config.pathSeparator .. config.mainAppFile,"rb");
			if fh then
				tweetyio.close(fh);
				self.hostVCC = "system";
				vccName = self.hostVCC;
			else
				self.hostVCC = self.vccName;
				vccName = self.hostVCC;
			end
		else
			vccName = self.hostVCC
		end
		return config.usersPath .. config.pathSeparator .. vccName .. config.pathSeparator .. appName
	end
	local function getAppPath(appName)
		return self:getAppPath(appName);
	end
	function self:isFormAvailable(formName)
		for idx,form in ipairs(_app.catalog.views) do	-- lo busca en el catalogo para evitar grabar cualquier cosa
			if form.name == formName then
				return true
			end
		end
		return false
	end
	function self:getVCCName()
		return self.vccName;
	end
	function self:getName()
		return _appName
	end
	function self:getVersion()
		return _app.version or 1
	end
	function self:save()
		local fh
		fh = tweetyio.open(getAppPath(_appName) .. config.pathSeparator .. config.mainAppFile,"wb");
		if not fh then
			LogDebug("Can't open %s ",getAppPath(_appName) .. config.pathSeparator .. config.mainAppFile)
			return false,messages.messageKey('write_file_error',{file=getAppPath(_appName) .. config.pathSeparator .. config.mainAppFile})
		end
		fh:write(json.encode(_app,true))
		tweetyio.close(fh)
		return true
	end
	function self:getSettings()
		local res = {}
		local catalog = self:getCatalog();
		for k,v in pairs(catalog) do
			if k~= "datasources"  and k~= "resources" then
				res[k] = v;
			end

		end
		return res;
	end
	function self:initialize(appName)
		local fh;
		local txt;
		_appName=appName
		-- Carga el archivo de descripcion de la aplicacion

		local fullAppName = getAppPath(_appName) .. config.pathSeparator .. config.mainAppFile
		--LogDebug("[APPLICATION:initialize] --> appName: '%s', fullAppName: '%s'", appName, fullAppName)

		fh = tweetyio.open(fullAppName,"rb");
		if not fh then
			--LogDebug("Can't open %s ",getAppPath(_appName) .. config.pathSeparator .. config.mainAppFile)
			return false,messages.messageKey('read_file_error',{file=getAppPath(_appName) .. config.pathSeparator .. config.mainAppFile})
		end

		txt = fh:read("*a")
		tweetyio.close(fh);
		_app = json.decode(txt)
		_app.data.resources = _app.data.resources or {}
--		messages.setLocale(self:getLocale())
		-- upgrade
		if not _app.version  then --  1er version
			local views = {}
			for _,name in ipairs(_app.catalog.forms) do
				table.insert(views,{name=name,type = "form"})
			end
			_app.data.views = _app.data.forms

			_app.data.forms = nil;
			_app.catalog.views = views;
			_app.catalog.forms = nil;
		end
		return true
	end
	-- function self:getLocale()
		-- return _app.catalog.locale or "";
	-- end
	function self:loadFormFile(formName)
		local fh;
		local txt;
		local code;
		local fname = getAppPath(_appName) .. config.pathSeparator .. formName .. config.formFileExtention
		fh = tweetyio.open(fname,"rb");
		if not fh then
			LogDebug("Can't load file %s",fname);
			return false,messages.messageKey('read_file_error',{file=fname})
		end
		txt = fh:read("*a")
		tweetyio.close(fh);
		if not txt or txt == "null" then
			LogDebug("Can't load file %s",fname);
			return false,messages.messageKey('read_file_error',{file=fname})
		end
		_app.data.views[formName] = json.decode(txt)
		_app.data.views[formName].name = formName;
		_app.data.views[formName].id = formName;
		fname = getAppPath(_appName) .. config.pathSeparator .. formName .. config.formCodeFileExtention
		if not _app.data.views[formName].__code  then -- si no tiene __code es porque es formato nuevo
			fh = tweetyio.open(fname,"rb");
			if not fh then
				LogDebug("Can't load file %s",fname);
				return false,messages.messageKey('read_file_error',{file=fname})
			end
			code = fh:read("*a")

			tweetyio.close(fh);
			_app.data.views[formName].__code = code;
		end
		return true
	end
	function self:loadDatasourceFile(datasourceName,isSystemDS)
		local fh;
		local txt;
		local fname;
		if not isSystemDS then
			fname = getAppPath(_appName) .. config.pathSeparator .. datasourceName  .. config.datasourceFileExtention
		else
			fname = config.resourcePath .. config.pathSeparator .. "datasources" .. config.pathSeparator .. datasourceName  .. config.datasourceFileExtention
		end
		fh = tweetyio.open(fname,"rb");
		if not fh then
			LogDebug("Can't load file %s",fname);
			return false,messages.messageKey('read_file_error',{file=fname})
		end
		txt = fh:read("*a")
		tweetyio.close(fh);
		_app.data.datasources[datasourceName] = json.decode(txt)
		_app.data.datasources[datasourceName].applicationName = _appName;
		_app.data.datasources[datasourceName].name = datasourceName;


		-- 2011-12-30
		-- patch para datasource interaccion que arregla el problema del
		-- campo Campaign en la operacion update que esta mapeado con la columna Id


		local data = _app.data.datasources[datasourceName]
		data.version = data.version or 0
		if data.version < 3 then

			if data.type == "icc_interaction_ds" then
				for _,param in ipairs(data.description.columns) do
					if param.name == "Contact" then
						param.readOnly = true
					end

				end

				for _,param in ipairs(data.description.update.inputParams) do
					--2012-01-14 Parche de Campaign que queda mal mapeado
					if param.name == "Campaign" then
						param.column = "Campaign"
					end
					--2012-01-14 Parche de Contact que es nullable porque se pone
					-- 'nothing' si no se pasa nada
					if param.name == "Contact" then
						param.allowNull = true
					end

				end
			end
			-- 2012-01-17
			-- las tablas mssql tambien mapean las columnas en retrieve
			local columns = {}
			for _,column in ipairs(data.description.columns) do
				columns[column.name]=column
			end
			if 	data.type == "mssql" and
				data.description.retrieve.enabled and
				data.description.retrieve.type == "msqltable" then
				for _,param in ipairs(data.description.retrieve.inputParams) do
					if not param.column and columns[param.name] then
						param.column = param.name
					end
				end
			end
			data.version = 3;
		end
		-- 2012-02-20
		-- Al implementar la busqueda de contactos
		-- todo los parametros de retrieve son nullables y
		-- estan todos mapeados a las columnas
		if data.version < 4 then
			if 	data.type == "icc_contact_ds" then
				data.description.retrieve.inputParams = {}
				for _,param in ipairs(data.description.columns) do
					if param.type ~= "phonelist" and param.type ~= "dataitems" and param.type ~= "skilllist" then

						table.insert(data.description.retrieve.inputParams,
							{
								name=param.name,
								column=param.column,
								allowNull=true,
								type=param.type
							}
						)
					end
				end

			end

			data.version = 4
		end
		--2012-02-23 Correccion de nombre y tipo para los campos isVIP e isClient

		if data.version < 5 then
			if 	data.type == "icc_contact_ds" then
				for _,param in ipairs(data.description.columns) do
					if param.name == "IsClient" then
						param.name = "isClient"
						param.type = "boolean"
					end
					if param.name == "VIP" then
						param.name = "isVIP"
						param.type = "boolean"
					end
				end
				for _,colection in ipairs({data.description.retrieve,data.description.update,data.description.store}) do

					for _,param in ipairs(colection.inputParams)do
						if param.name == "IsClient" then
							param.name = "isClient"
							param.type = "boolean"
							param.column = "isClient"
						end
						if param.name == "VIP" then
							param.name = "isVIP"
							param.type = "boolean"
							param.column = "isVIP"
						end

					end
				end
			end
			data.version = 5
		end
		-- 2012-02-28   when se transforma en 2 opciones required conditions y enabled conditions
		if data.version < 6 then
			for _,col in ipairs(data.description.columns) do
				col.requiredCondition = nil;
				col.enabledCondIsField = nil;
				col.enabledCondIsExpr = nil;
				col.requiredCondIsField = nil;
				col.requiredCondIsExpr = nil;
				col.alwaysEnabled = nil;
				if not col.whenRequired and not col.whenEnabled then    -- 0  0
					col.alwaysEnabled = true
					col.requiredCondition = col.required

				elseif not col.whenRequired and col.whenEnabled then    --   0   1

					if col.whenIsField and col.whenDatasourceField and col.whenOperator  then

						col.enabledCondIsField = col.whenIsField;
						col.enabledCondDatasourceField = col.whenDatasourceField
						col.enabledCondOperator = col.whenOperator
						col.enabledCondFieldExpr = col.whenValue
						col.requiredCondition = col.required;

					elseif col.whenIsExpr and col.whenExpr then

						col.enabledCondIsExpr = col.whenIsExpr
						col.enabledCondExpresion = col.whenExpr
						col.requiredCondition = col.required;

					else
						col.requiredCondition = col.required
					end

				elseif col.whenRequired and not col.whenEnabled then  -- 1 0
					col.alwaysEnabled = true

					if col.whenIsField and col.whenDatasourceField and col.whenOperator  then
						if not col.required then
							col.requiredCondIsField = col.whenIsField;
							col.requiredCondDatasourceField = col.whenDatasourceField
							col.requiredCondOperator = col.whenOperator
							col.requiredCondFieldExpr = col.whenValue

						end
						col.requiredCondition = true
					elseif col.whenIsExpr and col.whenExpr then

						col.requiredCondIsExpr = col.whenIsExpr
						col.requiredCondExpresion = col.whenExpr
						col.requiredCondition = col.required;
						col.requiredCondition = true
					else
						col.requiredCondition = col.required
					end
--					col.requiredCondition = (col.whenIsExpr ) or (col.whenIsField) or col.required

				elseif col.whenRequired and col.whenEnabled then   -- 1 1
					col.alwaysEnabled=false;


					if col.whenIsField and col.whenDatasourceField and col.whenOperator  then

						col.required = true;
						col.enabledCondIsField = col.whenIsField;
						col.enabledCondDatasourceField = col.whenDatasourceField
						col.enabledCondOperator = col.whenOperator
						col.enabledCondFieldExpr = col.whenValue
						col.requiredCondition = col.required

					elseif col.whenIsExpr and col.whenExpr then
						col.required = true;
						col.enabledCondIsExpr = col.whenIsExpr
						col.enabledCondExpresion = col.whenExpr
						col.requiredCondition = col.required;

					else
						col.requiredCondition = col.required
					end

				end

			end
			data.version = 6
		end
		if data.version < 7 then
			if data.type == "icc_interaction_ds" then
				for _,param in ipairs(data.description.columns) do
					if param.name == "Contact" then
						param.readOnly = false
					end
				end
			end

			data.version = 7
		end
--		2012-03-12 El campo contacto por defecto es requerido si hay reschedule o hay ContactCode
		if data.version < 8 then
			if data.type == "icc_interaction_ds" then
				for _,param in ipairs(data.description.columns) do
					if param.name == "Contact" then
						param.readOnly = false
						param.alwaysEnabled= true
						param.enabledCondIsExpr= false
						param.readOnly= false
						param.required= false
						param.requiredCondExpresion= "schedule || ContactCode!=null"
						param.requiredCondIsExpr= true
						param.requiredCondition= true
					end
				end
			end
			data.version = 8
		end
--		2012-03-20 El campo _id de los datasources de mongodb es mapeado en todas las operaciones y agregado
--      a las columnas en modo hidden
		if data.version < 9 then
			if data.type == "icc-native" then
				local found = false
				for _,param in ipairs(data.description.columns) do
						if param.name == "_id" then
							found = true
						end
				end
				if not found then
					table.insert(data.description.columns,
					{
						key=true,
						type="string",
						name="_id",
						hidden=true

					});
				end
				for _,param in ipairs(data.description.store.inputParams) do
					if param.name == "_id" then
						param.column = "_id";
					end
				end
				for _,param in ipairs(data.description.update.inputParams) do
					if param.name == "_id" then
						param.column = "_id";
					end
				end
				for _,param in ipairs(data.description.delete.inputParams) do
					if param.name == "_id" then
						param.column = "_id";
					end
				end

			end
			data.version=9
		end
		if data.version < 10 then
			if data.type == "icc-native" then


				for _,section in ipairs({
					data.description.retrieve,
					data.description.store,
					data.description.update,
					data.description.delete}) do
					local t = {}
					local found
					for _,param in ipairs(section.inputParams) do

						if param.name == "_id" then
							if not found then table.insert(t,param) end
							param.column="_id"
							param.allowNull=section == data.description.retrieve
							param.key = not (section == data.description.retrieve)
						else
							table.insert(t,param)
						end

						found=true
					end
					if not found then
						table.insert(t,
						{
							type="string",
							name="_id",
							column="_id",
							allowNull=section == data.description.retrieve.inputParams,
							key = not (section == data.description.retrieve.inputParams)

						});
					end
					section.inputParams=t;
				end


			end
			data.version = 10
		end
		-- el designer arruinaba el paso nueve lo volvemos a repetir
		if data.version < 11 then
			if data.type == "icc-native" then
				local found = false
				for _,param in ipairs(data.description.columns) do
						if param.name == "_id" then
							found = true
						end
				end
				if not found then
					table.insert(data.description.columns,
					{
						key=true,
						type="string",
						name="_id",
						hidden=true

					});
				end
				for _,param in ipairs(data.description.store.inputParams) do
					if param.name == "_id" then
						param.column = "_id";
					end
				end
				for _,param in ipairs(data.description.update.inputParams) do
					if param.name == "_id" then
						param.column = "_id";
					end
				end
				for _,param in ipairs(data.description.delete.inputParams) do
					if param.name == "_id" then
						param.column = "_id";
					end
				end

			end
			data.version=11
		end

		if data.version < 12 then
			LogDebug("STARTING DATA CONVERSION VERSION 12 %s %s",_appName,datasourceName);
			local conds={}

			if data.description.retrieve.enabled then
				local properties ={
					"alwaysEnabled",
					"enabledCondIsExpr",
					"enabledCondExpresion",
					"enabledCondIsField",
					"enabledCondDatasourceField",
					"enabledCondOperator",
					"enabledCondFieldExpr",
					"required",
					"requiredCondition",
					"requiredCondIsExpr",
					"requiredCondExpresion",
					"requiredCondIsField",
					"requiredCondDatasourceField",
					"requiredCondOperator",
					"requiredCondFieldExpr",
					"default",
					"blank",
				}

				for _,col in ipairs(data.description.columns) do
					if col.enabledCondIsExpr or col.enabledCondIsField or col.requiredCondIsExpr or col.requiredCondIsField then
						local settings = {}
						for _,k in ipairs(properties) do
							settings[k] = col[k]
							--col[k]=nil
						end
						settings.blank = false;
						conds[col.name] = {
							col=col,
							settings=settings
						}
					end
				end
				for k,v in ipairs(data.description.filters) do
					if v.name == "default" then
						for nseg,segment in ipairs(v.criteria) do
							segment.settings = {
								alwaysEnabled=true,
								enabledCondIsExpr=false,

								enabledCondIsField=false,



								required=false,
								requiredCondIsExpr=false,
								requiredCondIsField=false,
								blank=false;

							}

							LogDebug("procesando segmento %s de %s",nseg,#v.criteria);
							if segment.column and segment.expr and string.sub(segment.expr,1,1) =="@" then
								local cond = conds[segment.column]
								if cond then
									LogDebug("Convertida condicion de campo en %s %s %s input %s",
										_appName,datasourceName,cond.col.name,segment.expr);

									segment.settings = cond.settings

									if segment.settings.requiredCondIsField then
										LogDebug("requiredCondIsField %s %s",segment.settings.requiredCondIsField,segment.settings.requiredCondDatasourceField)
										local found = false;

										for _,s in ipairs(v.criteria) do
--											LogDebug("TEST %s %s %s [%s]",segment.settings.requiredCondDatasourceField,s.column,s.expr,string.sub(s.expr,1,1))
											if segment.settings.requiredCondDatasourceField == s.column and string.sub(s.expr,1,1) =="@" then
												LogDebug("FOUND FIELD %s",s.expr);
												segment.settings.requiredCondDatasourceField = s.column .. s.expr
												found = true
												break;
											end
										end
										if not found then
											LogDebug("REQUIRED CONDITION FIELD MISMATCH [%s] campo en %s %s %s input %s ",
											segment.settings.requiredCondDatasourceField,
											_appName,datasourceName,cond.col.name,segment.expr)
										end

									end

									if segment.settings.enabledCondIsField then
									LogDebug("enabledCondIsField %s %s",segment.settings.enabledCondIsField,segment.settings.enabledCondDatasourceField)
										local found = false
										for _,s in ipairs(v.criteria) do
	--										LogDebug("TEST %s %s %s [%s]",segment.settings.enabledCondDatasourceField,s.column,s.expr,string.sub(s.expr,1,1))

											if segment.settings.enabledCondDatasourceField == s.column and  string.sub(s.expr,1,1) =="@" then
												segment.settings.enabledCondDatasourceField = s.column .. s.expr
												LogDebug("FOUND FIELD %s",s.expr);
												found = true
												break;
											end
										end
										if not found then
											LogDebug("ENABLED CONDITION FIELD MISMATCH [%s] campo en %s %s %s input %s ",
											segment.settings.enabledCondDatasourceField,
											_appName,datasourceName,cond.col.name,segment.expr)
										end

									end

									if segment.settings.requiredCondIsExpr then
										LogDebug("REQUIRED EXPRESION [%s]",segment.settings.requiredCondExpresion)
									end
									if segment.settings.enabledCondIsExpr then
										LogDebug("ENABLED EXPRESION [%s]",segment.settings.enabledCondExpresion)
									end
									if 	data.description.retrieve.enabled and
											not data.description.store.enabled and
											not data.description.update.enabled and
											not data.description.delete.enabled then

										for _,key in ipairs(properties) do
											cond.col[key]=nil
										end
										LogDebug("OK CONVERSION")
									else
										LogDebug("CONFLICT CONVERSION")
									end
								end

							end
						end
					end
				end

			end
			data.version = 12;
		end
		if data.version < 13 then
			if 	data.type == "icc_contact_ds" then
				for _,param in ipairs(data.description.retrieve.inputParams) do
					param.column = param.name
					param.allowNull = true
				end
			end
			data.version = 13;
		end
		if data.version < 14 then
			if data.type == "icc_interaction_ds" then
				for _,param in ipairs(data.description.columns) do
					if param.name == "RescheduledToDate" or param.name == "RescheduledToTime" then
						param.required = false;
						param.requiredCondExpresion =  "schedule == true";
						param.requiredCondIsExpr = true;
						param.requiredCondition = true;
					end
				end
			end

			data.version = 14
		end
		if data.version < 15 then
			for _,param in ipairs(data.description.columns) do
				if param.validate and param.validate.displayField then
					param.validate.codeField = param.validate.displayField
				end
			end
			data.version = 15
		end
		if data.version < 16 then
			if data.type == "icc_interaction_ds" then
				local col = {
					name="RescheduledToPhoneType",
					type="string",
					column="RescheduledToPhoneType",
					requiredCondExpresion = "RescheduledToPhone != undefined",
					requiredCondIsExpr = true,
					requiredCondition = true
				};
				for idx,param in ipairs(data.description.columns) do
					if param.name == "RescheduledToPhone" then
						table.insert(data.description.columns,idx+1,col)
						break
					end
				end
				table.insert(data.description.update.inputParams,col)
			end
			data.version = 16
		end
		if data.version < 17 then
			if data.type == "icc_interaction_ds" then
				for idx,param in ipairs(data.description.update.inputParams) do
					if param.name == "RescheduledToPhoneType" then
						param.allowNull = true;
						break
					end
				end
				table.insert(data.description.update.inputParams,col)
			end
			data.version = 17
		end
		if data.version < 18 then
			if data.type == "icc_interaction_ds" then
				for idx,param in ipairs(data.description.columns) do
					if param.name == "RescheduledToPhoneType" and not param.default then
						param.default= "CELLULAR"
						break
					end
				end
				table.insert(data.description.update.inputParams,col)
			end
			data.version = 18
		end
		if data.version < 19 then
			if data.type == "icc_interaction_ds" then
				for idx,param in ipairs(data.description.columns) do
					if param.name == "ContactName" then
						param.readOnly = false;
						break
					end
				end
				local col = {
					name="ContactName",
					column="ContactName",
					type="string",
					allowNull=true
				}
				table.insert(data.description.update.inputParams,col)
			end
			data.version = 19
		end
		if data.version < 20 then
			if data.type == "icc_interaction_ds" then
				for idx,param in ipairs(data.description.columns) do

					if param.name == "schedule" then
						param.hidden = false
					end
				end
				table.insert(data.description.columns,2,{
					name='Sections',
					readOnly=true,
					type="integer"
				});
			end
			data.version = 20;
		end
		if data.version < 21 then
			if data.type == "icc-summarizer" then
				for idx,param in ipairs(data.description.columns) do
					if param.name == "ID" or param.name == "timeStamp" then
						param.hidden = true
					end
				end
			end
			data.version = 21;
		end
		if data.version < 22 then
			if data.type == "icc-summarizer" then
				local col = {
					name="_survey_data",
					column="_survey_data",
					type="string",
					allowNull=true,
					hidden=true
				}
				table.insert(data.description.update.inputParams,col);
				table.insert(data.description.store.inputParams,col);
				table.insert(data.description.columns,col)
			end
			data.version = 22;
		end		
		if data.version < 23 then
			if data.type == "icc-summarizer" then
				for idx,param in ipairs(data.description.columns) do
					if param.name == "ID" and not (param.default or param.default == "") then
						param.default = "=args.interaction_id || args.INTERACTIONID";
					end
				end
			end
			data.version = 23;
		end		
		if data.type == "icc-native" then
			local c = {}
			local ch = {}
			for _,col in ipairs(data.description.columns) do
				if not ch[col.name] then
					ch[col.name] = col
					table.insert(c,col)
				end
			end
			data.description.columns = c;
		end

		return true,_app.data.datasources[datasourceName]
	end

	function self:getFormData(formName)
		local function split(self,sep)
			local sep, fields = sep or ":", {}
			local pattern = string.format("([^%s]+)", sep)
			self:gsub(pattern, function(c) fields[#fields+1] = c end)
			return fields
		end


		for _,form in ipairs(_app.catalog.views) do
			if form.name == formName then
				if _app.data.views[form.name] then
					return true,_app.data.views[form.name]
				else
					local status,err = self:loadFormFile(form.name)
					if status then
						return true,_app.data.views[form.name],formName
					else
						return status,err
					end
				end
			end
		end
		local t = split(formName,'~')
		local resname = t[1]
		local sectionName = t[2]
		if resname and sectionName then
			for _,res in ipairs(_app.catalog.resources) do
				if res.src == resname and res.mimeType == "x-application/icc-flowactivity" then
					status,result = self:readResource(res.src,true)
					if status then
						local jsonData = json.decode(result.content);
						local properties = jsonData.properties or {}
						local property
						for k,v in ipairs(properties) do
							if (v.type == "custom" or v.type == "custom-panel") and v.customDialog then
								LogDebug("FORM %s",json.encode(v.customDialog))
								local t = split(resname,".")
								return true,v.customDialog,table.concat(t,"_") .. "_" .. sectionName
							end
						end
					else
						return false,result
					end
				end
			end
		end
		return false,messages.messageKey('argument_error',{module="application.lua",func="getFormData",param="formName",value=formName})
	end
	function self:getPopupSize(formName)
		local res = {width=0;height=0}
		local cat;
		if formName then
			cat = {{name=formName}}
		else
			cat = _app.catalog.views
		end
		for _,form in ipairs(cat) do
			local status,err = self:loadFormFile(form.name)
			if status then
				local formData = _app.data.views[form.name]
				if formData.formSubType ~= "bireport" and formData.formSubType ~= "report" then
					for _,child in ipairs(formData.childs) do
						if child.x and child.width and child.x + child.width > res.width then res.width = child.x + child.width end
						if child.y and child.height and child.y + child.height > res.height then res.height = child.y+child.height end
					end
				end
			end
		end
		return res;
	end
	function self:getReportInfo(reportName)
		for _,form in ipairs(_app.catalog.views) do
			if form.name == reportName then
				local status,err = self:loadFormFile(form.name)
				if status then
					local formData = _app.data.views[form.name]

					if formData.formSubType == "bireport" then
						local paramKey = {}
						inputParams = {}
						for _,band in ipairs(formData.childs) do
							if band['band-type'] ==  "reportdetails" then

								for _,row in ipairs(band.childs[1].childs) do
									if  row.source and row.source.datasource then
										local status,dsdata = self:getDatasourceData(row.source.datasource)
										if status then

											if dsdata.description.retrieve.enabled then
												-- buscamos los parametros de retrieve que
												-- sean obligatorios
												for _,param in ipairs(dsdata.description.retrieve.inputParams) do
													-- si no esta mapeado a una columna y no permite nulos
													-- es requerido siempre
													local found = false;

													if param.column then
														for _,criteria in ipairs(
															(((dsdata.description.filters or {{}})[1] or {}).criteria or {})

															) do
															if param.column == criteria.column  then --
																if criteria.expr and string.sub(criteria.expr,1,1) == "@" and not paramKey[criteria.expr] then
																	paramKey[criteria.expr]=true;
																	table.insert(inputParams,{column = "",nullable = param.allowNull and criteria.nullable,coltype = param.type,comparator="==",expr=criteria.expr})
																end
																found = true;
															end
														end
													end
													if not found and not param.allowNull then
														if not paramKey[param.name] then
															paramKey[param.name] = true
															table.insert(inputParams,{column="",coltype = param.type,nullable=false,comparator="==",expr=param.name,isInput=true})
														end
													end
												end
											end
										end
									end
								end
							end
						end
						if formData.paramsSettings then
							for _,param in ipairs(inputParams) do
								local paramData = formData.paramsSettings[param.expr]
								for k,v in pairs(paramData or {}) do
									param[k]=v
								end
							end
						end
						return true,{filters={{criteria=inputParams}}}
					else
						return true,{filters=_app.data.views[form.name].filters}
					end
				else
					return status,err
				end
			end
		end
		return false,messages.messageKey('argument_error',{module="application.lua",func="getFormData",param="formName",value=formName})
	end

	function self:saveDatasourceData(datasourceName,str_data)
		local iccSummarizerFixedColumns = {
			ID = true,
			timeStamp = true,
			DateFrom = true,
			DateTo = true,
			_HasLogin = true,
			_HasPreSurvey = true,
			_HasPostSurvey = true,
			_survey_data = true
		}
		iccSummarizerSearchableColumns = {}
		local datasourceFound = false
		for _,name in ipairs(_app.catalog.datasources) do	-- lo busca en el catalogo para evitar grabar cualquier cosa
			local status,dsdata
			if name == datasourceName then
				status,dsdata = pcall(json.decode, str_data)
				datasourceFound = true
			else
				status,dsdata = self:getDatasourceData(name)
			end
			if not status then
				dsdata = nil; -- Ignoramos el error, es porque falta el archivo de datos de un ds corrupto
				LogWarning("Error getting datasource: '%s'",name)
				-- return status,dsdata -- dsdata es el error
			end
			if dsdata and dsdata.type == "icc-summarizer" then
				for colNum, col in pairs(dsdata.description.columns) do
					if not iccSummarizerFixedColumns[col.name] and col.searchable then
						iccSummarizerSearchableColumns[col.type] = iccSummarizerSearchableColumns[col.type] and iccSummarizerSearchableColumns[col.type] + 1 or 1
					end
				end
			end
		end
		if datasourceFound then
			for colType, colCount in pairs(iccSummarizerSearchableColumns) do
				if colCount > MAX_SEARCHABLE_FIELDS then
					LogWarning("Can't save more than %d %s searchable fields", MAX_SEARCHABLE_FIELDS, colType)
					return false,messages.messageKey('max_searchable_fields_reached',{maxSearchableFields=MAX_SEARCHABLE_FIELDS,searchableType=colType})
				end
			end
			local fname = getAppPath(_appName) .. config.pathSeparator .. datasourceName .. config.datasourceFileExtention
			local fh = tweetyio.open(fname,"wb")
			--LogDebug("[application:saveDatasourceData] ---> fname: '%s', str_data: '%s'", fname, str_data)
			if not fh then
				LogDebug("Can't open file %s",fname);
				return false,messages.messageKey('read_file_error',{file=name})
			end
			fh:write(str_data);
			tweetyio.close(fh);
			_app.data.datasources[datasourceName]=data
		else
			return false,messages.messageKey('argument_error',{module="application.lua",func="saveDatasourceData",param="datasourceName",value=datasourceName})
		end
		return true
	end
	function self:getDatasourceData(datasourceName, returnPseudoDatasources)
		--LogDebug("[application:getDatasourceData] 1 - datasourceName: '%s', returnPseudoDatasources: '%s'", datasourceName, returnPseudoDatasources)
		--LogDebug("[application:getDatasourceData] 2 - _app.catalog.datasources: '%s'", json.encode(_app.catalog.datasources))

		for _,name in ipairs(_app.catalog.datasources) do
			if name == datasourceName then
				if _app.data.datasources[name] then
					local datasource = _app.data.datasources[name]
					--LogDebug("[application:getDatasourceData] 3 - _app.data.datasources['%s']: '%s'", name, json.encode(datasource))
					return true, datasource
				else
					local status,err = self:loadDatasourceFile(datasourceName)
					if status then
						local datasource = _app.data.datasources[name]
						--LogDebug("[application:getDatasourceData] 4 - _app.data.datasources['%s']: '%s'", name, json.encode(datasource))
						return true, datasource
					else
						return false,err
					end
				end
			end
		end
		-- No encontro el datasource requerido,es asqueroso pero no hay mas remedio que cargarlos todos de a uno y buscar si es un outstream
		for _,name in ipairs(_app.catalog.datasources) do
			local status,err = self:loadDatasourceFile(name)
			if status then
				local dsdata = _app.data.datasources[name]
				if dsdata.description.outstreams then
					for _,outstream in ipairs(dsdata.description.outstreams) do
						if outstream.name == datasourceName then
							-- creamos un pseudo datasource
							if returnPseudoDatasources then
								-- toda esta macana esta hecha para hace andar el sumarizador de i6, sin pensar mucho
								local dsresult = {

									description = {
										sourceName = name,
										columns = dsdata.description.columns,
										retrieve = {
											type=dsdata.description.retrieve.type,
											enabled=true,
											inputParams={}
										},
										delete = {
											enabled=false,
											inputParams={}
										},
										update = {
											enabled=false,
											inputParams={}
										},
										store = {
											enabled=false,
											inputParams={}

										},
										type = dsdata.description.retrieve.type,
										filters=outstream.description.filters
									}
								}
								local inputParamsHash={}
								if outstream.type == "summary" then
									for _,indicator in ipairs(outstream.description.indicators) do
										table.insert(dsresult.description.columns,{
											name=indicator.name,
											type=indicator.type,
											readOnly=true
										})
										inputParamsHash[indicator.name]={
											name=indicator.name,
											allowNull=true,
											type=indicator.type,
											column=indicator.name,
											type="string"
										}
									end
								elseif outstream.type == "details" then
									local colhash={}
									for _,column in ipairs(dsdata.description.columns) do
										colhash[column.name]=column;
									end
									for _,column in ipairs(outstream.description.projectedcolumns) do
										table.insert(dsresult.description.columns,{
											name=column.name,
											type=(colhash[column.name] or {type="string"}).type,
											readOnly=true
										})
										inputParamsHash[column.name]={
											name=column.name,
											allowNull=true,
											type=(colhash[column.name] or {type="string"}).type,
											column=column.name
										}
									end
								end
								for _,segment in ipairs((outstream.description.filters[1] or {}).criteria) do
									if segment.column and segment.expr then

										local allowNull=false
										if segment.column == "VCC" and segment.expr == "@VCC" then -- el VCC ta por todos lados :)
											allowNull=true
										end
										inputParamsHash[segment.column]=inputParamsHash[segment.column] or {
											name=segment.column;
											type="string",
											allowNull=true,
											column=segment.column
										}
									end
								end
								for _,param in pairs(inputParamsHash) do
									table.insert(dsresult.description.retrieve.inputParams,param)
								end
								return true,dsresult
							else
								return true,dsdata;
							end
						end
					end
				end
			end
		end

		-- si llego aca puede tratarse de un datasource de sistema

		LogDebug("Attempt loading system datasource");
		local status,err = self:loadDatasourceFile(datasourceName,true)
		if status then
			return true,_app.data.datasources[datasourceName]
		else
			return false,err
		end
		return false,messages.messageKey('argument_error',{module="application.lua",func="getDatasourceData",param="datasourceName",value=datasourceName})
	end
	local buildControl
	local buildDatasourceList
	buildDatasourceList = function(ctrldata,output)
		local cchilds = ctrldata.childs
		output[ctrldata.datasource or 0] = ctrldata.datasource
		output[ctrldata.rowsSource or 0] = ctrldata.rowsSource

		if ctrldata.childs then
			for _,clctrldata in ipairs(ctrldata.childs) do
				buildDatasourceList(clctrldata,output)
			end
		end
	end
	buildControl = function (ctrldata,output,avoidOmited)
		local cchilds = ctrldata.childs
		local ctype = ctrldata['runtime-type'] or ctrldata.type
		local childsid={}
		setmetatable(childsid,{
			__toJson=function(self)
				local res = {"["}
				local r ={}
				for _,v in ipairs(self) do
					table.insert(r,v)
				end
				table.insert(res,table.concat(r,","))
				table.insert(res,"]");
				return table.concat(res);
			end
		})
		ctrldata.type = nil
		if ctrldata.childs then
			ctrldata.childs = childsid
		end
		if cchilds then
			for _,clctrldata in ipairs(cchilds) do
				if not avoidOmited or not clctrldata['__omited__in_buildform_'] then
					table.insert(childsid,clctrldata.name or clctrldata.id);
					buildControl(clctrldata,output,avoidOmited)
				end
			end

		end
		table.insert(output,table.concat{
			"var ",ctrldata.name or ctrldata.id,"=new icc.",ctype,"(",json.encode(ctrldata),",environment);"
		})
		ctrldata.type = ctype
		ctrldata.childs = cchilds
	end

	-- metodos para el webHandler
	function self:getDefaultView()
		if _app.catalog.defaultView then
			local name;
			local firstForm
			for _,view in ipairs(_app.catalog.views) do
				firstForm = firstForm or view.name
				if view.name == _app.catalog.defaultView then
					name = _app.catalog.defaultView
				end

			end
			return name or firstForm or _app.catalog.defaultView
		elseif #_app.catalog.views>0 then
			return _app.catalog.views[1].name
		end
	end
	function self:listResources(mimeType)
		local res = {}
		for _,reg in ipairs(_app.catalog.resources) do
			if mimeType then
				if reg.mimeType == mimeType then
					table.insert(res,reg);
				end
			else
				table.insert(res,reg);
			end
		end
		return res;
	end
	function self:processJavascriptCode(code)

		code=string.gsub(code,"<%%([%w]+)[:]*([%w%.~]*)%%>",
			function(cmd,param)
				if cmd == "VIEW" and param ~= nil then
					local status,result = self:buildInlineForm(param)
					if status then
						return result
					else
						return "alert('" .. result.description .."');"
					end
				elseif cmd == "FRAME" and param  then
					local form,frame
					form  = string.gsub(param,"%.(%w+)$",function(s) frame = s return "" end)

					if form and frame then
						local status,result = self:buildInlineFrame(form,frame)
						if status then
							return result
						else
							return "alert('" .. result.description.."');"
						end
					end
				end
			end
		)
		return code
	end
	function self:readResource(resName,rawread)
		local result = messages.messageKey('argument_error',{module="application.lua",func="readResource",param="resName",value=resName})
		local status = false;
		for _,res in ipairs(_app.catalog.resources) do
			if res.src == resName then
				if _app.data.resources[resName] then
					return true,{mimeType=res.mimeType,src=res.src,content=_app.data.resources[resName]}
				else
					local fname
					local iohandler
					if (_app.catalog.appClass == "chatcampaign" or _app.catalog.appClass == "webcallcampaign") and resName == "___helpers.js" then
						resName = "!chatcampaign___helpers.js"
					end
					if string.sub(resName,1,1) == "!" then
						fname = config.resourcePath .. config.pathSeparator .. resName
						iohandler = io;
					else
						iohandler = tweetyio;
						fname = getAppPath(_appName) .. config.pathSeparator .. resName
					end

					local fh = iohandler.open(fname,"rb");
					if not fh then
						LogDebug("Can't load file %s",fname);
						return false,messages.messageKey('read_file_error',{file=fname})
					end
					local txt = fh:read("*a")
					iohandler.close(fh);
					if not rawread then

						if res.mimeType == "text/javascript" then
							txt = self:processJavascriptCode(txt);
						end
					end
					_app.data.resources[res.src] = txt;
					result={mimeType=res.mimeType,src=res.src,content=txt}
					return true,result
				end
			end
		end
		return status,result
	end
	function self:saveResource(resName,content)
		for _,res in ipairs(_app.catalog.resources) do	-- lo busca en el catalogo para evitar grabar cualquier cosa
			if res.src == resName then
				local fname
				if string.sub(resName,1,1) == "!" then
					fname = config.resourcePath .. config.pathSeparator .. resName
				else
					fname = getAppPath(_appName) .. config.pathSeparator .. resName
				end

				local fh = tweetyio.open(fname,"wb")
				if not fh then
					LogDebug("Can't open file %s",fname);
					return false,messages.messageKey('write_file_error',{file=name})
				end
				if res.mimeType == "x-application/icc-flowactivity" then
					content = json.encode(json.decode(content),true)
				end
				fh:write(content);
				tweetyio.close(fh);
				return true
			end
		end
		return false,messages.messageKey('argument_error',{module="application.lua",func="saveResource",param="resName",value=resName})
	end
	function self:addResource(resName,mimeType,content,overwrite)
		local duplicated
		for idx,res in ipairs(_app.catalog.resources) do	-- lo busca en el catalogo para evitar sobreescribir
			if res.src == resName then
				duplicated = true
				if not overwrite then
					return false,messages.messageKey('duplicated_entry',{module="application.lua",func="addResource",param="resName",value=resName})
				end
			end
		end
		local fname
		if string.sub(resName,1,1) == "!" then
			fname = config.resourcePath .. config.pathSeparator .. resName
		else
			fname = getAppPath(_appName) .. config.pathSeparator .. resName
		end
		local fh = tweetyio.open(fname,"wb")
		if not fh then
			LogDebug("Can't open file %s",fname);
			return false,messages.messageKey('read_file_error',{file=fname})
		end
		if mimeType=="x-application/icc-flowactivity" then
			content = json.encode(json.decode(content),true)
		end
		fh:write(content);
		tweetyio.close(fh);
		if not duplicated then
			table.insert(_app.catalog.resources,{src=resName,mimeType=mimeType});
			return self.save()
		end
		return true
	end
	function self:deleteResource(resName)
		for idx,res in ipairs(_app.catalog.resources) do	-- lo busca en el catalogo para ver si existe
			if res.src == resName then
				table.remove(_app.catalog.resources,idx);
				if self:save() then
					local fname;
					if string.sub(resName,1,1) == "!" then
						fname = config.resourcePath .. config.pathSeparator .. resName
					else
						fname = getAppPath(_appName) .. config.pathSeparator .. resName
					end
					return tweetyio.remove(fname)
				end
			end
		end
		return false,messages.messageKey('argument_error',{module="application.lua",func="deleteResource",param="resName",value=resName})
	end

	function self:addDatasource(datasourceName,datasourceData)
		local status,err
		for idx,name in ipairs(_app.catalog.datasources) do	-- lo busca en el catalogo para evitar sobreescribir
			if name == datasourceName then
				return false,messages.messageKey('argument_error',{module="application.lua",func="addDatasource",param="resName",value=resName})
			end
		end
		table.insert(_app.catalog.datasources,datasourceName);
		status,err = self:save()
		if status then
			return self:saveDatasourceData(datasourceName,datasourceData)
		else
			return status,err
		end

	end

	function self:deleteDatasource(datasourceName)
		local status,err
		for idx,name in ipairs(_app.catalog.datasources) do	-- lo busca en el catalogo para ver si existe
			if name == datasourceName then
				local dsdata;
				status,dsdata = self:getDatasourceData(datasourceName)
				if not status then
					dsdata = nil; -- En dsdata esta un error que ignoramos. Se nilea para que lo de abajo continue sin problemas
								  -- Debe quitar solo el nombre del catalogo
					LogWarning("Error getting datasource: '%s'",datasourceName)
					--return status,dsdata -- dsdata es el error
				end
				if dsdata and dsdata.description["store"].enabled then
					local ds = require (tweety_modules_prefix .. dsdata.description["store"].type).new(dsdata)
					if ds then
						ds.name = datasourceName;
						ds.appName = _appName;
						ds.inconcert_authUser = inconcert.authUser;
						ds.inconcert_session = inconcert.session;
						if ds and ds.deconfigure then
							status,result = ds:deconfigure(dsdata);
						end
					end
				end
				table.remove(_app.catalog.datasources,idx);
				_app.data.datasources[datasourceName]=nil
				status,err = self:save()
				if status then
					local fname = getAppPath(_appName) .. config.pathSeparator .. datasourceName .. config.datasourceFileExtention
					if not tweetyio.remove(fname) then
						return false,messages.messageKey('delete_file_error',{file=fname})
					else
						return true
					end
				else
					LogDebug("application:deleteDatasource:Error saving application data")
					return status,err
				end
			end
		end
		LogDebug("application:deleteDatasource:Error finding datasource named %s",datasourceName)
		return false,messages.messageKey('argument_error',{module="application.lua",func="deleteDatasource",param="datasourceName",value=datasourceName})
	end


	function self:addForm(formName,content)
		local status,err
		for idx,form in ipairs(_app.catalog.views) do	-- lo busca en el catalogo para evitar grabar cualquier cosa
			if form.name == formName then
				return false,messages.messageKey('argument_error',{module="application.lua",func="addForm",param="formName",value=formName})
			end
		end
		content = json.decode(content)
		table.insert(_app.catalog.views,{name=formName,type=content.formType or "form",class=content.class});
		status,err = self:save();
		if status then
			return self:saveForm(formName,content)
		else
			return status,err
		end
	end
	function self:deleteForm(formName)
		local status,err
		for idx,form in ipairs(_app.catalog.views) do -- lo busca en el catalogo para evitar grabar cualquier cosa
			if form.name == formName then
				status,result = self:getFormData(formName)
				if not status then
					LogWarning("Error getting form: '%s'",formName)
					--return status,result
				end
				if result and result.class then
					local m
					pcall(function()
						m = require (tweety_modules_prefix  .. "formconfig_" .. result.class)
					end)
					if m then
						status,result = m.deconfigure(self,formName,result);
						if not status then
							return status,result
						end
					end
				end

				table.remove(_app.catalog.views,idx);
				_app.data.views[formName]=nil
				status,err = self:save()
				if status then
					local fname = getAppPath(_appName) .. config.pathSeparator .. formName .. config.formFileExtention
					if not tweetyio.remove(fname) then
						return false,messages.messageKey('delete_file_error',{file=fname})
					end
					fname = getAppPath(_appName) .. config.pathSeparator .. formName .. config.formCodeFileExtention
					if not tweetyio.remove(fname) then
						return false,messages.messageKey('delete_file_error',{file=fname})
					end
					return true
				else
					return status,err
				end
			end
		end
		return false,messages.messageKey('argument_error',{module="application.lua",func="deleteForm",param="formName",value=formName})
	end

	function self:renameComponent(componentName,newName,componentType)
		local status,result;
		if componentType == "form" then
			for idx,form in ipairs(_app.catalog.views) do	-- lo busca en el catalogo para evitar grabar cualquier cosa
				if form.name == componentName then
					form.name = newName;
					local oldfname = getAppPath(_appName) .. config.pathSeparator .. componentName .. config.formFileExtention
					local newfname = getAppPath(_appName) .. config.pathSeparator .. newName .. config.formFileExtention
					if not tweetyio.rename(oldfname,newfname) then
						return false,messages.messageKey('rename_file_error',{file=oldfname})
					end
					oldfname = getAppPath(_appName) .. config.pathSeparator .. componentName .. config.formCodeFileExtention
					newfname = getAppPath(_appName) .. config.pathSeparator .. newName .. config.formCodeFileExtention
					if not tweetyio.rename(oldfname,newfname) then
						return false,messages.messageKey('rename_file_error',{file=oldfname})
					end

					return self:save();
				end
			end
			return false,messages.messageKey('argument_error',{module="application.lua",func="renameComponent",param="componentName",value=componentName})
		elseif componentType == "resource" then
			for idx,res in ipairs(_app.catalog.resources) do	-- lo busca en el catalogo para evitar grabar cualquier cosa
				if res.src == componentName then
					res.src = newName;
					local oldfname = getAppPath(_appName) .. config.pathSeparator .. componentName
					local newfname = getAppPath(_appName) .. config.pathSeparator .. newName
					if not tweetyio.rename(oldfname,newfname) then
						return false,messages.messageKey('rename_file_error',{file=oldfname})
					end
					return self:save();
				end
			end
			return false,messages.messageKey('argument_error',{module="application.lua",func="renameComponent",param="componentName",value=componentName})
		elseif componentType == "datasource" then
			for idx,name in ipairs(_app.catalog.datasources) do	--
				if name == componentName then
					_app.catalog.datasources[idx]=newName;
					local oldfname = getAppPath(_appName) .. config.pathSeparator .. componentName .. config.datasourceFileExtention
					local newfname = getAppPath(_appName) .. config.pathSeparator .. newName .. config.datasourceFileExtention
					if not tweetyio.rename(oldfname,newfname) then
						return false,messages.messageKey('rename_file_error',{file=oldfname})
					end
					status,result = self:save();
					if status then
						local dsdata;
						status,dsdata = self:getDatasourceData(newName)
						if not status then
							return status,dsdata -- dsdata es el error
						end
						if dsdata.description["store"].enabled then
							local ds = require (tweety_modules_prefix .. dsdata.description["store"].type).new(json.decode(json.encode(dsdata)));
							if ds then
								dsdata.name = newName;
								dsdata.appName = _appName;
								dsdata.inconcert_authUser = inconcert.authUser;
								dsdata.inconcert_session = inconcert.session;

								ds.name = componentName;
								ds.appName = _appName;
								ds.inconcert_authUser = inconcert.authUser;
								ds.inconcert_session = inconcert.session;
								if ds and ds.configure then
									status,result = ds:configure(dsdata);
								end
							--	if status then
							--		status,result = self:saveDatasourceData(newName,json.encode(dsdata,true))
							--	end
							end
						end
					end
					return status,result
				end
			end
			return false,messages.messageKey('argument_error',{module="application.lua",func="renameComponent",param="componentName",value=componentName})
		else
			return false,messages.messageKey('argument_error',{module="application.lua",func="renameComponent",param="componentType",value=componentType})
		end
	end

	function self:saveForm(formName,content)
		LogInfo("[APPLICATION:save_form] formName: '%s'", formName)
		local status,result
		for _,form in ipairs(_app.catalog.views) do	-- lo busca en el catalogo para evitar grabar cualquier cosa
			if form.name == formName then
				local fname = getAppPath(_appName) .. config.pathSeparator .. formName .. config.formFileExtention
				local code = content.__code or "";
				if content.class then
					local m
					
					pcall(function()
						m = require (tweety_modules_prefix  .. "formconfig_" .. content.class)
					end)
					if m then
						status,oldcontent = self:getFormData(formName)
						if not status then
							oldcontent = nil
						end
						local status,result = m.configure(self,formName,content,oldcontent);
						if not status then
							return status,result
						end
					end
				end
				content.__code = nil;
				local fh = tweetyio.open(fname,"wb")
				if not fh then
					LogDebug("Can't open file %s",fname);
					return false,messages.messageKey('write_file_error',{file=fname})
				end
				fh:write(json.encode(content,true));
				tweetyio.close(fh);

				fname = getAppPath(_appName) .. config.pathSeparator .. formName .. config.formCodeFileExtention
				local fh = tweetyio.open(fname,"wb")
				if not fh then
					LogDebug("Can't open file %s",fname);
					return false,messages.messageKey('write_file_error',{file=fname})
				end
				fh:write(code);
				tweetyio.close(fh);
				return true
			end
		end
		return false,messages.messageKey('argument_error',{module="application.lua",func="saveForm",param="formName",value=formName})
	end


	function self:saveFormParams(formName,params)
		local status,result = self:getFormData(formName)
		if not status then
			return json.encode({status=status,err=result})
		end
		result.testparams = params;
		return self:saveForm(formName,result)
	end

	function self:getDatasourceDef(datasourceName)
		local status,ds = self:getDatasourceData(datasourceName)
		if status then
			return true,ds.description
		end
		return status,ds	-- ds tiene un mensaje de error
	end
	local function validVariableName(name)
		if string.sub(name,1,1):match("[^%a]") then
			return false
		end
		if name:match("[ %%%$!=()/\\#\"\*]") then
			return false
		end

		return true

	end

	function self:buildForm(formName)
		local out = {}
		local usedDatasources={}
		local status,formData,formName = self:getFormData(formName)

		if not status then
			-- TODO: Generar un mensaje de error en javascript
			return json.encode({status=status,err=formData})
		end

		if formData.formType == "report" then
			local newForm = json.decode([[{"type":"ui.panel","childs":[{"type":"ui.reportpanel","id":"reportpanel1","borderWidth":0,"width":560,"y":0,"x":0,"borderColor":"gray","height":235,"runAtStart":true,"reportName":"reporte1"}],"overflow":"visible","id":"l2","name":"l2"}]])
			newForm.fillWidth=true;
			newForm.fillHeight=true;
			newForm.pr=2;
			newForm.pb=2;
			newForm.filters= formData.filters;-- el control de reportes usa esto para generar el dialog de los parametros
			newForm.childs[1].width=nil;
			newForm.childs[1].height=nil;
			newForm.childs[1].fillWidth=true;
			newForm.childs[1].fillHeight=true;
			newForm.childs[1].reportName=formName;

			formData = newForm;
		end

--		formData.width=nil;
--		formData.height=nil;
--		formData.fillWidth=true;
--		formData.fillHeight=true;
		local formId = formName;
		formName = string.gsub(formName, "%s+", "_")
		local res= {"/*GENERATED CODE */icc.cache.",formName,"=function(icc,environment){designer=false;var args=environment.arguments;window._form_env=environment;"}
		local code = formData.__code
		local layoutcode = formData.__layoutcode;
		formData.__code = nil;
		formData.__layoutcode = nil
		buildDatasourceList(formData,usedDatasources);

		for _,name in pairs(usedDatasources) do
			local status,ds = self:getDatasourceData(name,true)
			if status then
				ds.description.name = name;
				if validVariableName(name) then
					table.insert(out,table.concat{
						"if(!environment.datasources['",name,"'])",
						"var ",name,"=new icc.ui.datasource","(",json.encode(ds.description),",environment","); ",
						"else ",
						"var ",name,"=environment.datasources['",name,"'];"
					})
				else
					table.insert(out,table.concat{
						"if(!environment.datasources['",name,"'])",
						"environment.datasources['",name,"']=new icc.ui.datasource","(",json.encode(ds.description),",environment","); ",
					})
				end
			else
				-- TODO : Loguear el error ignorado de alguna manera

			end
		end
		formData.id = formId;
		formData.name = formName;
		buildControl(formData,out,true);

		table.insert(res,table.concat(out,";"))
		table.insert(res,";var self=");table.insert(res,formName);


		table.insert(res,";");

		if formData.formSubType == "dialog" then
			local fh = io.open(config.resourcePath .. config.pathSeparator .. "dialog-runtime-code-min.js","rb");
			if fh then
				table.insert(res,fh:read("*a"));
				io.close(fh);
			end
		end

		if formData.class then
			local fh = io.open(config.resourcePath .. config.pathSeparator .. formData.class .. "-code-min.js","rb");
			if fh then
				layoutcode = fh:read("*a");
				io.close(fh);
			end

		end


		if layoutcode then
			layoutcode = self:processJavascriptCode(layoutcode);
			table.insert(res,layoutcode);
		end
		if code then
			code = self:processJavascriptCode(code);
			table.insert(res,code);
		end

		table.insert(res,"\n// END USER CODE\n");
		table.insert(res," return " .. formName ..";");
		table.insert(res,"};");
		return table.concat(res);
	end

	function self:getForm(formName)
		local status, formData, formName = self:getFormData(formName)
		if not status then
			return { status = false, err = formData }
		end
		return { status = true, data = formData }
	end

	local __chatForms = {
		"login",
		"preSurvey",
		"chat",
		"___chatv3",
		"webcall",
		"___webcall",
		"Submit"
	}
	function self:getChatFirstForm()
		for _, formName in pairs(__chatForms) do
			local result = self:getForm(formName)
			if result.status then
				return json.encode(result)
			end
		end
		return json.encode({ status = false, err = "initial form not found" })
	end

	function self:buildInlineForm(formName)
		local out = {}
		local usedDatasources={}
		local status,formData = self:getFormData(formName)
		if not status then
			-- TODO: Generar un mensaje de error en javascript
			return false,formData
		end

		local res= {"/* INLINE GENERATED FORM */var environment = {};"}
		local code = formData.__code
		local layoutcode = formData.__layoutcode;
		formData.__code = nil;
		formData.__layoutcode = nil;
		formData.name = formName;
		formData.id = formName;
		formData.width=nil;
		formData.height=nil;
		formData.fillWidth=true;
		formData.fillHeight=true;
		buildDatasourceList(formData,usedDatasources);

		for _,name in pairs(usedDatasources) do
			local status,ds = self:getDatasourceData(name)
			if status then
				ds.description.name = name;
				table.insert(out,table.concat{
					"if(!environment.datasources['",name,"']) ",
					"var ",name,"=new icc.ui.datasource","(",json.encode(ds.description),",environment",");",
					"else ",
					"var ",name,"=environment.datasources['",name,"'];"
				})
			else
				-- TODO : Loguear el error ignorado de alguna manera
			end
		end
		buildControl(formData,out,true);

		table.insert(res,table.concat(out,";"))
--		table.insert(res,";\n//  End form layout\n\n\n");

		if layoutcode then
			layoutcode = self:processJavascriptCode(layoutcode);
			table.insert(res,layoutcode);
		end
		if code then
			code = self:processJavascriptCode(code);
			table.insert(res,code);
		end
		return true,table.concat(res);
	end

	function self:buildInlineFrame(formName,frameName)
		local out = {}
		local usedDatasources={}
		local status,formData = self:getFormData(formName)
		if not status then
			-- TODO: Generar un mensaje de error en javascript
			return false,formData
		end

		local res= {"/* INLINE GENERATED FRAME */var environment = {};"}
		local function scanframe(frameName,frame)
			if frame.id == frameName then
				return frame
			end
			if frame.childs and #frame.childs >0 then
				local res
				for _,frame in ipairs(frame.childs) do
					res = scanframe(frameName,frame)
					if res then
						return res
					end
				end
			end
		end
		local frame = scanframe(frameName,formData);
		if frame then
			frame.borderWidth=0;
			frame.x = nil;
			frame.y = nil;
			buildControl(frame,out,false);
		else
			return false,messages.messageKey('argument_error',{module="application.lua",func="buildInlineFrame",param="frameName",value=frameName})
		end

		table.insert(res,table.concat(out,";"))
--		table.insert(res,";\n//  End frame layout\n\n\n");
		return true,table.concat(res);
	end

	function self:getCatalog()
		return _app.catalog
	end

	function self:setCatalog(newCatalog)
		_app.catalog = newCatalog;
	end

	function self:HasCaptchaConfigured()
		local _, appInfo = self:getInformation()
		local catalog = _app.catalog
		local appName = appInfo.workspaceName
		local vccName = self.vccName

		LogInfo("[APPLICATION.HasCaptchaConfigured] Validating if the App '%s' VCC '%s' has CAPTCHA Configured..", appName, vccName)

		if not catalog then
			LogWarning("[APPLICATION.HasCaptchaConfigured] 1 - Error retrieve App '%s' Catalog. Catalog: %s'.", formName, appName, vccName, catalog)
			return false
		end
	
		local function FindRecursiveCaptchaCtrl (ctrl)
			if type(ctrl) ~= "table" then
				return false
			end
			if (ctrl.type or ""):lower() == "ui.captcha" then
				return ctrl
			end
			local foundCaptchaCtrl
			if type(ctrl.childs) == "table" then
				for _, childCtrl in pairs(ctrl.childs) do
					foundCaptchaCtrl = FindRecursiveCaptchaCtrl(childCtrl)
					if foundCaptchaCtrl then
						break
					end
				end
			end
			return foundCaptchaCtrl
		end
	
		--LogDebug("[APPLICATION.HasCaptchaConfigured] ---> app '%s' vcc: '%s'. Catalog: '%s'.", appName, vccName, json.encode(catalog or {}))
		for _, view in ipairs(catalog.views or {}) do
			--LogDebug("[APPLICATION.HasCaptchaConfigured] ---> view: '%s', .", json.encode(view))
			if (view.type or ""):lower() == "form" then
				local status, formData = self:getFormData(view.name)
				--LogDebug("[APPLICATION.HasCaptchaConfigured] -----> frmName: '%s', status: '%s', formData: %s.", view.name, status, json.encode(formData or {}))
				local captchaCtrl = FindRecursiveCaptchaCtrl(formData)
				if captchaCtrl then
					LogInfo("[APPLICATION.HasCaptchaConfigured] + The App '%s' in Vcc '%s' HAVE CAPTCHA CONFIGURED.", appName, vccName)
					return true
				end
			end
		end

		LogInfo("[APPLICATION.HasCaptchaConfigured] - The App '%s' in Vcc '%s' does not have CAPTCHA configured.", appName, vccName)
		return false
	end

	function self:buildExportBundle()
		local status,result
		local catalog = self:getCatalog()


		for _,viewEntry in ipairs(catalog.views) do
			status,result = self:loadFormFile(viewEntry.name)
			if not status then
				LogDebug("application export_fix: %s",json.encode(result))
--				return status,result
			end
		end



		for _,dsname in ipairs(catalog.datasources) do
			status,result = self:loadDatasourceFile(dsname)
			if not status then
				LogDebug("application export_fix: %s",json.encode(result))

--				return status,result
			end
		end



		for _,resourceEntry in ipairs(catalog.resources) do
			status,result = self:readResource(resourceEntry.src,true)
			_app.data.resources[resourceEntry.src] = base64.encode(_app.data.resources[resourceEntry.src])
			if not status then
				LogDebug("application export_fix: %s",json.encode(result))

				--return status,result
			end
		end
		_app.exportedBy = inconcert.authUser.userName;
		_app.exportedAt = os.date("%Y-%m-%d %H:%M:%S")

		_app.name = _appName
		_app.magic = config.appbundle_magic
		_app.bundleVersion = 2;

		return true,json.encode(_app);
	end	
	function self:readPrivateData()
		local status,result
		local catalog = self:getCatalog()
		local res = {}
		for _,dsname in ipairs(catalog.datasources) do
			status,result = self:getDatasourceData(dsname)
			if not status then
				return status,result
			else
				if result.type == "mssql" then	-- solo los de sql tienen ip configurable hasta ahora
					result.private.userPassword = "***********"; -- pisa el password
					res[dsname] = result.private
				end
			end
		end
		return true,res
	end
	function self:savePrivateData(data)
		local status,result
		for dsname,privatedata in pairs(data) do
			status,result = self:getDatasourceData(dsname)
			if not status then
				return status,result
			else
				local private = result.private or {}
				local userPassword = private.userPassword;
				if privatedata.userPassword == "***********" then
					privatedata.userPassword = userPassword;
				end
				result.private = privatedata
				status,result = self:saveDatasourceData(dsname,json.encode(result,true))
				if not status then
					return status,result
				end
			end
		end
		return true
	end
	function self:initFromBundle ( fileName)
		local status
		local fh = tweetyio.open(fileName,"rb")
		local txt
		if not fh then
			return false,messages.messageKey('read_file_error',{file=fileName});
		end
		txt = fh:read("*a")
		tweetyio.close(fh);
		status,_app = pcall(json.decode,txt)
		if not status or _app.magic ~= config.appbundle_magic then
			return false,messages.messageKey('invalid_binary_format',{file=fileName});
		end
		return true
	end
	function self:getInformation()
		if _app then
			return true,{
				workspaceName = _appName,
				name=_app.name or _appName,
				exportedAt=_app.exportedAt,
				exportedBy=_app.exportedBy
			}
		else
			return false,messages.messageKey('invalid_operation',{});
		end
	end

	function self:importFromBundle (fileName,appName,overwrite,hidden)

		local status,result
		local function deconfigure_me()
				LogDebug("Deconfigure me %s %s",tostring(self.vccName),tostring(appName))
				local app = new({vccName=self.vccName})
				local status,result = app:initialize(appName)
				if status then
					status,result = app:deconfigure();
				end
				return status,result;
		end

		status,result = self:initFromBundle(fileName)
		_appName = appName;
		if status then
			local app_path = config.usersPath .. config.pathSeparator .. self.vccName .. config.pathSeparator .. appName
			local res,err = tweetyio.mkdir(app_path)
			if not res  and not overwrite then -- si se puede sobreescribir puede que
												-- el mkdir falle ya que el dir existe
				if overwrite then
					status,result = deconfigure_me()
					if not status then
						if result.code == 1040 or result.code == 1001 then -- database error o database connection error
							return status,result
						else
							return status,messages.messageKey('mkdir_error',{path=app_path}) -- retornamos el mkdir error que es lo mas seguro.
						end
					end
				else
					return false,messages.messageKey('mkdir_error',{path=app_path})
				end
			end
			local catalog = self:getCatalog()
			for _,viewEntry in ipairs(catalog.views) do
				if _app.data.views[viewEntry.name] then
					status,result = self:saveForm(viewEntry.name,_app.data.views[viewEntry.name])
					if not status then
						return status,result
					end
				else
					LogDebug("application_export_fix: Can't read view data %s",viewEntry.namesu)
				end
			end
			for _,dsname in ipairs(catalog.datasources) do
				local dsdata = _app.data.datasources[dsname]
				if dsdata then
					if dsdata.description["store"].enabled then
						local ds = require (tweety_modules_prefix .. dsdata.description["store"].type).new(dsdata)
						if ds then
							ds.name = dsname;
							ds.appName = appName;
							ds.inconcert_authUser = inconcert.authUser;
							ds.inconcert_session = inconcert.session;
							if ds and ds.configure then
								status,result = ds:configure(dsdata);
							end
						end
					end
					if status then
						status,result = self:saveDatasourceData(dsname,json.encode(dsdata,true))
					end
					if not status then
						return status,result
					end
				else
					LogDebug("application_export_fix: Can't read datasource data from bundle %s",dsname)
				end
			end
			for _,resourceEntry in ipairs(catalog.resources) do
				if _app.data.resources[resourceEntry.src] then
					if _app.bundleVersion == 2 then
						status,result = self:saveResource(resourceEntry.src,base64.decode(_app.data.resources[resourceEntry.src]))
					else
						status,result = self:saveResource(resourceEntry.src,_app.data.resources[resourceEntry.src])

					end
					if not status then
						return status,result
					end
					LogDebug("application_export_fix: Can't read resource data from bundle %s",resourceEntry.src)

				end
			end
			_app.data = {
				views={},
				datasources={},
				images={},
				resources={}
			} -- descarta los datos adjuntos
			fh = tweetyio.open(getAppPath(_appName) .. config.pathSeparator .. config.mainAppFile,"wb");
			if not fh then
				LogDebug("Can't open %s ",getAppPath(_appName) .. config.pathSeparator .. config.mainAppFile)
				return false,messages.messageKey('write_file_error',{file=getAppPath(_appName) .. config.pathSeparator .. config.mainAppFile})
			end
			_app.name = nil
			_app.exportedAt =nil
			_app.exportedBy = nil
			fh:write(json.encode(_app,true));
			tweetyio.close(fh);

			local catalog_entry = {
				name=appName,
				version=0,					-- si es una app vieja que exporte con la fecha de hoy
				created_at=_app.created_at or LuaOrm.Date():toString(),--os.date("%Y-%m-%d %H:%M:%S"),
				status="untracked",
				hidden=hidden
			}
			fh = tweetyio.open(app_path .. config.pathSeparator .. config.catalogFile,"wb");
			if not fh then
				return false,messages.messageKey('write_file_error',{file=app_path .. config.pathSeparator .. config.catalogFile})
			end
			fh:write(json.encode(catalog_entry,true))
			tweetyio.close(fh);
			catalog_entry.created_at = string.gsub(LuaOrm.Date(catalog_entry.created_at):toDifferentTimezoneString(inconcert.authUser.timezone),"%.%d*$","");

			result = catalog_entry;
		end
		return status,result
	end
	function self:configure()
		local status,result;
		status = true;
		local catalogApp = self:getCatalog();
		local datasources = catalogApp.datasources;
		for _,dsname in ipairs(datasources) do
			local dsdata;
			status,dsdata = self:getDatasourceData(dsname)
			if not status then
				dsdata = nil-- dsdata es el error
			end
			if dsdata and dsdata.description["store"].enabled then
				local ds = require (tweety_modules_prefix .. dsdata.description["store"].type).new(dsdata)
				if ds then
					ds.name = dsname;
					ds.appName = _appName;
					ds.inconcert_authUser = inconcert.authUser;
					ds.inconcert_session = inconcert.session;
					if ds and ds.configure then
						status,result = ds:configure(dsdata);
						if not status then
							return status,result
						end
					end
				end
			end
		end
		local views = catalogApp.views;
		for _,form in ipairs(views) do
			local formData;
			status,formData = self:getFormData(form.name)
			if not status then

				formData = nil-- formData es el error
			end
			if formData and formData.class then
				local m
				pcall(function()
					m = require (tweety_modules_prefix  .. "formconfig_" .. formData.class)
				end)
				if m then
					status,result = m.configure(self,form.name,formData);
					if not status then
						return status,result
					end
				end
			end
		end

		return status,result
	end

	function self:deconfigure()
		local cpyarray = function (ar)
			local res={}
			for k,v in ipairs(ar) do
				table.insert(res,v)
			end
			return res;
		end
		local status,result;
		status = true;
		local catalogApp = self:getCatalog();
		local datasources = cpyarray(catalogApp.datasources);
		for _,dsname in ipairs(datasources) do
			status,result = self:deleteDatasource(dsname)
			if not status then
				return status,result;
			end
		end
		local views = cpyarray(catalogApp.views)
		for _,form in ipairs(views) do
			status,result = self:deleteForm(form.name)
			if not status then
				return status,result;
			end
		end
		return status,result;
	end
	function self:renderReportXLS(processId,start_page,end_page)
		local renderControl;
		local function  rmvdup(tb)
			local tbinsert = table.insert
			local r = {}
			local c=-10e9

			for _,v in pairs(tb) do
				if c ~= v  then
					tbinsert(r,v)
					c=v
				end
			end
			return r
		end
		local function make2darray (rows,cols)
			local r={rows=rows,cols=cols}

			for j=1,rows do
				local c={}
				for i=1,cols do
					c[i]={}
				end
				r[j] = c
			end
			return r
		end
		local function posctrl (ctrl,scr)
			local c
			for x=1,scr.cols do
				c=scr[1][x]
				if ctrl.x  == c.x then
					for y=1,scr.rows do
						c=scr[y][x]

						if ctrl.y == c.y then

							if not c.nc and not c.ctrl then
								c.ctrl = ctrl
								local j, i = 0,0
								while scr[y][x+i]  and  scr[y][x+i].nc ~=true and  (c.ctrl.x + c.ctrl.width) > scr[y][x+i].x do
								    j=0;
									while scr[y+j] and scr[y+j][x]  and scr[y+j][x+i].nc ~= true and (c.ctrl.y + c.ctrl.height) > scr[y+j][x+i].y do
										scr[y+j][x+i].nc=true

										scr[y+j][x+i].ncctrl = ctrl;
										j=j+1

									end
									i=i+1
								end
								c.colspan = i
								c.rowspan = j
								return true
							else

								return false,c.ncctrl
							end
						end
					end
				end
			end
		end

		local function makeGrid(ctrls)

			local function generateStoringTable (ctrls)

				local v={}
				local h={}
				local l_tbinsert = table.insert
				l_tbinsert(h,0)
				l_tbinsert(v,0)

				for k,ctrl in ipairs(ctrls) do

					l_tbinsert(h,ctrl.x)
					l_tbinsert(h,(ctrl.x + (ctrl.width or 0)) )
					l_tbinsert(v,ctrl.y )
					l_tbinsert(v,(ctrl.y + (ctrl.height or 0)))

				end

				table.sort(v)
				table.sort(h)
				v = rmvdup(v)
				h = rmvdup(h)
				local rows = #v
				local cols = #h
				local scr=make2darray(rows,cols)
				for j,y in ipairs(v) do
					for i,x in ipairs(h) do
						scr[j][i].x = x
						scr[j][i].y = y

					end
				end
				return scr;
			end
			local rb = false;
			local scr;
			repeat
				rb = false;
				scr = generateStoringTable(ctrls)
				local r,c
				for _,ctrl in ipairs(ctrls) do
					r,c = posctrl(ctrl,scr)

					if not r then
						-- corremos este control hacia el costado

						ctrl.x = c.x+c.width
						rb = true
					end

				end
			until not rb
			return scr;
		end



		local function renderSheet(pageNum)
			local ctrllist = {}
			local fname = config.tempPath .. config.pathSeparator .. processId .. "_" .. "page_" .. tostring(pageNum)
			local fh = tweetyio.open(fname,"rb");
			local txt = fh:read("*a");
			tweetyio.close(fh);
			local cnt = json.decode(txt);

			local function unhierarchize(parentx,parenty,ctrl)

				ctrl.x = ctrl.x + parentx
				ctrl.y = ctrl.y + parenty
				if ctrl.childs then
					for _,c in ipairs(ctrl.childs) do
						unhierarchize(ctrl.x,ctrl.y,c)
					end
				else
					if ctrl.type == "ui.label" then
						table.insert(ctrllist,ctrl)
					end
				end
			end
			for _,ctrl in ipairs(cnt.content) do
				unhierarchize(0,0,ctrl)
			end
--[[
			for k,ctrl in ipairs(ctrllist) do
				ctrl.x = ctrl.x - math.fmod(ctrl.x,10)
				ctrl.y = ctrl.y - math.fmod(ctrl.y,10)
				ctrl.width = ctrl.width - math.fmod(ctrl.width,20)
				if ctrl.width == 0 then
					ctrl.width = 20
				end

				ctrl.height = ctrl.height - math.fmod(ctrl.height,20)
				if ctrl.height == 0 then
					ctrl.height = 20
				end

			end
--]]
			local sheet = excel.worksheet{name="sheet " .. pageNum,visibleGridLines=false}
			local scr = makeGrid(ctrllist)
			local c;

			for i=1,scr.rows do
				for j=1,scr.cols do
					c = scr[i][j]
					if c.ctrl then
						if c.ctrl.type == "ui.label" then
							sheet:cell(i,j,c.ctrl.caption,c.ctrl.font,c.ctrl.foreColor,c.ctrl.backColor,c.colspan,c.rowspan)
						end
					end
				end
			end
			local acum=0;
			local cinfo = {}
			for x=1,scr.cols do
				c=scr[1][x]
				table.insert(cinfo,(c.x-acum)/5)
				acum = c.x
			end
			for x,c in ipairs(cinfo) do
				sheet:columnWidth(x-1,x-1,tonumber(string.format("%d",(c)*1)));
			end
			acum=0;
			for y=1,scr.rows do
				c = scr[y][1]
				sheet:rowHeight(y-1,y-1,tonumber(string.format("%d",(c.y-acum)*1)));
				acum = c.y
			end
			return sheet;
		end
		local session = tweetyio.session();
		if not session.tweetyprocess then session.tweetyprocess = {} end
		local process = session.tweetyprocess[processId]
		session:release();

		if process then
			local data = process.data;
			start_page = start_page or 1
			if not end_page then
				end_page = start_page
			elseif end_page == -1 then
				end_page = data.pageNum
			end
			if data.pageNum >=start_page then
				local workbook = excel.excelDocument{}

				for page=start_page,end_page do
					workbook:addWorksheet(renderSheet(page,end_page));
				end
				return workbook:stream()
			else
				return ""
			end
		else
			return ""
		end
	end



	local units,scale,fontscale


	-- local scale = 1
	-- local units = "px"

	local pagesizes ={
		A4=1258				--297 / scala   -- largo de un A4 - 30 px por seguridad
	}

	function self:renderReportHTML(processId,start_page,end_page,prn)
--		if not prn then
--			scale = 0.275	-- esto es para mejorar el preview que se vea mas grande
--		--	scale = 1
--			fontscale = 1.00
--		else
--			fontscale = 0.95
--		end
		local pageOffset = 0;
		local function reportError(msg)
			msg = msg or ""
			return string.format([=[
				<html>
				<p>%s<p>
				</html>
			]=],msg)
		end
		local renderControl;
		local function styleStr ( ctrl)
			local res = {}
			local border = false
			for k,v in pairs(ctrl) do
				if k == "x" then
					table.insert(res,string.format("left:%.1f%s",v*scale,units))
				elseif k == "y" then
					table.insert(res,string.format("top:%.1f%s",v*scale,units))
				elseif k == "width" then
					local bz =0;
					if ctrl['borderLeft'] then
						bz = bz + (ctrl['borderWidth'] or 0)
					end
					if ctrl['borderRight'] then
						bz = bz + (ctrl['borderWidth'] or 0)
					end
					if not ctrl['borderLeft'] and not ctrl['borderRight'] and not ctrl['borderTop'] and not ctrl['borderBottom'] then
						bz = (ctrl['borderWidth'] or 0) * 2;
					end
					table.insert(res,string.format("width:%.1f%s",((v-bz)*scale),units))

				elseif k == "height" then
					local bz =0;

					if ctrl['borderTop'] then
						bz = bz + (ctrl['borderWidth'] or 0)
					end
					if ctrl['borderBottom'] then
						bz = bz + (ctrl['borderWidth'] or 0)
					end
					if not ctrl['borderTop'] and not ctrl['borderBottom'] and not ctrl['borderLeft'] and not ctrl['borderRight'] then
						bz = (ctrl['borderWidth'] or 0) * 2;
					end

					table.insert(res,string.format("height:%.1f%s",((v-bz)*scale),units))

				elseif k == "foreColor" then
					table.insert(res,string.format("color:%s",v))
				elseif k == "backColor" then
					table.insert(res,string.format("background:%s",v))

				elseif (k == "borderWidth" or k == "borderTop" or k == "borderBottom" or k == "borderLeft"  or k == "borderRight" or k == "borderColor") and not border then
					if ctrl["borderTop"] then
						table.insert(res,string.format("border-top:%.1f%s solid %s",(ctrl["borderWidth"] or 0) * scale,units,ctrl["borderColor"] or "transparent"))
					end
					if ctrl["borderBottom"] then
						table.insert(res,string.format("border-bottom:%.1f%s solid %s",(ctrl["borderWidth"] or 0) * scale,units,ctrl["borderColor"] or "transparent"))
					end
					if ctrl["borderLeft"] then
						table.insert(res,string.format("border-left:%.1f%s solid %s",(ctrl["borderWidth"] or 0) * scale,units,ctrl["borderColor"] or "transparent"))
					end
					if ctrl["borderRight"] then
						table.insert(res,string.format("border-right:%.1f%s solid %s",(ctrl["borderWidth"] or 0) * scale,units,ctrl["borderColor"] or "transparent"))
					end
					if not ctrl["borderTop"] and not ctrl["borderBottom"] and not ctrl["borderLeft"] and not ctrl["borderRight"] then
						table.insert(res,string.format("border:%.1f%s solid %s",(ctrl["borderWidth"] or 0) * scale,units,ctrl["borderColor"] or "transparent"))
					end


					border = true

				elseif k == "align" then
					table.insert(res,string.format("text-align:%s",v))

				elseif k == "font" then
					table.insert(res,string.format("font-family:%s",v.family))
					table.insert(res,string.format("font-weight:%s",v.weight))
					table.insert(res,string.format("font-size:%.2f%s",(v.size*scale)*fontscale,units))
				elseif k == "border" or k=="position" or k == "border-top" or k == "border-left" or k == "border-bottom" or k == "border-right" or k=="overflow" or k=="white-space" then
					table.insert(res,string.format("%s:%s",k,v))
				end
			end
			return "style=\"" .. table.concat(res,";") .. "\""
		end

		local function renderPanel(ctrl)
			local content = {}
			ctrl.borderColor = ctrl.borderColor or "#BFBFBF"
			if ctrl.childs then
				for _,c in ipairs(ctrl.childs) do
					table.insert(content,renderControl(c))
				end
			end
			return string.format("<div %s>%s</div>",styleStr(ctrl),table.concat(content,"\r\n"))
		end

		local function renderLabel(ctrl)
			local v
			local caption = ctrl.caption or ""
			ctrl.font = ctrl.font or {}
			ctrl.font.family = ctrl.font.family or 'arial,sans-serif'
			ctrl.font.size = ctrl.font.size or 11
			ctrl.font.weight = ctrl.font.weight or "normal"
--			LogDebug("Renderlabel %s",ctrl.format or "nada");
			if not ctrl.multiLine then
				ctrl['white-space']="nowrap";
				ctrl['overflow']="hidden";
			end
			if ctrl.format == "numeric" or ctrl.format == "fixed" then
				v = tonumber(ctrl.caption)
				local fmts = "%." .. (ctrl.decimals or 2) .. "f"
				if v then
					caption = string.format(fmts,v);
				end
			elseif ctrl.format == "integer" then
				v = tonumber(ctrl.caption)
				if v then
					caption = string.format("%d",v);
				end
			elseif ctrl.format == "currency" then
				local function comma_value(amount)
				  local formatted = amount
				  while true do
					formatted, k = string.gsub(formatted, "^(-?%d+)(%d%d%d)", '%1,%2')
					if (k==0) then
					  break
					end
				  end
				  return formatted
				end
				v = tonumber(ctrl.caption)
				if v then
					caption = comma_value(string.format("%.2f",v));
				end
			elseif ctrl.format == "date" then
				caption = string.gsub(caption,"(%d%d%d%d)-(%d%d)-(%d%d)(.*)","%3/%2/%1")
			elseif ctrl.format == "datetime" then
				caption = string.gsub(caption,"(%d%d%d%d)-(%d%d)-(%d%d) (%d%d):(%d%d):(%d%d)(.*)","%3/%2/%1 %4:%5:%6")
			elseif ctrl.format == "percent" then
				v = tonumber(ctrl.caption)
				if v then
					v = tostring(v)

					local pos = string.find(v,".",0,true)
					if pos then
						caption = string.sub(v,1,pos+(tonumber(ctrl.decimals) or 1)) .. "%"
					else
						caption = v .. "%"
					end

				end
--[[
				local fmts = "%." .. (ctrl.decimals or 1) .. "f%%"
				if v then
					caption = string.format(fmts,v);
				end
--]]
			end
			if ctrl.align == "right" then
				caption = caption .. "&nbsp;"
			end

			return string.format("<div %s ><div style=\"display: table;width:100%%;height:100%%\"><div style=\"display: table-cell; vertical-align: middle;\">%s</div></div></div>",styleStr(ctrl),caption or "")
		end
		local function renderHLine(ctrl)
			ctrl['border-top']=string.format("%dpx %s %s",ctrl.weight or 1,ctrl.penStyle or "solid",ctrl.foreColor or "black")
			ctrl['border-left']="none"
			ctrl['border-bottom']="none"
			ctrl['border-rigth']="none"
			ctrl.height = 0;
			return string.format("<div %s /></div>",styleStr(ctrl))
		end
		local function renderVLine(ctrl)
			ctrl['border-right']=string.format("%dpx %s %s",ctrl.weight or 1,ctrl.penStyle or "solid",ctrl.foreColor or "black")
			ctrl['border-left']="none"
			ctrl['border-bottom']="none"
			ctrl['border-top']="none"
			ctrl.width = 0;
			return string.format("<div %s /></div>",styleStr(ctrl))
		end
		local function renderImage(ctrl)
			local src;
			if ctrl.image then
				if string.sub(ctrl.image,1,1) == "!" then
					ctrl.src = string.format("resources/images/%s",string.sub(ctrl.image,2))
				else
					ctrl.src = string.format("designer/applications/%s/resources/%s",_appName,ctrl.image)
				end
			end
			return string.format("<img %s src='%s' >",styleStr(ctrl),ctrl.src or "")
		end
		renderControl = function(ctrl)
			ctrl.position = "absolute"
			if ctrl.type == "ui.panel" then
				if ctrl['band-type'] then -- and ctrl['band-type']~= 'pagefooter' then
					ctrl.y = nil
					ctrl.x = nil
					ctrl.position="relative";
				end
				return renderPanel(ctrl)
			elseif ctrl.type == "ui.label" then
				return renderLabel(ctrl)
			elseif ctrl.type == "ui.image" then
				return renderImage(ctrl)
			elseif ctrl.type == "ui.hline" then
				return renderHLine(ctrl)
			elseif ctrl.type == "ui.vline" then
				return renderVLine(ctrl)
			else
				LogDebug("Ctrl type desconocido %s %s",ctrl.type or "nil" ,ctrl.id)
			end
		end
		local function renderPage(pageNum,endPage)
			local res = {}
			local fname = config.tempPath .. config.pathSeparator .. processId .. "_" .. "page_" .. tostring(pageNum)
			local fh = tweetyio.open(fname,"rb");
			local txt = fh:read("*a");
			tweetyio.close(fh);
			local cnt = json.decode(txt);
			scale = cnt.scale;
			units = cnt.units;
			fontscale = cnt.fontscale
			if prn then
				scale = cnt.printerScale;
				fontscale = cnt.printerFontScale;
			end

			for _,ctrl in ipairs(cnt.content) do
				table.insert(res,renderControl(ctrl))
			end
--			table.insert(res,"</hr>");
			if pageNum < endPage then
				table.insert(res,string.format("<hr style=\"border:0px solid;width:1px;height:1px;page-break-before: always\"/>",pageOffset*scale,units))
			end
			pageOffset = pageOffset + cnt.pageSize;
			return table.concat(res,"\r\n");

		end

		local session = tweetyio.session();
		if not session.tweetyprocess then session.tweetyprocess = {} end
		local process = session.tweetyprocess[processId]
		session:release();
		if process then


			local data = process.data;
			start_page = start_page or 1

			if not end_page then
				end_page = start_page
			elseif end_page == -1 then
				end_page = data.pageNum
			end
			if data.pageNum >=start_page then
				local code = ""
				if prn == 1 then
					code = "onload='window.print();'"
				end
				local result = {"<html><head><title>","</title><meta http-equiv=\"Content-Type: text/html; charset=utf-8\"></head>",string.format("<body style=\"box-sizing: border-box\" %s>",code)}

				for page=start_page,end_page do
					table.insert(result,renderPage(page,end_page));
				end
				table.insert(result,"</body></html>");
				return table.concat(result,"\r\n");
			else
				return reportError("Invalid page range")
			end
		else
			return reportError("Invalid processId");
		end
	end

	function self:ReportProcess(processId,inputdata,formData)
		local reportheaders={}
		local pageheaders={}
		local groupheaders={}
		local reportdetails={}
		local groupfooters={}
		local pagefooters={}
		local reportfooters={}
		local pageData={}
		local grandtotals = {}
		local formulas={}
		local topMargin = 0; -- TODO iniciar de config del reporte
		local bottomMargin = 0;
		local pageSize
		local pageNum=0;

		if inputdata.pageSize and not tonumber(inputdata.pageSize) then
			pageSize = pagesizes[pageSize] or pagesizes.A4
		else
			pageSize = pagesizes.A4
		end
		local pageOffset=topMargin; --
		local now = string.gsub(LuaOrm.Date(LuaOrm.Date():toString()):toDifferentTimezoneString(inconcert.authUser.timezone),"%.%d*$","")
		local maxPageOffset = pageSize - bottomMargin;

		local pageHeader=true;
		local pageFooter=true;
		local datasources={}
		local session = tweetyio.session();
		process = session.tweetyprocess[processId];
		session:release();

		local function scanControlData(parentBand,ctrl)
			if ctrl.datasource and ctrl.field then
				datasources[ctrl.datasource] = true;
				if ctrl.type == "grandtotal" then
					grandtotals[ctrl.id]={
						datasource=ctrl.datasource,
						field=ctrl.field,
						['band-type']=parentBand['band-type'],
						bandId=parentBand.id,
						fieldName=ctrl.id,
						value=0
					}
					ctrl.type = "calculatedField";
					ctrl.fieldName = ctrl.id;
				end
			end
			if ctrl.type == "formula" then
				formulas[ctrl.id]={
					fieldName=ctrl.id,
					expresion=ctrl.expresion,
					value = tonumber(ctrl.initialValue) or ctrl.initialValue,
					initialValue=ctrl.initialValue,
					bandId=parentBand.id
				}
				ctrl.type = "calculatedField";
				ctrl.fieldName = ctrl.id;
			end


			if ctrl.childs then
				for _,cl in ipairs(ctrl.childs) do
					scanControlData(parentBand,cl)
				end
			end
		end
		local function instanceDatasources()
			for name,_ in pairs(datasources) do
				LogDebug ( "build datasource %s ",self.vccName);
				local status,ds = datasource.buildDatasource(self.vccName,_appName,name,"retrieve")
				if status then
					if ds.description.retrieve.enabled and ds.cursor then	-- si se puede hacer retrieve ,y ademas tiene el nuevo metodo cursor lo damos como valido
						datasources[name]=ds;
					else
						datasources[name]=nil
					end
				else
					return false,ds
				end
			end
			for name,ds in pairs(datasources) do

				if ds.description.relationships then
					for _,relationship in ipairs(ds.description.relationships) do
						local tgtds = datasources[relationship.datasourceName]
						if tgtds then
							ds.childs = ds.childs or {}
							tgtds.child = true;

							table.insert(ds.childs,{ds=tgtds,fields=relationship.fields});

						end
					end
				end
			end
			for name,ds in pairs(datasources) do
				if ds.child then
					datasources[name]=nil
				else
					LogDebug(" datasource input %s",name)
				end
			end
			return true
		end
		local function getJoinedData(ds,rec,cdata)
			if ds.childs then
				for _,cds in ipairs(ds.childs) do
					local q = {}
					for _,field in ipairs(cds.fields) do
						if field.source and field.target then
							table.insert(q,{name=field.target,value=rec[field.source]})
						end
					end
					local status,ite = cds.ds:cursor({parameters=q})
					if  status then
						if not ite.eof() then
							cdata[cds.ds.name]=ite.next();
							getJoinedData(cds.ds,cdata[cds.ds.name],cdata);
						end
					end
				end
			end
		end
		local function clearJoinedData(ds,cdata)
		end
		local function evalexpr(expr,environment)
			if type(expr) == "string" then
				expr = string.gsub(expr,"!=","~=")
				expr = string.gsub(expr,"<>","~=")
			end
			environment.page = pageNum
			environment.now = now;

			environment = environment or {}
			environment.print = print
			environment.tostring = tostring
			environment.tonumber = tonumber
			environment.str = tostring
			environment.num = tonumber
			environment.type = type
			environment.iif = function(cond,rtrue,rfalse)
				if cond == true then
					return rtrue
				else
					return rfalse
				end
			end
			environment.getmetatable = getmetatable
			local proc,err = loadstring("return " .. expr)
			if proc then

				local mt = getmetatable("")
				local __add = mt.__add
				mt.__add = function(a,b)
					if tonumber(a) and tonumber(b) then
						return tostring(tonumber(a) + tonumber(b))
					else
						return tostring(a) .. tostring(b)
					end
				end
				setfenv(proc,environment)
				local result,value = pcall(proc)
				mt.__add = __add
				if result then
					return value
				end
			end
		end
		local function initializeFields(cdata)
			for _,fld in pairs(grandtotals) do
				cdata[fld.fieldName] = 0
			end
			for _,fld in pairs(formulas) do
				cdata[fld.fieldName] = tonumber(fld.initialValue) or fld.initialValue
			end

		end
		local function calculateFields(cdata)
			for _,fld in pairs(grandtotals) do
				cdata[fld.fieldName] = cdata[fld.fieldName] +  (tonumber((cdata[fld.datasource] or {})[fld.field]) or 0)
			end
			for _,fld in pairs(formulas) do
				cdata[fld.fieldName] = evalexpr(fld.expresion or "",cdata)
			end
		end

		local function moveFirst(cdata)
			local r
			local status;
			cdata.rec = nil
			for name,ds in pairs(datasources) do
				--LogDebug("FILTRO %s",tostring(ds._filter))
				status, ds._iterator = ds:cursor({ orderby=ds._orderBy,filter=ds._filter,parameters=inputdata.parameters})
				if status then
					if not ds._iterator.eof() then
						cdata.rec = true;
						r = ds._iterator.next();
						cdata[name]=r;
						getJoinedData(ds,r,cdata)
					else
						clearJoinedData(ds,cdata);
					end
				else
					ds._iterator = nil;
				end

			end
			if cdata.rec then
				cdata.recordNum = 1
				cdata.evenOdd = "even"
			end
		end
		local function moveNext(cdata)
			local r
			cdata.rec = nil
			for name,ds in pairs(datasources) do
				if ds._iterator then
					if not ds._iterator.eof() then
						cdata.rec = true;
						r = ds._iterator.next();
						cdata[name]=r;
						getJoinedData(ds,r,cdata)

					else
						clearJoinedData(ds,cdata);
					end
				end
			end
			if cdata.rec then
				cdata.recordNum = cdata.recordNum +1
			end
		end

		for _,band in ipairs(formData.childs) do
			band.height = band.height-23;
			if band['band-type'] == "reportheader" then
				table.insert(reportheaders,band);
			elseif  band['band-type'] == "pageheader" then
				table.insert(pageheaders,band);
			elseif  band['band-type'] == "groupheader" then
				table.insert(groupheaders,band);
			elseif  band['band-type'] == "reportdetails" then
				table.insert(reportdetails,band);
			elseif  band['band-type'] == "groupfooter" then
				table.insert(groupfooters,band);
			elseif  band['band-type'] == "pagefooter" then
				table.insert(pagefooters,band);
			elseif  band['band-type'] == "reportfooter" then
				table.insert(reportfooters,band);
			end
			scanControlData(band,band);
			for idx,cl in ipairs(band.childs) do
				if not cl.bandDecoration then
					cl.y = cl.y or 20
					cl.y = cl.y  - 20
				end
			end
		end
		status,result = instanceDatasources();
		if not status then
			return status,result
		end
		if formData.filters and formData.filters[1] and datasources[formData.filters[1].datasource] then
			datasources[formData.filters[1].datasource]._filter = formData.filters[1];
		end

		-- recorre los grupos, para establecer el orden de los registros
		for _,gheader in ipairs(groupheaders) do
			if gheader.groupBy and gheader.groupBy.datasourceName and datasources[gheader.groupBy.datasourceName] and #gheader.groupBy.columns then

				datasources[gheader.groupBy.datasourceName]._orderBy = datasources[gheader.groupBy.datasourceName]._orderBy or {}
				for _,col in ipairs(gheader.groupBy.columns) do
					-- al principio el group seran los nombres de las columnas y predefinimos orden ascendente
					--
					if type (col) == "string" then
						table.insert(datasources[gheader.groupBy.datasourceName]._orderBy,{column=col,order="ASC"})
					else
						table.insert(datasources[gheader.groupBy.datasourceName]._orderBy,col)
					end
				end
			end
		end

		for _,d in pairs(datasources) do
			local hcols={}
			local t = {}
			if d._orderBy then
				for _,col in ipairs(d._orderBy) do
					if not hcols[col] then
						table.insert(t,col)
						hcols[col]=true
					end
				end
				d._orderBy = t;
			end
		end



		local cdata={} ;--= {} --  cdata debe tener el registro actual de trabajo , que puede ser un mix de todos los datasources
		for _,param in ipairs(inputdata.parameters or {}) do
			cdata[param.name] = param.value;
			local pos = string.find(param.name,"@")
			if pos and pos > 0 then
				local pname = string.sub(param.name,pos+1)
				if #pname > 0 then
					cdata[pname]=param.value
				end
			end
		end
		LogDebug("DATOS %s",json.encode(cdata,true));
		-- checkea si debe hacer un salto de pagina porque la proxima banda no entra
		local function checkPageBreak(nextBand,cdata)
			if not nextBand.printCondition or (nextBand.printCondition and evalexpr(nextBand.printCondition,cdata) == true) then
				if pageOffset+nextBand.height > maxPageOffset then

					--LogDebug ( "pageBreak %d, bandheight %d maxoffset = %d",pageOffset,nextBand.height,maxPageOffset)
					return true;
				end
			else
				return false
			end
		end
		-- checkea si debe hacer un salto de pagina porque la ultime banda insertada requiera un salto
		local function checkForcedPageBreak(lastBand,cdata)
			if not lastBand.printCondition or (lastBand.printCondition and evalexpr(lastBand.printCondition,cdata) == true) then
				return lastBand.breakAfter
			else
				return false;
			end
		end
		local function saveCurrentPage()
			local fname = config.tempPath .. config.pathSeparator .. processId .. "_" .. "page_" .. tostring(pageNum)
			--LogDebug ( "save report page %s",fname);
			local f = tweetyio.open(fname,"w");

			f:write(json.encode(
				{
					scale=0.275,
					units="mm",
					fontscale=1,
					printerScale = 0.23,
					printerFontScale=0.95,
					pageNum=pageNum,
					pageSize=pageSize,
					content=pageData
				}
			))
			tweetyio.close(f);
			local session = tweetyio.session(); -- aprovechando que debe indicar que hay una nueva pagina refresca el objeto proccess por si desde
												-- el cliente se mando un stop
			process = session.tweetyprocess[processId];
			process.pages = pageNum;
			process.data=process.data or {};
			process.data.pageNum=pageNum;
			session:release();
			--LogDebug("sleep")
			socket.sleep(0.2);
		end
		local addNode;
		-------------------------------------------------------------------------------------
		local function addPanel (parent,node,cdata)
			local p = rpwidget.panel{
				type="ui.panel",
				x=node.x,
				y=node.y,
				width = node.width,
				height = node.height,
				backColor = node.backColor,
				foreColor = node.foreColor,
				borderWidth = node.borderWidth,
				borderColor = node.borderColor,
				childs={}
			}
			for _,cl in ipairs(node.childs or {}) do
				if not cl.printCondition or (cl.printCondition and evalexpr(cl.printCondition,cdata) == true) then
					addNode ( p,cl,cdata)
				end
			end
			table.insert(parent.childs,rpwidget.panel(p));
		end

		addNode = function (parent,nodeData,cdata)
			local function deepcopy(object)
				local lookup_table = {}
				local function _copy(object)
					if type(object) ~= "table" then
						return object
					elseif lookup_table[object] then
						return lookup_table[object]
					end
					local new_table = {}
					lookup_table[object] = new_table
					for index, value in pairs(object) do
						new_table[_copy(index)] = _copy(value)
					end
					return setmetatable(new_table, getmetatable(object))
				end
				return _copy(object)
			end
			nodeData = deepcopy(nodeData);
			if nodeData.type == "ui.panel" then
				addPanel(parent,nodeData,cdata)
			elseif nodeData.type == "calculatedField" then
				nodeData.caption = cdata[nodeData.fieldName]
				table.insert(parent.childs,rpwidget.label(nodeData))
			elseif nodeData.type == "ui.reportfield" then
				nodeData.datasource = nodeData.datasource or ""
				nodeData.field = nodeData.field or ""
				nodeData.caption = (cdata[nodeData.datasource] or {})[nodeData.field]
				table.insert(parent.childs,rpwidget.label(nodeData))
			elseif nodeData.type == "ui.label" then
				table.insert(parent.childs,rpwidget.label(nodeData))
			elseif nodeData.type == "ui.image" then
				table.insert(parent.childs,rpwidget.image(nodeData))
			elseif nodeData.type == "ui.hline" then
				table.insert(parent.childs,rpwidget.hline(nodeData))
			elseif nodeData.type == "ui.vline" then
				table.insert(parent.childs,rpwidget.vline(nodeData))
			else
				LogDebug("Unknown node type %s",nodeData.type)
			end
		end
		-------------------------------------------------------------------------------------
		local function addBandContent(pageData,band,band_cdata)
			LogDebug("ADD BAND %s",band['band-type']);
			if band['type']== "ui.bireportdetails" then
				assert(false); -- Como va a aparecer una banda de estas en un reporte comun ? . Seguro en algun momento quise aprovechar el proceso
				return biReport.addBandContent(band,pageData,self,pageOffset,process,inputdata)
			else

				local backColor = band.backColor;
				if band_cdata.evenOdd == "odd" then
					backColor = band.oddColor
				end
				if band_cdata.evenOdd == "even" then
					backColor = band.evenColor
				end

				if not band.printCondition or (band.printCondition and  evalexpr(band.printCondition,band_cdata)) then
					local p = {
						type="ui.panel",
						['band-type'] = band['band-type'],
						x=0,
						y = pageOffset,
						width = band.width,
						height = band.height,
						backColor = backColor,
						borderWidth = band.borderWidth,
						borderColor = band.borderColor,
						childs={}
					}

					band.childs = band.childs or {}
					for idx,cl in ipairs(band.childs) do

						if not cl.bandDecoration then
							if not cl.printCondition or (cl.printCondition and evalexpr(cl.printCondition,band_cdata) == true) then
								addNode ( p,cl,band_cdata)
							end
						end
					end
					table.insert(pageData,p);
					-- Cuando imprime una banda si es de tipo pagefooter o groupfooter resetea los sumarizadores
					-- y formulas que contiene
					if band['band-type'] == "pagefooter" or band['band-type']=="groupfooter" then
						for _,fld in pairs(grandtotals) do
							if fld.bandId == band.id then
								--LogDebug("Reseteando sumarizador %s ",fld.fieldName);
								cdata[fld.fieldName]=0;
								cdata[fld.fieldName] = cdata[fld.fieldName] +  (tonumber((cdata[fld.datasource] or {})[fld.field]) or 0)
							end
						end
						for _,fld in pairs(formulas) do
							if fld.bandId == band.id then
								cdata[fld.fieldName]=tonumber(fld.initialValue) or fld.initialValue;
								cdata[fld.fieldName] = evalexpr(fld.expresion or "",cdata)
							end
						end

					end


					return band.height;
				else
					return 0
				end
			end

		end

		maxPageOffset = pageSize - bottomMargin;
		for _,band in ipairs(pagefooters) do
			-- TODO deberia tomar en cuenta si se renderizan o no
			maxPageOffset = maxPageOffset - band.height
		end

		local function add_newpage(cdata,prev_cdata)
			if #pageData and pageNum > 0  then
				local gapHeight = maxPageOffset - pageOffset;
				if gapHeight > 0 and #pagefooters>0 then
					addBandContent(pageData,{width=1,height=gapHeight,['band-type']="gap"},prev_cdata)
				end
				pageOffset=maxPageOffset
				for _,band in ipairs(pagefooters) do
					pageOffset = pageOffset + addBandContent(pageData,band,prev_cdata)
				end

				saveCurrentPage();
			end
			pageNum =pageNum+1
			pageOffset = topMargin
			pageHeader=true;
			pageData = {}
			if pageNum>1 then

				for _,band in ipairs(pageheaders) do
					pageOffset = pageOffset + addBandContent(pageData,band,cdata)
				end
			else
				for _,band in ipairs(pageheaders) do
					pageOffset = pageOffset + addBandContent(pageData,band,cdata)
				end

				for _,band in ipairs(reportheaders) do
					if checkPageBreak(band,cdata) then
						add_newpage(cdata,cdata)
					end
					pageOffset = pageOffset + addBandContent(pageData,band,cdata)
					if checkForcedPageBreak(band,cdata) then
						add_newpage(cdata,cdata)
					end
				end
			end
		end
		local function init_startpage(cdata)
			pageOffset = topMargin
			pageHeader=true;
			pageData = {}
			for _,band in ipairs(pageheaders) do
				pageOffset = pageOffset + addBandContent(pageData,band,cdata)
			end

			for _,band in ipairs(reportheaders) do
				if checkPageBreak(band,cdata) then
					add_newpage(cdata,cdata)
				end
				pageOffset = pageOffset + addBandContent(pageData,band,cdata)
				if checkForcedPageBreak(band,cdata) then
					add_newpage(cdata,cdata)
				end
			end
		end
		pageNum=1;
		initializeFields(cdata)
		moveFirst(cdata);
		local prev_cdata = cdata;
		calculateFields(cdata);
		init_startpage(cdata)
		repeat
			local chggroup = false;
			for idx,band in ipairs(groupheaders) do
				if band.groupBy and band.groupBy.datasourceName and #band.groupBy.columns>0 then
					local ckey={}
					for _,col in ipairs(band.groupBy.columns) do
						table.insert(ckey,(cdata[band.groupBy.datasourceName] or {})[col] or "")
					end
					ckey = table.concat(ckey)
					if ckey ~= band.ckey then
						cdata.evenOdd="even";
						chggroup = true
						for c=idx+1,#groupheaders do
							if groupheaders[c].ckey then
								local footerBand = groupfooters[#groupheaders - (c-1)]
								if footerBand then
									if checkPageBreak(footerBand,prev_cdata) then
										add_newpage(cdata,prev_cdata)
									end
									pageOffset = pageOffset + addBandContent(pageData,footerBand,prev_cdata)
								end
								groupheaders[c].ckey = nil
							end
						end

						if band.ckey then	-- ya lo imprimimos en otra oportunidad
										-- buscamos un footer
							local footerBand = groupfooters[#groupheaders - (idx-1)]
							if footerBand then
								if checkPageBreak(footerBand,prev_cdata) then
									add_newpage(cdata,prev_cdata)
								end
								pageOffset = pageOffset + addBandContent(pageData,footerBand,prev_cdata)
							end

						end

						if checkPageBreak(band,prev_cdata) then
							add_newpage(cdata,prev_cdata)
						end

						pageOffset = pageOffset + addBandContent(pageData,band,cdata)
						band.ckey = ckey;

					end
				end
			end
			local prevPageOffset = pageOffset
			for _,band in ipairs(reportdetails) do
				if checkPageBreak(band,prev_cdata) then
					add_newpage(cdata,prev_cdata)
				end
				pageOffset = pageOffset + addBandContent(pageData,band,cdata)
			end
			if pageOffset > prevPageOffset then
				if cdata.evenOdd == "even" then
					cdata.evenOdd = "odd"
				else
					cdata.evenOdd = "even"
				end
			end


			prev_cdata = cdata;
			cdata = {}
			for k,v in pairs(prev_cdata) do
				cdata[k] = v
			end
			--cdata = {recordNum=cdata.recordNum,evenOdd=cdata.evenOdd};
			moveNext(cdata)
			calculateFields(cdata);

		until not cdata.rec or process.stop

		if not process.stop then

			for idx,band in ipairs(groupheaders) do
				if band.ckey then
					local footerBand = groupfooters[#groupheaders - (idx-1)]
					if footerBand then
						if checkPageBreak(footerBand,prev_cdata) then
							add_newpage(cdata,prev_cdata)
						end
						pageOffset = pageOffset + addBandContent(pageData,footerBand,prev_cdata)
					end
				end
			end

			for _,band in ipairs(reportfooters) do
				if checkPageBreak(band,prev_cdata) then
					add_newpage(cdata,prev_cdata);
				end
				pageOffset = pageOffset + addBandContent(pageData,band,prev_cdata)
				if checkForcedPageBreak(band) then
					add_newpage(cdata,prev_cdata);

				end
			end

			if #pageData then
				local gapHeight = maxPageOffset -pageOffset;

				if gapHeight > 0 and #pagefooters>0 then
					addBandContent(pageData,{width=10,height=gapHeight,['band-type']="gap"},prev_cdata)
				end

				pageOffset = maxPageOffset;
				for _,band in ipairs(pagefooters) do
					pageOffset = pageOffset + addBandContent(pageData,band,prev_cdata)
				end

				saveCurrentPage();
			end
		end
		session = tweetyio.session();
		process = session.tweetyprocess[processId];
		process.processCompletion=100;
		process.isCompleted = true;
		session:release();

		return true,{processCompletion=process.processCompletion,isCompleted=process.isCompleted,haveError=process.haveError,data={pageNum=pageNum},messages=msgs}
	end
	---------------------------------------------------------------------------------------------------------
	function self:BIReportProcess(processId,process,inputdata,formData)

		local reportbands={}
		local pageData={}
		local formulas={}
		local topMargin = 0; -- TODO iniciar de config del reporte
		local bottomMargin = 0;
		local pageSize
		local pageNum=0;

		local pageOffset=topMargin; --
		local now = string.gsub(LuaOrm.Date(LuaOrm.Date():toString()):toDifferentTimezoneString(inconcert.authUser.timezone),"%.%d*$","")


		local function scanControlData(parentBand,ctrl)
			if ctrl.type == "formula" then
				formulas[ctrl.id]={
					fieldName=ctrl.id,
					expresion=ctrl.expresion,
					value = tonumber(ctrl.initialValue) or ctrl.initialValue,
					initialValue=ctrl.initialValue,
					bandId=parentBand.id
				}
				ctrl.type = "calculatedField";
				ctrl.fieldName = ctrl.id;
			end


			if ctrl.childs then
				for _,cl in ipairs(ctrl.childs) do
					scanControlData(parentBand,cl)
				end
			end
		end
		local function evalexpr(expr,environment)
			if type(expr) == "string" then
				expr = string.gsub(expr,"!=","~=")
				expr = string.gsub(expr,"<>","~=")
			end
			environment.page = pageNum
			environment.now = now;

			environment = environment or {}
			environment.print = print
			environment.tostring = tostring
			environment.tonumber = tonumber
			environment.str = tostring
			environment.num = tonumber
			environment.type = type
			environment.fmt = string.format;
			environment.os = os;
			environment.iif = function(cond,rtrue,rfalse)
				if cond == true then
					return rtrue
				else
					return rfalse
				end
			end
			environment.getmetatable = getmetatable
			expr = string.gsub(expr,"@","")
			local proc,err = loadstring("return " .. expr)
			if proc then

				local mt = getmetatable("")
				local __add = mt.__add
				mt.__add = function(a,b)
					if tonumber(a) and tonumber(b) then
						return tostring(tonumber(a) + tonumber(b))
					else
						return tostring(a) .. tostring(b)
					end
				end
				setfenv(proc,environment)
				local result,value = pcall(proc)
				mt.__add = __add
				if result then
					return value
				end
			end
		end
		local function initializeFields(cdata)
			for _,fld in pairs(formulas) do
				cdata[fld.fieldName] = tonumber(fld.initialValue) or fld.initialValue
			end

		end
		local function calculateFields(cdata)
			for _,fld in pairs(formulas) do
				cdata[fld.fieldName] = evalexpr(fld.expresion or "",cdata)
			end
		end

		for _,band in ipairs(formData.childs) do
			band.height = band.height-23;
			scanControlData(band,band);
			for idx,cl in ipairs(band.childs or {}) do
				if not cl.bandDecoration then
					cl.y = cl.y or 20
					cl.y = cl.y - 20
				end
			end
		end

		local cdata={} ;--= {} --  cdata debe tener el registro actual de trabajo , que puede ser un mix de todos los datasources
		for _,param in ipairs(inputdata.parameters or {}) do
			cdata[string.gsub(param.name,"@","")] = param.value;
		end

		local function saveCurrentPage()
			local fname = config.tempPath .. config.pathSeparator .. processId .. "_" .. "page_" .. tostring(pageNum)
			process.pages = pageNum;
			local f = tweetyio.open(fname,"w");
			f:write(json.encode({
				pageNum=pageNum,
				pageSize=pageOffset,
				content=pageData,

				scale=1,
				units="px",
				fontscale=1,
				printerScale = 1,
				printerFontScale=1
			}))
			tweetyio.close(f);
			process.data=process.data or {};
			process.data.pageNum=pageNum;
			socket.sleep(0.2);
		end
		local addNode;
		-------------------------------------------------------------------------------------
		local function addPanel (parent,node,cdata)
			local p = rpwidget.panel{
				x=node.x,
				y=node.y,
				width = node.width,
				height = node.height,
				backColor = node.backColor,
				foreColor = node.foreColor,
				borderWidth = node.borderWidth,
				borderColor = node.borderColor,
				childs={}
			}
			for _,cl in ipairs(node.childs or {}) do
				addNode ( p,cl,cdata)
			end
			table.insert(parent.childs,p);
		end

		addNode = function (parent,nodeData,cdata)
			local function deepcopy(object)
				local lookup_table = {}
				local function _copy(object)
					if type(object) ~= "table" then
						return object
					elseif lookup_table[object] then
						return lookup_table[object]
					end
					local new_table = {}
					lookup_table[object] = new_table
					for index, value in pairs(object) do
						new_table[_copy(index)] = _copy(value)
					end
					return setmetatable(new_table, getmetatable(object))
				end
				return _copy(object)
			end
			nodeData = deepcopy(nodeData);
			if nodeData.type == "ui.panel" then
				addPanel(parent,nodeData,cdata)
			elseif nodeData.type == "calculatedField" then
				nodeData.caption = cdata[nodeData.fieldName]
				table.insert(parent.childs,rpwidget.label(nodeData))
			elseif nodeData.type == "ui.label" then
				table.insert(parent.childs,rpwidget.label(nodeData))
			elseif nodeData.type == "ui.image" then
				table.insert(parent.childs,rpwidget.image(nodeData))
			elseif nodeData.type == "ui.hline" then
				table.insert(parent.childs,rpwidget.hline(nodeData))
			elseif nodeData.type == "ui.vline" then
				table.insert(parent.childs,rpwidget.vline(nodeData))
			else
				LogDebug("Unknown node type %s",nodeData.type)
			end
		end
		-------------------------------------------------------------------------------------
		local function addBandContent(pageData,band,band_cdata)
			LogDebug("ADD BAND %s",band['band-type'] or "");
			if band['type']== "ui.bireportdetails" then
				return biReport.addBandContent(band,pageData,self,pageOffset,process,inputdata)
			else

				local backColor = band.backColor;
				if band_cdata.evenOdd == "odd" then
					backColor = band.oddColor
				end
				if band_cdata.evenOdd == "even" then
					backColor = band.evenColor
				end

				local p = {
					type="ui.panel",
					['band-type'] = band['band-type'],
					x=0,
					y = pageOffset,
					width = band.width,
					height = band.height,
					backColor = backColor,
					borderWidth = band.borderWidth,
					borderColor = band.borderColor,
					childs={}
				}

				band.childs = band.childs or {}
				for idx,cl in ipairs(band.childs) do

					if not cl.bandDecoration then
						if not cl.printCondition or (cl.printCondition and evalexpr(cl.printCondition,band_cdata) == true) then
							addNode ( p,cl,band_cdata)
						end
					end
				end
				table.insert(pageData,p);
				return band.height;
			end

		end


		local function add_newpage(cdata)
			if #pageData and pageNum > 0  then
				saveCurrentPage();
			end
			pageNum =pageNum+1


			pageOffset = topMargin
			pageData = {}
		end
		initializeFields(cdata)
		calculateFields(cdata);

		for _,band in ipairs(formData.childs) do
			if band['band-type'] == 'pageheader' then
				add_newpage(cdata);
			end
			pageOffset = pageOffset + addBandContent(pageData,band,cdata)
			calculateFields(cdata);

			if process.stop then
				break
			end
		end
		if #pageData then
			saveCurrentPage();
		end
		process.processCompletion=100;
		process.isCompleted = true;
		return true,{processCompletion=process.processCompletion,isCompleted=process.isCompleted,haveError=process.haveError,data={pageNum=pageNum},messages=msgs}

	end

	---------------------------------------------------------------------------------------------------------

	function self:startReportProcess(processId)
		local status,result
		local res={} ;
		local session = tweetyio.session();
		if not session.tweetyprocess then session.tweetyprocess = {} end
		local process = session.tweetyprocess[processId]
		session:release();
		if process then
			local inputdata = process.inputdata;
			local reportName = inputdata.reportName
			process.data = {}
			local status,formData = self:getFormData(reportName)
			if not status then
				return status,formData
			else
				if formData.formSubType == "bireport" then
					return self:BIReportProcess(processId,process,inputdata,formData)
				else
					return self:ReportProcess(processId,inputdata,formData)
				end
			end
		end
	end

	return self
end

