module(..., package.seeall)
local svgbutton = require (tweety_modules_prefix .. "svgbutton")
local svgpanel = require (tweety_modules_prefix .. "svgpanel")

--require tweety_modules_prefix .. "slice"
local config = require (tweety_modules_prefix .. "config")
local base64 = require ("base64");
local tempPath=config.tempPath
local rsvg_convert=config.rsvgconvert
local render = {}
local m_appLocation

local function cache ( handler) 
	return function (web,...)
		web.headers["Expires"]=os.date("!%a, %d %b %Y %H:%M:%S GMT",os.time()+(3600*1440));
		return handler(web,...)
	end
end

local function loadimage(id)
	local res
	local fh = io.open(tempPath .. config.pathSeparator .. id .. ".png","rb")
	if fh then
		res = fh:read("*a")
		io.close(fh)
	end
	return res
end
local function renderimage(id,text_svg)
	local res
	local fh = io.open(tempPath .. config.pathSeparator .. id .. ".svg","wb")
	if not fh then return false end
	fh:write(text_svg)
	io.close(fh)
	local cmd = string.format("\"%s\" %s -o %s",
		rsvg_convert,
		tempPath .. config.pathSeparator .. id .. ".svg",
		tempPath .. config.pathSeparator .. id .. ".png")
	LogDebug("render command %s",cmd);
	res  = os.execute(cmd)
	--os.remove(tempPath .. "\\" .. id .. ".svg")
	return res==0
end
local function button(web,data)
	local id = md5.sumhexa(data)
	local res = loadimage(id);
	if not res then 
		local button_data = json.decode(base64.decode(data))
		local button = svgbutton.new(button_data)
		if renderimage(id,button:tostring()) then
			res = loadimage(id)
		end
	end
	web:content_type("image/png")
	return res
end
local function panel(web,data)
	LogDebug("BASE 64 %s",data);
	local id = md5.sumhexa(data)
	local res = loadimage(id);
	if not res then 

		LogDebug("JSON 64 %s",base64.decode(data));
		
		local panel_data = json.decode(base64.decode(data))
		local panel = svgpanel.new(panel_data)
		if renderimage(id,panel:tostring()) then
			res = loadimage(id)
		end
	end
	web:content_type("image/png")
	return res
end
local function slinfo(web,slicename)
	local filename = config.resourcePath .. config.pathSeparator .. slicename .. ".slices"
	local file = io.open(filename,"rb")
	local jsonData
	local buildData;
	buildData= function(sliceData)
		local res = {slices={}}
		for _,slice in pairs(sliceData.slices) do
			res.slices[slice.id] = {w=slice.width;h=slice.height,x=slice.x,y=slice.y}
		end
		res.client = {
			x=sliceData.data.client.x,
			y = sliceData.data.client.y,
			w = sliceData.data.client.width,
			h = sliceData.data.client.height
		}
		
		return res;	
	end
	if file then
		jsonData = file:read("*a")
		io.close(file);
		return json.encode(buildData(json.decode(jsonData)));
	end
end
local function slget (web,slicename,sliceid)
	local filename = config.resourcePath .. config.pathSeparator .. slicename .. ".slices"
	local file = io.open(filename,"rb")
	local jsonData 
	if file then
		jsonData = file:read("*a")
		io.close(file);
		jsonData = json.decode(jsonData);
		for _,slice in pairs(jsonData.slices) do
			if slice.id == sliceid then
				web:content_type(slice.mimeType)
				return slice.data
			end
		end
	end
end

function initialize(_M, prefix,appLocation)

	prefix = prefix or ""
	_M:dispatch_get(cache(button), prefix .. "/render/button/([^/]+)")
	_M:dispatch_get(cache(panel), prefix .. "/render/panel/([^/]+)")
	_M:dispatch_get(cache(slinfo), prefix .. "/render/slinfo/([^/]+)")
	_M:dispatch_get(cache(slget), prefix .. "/render/slget/([^/]+)/([^/]+)")
	

end

