--[[
	reCAPTCHA V2:
	-------------
	Las claves pública y privada se crearon en el link de abajo,
	y se crearon con el usuario de gmail de inconcert:

		Usr: inconcertmcc@gmail.com
		Pwd: Inc0nc3rt!

		Clave del sitio:
		----------------
		(client side) Usada en el código HTML que se muestra a los usuarios.

			6Lf-J0UUAAAAAGmR_IKYvcsFhjsu9v_AXeTnsg87

		Clave secreta  :
		----------------
		(server side) Usada para las comunicaciones entre tu sitio y Google. No revelársela a nadie.

			6Lf-J0UUAAAAAFFGy0pjfQgF0tZRtaGYOc2NuWdw

	Quedó desactivada la validacion de dominios y subdominios, según la documentación no es recomendable desactivarlos pero bueno.
	Los dominios de prueba que había configurado eran: 
		i6.inconcert.cc        (este utilizado desde el designer en modo diseño)
		chat1-i6.inconcert.cc  (este utilizado desde el designer en modo runtime)

	Referencias:
		https://developers.google.com/recaptcha/docs/display#AJAX
		https://developers.google.com/recaptcha/docs/faq
		https://www.google.com/recaptcha/admin (aca se crea o administran las claves publica y privada)
		https://www.google.com/recaptcha/admin#site/340076542?setup
		https://developers.google.com/recaptcha/docs/domain_validation (documentacion de los dominios y subdominios)
]]--

module(..., package.seeall)

--[[
	ATENCION:
		Se utiliza "ssl.https" en lugar "socket.http" para poder realizar request "https",
		ya que la versión 2.0 del API CAPTCHA, solo soporta "https"
]]--
local https = require "ssl.https"
local config = require (tweety_modules_prefix .. "config")
local messages = require (tweety_modules_prefix .. "messages")
local tweetyio = require(tweety_modules_prefix .. "tweetyio")
local verifyUriV2 = "https://www.google.com/recaptcha/api/siteverify"
local secretKey = "6Lf-J0UUAAAAAFFGy0pjfQgF0tZRtaGYOc2NuWdw"

--[[
RESPONSE BODY:
--------------
OK:
 {
  "success": true,
  "challenge_ts": "2018-02-07T13:50:30Z",
  "hostname": "chat1-i6.inconcert.cc"
}

ERROR:
{
  "success": false,
  "error-codes": [
    "invalid-input-secret"
  ]
}

]]--
local function Post (url, body, headers, cookie)
	headers = headers or {}

	LogInfo ("[CAPTCHA:Post] HTTP Post invoqued '%s'\nbody: '%s'\nheaders: '%s'\ncookie: '%s'", url, body, json.encode(headers), cookie)

	headers["content-length"] = string.len(body)

	if cookie and string.len(cookie) then
		headers["Cookie"] = cookie
	end

	local responseBody = {}
	local result, code, responeHeaders, statusLine = https.request{
		url = url,
		method = "POST",
		headers = headers,
		source = ltn12.source.string(body),
		sink = ltn12.sink.table(responseBody)
	}

	local encodedResponseBody = table.concat(responseBody, "")

	LogInfo("[CAPTCHA:Post] HTTP Post result: '%s',\ncode: '%s',\nresponeHeaders: '%s',\nstatusLine: '%s',\nresponseBody: '%s'", result, code, json.encode(responeHeaders or {}), statusLine, encodedResponseBody)

	code = tonumber(code)
	if code == 200 then
		return true, encodedResponseBody
	end

	LogWarning("[CAPTCHA:Post] error getting url %s  result code: %s\nbody:%s", url, tostring(code), encodedResponseBody)

	return false, encodedResponseBody, code
end

local function makePostBody (response, clientIP)
	local body = {
		"secret=" .. secretKey,
		"response=" .. response,
		"remoteip=" .. clientIP
	}
	return table.concat(body,"&")
end

--https://developers.google.com/recaptcha/docs/verify

local function getErrorCodes (result)
	if type(result) ~= "table" then
		return "Unknow Error"
	end
	local errors = result["error-codes"]
	if type(errors) == "string" then
		return errors
	end
	if type(errors) ~= "table" then
		return "Unknow Error"
	end
	return json.encode(errors)
end

function validateCaptcha (response, clientIP)
	local body = makePostBody(response, clientIP)
	local status, encodedResult, code = Post(verifyUriV2, body, {["Content-Type"]="application/x-www-form-urlencoded"})
	local result = json.decode(encodedResult)

	if status then
		if result.success then
			return true
		else
			--result = messages.messageKey('wrong_captcha',{reason=r[2],challenge=challenge})
			local errors = getErrorCodes(result)
			--LogDebug("errors: %s", errors)
			result = messages.messageKey("wrong_captcha", {reason=errors})
			return false, result
		end
	else
		if code == 403 then
			result = messages.messageKey("wrong_captcha", {reason="forbidden"})
			return false, result
		end
		result.reason = "Network error"
		return false, result
	end
end

function verify (app,args,clientIP)
	local status,result,formdata
	status = true

	if args.context and args.context.formid and args.content then
		status,formdata = app:getFormData(args.context.formid)
		if status then
			local search_captcha_control
			search_captcha_control = function(ctrl)
				local res
				if ctrl.type == "ui.captcha" then
					return ctrl
				end
				if ctrl.childs then
					for _,c in ipairs(ctrl.childs) do
						res = search_captcha_control(c)
						if res then
							return res
						end
					end
				end
			end
			local captcha_control = search_captcha_control(formdata)
			if captcha_control then
				status = false
				result = messages.messageKey("wrong_captcha")
				for _,param in ipairs(args.content) do
					if param.name == "__captcha_challenge" then
						if param.value and param.response and param.response ~= "" then
							return validateCaptcha(param.response, clientIP)
						end
					end
				end
				result.reason = "Can't find challenge/response data"
			end
		else
			result = messages.messageKey('wrong_captcha',{reason='form not found'})
		end
	end
	return status,result
end

