local wsapi_request = require "wsapi.request"

module(...,package.seeall,inconcert.orbit.application.Decorate)
require "inconcert.orbit.application"
local messages = require (tweety_modules_prefix .. "messages")
require "uuid"
-- local mongo = require "luamongo"
local md5 = require "md5";
local config = require (tweety_modules_prefix .. "config")
local runtime = require (tweety_modules_prefix .. "runtime")
local tweetyio =require (tweety_modules_prefix .. "tweetyio")


local dbconn
-- Generacion de un uuid basado en time para los registros de la tabla
require "socket"

local lwuid_count = 0;
local lwuid_pid = string.gsub(tostring({}),"table: ","")
local lwuid_mcid = string.sub(md5.sumhexa(os.getenv("COMPUTERNAME") or os.getenv("HOSTNAME") or tostring(os.time())),1,8);
math.randomseed( os.time() )
if config.mongodb_one_db_by_table then
	function getNamespace(tableName)
		return config.mongodb_namespace_prefix .. runtime.getCurrentVCCName() .. "_" .. tableName .. ".data";
	end

else
	function getNamespace ( tableName)
		return config.mongodb_namespace_prefix .. runtime.getCurrentVCCName() .. "." .. tableName
	end
end

function lwuid ()
	lwuid_count=lwuid_count+1
	if lwuid_count == 1e4 then
		lwuid_count=0
	end
	local res = {
		tostring(os.time()),
		string.format("%05d",lwuid_count),
		lwuid_mcid,
		lwuid_pid,
		string.format("%05d",math.random(1e4)),
	}
	return table.concat(res,"-")
end

local function nativeError(err)
	if err.code == 11000 then	-- este tiene informacion de que indice fue violado hay que obtener info de indice para saber que campo
		return messages.messageKey('duplicated_key',{nativeError=err});
	elseif err.code == 11001 then	-- en este mongo no da informacion de que indice fallo.
		return messages.messageKey('duplicated_key',{nativeError=err});
	else
		return messages.messageKey('unknown',{nativeError=err});
	end
end



local function trim(s)
	return (string.gsub(s, "^%s*(.-)%s*$", "%1")) -- trim
end
local function lread (fh)	-- lee una linea - cuenta las comillas , si el numero es impar , lee otra linea y sigue contando
	local t = {}
	local c;
	local count =0
	local l
	repeat
		l = fh:read("*l")
		if not l then
			return
		end
		table.insert(t,l);
		_,c =string.gsub(l,'"','')
		count = count +c
	until math.fmod(count,2)== 0
	if  t[1] and string.sub(t[1],1,3) == string.char(0xEF,0xBB,0xBF) then -- salteamos un bom si lo hay 
																		  -- lamentablemente no solo en el principio 
																		  -- sino que lo hacemos linea por linea
		t[1]= string.sub(t[1],4)
	end
	return table.concat(t,"\r\n");
end

function escapeCSV (s,delimiter)
  if s == nil then
	return 
  end

  s = tostring(s)
  if string.find(s, '[' .. delimiter .. '"\r\n]') then
    s = '"' .. string.gsub(s, '"', '""') .. '"'
  end
  return s
end
local function ParseCSVLine (line,sep)
	local res = {}
	local pos = 1
	sep = sep or ','
	while true do
		local c = string.sub(line,pos,pos)
		if (c == "") then break end
		if (c == '"') then
			-- quoted value (ignore separator within)
			local txt = ""
			repeat
				local startp,endp = string.find(line,'^%b""',pos)
				txt = txt..string.sub(line,startp+1,endp-1)
				pos = endp + 1
				c = string.sub(line,pos,pos)
				if (c == '"') then txt = txt..'"' end
				-- check first char AFTER quoted string, if it is another
				-- quoted string without separator, then append it
				-- this is the way to "escape" the quote char in a quote. example:
				--   value1,"blub""blip""boing",value3  will result in blub"blip"boing  for the middle
			until (c ~= '"')

			table.insert(res,txt)
			assert(c == sep or c == "")
			pos = pos + 1
		else
			-- no quotes used, just look for the first separator
			local startp,endp = string.find(line,sep,pos)
			if (startp) then
				table.insert(res,string.sub(line,pos,startp-1))
				pos = endp + 1
			else
				-- no separator found -> use rest of string and terminate
				table.insert(res,string.sub(line,pos))
				break
			end
		end
	end
	return res
end

local function detect_columns(txLine,delimiter)
	local res = {}
	if delimiter == "" then
		return res
	end
	txLine = txLine .. delimiter

	delimiter = string.gsub(delimiter,"([%%%.])","%%%1")
	local pattern = string.format("([^%s]*)[%s]",delimiter,delimiter)
	local ite = string.gmatch(txLine,pattern)
	local col
	repeat
		col = ite();
		if col then
			col = trim(col)
			table.insert(res,{name=col,alias=col,type="string"});
		end
	until not col
	return res;
end

local function detect_columns(txLine,delimiter)
	local res = {}
	if delimiter == "" then
		return res
	end
	local r = ParseCSVLine(txLine,delimiter);
	for _,col in ipairs(ParseCSVLine(txLine,delimiter)) do
		col = trim(col)
		table.insert(res,{name=col,alias=col,type="string"});
	end
	return res;
end
local function dataconvert(value,type,format,nullExpr)
	local c
	if value == nil then
		return nullExpr
	end
	if type == "date" then
		format = format or "dd/mm/yyyy"
		if format == "dd/mm/yyyy" then
			value,c = string.gsub(value,"(%d%d)/(%d%d)/(%d%d%d%d)","%3-%2-%1")
		elseif format == "mm/dd/yyyy" then
			value,c = string.gsub(value,"(%d%d)/(%d%d)/(%d%d%d%d)","%3-%1-%2")
		elseif format == "yyyymmdd" then
			value,c = string.gsub(value,"(%d%d%d%d)(%d%d)(%d%d)","%1-%2-%3")
		elseif format == "yyyy-mm-dd" then
			value,c = string.gsub(value,"(%d%d%d%d)-(%d%d)-(%d%d)","%1-%2-%3")
		end
		if c == 0 then
			value = nil
		end

	elseif type == "datetime" then

		format = format or "dd/mm/yyyy hh:mm:ss.mmm"
		if format == "dd/mm/yyyy hh:mm:ss.mmm" then
			value,c = string.gsub(value,"(%d%d)/(%d%d)/(%d%d%d%d) (%d%d):(%d%d):(%d%d).(%d%d%d)","%3-%2-%1 %4:%5:%6")
			if c == 0 then
				value,c = string.gsub(value,"(%d%d)/(%d%d)/(%d%d%d%d) (%d%d):(%d%d):(%d%d)","%3-%2-%1 %4:%5:%6") -- horas:minutos:segundos
			end
			if c == 0 then
				value,c = string.gsub(value,"(%d%d)/(%d%d)/(%d%d%d%d) (%d%d):(%d%d)","%3-%2-%1 %4:%5:00") -- horas:minutos
			end

			if c == 0 then
				value,c = string.gsub(value,"(%d%d)/(%d%d)/(%d%d%d%d)","%3-%2-%1 00:00:00") -- solo la fecha
			end
		elseif format == "mm/dd/yyyy hh:mm:ss.mmm" then

			value,c = string.gsub(value,"(%d%d)/(%d%d)/(%d%d%d%d) (%d%d):(%d%d):(%d%d).(%d%d%d)","%3-%1-%2 %4:%5:%6") -- completo

			if c == 0 then
				value,c = string.gsub(value,"(%d%d)/(%d%d)/(%d%d%d%d) (%d%d):(%d%d):(%d%d)","%3-%1-%2 %4:%5:%6") -- horas:minutos:segundos
			end
			if c == 0 then
				value,c = string.gsub(value,"(%d%d)/(%d%d)/(%d%d%d%d) (%d%d):(%d%d)","%3-%1-%2 %4:%5:00") -- horas:minutos
			end
			if c == 0 then
				value,c = string.gsub(value,"(%d%d)/(%d%d)/(%d%d%d%d)","%3-%1-%2 00:00:00")
			end

		elseif format == "yyyymmddhhmmssmmm" then
			value,c = string.gsub(value,"(%d%d%d%d)(%d%d)(%d%d)(%d%d)(%d%d)(%d%d)(%d%d%d)","%1-%2-%3 %4:%5:%6")
			if c == 0 then
				value,c = string.gsub(value,"(%d%d%d%d)(%d%d)(%d%d)(%d%d)(%d%d)(%d%d)","%1-%2-%3 %4:%5:%6")
			end
			if c == 0 then
				value,c = string.gsub(value,"(%d%d%d%d)(%d%d)(%d%d)(%d%d)(%d%d)","%1-%2-%3 %4:%5:00")
			end

			if c == 0 then
				value,c = string.gsub(value,"(%d%d%d%d)(%d%d)(%d%d)","%1-%2-%3 00:00:00")
			end


		elseif format == "yyyy-mm-dd hh:mm:ss.mmm" then
			value,c = string.gsub(value,"(%d%d%d%d)-(%d%d)-(%d%d) (%d%d):(%d%d):(%d%d).(%d%d%d)","%1-%2-%3 %4:%5:%6")
			if c == 0 then
				value,c = string.gsub(value,"(%d%d%d%d)-(%d%d)-(%d%d) (%d%d):(%d%d):(%d%d)","%1-%2-%3 %4:%5:%6")
			end
			if c == 0 then
				value,c = string.gsub(value,"(%d%d%d%d)-(%d%d)-(%d%d) (%d%d):(%d%d)","%1-%2-%3 %4:%5:00")
			end
			if c == 0 then
				value,c = string.gsub(value,"(%d%d%d%d)-(%d%d)-(%d%d)","%1-%2-%3 00:00:00")
			end
		end
		if c == 0 then
			value = nil
		end
	elseif type == "integer" then
		if tonumber(value) then
			value= tonumber(string.format("%d",tonumber(value)))
		else
			value = nil

		end
	elseif type == "numeric" then
		if tonumber(value) then
			value= tonumber(string.format("%f",tonumber(value)))
		else
			value = nil

		end
	elseif type == "boolean" then
		if value ~= nil then
			value = string.lower(value)
		end	
		if value == "1" or value == "si" or value =="true" or value == "yes" then
			value = true
		else
			value = false;
		end
	end
	if value == nil then
		return nullExpr
	end	
	
	return value
end
local function parseLine(txLine,delimiter,detectedCols,outputCols,nullExpr)
	local r = ParseCSVLine(txLine,delimiter);
	local reg = {}
	local res = {}
	for idx,col in ipairs(detectedCols) do
		reg[col.name] = r[idx]
	end
	for idx,col in ipairs(outputCols) do
		table.insert(res,dataconvert(reg[col.name],col.type,col.format,nullExpr))
	end
	return res;
end




function csv_detectcolumns(web)
	local line
	local res={}
	local csvdata = web.input["post_data"];
	csvdata = json.decode(csvdata)
	local session = tweetyio.session();
	if not session.userfiles then session.userfiles = {} end
	fileEntry = session.userfiles[csvdata.fileId]
	if not fileEntry then
		return json.encode({status=false,result=messages.messageKey('argument_error',{module="csvutils.lua",func="columns",param="fileId",value=csvdata.fileId})});
	end
	fh = tweetyio.open(fileEntry.fullFileName,"rb");
	session:release();

	if not fh then
		return json.encode({status=false,result=messages.messageKey('read_file_error',{file = fileEntry.fullFileName})});
	end
	repeat
		line = lread(fh)
		if not line then
			break
		end
		line = trim(line)
	until line ~= ''
	if line then
		res=detect_columns(line,csvdata.delimiter)
		if not csvdata.firstRowHaveColumnNames then
			for nro,col in ipairs(res) do
				col.name = "field_" .. nro
				col.alias="field_" .. nro
			end
		end
	end
	tweetyio.close(fh);
	return json.encode({status=true,result = res})
end


function csv_getresulset(web)
	local line
	local res={colnames={},coltypes={},rows={}}
	local cols={}
	local outputcols={}
	local csvdata = web.input["post_data"];
	csvdata = json.decode(csvdata)

	for _,col in ipairs(csvdata.columns) do
		if col.enabled ~= false then
			table.insert(outputcols,col);
			table.insert(res.colnames,col.alias);
			table.insert(res.coltypes,col.type);
		end
	end
	local session = tweetyio.session();
	if not session.userfiles then session.userfiles = {} end
	fileEntry = session.userfiles[csvdata.fileId]
	if not fileEntry then
		session:release();
		return json.encode({status=false,result=messages.messageKey('argument_error',{module="csvutils.lua",func="columns",param="fileId",value=csvdata.fileId})});
	end
	fh = tweetyio.open(fileEntry.fullFileName,"rb");
	session:release();
	if not fh then
		return json.encode({status=false,result=messages.messageKey('read_file_error',{file = fileEntry.fullFileName})});
	end
	repeat
		line = lread(fh)
		if not line then
			break
		end
		line = trim(line)
	until line ~= ''
	if line then
		cols=detect_columns(line,csvdata.delimiter)
		if not csvdata.firstRowHaveColumnNames then
			for nro,col in ipairs(cols) do
				col.name = "field_" .. nro
				col.alias="field_" .. nro
			end
		end
	end
	local row;

	for c=1,csvdata.rows or 50 do
		if csvdata.firstRowHaveColumnNames or  c>1 then
			line = lread(fh)
		end
		if not line then
			break
		end
		
		line = trim(line)
		row = parseLine(line,csvdata.delimiter,cols,outputcols,json.null);
		-- chequear por duplicados aqui
		table.insert(res.rows,row);
	end
	tweetyio.close(fh);
	return json.encode({status=true,result = res})
end

function csv_import_async(web)
	local cookie = "AUTH_SID_WEBDESIGNER"
	local value;
	local data = web.input["post_data"];
	data = json.decode(data);
	local processId = data.processId;
	if inconcert.authUser then
		value = inconcert.authUser.authToken;
		LogDebug("Cookie = %s value = %s",cookie,value)
	end

	local task = AsyncTasks{
		url = config.myself .. "/inconcert/apps/webdesigner/designer/csv/startprocess/" .. processId,
		cookies = {
			[cookie] = value
		}
	}
	task:Add("webdesigner");
	return json.encode({status=true})
end
function csv_import(web,processId)

	local status,result
--	local data = web.input["post_data"];
	local keys = {};
	
--	data = json.decode(data);
--	local processId = data.processId;
	local session = tweetyio.session();
	if not session.tweetyprocess then session.tweetyprocess={} end
	local process =  session.tweetyprocess[processId]
	if process then
		local csvdata = process.inputdata;
		local line
		local cols={}
		local outputcols={}
		local linenumber=0;
		for _,col in ipairs(csvdata.columns) do
			if col.enabled ~= false then
				table.insert(outputcols,col);
			end
		end
		if not session.userfiles then session.userfiles={} end
		fileEntry = session.userfiles[csvdata.fileId]
		if not fileEntry then
			process.haveError = true;
			session:release();
			return json.encode({status=false,result=messages.messageKey('argument_error',{module="csvutils.lua",func="csv_import",param="fileId",value=csvdata.fileId})});
		end

		session:release();

		fh = tweetyio.open(fileEntry.fullFileName,"rb");
		if not fh then
			process.haveError = true;
			return json.encode({status=false,result=messages.messageKey('read_file_error',{file = fileEntry.fullFileName})});
		end
		-- obtenemos el tamao del archivo para el informe de progreso
		local fsize;
		local filepos=0;
		fsize = fh:seek ("end",0)
		-- volvemos al principio
		fh:seek ("set",0)

		-- conectamos a mongo usando la conf
		
		if not dbconn then
			dbconn = mongo.Connection.New();

			status,result = pcall(dbconn.connect,dbconn,config.mongodb_connection_string);
			if not status then
				dbconn = nil; -- gc collected
				tweetyio.close(fh);
				process.haveError = true;
				return json.encode({status=false,result=messages.messageKey('database_conection_error',{description=result})})
			end
		end
		-- Comunica estado del proceso
		session = tweetyio.session();
		process = session.tweetyprocess[processId]
		process.messages[#process.messages+1]=string.format("Begin import process.");
		session:release();
		local namespace = getNamespace(csvdata.tableName);
		LogDebug("TABLE NAME %s",namespace)

		-- dropeamos todo lo que tenga
		if csvdata.emptyCollection then
			status,result = dbconn:remove(namespace,{});
			if not status then
				tweetyio.close(fh);
				process.haveError = true;
				return json.encode({status=false,result=messages.messageKey('database_conection_error',{description=result})})
			end
		end

		repeat
			line = lread(fh)
			if not line then
				break
			end
			line = trim(line)
			linenumber=linenumber+1
		until line ~= ''

		if line then
			cols=detect_columns(line,csvdata.delimiter)
			if not csvdata.firstRowHaveColumnNames then
				for nro,col in ipairs(cols) do
					col.name = "field_" .. nro
					col.alias="field_" .. nro
				end
			end
		end
		local nrows=0;
		local row;
		local c=0;
		repeat
			

			if csvdata.firstRowHaveColumnNames or c>0 then
				line = lread(fh)
			end
			if not line then
				break
			end
			line = trim(line)
			linenumber=linenumber+1
			row = parseLine(line,csvdata.delimiter,cols,outputcols,json.null);
			local t={}
			local k={}
			local r={}

			-- crea la tupla para mongo
			for idx,col in ipairs(outputcols) do
				if col.key then
					if row[idx] ~= json.null then
						table.insert(k,row[idx])
					else
						table.insert(k,'(null)')
					end
				end
				table.insert(r,tostring(row[idx]))
				
				if row[idx] ~= json.null then
					t[col.alias] = row[idx]
				end
			end
			--LogDebug("ROW [%s]",table.concat(r));
			
			k = table.concat(k,",");
			if #k>0 and keys[k] then
				t._id = keys[k]
				if csvdata.duplicatedBehavior=="overwrite" then
					local session = tweetyio.session();
					process =session.tweetyprocess[processId]
					process.messages[#process.messages+1]=string.format("Overwriting duplicated entry at line %d key = %s.",linenumber,k);
					if dbconn:update(namespace,{_id=t._id},t) then
						result = dbconn:get_last_error();
						if result.code then
							if result.code == 11000 or result.code == 11001 then
								process.messages[#process.messages+1]=string.format("Index violation at line %d.",linenumber)
							else
								process.messages[#process.messages+1]=string.format("Database unknown error at line %d , process aborted.",linenumber);
								tweetyio.close(fh);
								process.haveError = true;
								session:release();
								return json.encode({status=false,result=nativeError(result)});
							end
						end
					else
						tweetyio.close(fh);
						process.haveError = true;
						session:release();
						return json.encode({status=false,result=messages.messageKey('database_conection_error')})
					end
					session:release();
				else
					process.messages[#process.messages+1]=string.format("Discard duplicated entry at line %d key = %s",linenumber,k)
				end
			else
				t._id = lwuid() -- un nuevo id
				if dbconn:insert(namespace,t) then
					result = dbconn:get_last_error();
					if result.code then

						local session = tweetyio.session();
						process =session.tweetyprocess[processId]

						if result.code == 11000 or result.code == 11001 then
							process.messages[#process.messages+1]=string.format("Index violation at line %d.",linenumber)
						else
							process.messages[#process.messages+1]=string.format("Database unknown error at line %d , process aborted.",linenumber)
							tweetyio.close(fh);
							process.haveError = true;
							session:release();
							return json.encode({status=false,result=nativeError(result)});
						end
						session:release();
					end
					nrows=nrows+1;
				else
					tweetyio.close(fh);
					process.haveError = true;
					return json.encode({status=false,result=messages.messageKey('database_conection_error')})
				end
				keys[k]=t._id;
			end
			filepos=fh:seek ("cur",0)
			c=c+1
			process.processCompletion = tonumber(string.format("%d",(filepos * 100 )/fsize));
			if math.fmod(c,400)==0 then
				local session = tweetyio.session();
				process =session.tweetyprocess[processId]
				process.processCompletion = tonumber(string.format("%d",(filepos * 100 )/fsize));
				session:release();
				socket.sleep(0.001)
			end
		until not line
		session = tweetyio.session();
		------------
		process =session.tweetyprocess[processId]

		process.processCompletion=100
		process.isCompleted = true;
		process.haveError = false;
		process.messages[#process.messages+1]=string.format("Import process finished.");
		process.messages[#process.messages+1]=string.format("%d new rows.",nrows);

		local msgs=process.messages;
--		process.messages={}
		tweetyio.close(fh);
		local res = json.encode({status=true,result={processCompletion=process.processCompletion,isCompleted=process.isCompleted,haveError=process.haveError,messages=msgs}})
--		session.tweetyprocess[processId]=nil;

		-----------
		session:release();
		return res;
	else
		session:release();
		return json.encode({status=false,result=messages.messageKey('argument_error',{module="csvutils.lua",func="csv_import",param="processId",value=processId})});
	end

end



function csv_exportdata(wsapi_env,exportParams,fileName)

	local request = wsapi_request.new(wsapi_env)
	local headers = {}
	local status = "404 Not Found" -- not found por defecto
	LogInfo("Start export data {%s} filename %s",exportParams,fileName)
	headers["Response"] = "defaultError"
	headers["Content-Type"] = "application/octet-stream"
	local exportParams = json.decode(exportParams)
	exportParams.delimiter = exportParams.delimiter or ","
	local tbinsert = table.insert;
	if not dbconn then
		dbconn = mongo.Connection.New();
		status,result = pcall(dbconn.connect,dbconn,config.mongodb_connection_string);
		if not status then
			dbconn = nil; -- gc collected
			headers["Response"] = messages.messageKey('database_conection_error',{description=result}).description;
			return status, headers, coroutine.wrap(function() coroutine.yield("") end)
		end
	end
	local namespace = getNamespace(exportParams.tableName);
	local cursor,err = dbconn:query(namespace,{['$query']={},['$orderby']={['_id']=1}},0,0,retrieve_keys);
	if not cursor then
		headers["Response"] = messages.messageKey('database_conection_error',{description=err}).description;
		return status, headers, coroutine.wrap(function() coroutine.yield("") end)
	end
	local rows={}
	local row = {}
	if exportParams.firstRowHaveColumnNames then
		for _,col in ipairs(exportParams.columns) do
			if col.enabled then
				table.insert(row,escapeCSV(col.alias or col.name,exportParams.delimiter));
			end
		end
		table.insert(rows,table.concat(row,exportParams.delimiter));
	end
	local row_count = 0
	local function moredata()
		while cursor:has_more() do
			row = {}
			reg = cursor:next();
			for _,col in ipairs(exportParams.columns) do
				if col.enabled then
					tbinsert(row,escapeCSV(reg[col.name],exportParams.delimiter) or "")
				end
			end
			tbinsert(rows,table.concat(row,exportParams.delimiter));
			row_count = row_count +1
			if math.fmod(row_count,200) == 0 then
				if not coroutine.yield(table.concat(rows,"\r\n") .."\r\n") then -- se corto la conexion del cliente
					return nil
				end	
				rows = {}
			end
		end
		if #rows then
			if not coroutine.yield(table.concat(rows,"\r\n") .."\r\n") then -- se corto la conexion del cliente
				return nil
			end	
		
			return nil;
		else
			return nil;
		end
	end
		
	
	headers["Response"] = "ok"
	headers["Content-Type"]="application/octet-stream"
	headers["Content-Disposition"] = [[attachment; filename="]] .. fileName .. "\""

	-- Return an iterator co-rutine to send chunks of opened file in case blob is huge.
	return "200 Ok", headers, 
		coroutine.wrap(function()
			moredata();
		end)
end


function initialize(_M, prefix,appLocation)
	prefix = prefix or ""
	_M:dispatch_post(csv_detectcolumns, prefix .. "/columns");
	_M:dispatch_post(csv_getresulset, prefix .. "/get");
	_M:dispatch_post(csv_import, prefix .. "/startprocess/([^/]+)");
	_M:dispatch_post(csv_import_async, prefix .. "/startprocess_async");
	_M:dispatch_wsapi(csv_exportdata,prefix .."/exportdata/([^/]+)/([^/]+)");
end
