

module(..., package.seeall)
local application = require (tweety_modules_prefix .. "application")
local runtime = require (tweety_modules_prefix .. "runtime");
local csvutils = require (tweety_modules_prefix .. "csvutils");
local messages = require (tweety_modules_prefix .. "messages"); 
local captcha = require (tweety_modules_prefix .. "captcha"); 
local tweetyio = require (tweety_modules_prefix .. "tweetyio"); 
local socket = require("socket");
local base64 = require ("base64");
local utils = require (tweety_modules_prefix .. "utils");
-- se recibe opcionalmente la aplicacion ya debidamente inicializada cuando esta disponible

function buildDatasource(vccName,appName,datasourceName,operationName,vApp) 
	local status,result	
	local dsdata
	local ds
	status=true;
	local app;
	if not vApp then			
		app = application.new{vccName=vccName}
		status,result = app:initialize(appName);
	else
		app = vApp;
	end
	
	if status then
		status,dsdata = app:getDatasourceData(datasourceName);
		if status then
			ds = require (tweety_modules_prefix  .. dsdata.description[operationName].type).new(dsdata)
			ds.appName = appName;
			ds.description.name = datasourceName;
			ds.inconcert_authUser = inconcert.authUser;
			ds.vccName=vccName;
			ds.inconcert_session = inconcert.session;
			return status,ds
		else
			return status,dsdata -- dsdata tiene la descripcion del error
		end
	end
	return status,result
end


local function export(wsapi_env,tokenId,appName,datasourceName,exportParams,fileName)
	local vccName = runtime.getCurrentVCCName() or tweetyio.get_token_data(tokenId).vccName;
	local headers = {}
	local status = "404 Not Found" -- not found por defecto
	LogDebug("EXPORT 1");

	local exportParams = json.decode(base64.decode(exportParams))
	exportParams.context.delimiter = exportParams.context.delimiter or ","
	local tbinsert = table.insert;
	local result;

	headers["Response"] = "defaultError"
	exportParams.context.fileName = datasourceName;
	if exportParams.context.firstRowHaveColumnNames == nil then
		exportParams.context.firstRowHaveColumnNames=true;
	end
	local _,mainDS = buildDatasource(vccName,appName,datasourceName,"retrieve");
	local joindatasources = {}
	local function initDatasources()
		if exportParams.context.columns then
			for _,col in ipairs(exportParams.context.columns) do
				joindatasources[col.datasource]=true;
			end
			joindatasources[datasourceName]=nil;
			
			if mainDS.description.relationships then
				
				for _,relationship in ipairs(mainDS.description.relationships) do
					if joindatasources[relationship.datasourceName] then
						local st,ds = buildDatasource(vccName,appName,relationship.datasourceName,"retrieve");
						if st then
							joindatasources[relationship.datasourceName] = {ds=ds;fields=relationship.fields}
						else
							joindatasources[relationship.datasourceName] = nil;
						end
					end
				end
			end
		end
	end
	local mainCursor
	local function moveNext()
		local stat
		local res={};
		if not mainCursor then
			local filter = mainDS:get_filter_ref(exportParams.context.filter) or {criteria={}}
			stat,mainCursor = mainDS:cursor({orderby=filter.order,filter=filter,parameters=exportParams.data,name=exportParams.context.name,sourceName=exportParams.context.sourceName});
			if not stat then	
				return false,mainCursor -- error
			end
		end
		if not mainCursor.eof() then
			local r = mainCursor.next();
			for k,v in pairs(r) do
				res[datasourceName .. "." .. k]=v;
			end
			
			for dsname,joinds in pairs(joindatasources) do
				local q = {}
				for _,field in ipairs(joinds.fields) do
					if field.source and field.target then
						local v = r[field.source];
						if v == json.null then
							v = nil
						end
						table.insert(q,{name=field.target,value=v})
					end
				end
				local status,ite = joinds.ds:cursor({parameters=q})
				if  status then
					if not ite.eof() then
						for k,v in pairs(ite.next()) do
							res[dsname .. "." .. k]=v;
						end
					end
				else
					return false,ite -- error
				end
			end
			return true,res
		else
			return true,nil
		end
			
	end
	
	

	local rows={}
	local row = {}
	local colindex = {}
	if exportParams.context.columns then
		for idx,col in ipairs(exportParams.context.columns) do
			table.insert(colindex,col.id);
		end
	else
		if exportParams.context.sourceName and exportParams.context.sourceName ~= exportParams.context.name and mainDS.description.outstreams then
			for _,outstream in ipairs(mainDS.description.outstreams) do
				if outstream.name == exportParams.context.name then
					for _,indicator in ipairs(outstream.description.indicators) do
						if indicator.hidden ~= true then
							table.insert(colindex,datasourceName .. "." .. indicator.name)
						end
					end
				end
			end
		else
			for idx,col in ipairs(mainDS.description.columns) do
				table.insert(colindex,datasourceName .. "." .. col.name)
			end
		end
	end
	if exportParams.context.firstRowHaveColumnNames then
		if exportParams.context.columns then
			for idx,col in ipairs(exportParams.context.columns) do
				table.insert(row,csvutils.escapeCSV(col.alias or col.name,exportParams.context.delimiter));
			end
		else
			if exportParams.context.sourceName and exportParams.context.sourceName ~= exportParams.context.name and mainDS.description.outstreams then
				for _,outstream in ipairs(mainDS.description.outstreams) do
					if outstream.name == exportParams.context.name then
						for _,indicator in ipairs(outstream.description.indicators) do
							if indicator.hidden ~= true then
								table.insert(row,csvutils.escapeCSV(indicator.name,exportParams.context.delimiter));
							end
						end
					end
				end
			else
		
				for idx,col in ipairs(mainDS.description.columns) do
					table.insert(row,csvutils.escapeCSV(col.name,exportParams.context.delimiter));
				end
			end
		end
		table.insert(rows,table.concat(row,exportParams.context.delimiter));
	end
	local rownum = 0
	local outs = 0;
	local function BOM()
		if outs == 0 and exportParams.context.withBOM then
			return string.char(0xEF,0xBB,0xBF)
		else
			return "";
		end
	end
	initDatasources();
	local statop,reg = moveNext();
	if not statop then
		headers["Response"] = messages.messageKey('database_conection_error',{description=err}).description;
		return status, headers, coroutine.wrap(function() coroutine.yield("") end)
	end

	function moredata()
		while statop and reg do
			row = {}
			LogDebug("rownum %d ", rownum);
			for _,idx in ipairs(colindex) do
				local value = reg[idx]
				if value == json.null then value = nil end
				tbinsert(row,csvutils.escapeCSV(value,exportParams.context.delimiter) or '')
			end
			tbinsert(rows,table.concat(row,exportParams.context.delimiter));
			rownum=rownum+1
			if math.fmod(rownum,100) == 0 then
				socket.sleep(0.2);
			end
			if math.fmod(rownum,200) == 0 then
				LogDebug(" output %d rows -1",#rows);

				if not coroutine.yield(BOM() .. table.concat(rows,"\r\n") .."\r\n") then -- se corto la conexion del cliente
					return nil
				end
				outs=outs+1
				rows = {}
			end
			statop,reg = moveNext();
		end
		if #rows>0 then
			LogDebug(" output %d rows -2",#rows);
			if not coroutine.yield(BOM() .. table.concat(rows,"\r\n") .."\r\n") then -- se corto la conexion del cliente
				return nil
			end

			return nil;
		else
			return nil;
		end
	end


	headers["Response"] = "ok"
	headers["Content-Type"]="application/octet-stream"
	headers["Content-Disposition"] = [[attachment; filename="]] .. fileName .. "\""
	headers["Set-Cookie"] = "downloadStarted=true; Path=/" -- used by the jquery download plugin

	-- Return an iterator co-rutine to send chunks of opened file in case blob is huge.
	return "200 Ok", headers,
		coroutine.wrap(function()
			moredata();
		end)
end
local function get(web,tokenId,appName,datasourceName)
	local ts = os.clock();
	local result
	local r
	local args
	status,args = utils.check_json_params(web.input["post_data"] or "{}")
	if not status then
		return json.encode({status=false,result=args})
	end
	local vccName = runtime.getCurrentVCCName() or tweetyio.get_token_data(tokenId).vccName;
	local status,ds=buildDatasource(vccName,appName,datasourceName,"retrieve");
	if status then
		status,result =ds:get(args.content,args.context)
		r = json.encode({status=status,result=result});
		if status then
			LogDebug("GET completed %f seconds %d rows",os.clock()-ts,#result.rows);
		end
		return r
	end
	return json.encode({status=false,result=ds});
end


local function update(web,tokenId,appName,datasourceName)
	local status,result,ds,args;
	local vccName = runtime.getCurrentVCCName() or tweetyio.get_token_data(tokenId).vccName;
	status,args = utils.check_json_params(web.input["post_data"] or "{}")
	if not status then
		return json.encode({status=false,result=args})
	end
	local app = application.new{vccName=vccName}
	status,result = app:initialize(appName);
	if status then	
--		status,result = captcha.verify(app,args,web.vars.REMOTE_ADDR);
--		if status then
			status,ds=buildDatasource(vccName,appName,datasourceName,"update",app);
			if status then
				status,result = ds:update(args.content,args.context)
				return json.encode({status=status,result=result});
			end
			return json.encode({status=false,result=ds});
--		end
	end
	return json.encode({status=false,result=result});
end

local function save(web,tokenId,appName,datasourceName)
	local status,result,ds,args;
	local vccName = runtime.getCurrentVCCName() or tweetyio.get_token_data(tokenId).vccName;
	status,args = utils.check_json_params(web.input["post_data"] or "{}")
	if not status then
		return json.encode({status=false,result=args})
	end
	local app = application.new{vccName=vccName}
	status,result = app:initialize(appName);
	if status then
--		status,result = captcha.verify(app,args,web.vars.REMOTE_ADDR);
--		if status then
			status,ds=buildDatasource(vccName,appName,datasourceName,"store",app);
			if status then
				status,result = ds:save(args.content,args.context);
				return json.encode({status=status,result=result});
			end
			return json.encode({status=false,result=ds});
--		end
	end
	return json.encode({status=false,result=result});
end


local function delete(web,tokenId,appName,datasourceName)
	local status,result,ds,args
	local vccName = runtime.getCurrentVCCName() or tweetyio.get_token_data(tokenId).vccName;	
	status,args = utils.check_json_params(web.input["post_data"] or "{}")
	if not status then
		return json.encode({status=false,result=args})
	end
	local app = application.new{vccName=vccName}
	status,result = app:initialize(appName);
	if status then
--		status,result = captcha.verify(app,args,web.vars.REMOTE_ADDR)
--		if status then
			status,ds = buildDatasource(vccName,appName,datasourceName,"delete",app);
			if status then
				status,result = ds:delete(args.content,args.context);
				return json.encode({status=status,result=result});
			end
			return json.encode({status=false,result=ds});
--		end
	end
	return json.encode({status=false,result=result});
end


function initialize(_M, prefix,appLocation)
	prefix = prefix or ""
	_M:dispatch_post(get, prefix .. "/([^/]+)/applications/([^/]+)/datasource/([^/]+)/get")
	_M:dispatch_post(save, prefix .. "/([^/]+)/applications/([^/]+)/datasource/([^/]+)/save")
	_M:dispatch_post(update, prefix .. "/([^/]+)/applications/([^/]+)/datasource/([^/]+)/update")
	_M:dispatch_post(delete, prefix .. "/([^/]+)/applications/([^/]+)/datasource/([^/]+)/delete")
	_M:dispatch_wsapi(export,prefix .."/([^/]+)/applications/([^/]+)/datasource/([^/]+)/export/([^/]+)/([^/]+)");

	_M:dispatch_post(get, prefix .. "/([^/]+)/applications/([^/]+)/datasource_cti/([^/]+)/get")
	_M:dispatch_post(save, prefix .. "/([^/]+)/applications/([^/]+)/datasource_cti/([^/]+)/save")
	_M:dispatch_post(update, prefix .. "/([^/]+)/applications/([^/]+)/datasource_cti/([^/]+)/update")
	_M:dispatch_post(delete, prefix .. "/([^/]+)/applications/([^/]+)/datasource_cti/([^/]+)/delete")
	_M:dispatch_wsapi(export,prefix .."/([^/]+)/applications/([^/]+)/datasource_cti/([^/]+)/export/([^/]+)/([^/]+)");

end

