module(..., package.seeall)
local function ep()

	local expr = require "loop.compiler.Expression"
	local eparse = expr {
		values = {
			number = "(%d*%.?%d+)",
			keyname = "([%a_][%w_]*)",
			const = "['\"]([%%]*[^'\"]*[%%]*)['\"]",
			param = "(@[%a_][%w_]*)",

		},
		operators = {
			or_op   = " OR ",
			and_op = " AND ",
			gte_op   = " >= ",
			lte_op   = " <= ",

			eq_op = " = ",
			gt_op = " > ",
			lt_op = " < ",
			like_op = " LIKE ",

		},
		precedence={
			{"and_op"},{"or_op"},

			{
				"gte_op","lte_op",

				"eq_op","lt_op","gt_op",
				"like_op"
			}

		},
	}

	function eparse:number  (value)  return {_type="number",value=value} end
	function eparse:const(value)     return {_type="const",value=value} end
	function eparse:keyname(name)   return {_type="keyname",value=name} end
	function eparse:param(name)   return {_type="param",value=name} end

	function eparse:eq_op(left,right) return {_type="op",optype="=",left = left,right =right} end
	function eparse:gt_op(left,right) return {_type="op",optype=">",left = left,right =right} end
	function eparse:gte_op(left,right) return {_type="op",optype=">=",left = left,right =right} end
	function eparse:lt_op(left,right) return {_type="op",optype="<",left = left,right =right} end
	function eparse:lte_op(left,right) return {_type="op",optype="<=",left = left,right =right} end

	function eparse:and_op(left,right) return {_type="op",optype="and",left = left,right =right} end
	function eparse:or_op(left,right) return {_type="op",optype="or",left = left,right =right} end
	function eparse:like_op(left,right) return {_type="op",optype="like",left = left,right =right} end
	return eparse;
end

function datasourceBase(self)
	function self:parse_query(query)
		LogDebug("EVAL QUERY [%s]",query);
		local eparse = ep();
		local result,query_tree = pcall(eparse.evaluate,eparse,query)
		if not result then
			LogDebug( "PARSE ERROR %s ",query_tree);
		end
		return result,query_tree
	end
	local mappedColumnsResult 
	function self:mappedColumns() -- devuelve una lista de las columnas
								  -- pero aplicando el mapeo de los retrieve.inputParams
		local map = {}
		if mappedColumnsResult then
			return mappedColumnsResult
		end
		mappedColumnsResult = {}
		if self.description.retrieve.enabled then
		
			for _,param in ipairs(self.description.retrieve.inputParams) do
				if param.column then
					map[param.column] = param.name
				end
			end
		end
		for _,column in ipairs(self.description.columns) do
			local cname = map[column.name]
			column.field = cname
			table.insert(mappedColumnsResult,column)
		end
		return mappedColumnsResult;
	end
	function self:filterOptimizer ( filter)
		local i=1
		local segment
		local function promoveCloseParenthesis(src,dst)

			if src.close then
				local ic = #src.close
				local io = #(src.open or "")
				while ic>0 do
					if io == 0 then
						dst.close = (dst.close or "") .. ")"
					else
						io=io-1
					end
					ic = ic-1
				end
			end
		end
		local function promoveOpenParenthesis(src,dst)
			if src.open then
				local io = #src.open
				local ic = #(src.close or "")
				while io>0 do
					if ic == 0 then
						dst.open = (dst.open or "") .. "("
					else
						ic=ic-1
					end
					io = io-1
				end
			end

		end

		while filter.criteria[i] do
			segment = filter.criteria[i]

			if not segment.expr then
				-- si es un or se lleva hacia atras
				if not segment.open and segment.combinator == "OR"  and i>1 then
					filter.criteria[i-1].combinator = "OR"
				end
				if segment.close and i>1 then
					promoveCloseParenthesis(segment,filter.criteria[i-1])
				end
				if segment.open and i<#filter.criteria then
					promoveOpenParenthesis(segment,filter.criteria[i+1])
				end

				table.remove(filter.criteria,i)
			else
				i=i+1
			end
		end
	--	print(filterTostring(filter))

		--for idx,segment in ipairs(filter.criteria) do
			--if segment.open then
			--	l = 1
			--[[
				while segment.open and segment.close do

					segment.open = string.sub(segment.open,1,#segment.open-1)

					if segment.open == "" then segment.open = nil end

					segment.close = string.sub(segment.close,1,#segment.close-1)

					if segment.close == "" then segment.close = nil end
				end
			--]]


	 --[[
				if segment.open then
					local r = #segment.open
					for x=idx+1,#filter.criteria do
						r = r+ #(filter.criteria[x].open or "")
						r = r- #(filter.criteria[x].close or "")
						if r<=0 then
							q = #filter.criteria[x].close-r
							q = q -1
							if q >0 then

								segment.open = string.sub(segment.open,q+1)
								print(filter.criteria[x].close,q)
								filter.criteria[x].close = string.sub(filter.criteria[x].close,1,#filter.criteria[x].close-q)
								print(filter.criteria[x].close)

								if filter.criteria[x].close == "" then
									filter.criteria[x].close = nil
								end
								if segment.open == "" then
									segment.open = nil
								end
							end
							--print(filter.criteria[x].column,q)
						end
					end
				end
				--]]

			--end
		--end
	--	print(filterTostring(filter))

		return filter
	end
	function self:get_filter_ref(filterName) 
		local filter;
		if type (filterName) == "string" then
			for _,fl in ipairs(self.description.filters) do
				if fl.name == filterName then
					filter = fl
					break
				end
			end
		else
			filter = filterName;
		end
		return filter;
	end
	function self:get_filter(filterName,params,fieldFormatFn)
		local filter
		local hparams ={}
		local hvalues ={}
		for _,param in ipairs(self.description.retrieve.inputParams) do
			hparams[param.column or param.name]=param
		end
		for _,param in ipairs(params) do
			hvalues[param.name]=param.value
		end
		if type (filterName) == "string" then
			for _,fl in ipairs(self.description.filters) do
				if fl.name == filterName then
					filter = fl
					break
				end
			end
		else
			filter=filterName;
		end
		LogDebug("FILTER %s = %s ",tostring(filterName),tostring(filter))
		if filter then

			local cp = 0;
			for idx,segment in ipairs(filter.criteria) do
				if segment.column and segment.expr and segment.comparator then
					if hparams[segment.column] then -- sino esta en los inputParams es porque es un campo nuevo de una tabla
													-- y no se ha ido a operations a actualizar
					
						if string.sub(segment.expr,1,1) == "@" then

							if hvalues[segment.column .. segment.expr] then
								segment.expr = fieldFormatFn(hparams[segment.column],hvalues[segment.column .. segment.expr],segment.comparator=='%')
								hvalues[segment.column]=nil
							else
								segment.expr=nil;
							end
						else
							segment.expr = fieldFormatFn(hparams[segment.column],segment.expr,segment.comparator=='%')
							hvalues[segment.column]=nil
						end
					end

				end
			end
			for k,v in pairs(hvalues) do
				if hparams[k] then
					table.insert(filter.criteria,1,{column=k,comparator="=",expr=fieldFormatFn(hparams[k],v,false),combinator="AND"})
				end
			end
			return self:filterOptimizer(filter);
		else
			filter = {criteria={}}
			for k,v in pairs(hvalues) do
				if hparams[k] then
					table.insert(filter.criteria,{column=k,comparator="=",expr=fieldFormatFn(hparams[k],v,false),combinator="AND"})
				end
			end
		end
		return filter;
	end
	function self:cursor(queryData)
		local nf;
		local filterName;
		LogDebug("GENERIC CURSOR")
--		LogDebug("%s",json.encode(queryData,true))
		
		if queryData.filter or queryData.orderby then
			filterName = "__cursor__"
			nf = {criteria={},order={},name=filterName}
			if queryData.filter then
				for _,entry in ipairs(queryData.filter.criteria) do
					table.insert(nf.criteria,entry)
				end
			end
			if queryData.orderby then
				for _,entry in ipairs(queryData.orderby) do
					table.insert(nf.order,entry)
				end
			end
--			LogDebug("NF %s ",json.encode(nf,true))
			self.description.filters = self.description.filters or {}
			table.insert(self.description.filters,nf)
		end
		LogDebug("CALL GET %s,%s ",json.encode(queryData.parameters or {}),json.encode({filter=filterName,limit=-1}));
		local stat,rset = self:get(queryData.parameters,{filter=filterName,name=queryData.name,sourceName=queryData.sourceName});
		
		if stat then
			local nrow = 1;
			LogDebug("cursor return iterator for %d rows",#rset.rows)
			return true,{
				next = function()
					local reg = rset.rows[nrow]
					nrow = nrow+1
					for idx,name in ipairs(rset.colnames) do
						reg[name]=reg[idx]
					end
					return reg
				end,
				eof = function()
					return rset.rows[nrow]==nil
				end
			}
		else
			return false,rset -- error 
		end
	
	end
	return self
end


