module(...,package.seeall,inconcert.orbit.application.Decorate)

local workspace = require (tweety_modules_prefix .. "workspace")
local template = require (tweety_modules_prefix .. "template")
local application = require (tweety_modules_prefix .. "application")
local messages = require (tweety_modules_prefix .. "messages")
require "inconcert.orbit.application"
local runtime = require (tweety_modules_prefix .. "runtime")
local config = require (tweety_modules_prefix .. "config")
local manage = require ("reporter.manage")
local cassandra = require ("lua_cassandra")
local tweetyio = require (tweety_modules_prefix .. "tweetyio");
local utils = require (tweety_modules_prefix .. "utils")

local function workspace_catalog_app(web)
	local vccName = runtime.getCurrentVCCName();
	local status,catalog = workspace.new{vccName=vccName}:getCatalog();
	if status then
		local catalogViews = {}
		for _,entry in pairs(catalog) do
			local  app = application.new{vccName=vccName}
			local status, result = app:initialize(entry.name)
			if status then
				local catalogApp = app:getCatalog();
				for _,view in pairs(catalogApp.views) do
					local entryView = {}
					entryView.id = entry.id .. "." .. view.name;
					entryView.name = entry.name .. "." .. view.name;
					entryView.description = entry.description .. "." .. view.name;
					entryView.version = entry.version;
					entryView.created_at = entry.created_at;
					table.insert(catalogViews, entryView);
				end
			else
				return json.encode({status=status,result=result});
			end
		end
		for _,entry in pairs(catalogViews) do table.insert(catalog, entry) end
		table.sort(catalog,function(a,b)
			return b.created_at > a.created_at
		end);
	end
	return json.encode({status=status,result=catalog});
end

local function workspace_catalog(web)
	local vccName = runtime.getCurrentVCCName();
	local status,result=workspace.new{vccName=vccName}:getCatalog()
	return json.encode({status=status,result=result})
end

local function workspace_catalog_post(web)
	local vccName = runtime.getCurrentVCCName();
	local filter = json.decode(web.input["post_data"] or "{}")
	local status,result=workspace.new{vccName=vccName}:getCatalog(nil,filter)
	return json.encode({status=status,result=result})
end

local function workspace_delete_application(web,appName)
	local status,result;
	local vccName = runtime.getCurrentVCCName();
	local wks=workspace.new{vccName=vccName}
	status,result = wks:deleteApplication(appName);
	return json.encode({status=status,result=result})
end

local function template_catalog (web, appClass, appSubClass)
	if appClass == "" then appClass = nil end
	if appSubClass == "" then appSubClass = nil end

	LogInfo("[DESIGNER:template_catalog] TEMPLATE CLASS %s SUBCLASS %s",appClass or "undefined",appSubClass or "undefined")

	local languageId, webLanguageCode = utils.GetLanguageFromWebInfo(web)
	local _T = utils.MakeTextTranslator(languageId, webLanguageCode)

	local status,result = template.list(appClass, appSubClass, _T)
	return json.encode({status=status,result=result})
end

local function image_catalog(web)
	local result = {}
	local imagePath = config.resourcePath .. config.pathSeparator .. "images"
	local iter, dir_obj = lfs.dir (imagePath)
	local dir_entry = iter(dir_obj)
	if not dir_entry then
		LogDebug("can't open path %s",imagePath);
		return json.encode({status=false,result=messages.messageKey('path_access_error',{path=imagePath})})
	end
	while dir_entry do
		if dir_entry ~= "." and dir_entry ~= ".." then
			table.insert(result,dir_entry)
		end
		dir_entry = iter(dir_obj)
	end
	return json.encode({status=true,result=result})
end

local function datasource_catalog(web)
	local result = {}
	local datasourcePath = config.resourcePath .. config.pathSeparator .. "datasources"
	local iter, dir_obj = lfs.dir (datasourcePath)
	local dir_entry = iter(dir_obj)
	if not dir_entry then
		LogDebug("can't open path %s",datasourcePath);
		return json.encode({status=false,result=messages.messageKey('path_access_error',{path=datasourcePath})})
	end
	while dir_entry do
		if dir_entry ~= "." and dir_entry ~= ".."  and string.sub(dir_entry,-3) == ".ds" then
			table.insert(result,string.sub(dir_entry,1,#dir_entry-3))
		end
		dir_entry = iter(dir_obj)
	end
	return json.encode({status=true,result=result})
end

local function odbc_info (web)
	local status,odbc = pcall (require,"odbc")
	if not status then
		return json.encode({status=false,result=messages.messageKey('module_not_found',{modulename="odbc"})});
	end
	local env = odbc.environment();
	local drivers = {}
	local sources = {}
	local t = env:drivers()
	for k,dentry in ipairs(t) do
		if type(dentry[2]) == "table" and dentry[2].Driver then
			table.insert(drivers,{name=dentry[1],description=dentry[2].Description})
		end
	end
	local t = env:datasources()
	for k,dentry in ipairs(t) do
		table.insert(sources,{name=dentry[1],driver=dentry[2]})
	end

	env:destroy()
	return json.encode({status=true,result={drivers=drivers,datasources=datasources}})
end

--databaseName: "mmprodat"
--driverName: "MyODBC"
--serverName: "192.168.65.100"
--sqlStatement: "Select * from clientes where phone = '095458160'"
--storeResultVar: "elCliente"
--userName: "usraccmw"
--userPassword: "inc2001"
local function odbc_columns(web)
	local res;
	local queryData = json.decode(web.input["post_data"] or "{}")
	if not queryData.databaseName or
	   not queryData.driverName or
	   not queryData.serverName or
	   not queryData.userName or
	   not queryData.sqlStatement then
		return json.encode({status=false,result = messages.messageKey("invalid_request")})
	end
	local status,odbc = pcall (require,"odbc")
	if not status then
		return json.encode({status=false,result=messages.messageKey('module_not_found',{modulename="odbc"})});
	end
	local cnn,env = odbc.driverconnect(string.format("DRIVER=%s;SERVER=%s;UID=%s;PWD=%s;DATABASE=%s",
		queryData.driverName,
		queryData.serverName,
		queryData.userName,
		queryData.userPassword or "",
		queryData.databaseName))

	if not cnn then
		return json.encode({status=false,result=messages.messageKey('database_error',{description=tostring(env)})})
	end

	local stmt, err = cnn:statement()
	if not stmt then
		cnn:destroy()
		env:destroy()
		return json.encode({status=false,result=messages.messageKey('database_error',{description=tostring(err)})})
	end
	--stmt:setautocommit(false);
	local cur,err =stmt:execute(queryData.sqlStatement);
	if not cur then
		stmt:destroy()
		cnn:destroy()
		env:destroy()
		return json.encode({status=false,result=messages.messageKey('database_error',{description=tostring(err)})})
	else
		if type(cur) == 'number' then
			res = {}
		else
			local names=cur:colnames();
			local types=cur:coltypes();
			res = {}
			for k,v in ipairs(names) do
				table.insert(res,{name=v,type=types[k]});
			end
			
		end
		cur:close();
	end
	stmt:destroy()
	cnn:destroy()
	env:destroy()
	return json.encode({status=true,result = res})
end

local function application_catalog_load(web,appId)
	local status,result
	local app = application.new{vccName=runtime.getCurrentVCCName()}
	status,result = app:initialize(appId);
	if status then
		result = app:getCatalog()
	end
	return json.encode({status=status,result=result});
end

local function application_settokendata(web,appId,tokenId,tokenData)
	if	appId == "chat{9CA752E9-3BA3-4FEB-931D-3E61734151D8}" or 
		appId == "chatbottest{9CA752E9-3BA3-4FEB-931D-3E61734151D8}" or 
		appId == "webcall{9CA752E9-3BA3-4FEB-931D-3E61734151D8}" or 
		appId == "ctfrm{5E1E3B54-5ED0-42be-A2C1-FDFEC827199A}" 
	then
		return json.encode({status=true})
	end
	local err = true;
	pcall(
		function()
			tokenData = json.decode(tokenData)
			err = false;
		end
	)
	if err then
		return json.encode({status=false,result=messages.messageKey('argument_error',{module='designer',func='application_settokendata',param='tokenData'})});
	end
	local app = application.new{vccName=runtime.getCurrentVCCName()}
	status,result = app:initialize(appId);
	if status then
		local catalog = app:getCatalog()
		status,result = app:initialize(appId);
		if status then
			local catalog = app:getCatalog()
			catalog.tokenData = catalog.tokenData or {}
			catalog.tokenData[tokenId] = catalog.tokenData[tokenId] or {}
			for name,value in pairs(tokenData) do
				catalog.tokenData[tokenId][name] = value
			end
			status,result = app:save();
			return json.encode({status=status,result=result});
		else
			return json.encode({status=false,result=messages.messageKey('argument_error',{module='designer',func='setauthdata',param='appId',value=appId})})
		end
	end
	return json.encode({status=status,result=result});
end

local function application_setauthfilterarguments(web,appId,authArguments)
	local status,result
	local err = true;

	if	appId == "chat{9CA752E9-3BA3-4FEB-931D-3E61734151D8}" or 
		appId == "webcall{9CA752E9-3BA3-4FEB-931D-3E61734151D8}" or 
		appId == "ctfrm{5E1E3B54-5ED0-42be-A2C1-FDFEC827199A}" 
	then
		return json.encode({status=true})
	end

	pcall(
		function()
			authArguments = json.decode(authArguments)
			err = false;
		end
	)

	if err then
		return json.encode({status=false,result=messages.messageKey('argument_error',{module='designer',func='application_setauthfilterarguments',param='authArguments',value=authArguments})});
	end
	local app = application.new{vccName=runtime.getCurrentVCCName()}
	status,result = app:initialize(appId);
	if status then
		local catalog = app:getCatalog()
		catalog.authfilter = catalog.authfilter or {}
		catalog.authfilter._wholeapp = catalog.authfilter._wholeapp or {userName="guest"}
		catalog.authfilter._matcharguments = catalog.authfilter._matcharguments or {};
		for name,value in pairs(authArguments) do
			for i=1,#catalog.authfilter._matcharguments do
				local arg = catalog.authfilter._matcharguments[i];
				if arg.name == name and arg.value == value then
					found=true;
					break;
				end
			end
			if not found then
				table.insert(catalog.authfilter._matcharguments,{name=name,value=value});
			end
		end
		status,result = app:save();
		return json.encode({status=status,result=result});
	else
		return json.encode({status=false,result=messages.messageKey('argument_error',{module='designer',func='setauthdata',param='appId',value=appId})})
	end
	return json.encode({status=status,result=result});
end


local function application_setproperties(web,appId)
	local status,result
	local properties = json.decode(web.input['content'])
	local app = application.new{vccName=runtime.getCurrentVCCName()}
	status,result = app:initialize(appId);
	if status then
		result = app:getCatalog()
		for name,value in pairs(properties) do
			result[name]=value
		end
		status,result = app:save();
	end
	return json.encode({status=status,result=result});
end

local function save_form(web,appName,formName)
	local app = application.new{vccName=runtime.getCurrentVCCName()}
	local status,result = app:initialize(appName)
	if status then
		LogInfo("[DESIGNER:save_form] formName: '%s'", formName)
		status,result = app:saveForm(formName,json.decode(web.input['content']))
	end
	return json.encode({status=status,result=result});
end

local function save_form_params(web,appName,formName)
	local app = application.new{vccName=runtime.getCurrentVCCName()}
	local status,result = app:initialize(appName)
	if status then
		status,result = app:saveFormParams(formName,json.decode(web.input['content']))
	end
	return json.encode({status=status,result=result});
end

local function read_form(web,appName,formName)
	local app = application.new{vccName=runtime.getCurrentVCCName()}
	local status,result = app:initialize(appName)
	if status then
		status,result = app:getFormData(formName)
	end
	return json.encode({status=status,result=result})
end

local function add_form(web,appName,formName)
	local status,result
	local app = application.new{vccName=runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		status,result = app:addForm(formName,web.input['content'])
	end
	return json.encode({status=status,result=result});
end

local function delete_form(web,appName,formName)
	local status,result
	local app = application.new{vccName=runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		status,result = app:deleteForm(formName)
	end
	return json.encode({status=status,result=result});
end

local function add_datasource(web,appName,datasourceName)
	local status,result
	local app = application.new{vccName=runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		local dsdata = json.decode(web.input['content']);
		status,result = app:addDatasource(datasourceName,web.input['content'])
		if status then
			if dsdata.description["store"].enabled then
				ds = require (tweety_modules_prefix .. dsdata.description["store"].type).new(dsdata)
				ds.description.name = datasourceName;
				ds.name = datasourceName;
				ds.appName = appName;
				ds.inconcert_authUser = inconcert.authUser;
				if ds and ds.configure then
					status,result = ds:configure(dsdata);
				end
			end
			if status then
				status,result = app:saveDatasourceData(datasourceName,json.encode(dsdata,true))
			end
		end
	end
	return json.encode({status=status,result=result});
end

local function application_new (web, appName, parameters)
	local parameters=json.decode(parameters)

	LogInfo("[DESIGNER:application_new] Creating New Application '%s', parameters: '%s'.", appName, parameters)
	local languageId, webLanguageCode = utils.GetLanguageFromWebInfo(web)
	local _T = utils.MakeTextTranslator(languageId, webLanguageCode)

	local status,result = template.createApplication(runtime.getCurrentVCCName(), appName, parameters, _T)
	return json.encode({status=status,result=result});
end

local function application_delete (web,appName,parameters)
	local parameters=json.decode(parameters)
	local status, result = template.createApplication(runtime.getCurrentVCCName(), appName, parameters)
	return json.encode({status=status,result=result});
end

local function application_export(web,appName)
	local status,result
	local vccName = runtime.getCurrentVCCName();
	local app = application.new{vccName=vccName}
	status,result = app:initialize(appName)
	if status then
		status,result = app:buildExportBundle()
	end
	if status then
		web:content_type("application/octet-stream")
		web.headers["Set-Cookie"] = "downloadStarted=true; Path=/" -- used by the jquery download plugin
		return result
	end
end

local function delete_datasource(web,appName,datasourceName)
	local status,result,dsdata
	local app = application.new{vccName=runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		status,dsdata = app:getDatasourceData(datasourceName);
		if status then
			if dsdata.description["store"].enabled then
				if dsdata.description["store"].type then
					ds = require (tweety_modules_prefix  .. dsdata.description["store"].type).new(dsdata)
					if ds and ds.deleteAditionalData then
						status,result = ds:deleteAditionalData(dsdata);
					end
				end
			end
			if status then
				status,result = app:deleteDatasource(datasourceName)
			end
		else -- no existe el datasource, seguro es el problema de la renombrada con el mismo nombre y posterior borrado
			status,result = app:deleteDatasource(datasourceName)

			return json.encode({status=status,result=result});
		end
	end
	return json.encode({status=status,result=result});
end

local function truncate_datasource(web,appName,datasourceName)
	local status,result,dsdata
	local app = application.new{vccName=runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		status,dsdata = app:getDatasourceData(datasourceName);
		if status then
			
			if dsdata.description["delete"].enabled then
				if dsdata.description["delete"].type then
					ds = require (tweety_modules_prefix  .. dsdata.description["delete"].type).new(dsdata)
					if ds.truncate then
						status,result = ds.truncate();	
					else
						status=false;
						result=messages.messageKey('invalid_operation');
					end
				end
			else
				status=false;
				result=messages.messageKey('invalid_operation');
			end
		end
	end
	return json.encode({status=status,result=result});
end


local function read_datasource(web,appName,datasourceName)
	local status,result
	local app = application.new{vccName=runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		status,result = app:getDatasourceData(datasourceName)
		if status then
			if result.private then
				result.private.userPassword = "***********";
			end
		end
	end
	return json.encode({status=status,result=result});
end

local function save_datasource(web,appName,datasourceName)
	local status,result,dsdata
	local data = json.decode(web.input['content']);
	local app = application.new{vccName=runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		status,dsdata = app:getDatasourceData(datasourceName);
		if status then
			if dsdata.private and data.private then
				if data.private.userPassword == "***********" then
					data.private.userPassword = dsdata.private.userPassword;
				end
			end
			if dsdata.description["store"].type then	 -- por defecto del wizard no se graba el tipo en store
														-- cuando se dice que es un sp y se usa para read
				ds = require (tweety_modules_prefix .. dsdata.description["store"].type).new(dsdata)
				ds.appName = appName;
				ds.description.name = datasourceName;
				ds.name = datasourceName;
				ds.inconcert_authUser = inconcert.authUser;

				if ds and ds.configure then
					status,result = ds:configure(data);
				end
			end
		end
		if status then
			status,result = app:saveDatasourceData(datasourceName,json.encode(data,true))
		end
	end
	return json.encode({status=status,result=result});
end
local function read_details_projected_fields(web)
	local fields = {}
	local res = {}

	table.insert(res,{name="Agent",id="Interaction.Agent"});
	table.insert(res,{name="Campaign",id="Interaction.Campaign"});
    table.insert(res,{name= "AttentionTime",id="Interaction.AttentionTime"});
   
    table.insert(res,{
        name= "ChatSource",
        id= "Interaction.ChatSource"
    });
    table.insert(res,{
        name= "DispositionCode",
        id= "Interaction.DispositionCode"
    });
    table.insert(res,{
        name= "DispositionIsGoal",
        id= "Interaction.DispositionIsGoal"
    });
    table.insert(res,{
        name= "DurationTime",
        id= "Interaction.DurationTime"
    });
    table.insert(res,{
        name= "EndDate",
        id= "Interaction.EndDate"
    });
    table.insert(res,{
        name= "FirstAgent",
        id= "Interaction.FirstAgent"
    });
    table.insert(res,{
        name= "StartAttention",
        id= "Interaction.StartAttention"
    });
    table.insert(res,{
        name= "StartDate",
        id= "Interaction.StartDate"
    });
    table.insert(res,{
        name= "State",
        id= "Interaction.State"
    });
    table.insert(res,{
        name= "WaitingTime",
        id= "Interaction.WaitingTime"
    });
    table.insert(res,{
        name= "WrapupTime",
        id= "Interaction.WrapupTime"
    });

--	table.insert(res,{name="HasPostSurvey",id="Interaction.survey._HasPostSurvey"});
--	table.insert(res,{name="HasPreSurvey",id="Interaction.survey._HasPreSurvey"});


--[[
	local summarizerKS = cassandra:CassandraConnection("SummarizerData")
	local reportNames = manage.listReports(summarizerKS,nil,"detail","system")

	for _,reportName in ipairs(reportNames) do
		local report = manage.getReport(summarizerKS,"system",reportName);
		if report.projectFields then
			for _,field in ipairs(report.projectFields) do
				if not fields[field] then
					fields[field]=field;
					table.insert(res,{name=field,id="Interaction." .. field})
				end
			end
		end
	end
]]
	table.sort(res,function(a,b) return a.name < b.name end )
	return json.encode({status=true,result=res})
end
	

local function read_summary_cut(web,cutName)
	summarizerKS = cassandra:CassandraConnection("SummarizerData")
	local function getIndicatorName(indicatorName,cutName)
		local c=1
		local s =string.gsub(indicatorName,"(%w+)%.",function(segment)

			local t = 1
			local found
			string.gsub(cutName ..".","(%w+)%.",function(cutsegment)
				if t == c and cutsegment == segment then
					found=true
				end
				t=t+1
			end)
			c=c+1
			if found then
				return ""
			end
		end)
		return s;
	end
	
	local function hierarchize(tree,tree_hash,fullName)
		local res = {}
		local currentFolder=tree_hash
		local parentNode = tree;
		
		local folderName = string.gsub(fullName,"(%w+)%.",
			
			function (folderName)
				local folder = currentFolder[folderName] 
				if not folder then
					folder = {tree_node = {caption=folderName}}
					currentFolder[folderName]=folder
					parentNode.items = parentNode.items or {}
					table.insert(parentNode.items,folder.tree_node)
				end	
				
				parentNode = folder.tree_node;
				currentFolder = folder
				return ""
			end
		)
	
		currentFolder[folderName] = {tree_node = {caption=folderName}}
		parentNode.items = parentNode.items or {}
		table.insert(parentNode.items,currentFolder[folderName].tree_node)
		return currentFolder[folderName].tree_node;
	end
	local tree = {}
	local tree_hash = {}
	
	local cutNames;
	if not cutName or cutName == "" then
		cutNames = manage.listCuts(summarizerKS,"system");
	else
		cutNames = {cutName}
	end
	table.sort(cutNames);
	local cuts={}
	local cuts_tree={}
	local cuts_tree_hashes={}
	for _,cutName in ipairs(cutNames) do
		cuts[cutName] = manage.getCut(summarizerKS,"system",cutName)
		cuts[cutName].id = cutName;
		local cutnode =hierarchize({items=tree},tree_hash,cutName)
		cutnode.cut = cuts[cutName];
		local columns_items = {}
		for _,columnName in ipairs(cutnode.cut.columns) do
			if columnName ~= "Slice" then
				table.insert(columns_items,{
					caption=columnName,
					column={
						cut=cutnode.cut,
						name=columnName,
						displayName=columnName
					}
				})
			end
		end
		cutnode.items = {
			{caption = "Columns",
				items=columns_items
			},
			{caption = "Indicators",
				items={}
			}
		}
		cuts_tree[cutName]=cutnode.items[2]; -- para que los indicadores se inserten dentro de Indicators
	end
	local indicatorsNames=manage.listSummaryIndicators(summarizerKS,"system")
	table.sort(indicatorsNames)
	for _,indicatorName in ipairs(indicatorsNames) do
		local ind =  manage.getSummaryIndicator(summarizerKS,"system",indicatorName)
		if ind then
			local CutIDs = ind.CutIDs
			for cutID,_ in pairs(CutIDs) do
				local cuttree = cuts_tree[cutID]
				if cuttree then
					cuts_tree_hashes[cutID] = cuts_tree_hashes[cutID] or {}
					cuttree.items = cuttree.items or {}
					local i = hierarchize(cuttree,cuts_tree_hashes[cutID],getIndicatorName(indicatorName,cutID))
					i.indicator = {cut=cuts[cutID],displayName=i.caption,name=indicatorName}
				end
			end
		end
	end
	
	return(json.encode({status=true,result=tree}));
		
end




local function read_resource(web,...)
	local status,result
	local arg={...}
	
	local vccName,appName,fileName
	
	if #arg == 2 then
		appName=arg[1]
		fileName=arg[2]
		vccName=runtime.getCurrentVCCName()
	elseif #arg ==3 then
		vccName=runtime.getCurrentVCCName() or (tweetyio.get_token_data(arg[1]) or {}).vccName or arg[1]
		appName=arg[2]
		fileName=arg[3]

	end
	local app = application.new{vccName=vccName}
	status,result = app:initialize(appName)
	if status then
		status,result = app:readResource(fileName,true);
		if status then
			if result.mimeType == "text/javascript" then
				web.headers["Content-Type"] = result.mimeType .. "text/javascript; charset=utf-8";
				web.headers["scriptCharset"]="utf-8"
			end
			if app:getSettings().cacheTime then
				web.headers["Expires"]=os.date("!%a, %d %b %Y %H:%M:%S GMT",os.time()+(3600*app:getSettings().cacheTime));
			else
				_M.response.DontCache(web)		
			end
		else
			return _M.not_found(web)
		end

	end
	if not status then
		local e = {}
		for k,v in pairs(result) do
			table.insert(e,k .. "=" .. tostring(v));
		end
		LogDebug(table.concat(e,","));
		return _M.not_found(web)
	else
		web:content_type(result.mimeType)
		return result.content
	end
end
function rename_component(web,appName)
	local status,result
	local data = json.decode(web.input["post_data"] or "{}")	
	local app = application.new{vccName = runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		status,result = app:renameComponent(data.componentName or "",data.newName or "",data.componentType or "");
	end
	return json.encode({status=status,result=result});
	
end
function save_resource(web,appName,fileName)
	local status,result
	local app = application.new{vccName = runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		status,result = app:saveResource(fileName,web.input['content']);
	end
	return json.encode({status=status,result=result});
end

function add_resource(web,appName,fileName)
	local status,result
	local  data = json.decode(web.input['content']);
	local app = application.new{vccName = runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		status,result = app:addResource(fileName,data.mimeType,data.content)
	end
	return json.encode({status=status,result=result});
end

-- agrega un resource usando un archivo uploadeado

function add_resource_file(web,appName,fileid)
	local status,result
	local fh;
	status=true;
	local session = tweetyio.session();
	--session.userfiles = session.userfiles or {}
	if not session.userfiles then session.userfiles = {} end 
	local fileEntry = session.userfiles[fileid]
	if not fileEntry then
		session:release();
		return json.encode({status=false,result=messages.messageKey('argument_error',{module="designer.lua",func="add_resource_file",param="fileid",value=fileid})});
	end
	fh = tweetyio.open(fileEntry.fullFileName,"rb");
	if not fh then
		session:release();
		return json.encode({status=false,result=messages.messageKey('read_file_error',{file = fileEntry.fullFileName})});
	end
	local  content= fh:read("*a");
	tweetyio.close(fh);
	session.userfiles[fileid] = nil
	session:release();

	tweetyio.remove(fileEntry.fullFileName);

	local app = application.new{vccName = runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		status,result = app:addResource(fileEntry.filename,fileEntry.mimeType,content)
		if status then
			result = {
				fileName=fileEntry.filename,
				mimeType=fileEntry.mimeType,
				size=fileEntry.size
			}
		end
	end
	return json.encode({status=status,result=result});
end
-- agrega un resource usando un archivo uploadeado con parametros adicionales

function add_resource_file_ex(web,appName,fileid)
	local status,result
	local fh;
	status=true;
	-- LogDebug("POSTDATA %s",web.input["post_data"] );
	local  params = json.decode(web.input["post_data"] or "{}");
	
	local session = tweetyio.session();
	--session.userfiles = session.userfiles or {}
	if not session.userfiles then session.userfiles = {} end 
	local fileEntry = session.userfiles[fileid]
	if not fileEntry then
		session:release();
		return json.encode({status=false,result=messages.messageKey('argument_error',{module="designer.lua",func="add_resource_file",param="fileid",value=fileid})});
	end
	fh = tweetyio.open(fileEntry.fullFileName,"rb");
	if not fh then
		session:release();
		return json.encode({status=false,result=messages.messageKey('read_file_error',{file = fileEntry.fullFileName})});
	end
	local  content= fh:read("*a");
	tweetyio.close(fh);
	session.userfiles[fileid] = nil
	session:release();

	tweetyio.remove(fileEntry.fullFileName);

	local app = application.new{vccName = runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		status,result = app:addResource(params.filename or fileEntry.filename,params.mimeType or fileEntry.mimeType,content,params.overwrite)
		if status then
			result = {
				fileName=fileEntry.filename,
				mimeType=fileEntry.mimeType,
				size=fileEntry.size
			}
		end
	end
	return json.encode({status=status,result=result});
end


function del_resource(web,appName,fileName)
	local status,result
	local app = application.new{vccName = runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		status,result = app:deleteResource(fileName);
	end
	return json.encode({status=status,result=result});
end

function read_private_data(web,appName)
	local status,result
	local app = application.new{vccName = runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		status,result = app:readPrivateData();
	end
	return json.encode({status=status,result=result});

end
function save_private_data(web,appName)
	local status,result
	local data = json.decode(web.input['content']);
	local app = application.new{vccName = runtime.getCurrentVCCName()}
	status,result = app:initialize(appName)
	if status then
		status,result = app:savePrivateData(data);
	end
	return json.encode({status=status,result=result});
end
function app_bundle_info(web,fileid)
	local fileEntry
	local status,result
	status=true;
	local session = tweetyio.session()
	session:release();
	if not session.userfiles then session.userfiles  = {} end
	fileEntry = session.userfiles[fileid]
	if not fileEntry then
		status = false;
		result = messages.messageKey('argument_error',{module="designer.lua",func="app_bundle_info",param="fileid",value=fileid});
	end
	if status then
		local app = application.new{vccName = runtime.getCurrentVCCName()}
		LogDebug("call app:initFromBundle");
		status,result = app:initFromBundle(fileEntry.fullFileName)
		if status then
			status,result = app:getInformation();
		end
	end
	return json.encode({status=status,result=result});
end
function app_bundle_import(web,fileid)
	local fileEntry
	local status,result
	status = true;
	local appName = web.input['appName']
	local overwrite = web.input['overwrite']=='true'
	local hidden = web.input['hidden']=='true'

	LogDebug("IMPORT HIDDEN %s", tostring(hidden));
	if not appName then
		status = false
		result = messages.messageKey('argument_error',{module="designer.lua",func="app_bundle_import",param="POST:appName"});
	end
	if status then
		local session = tweetyio.session();
		if not session.userfiles then session.userfiles = {} end
		fileEntry = session.userfiles[fileid]
		session.userfiles[fileid] = nil
		session:release();
		if not fileEntry then
			status = false;
			result = messages.messageKey('argument_error',{module="designer.lua",func="app_bundle_import",param="fileid",value=fileid});
		end
		if status then
			local app = application.new{vccName = runtime.getCurrentVCCName()}
			status,result = app:importFromBundle(fileEntry.fullFileName,appName,overwrite)
			tweetyio.remove(fileEntry.fullFileName);
		end
	end
	return json.encode({status=status,result=result});
end
function published_applications(web,filterParams)
	web.headers["Content-Type"] = "application/json"
	_M.response.DontCache(web)
	local params = {}
	pcall(
		function()
			params = json.decode(filterParams)
			
		end
	);
	LogDebug("PUBLISHED ************* %s",json.encode(params,true))
	
	local vccName = runtime.getCurrentVCCName();
	local status,result=workspace.new{vccName=vccName}:getCatalog(vccName,params)
	if status then
		if params.include_forms then
			--add apps forms
			local catalogViews = {}
			for _,entry in pairs(result) do
				local  app = application.new{vccName=vccName}
				local status, result = app:initialize(entry.name)
				if status then
					local catalogApp = app:getCatalog();
					for _,view in pairs(catalogApp.views) do
						local entryView = {}
						entryView.id = entry.id .. "." .. view.name;
						entryView.name = entry.name .. "." .. view.name;
						entryView.description = entry.description .. "." .. view.name;
						entryView.version = entry.version;
						entryView.created_at = entry.created_at;
						table.insert(catalogViews, entryView);
					end
				else
					return json.encode({});
				end
			end
			for _,entry in pairs(catalogViews) do table.insert(result, entry) end
			table.sort(result,function(a,b)
				return b.created_at > a.created_at
			end)
		end
		return json.encode(result);
	end
	return json.encode({});
end

local m_designerAppPermission = nil
function setViewEnvironment(env)
	assert(env and env.designerAppPermission)
	m_designerAppPermission = env.designerAppPermission
end

function initialize(_M, prefix,appLocation)
	prefix = prefix or "/designer"
	require (tweety_modules_prefix .. "introspect").initialize(_M,prefix .. '/datasources',m_appLocation);

	require (tweety_modules_prefix .. "csvutils").initialize(_M,prefix .. '/csv',m_appLocation);

	_M:dispatch_get(published_applications,"/applications/published[/]*([^/]*)");
	_M:dispatch_get(application_setauthfilterarguments,"/applications/([^/]+)/setauthfilterarguments/([^/]+)");
	_M:dispatch_get(application_settokendata,"/applications/([^/]+)/settokendata/([^/]+)/([^/]+)");
	
	
--	_M:dispatch_get(published_applications,"/applications/published_ex/([^/]*)");

	_M:dispatch_get(runtime.nocache(read_summary_cut),prefix .. "/summarizer/read_summary_cut/([^/]*)")
	_M:dispatch_get(runtime.nocache(read_details_projected_fields),prefix .. "/summarizer/read_details_projected_fields/([^/]*)")

--   read application description
	_M:dispatch_get(runtime.nocache(application_catalog_load), prefix .. "/applications/([^/]+)/load")
	_M:dispatch_post(application_setproperties,prefix .. "/applications/([^/]+)/setproperties");


--   FORMS

	_M:dispatch_get(runtime.nocache(read_form),prefix .. "/applications/([^/]+)/forms/([^/]+)")
	_M:dispatch_post(save_form,prefix .. "/applications/([^/]+)/forms/save/([^/]+)");
	_M:dispatch_post(save_form_params,prefix .. "/applications/([^/]+)/forms/setparams/([^/]+)");
	
	_M:dispatch_post(add_form,prefix .. "/applications/([^/]+)/forms/add/([^/]+)");
	_M:dispatch_get(runtime.nocache(delete_form),prefix .. "/applications/([^/]+)/forms/delete/([^/]+)");

--   DATASOURCES

	_M:dispatch_post(add_datasource,prefix .. "/applications/([^/]+)/datasources/add/([^/]+)");
	_M:dispatch_get(runtime.nocache(delete_datasource),prefix .. "/applications/([^/]+)/datasources/delete/([^/]+)");
	_M:dispatch_get(runtime.nocache(read_datasource),prefix .. "/applications/([^/]+)/datasources/([^/]+)")
	_M:dispatch_get(runtime.nocache(read_private_data),prefix .. "/applications/([^/]+)/private/read");
	_M:dispatch_post(save_private_data,prefix .. "/applications/([^/]+)/private/save");


	_M:dispatch_post(save_datasource,prefix .. "/applications/([^/]+)/datasources/save/([^/]+)")
	_M:dispatch_post(truncate_datasource,prefix .. "/applications/([^/]+)/datasources/truncate/([^/]+)")

	_M:dispatch_get(workspace_catalog_app, prefix .. "/catalogapp")
	_M:dispatch_post(workspace_catalog_app, prefix .. "/catalogapp")
	
	_M:dispatch_get(m_designerAppPermission:CheckPermission(tweety_permisions_actionId, "entities", runtime.nocache(workspace_catalog)), prefix .. "/catalog")
	_M:dispatch_post(m_designerAppPermission:CheckPermission(tweety_permisions_actionId, "entities", workspace_catalog_post), prefix .. "/catalog")
	
	
-- ODBC
	_M:dispatch_get(m_designerAppPermission:CheckPermission(tweety_permisions_actionId, "entities", runtime.nocache(odbc_info)), prefix .. "/odbcinfo")

	_M:dispatch_post(m_designerAppPermission:CheckPermission(tweety_permisions_actionId, "entities", runtime.nocache(odbc_columns)), prefix .. "/odbccolumns")
	

--  RESOURCES  .js .png .gif etc

	_M:dispatch_get(read_resource,prefix .. "/applications/([^/]+)/([^/]+)/resources/([^/]+)")
	_M:dispatch_get(runtime.nocache(read_resource),"/reportview/designer/applications/([^/]+)/resources/([^/]+)")
	_M:dispatch_post(save_resource,prefix .. "/applications/([^/]+)/resources/save/([^/]+)");
	_M:dispatch_post(add_resource,prefix .. "/applications/([^/]+)/resources/add/([^/]+)");
	_M:dispatch_get(runtime.nocache(del_resource),prefix .. "/applications/([^/]+)/resources/delete/([^/]+)");
	_M:dispatch_post(add_resource_file,prefix .. "/applications/([^/]+)/resources/addfile/([^/]+)");
	_M:dispatch_post(add_resource_file_ex,prefix .. "/applications/([^/]+)/resources/addfile_ex/([^/]+)");

	_M:dispatch_post(rename_component,prefix .. "/applications/([^/]+)/rename_component");


--  TEMPLATES that are available to create a new proyect
	_M:dispatch_get(runtime.nocache(template_catalog), prefix .. "/templates/([^/]*)[/]*([^/]*)")

--  IMAGES that are available for all applications

	_M:dispatch_get(runtime.nocache(image_catalog),prefix .. "/images")

--  DATASOURCES that are available for all applications

	_M:dispatch_get(runtime.nocache(datasource_catalog),prefix .. "/datasources")

--  create a new application into the  workspace using a template
	_M:dispatch_get(m_designerAppPermission:CheckPermission(tweety_permisions_actionId, "entities", runtime.nocache(application_new)), prefix .. "/applications/create/([^/]+)/([^/]+)")
	_M:dispatch_get(m_designerAppPermission:CheckPermission(tweety_permisions_actionId, "entities", runtime.nocache(workspace_delete_application)), prefix .. "/applications/delete/([^/]+)")
--	export application bundle
	_M:dispatch_get(m_designerAppPermission:CheckPermission(tweety_permisions_actionId, "entities", runtime.nocache(application_export)), prefix .. "/applications/export/([^/%.]+)%.bundle")
--  import application bundle
	_M:dispatch_get(runtime.nocache(app_bundle_info), prefix .. "/applications/bundle/getinfo/([^/%.]+)")
	_M:dispatch_post(app_bundle_import, prefix .. "/applications/bundle/import/([^/%.]+)")
--	available application
--[[
	_M:dispatch_get(
		runtime.nocache(
			function(web, appName)
				local status, result = application.new{vccName = runtime.getCurrentVCCName()}:initialize(appName)
				LogInfo("Tweety application '%s' Is Available: '%s'", appName, status)
				return json.encode({status = status})
			end
		), prefix .. "/applications/available/([^/%.]+)"
	);
	
--]]
	
		_M:dispatch_get(function(web, appName)
			local m_appName,m_formname
			
			m_appName,c = string.gsub(appName or "","(%..*)","")
			if c> 0 then
				m_formname = string.gsub(appName or "","[^%.]*%.","")
			end		
		
			local app = application.new{vccName = runtime.getCurrentVCCName()}
			
			local status, result = app:initialize(m_appName)
			LogInfo("Tweety application '%s' form '%s' Is Available: '%s'", m_appName ,m_formname or "", status);
			if status and m_formname then
				status = app:isFormAvailable(m_formname);
			end
			return json.encode({status = status})
		end, prefix .. "/applications/available/(.+)")
	
end
