require "struct"
require "icu.ustring"

module (...,package.seeall)

local function bit(p)
  return 2 ^ (p - 1)  -- 1-based indexing
end
-- bit operators

-- Typical call:  if hasbit(x, bit(3)) then ...
local function hasbit(x, p)
		return x % (p + p) >= p
end

local function setbit(x, p)
  return hasbit(x, p) and x or x + p
end

local function clearbit(x, p)
  return hasbit(x, p) and x - p or x
end

local ctable = {
	aliceblue="#f0f8ff",	antiquewhite="#faebd7",	aqua="#00ffff",	aquamarine="#7fffd4",	azure="#f0ffff",	beige="#f5f5dc",
	bisque="#ffe4c4",	black="#000000",	blanchedalmond="#ffebcd",	blue="#0000ff",	blueviolet="#8a2be2",	brown="#a52a2a",
	burlywood="#deb887",	cadetblue="#5f9ea0",	chartreuse="#7fff00",	chocolate="#d2691e",	coral="#ff7f50",	cornflowerblue="#6495ed",
	cornsilk="#fff8dc",	crimson="#dc143c",	cyan="#00ffff",	darkblue="#00008b",	darkcyan="#008b8b",	darkgoldenrod="#b8860b",	darkgray="#a9a9a9",
	darkgrey="#a9a9a9",	darkgreen="#006400",	darkkhaki="#bdb76b",	darkmagenta="#8b008b",	darkolivegreen="#556b2f",	darkorange="#ff8c00",
	darkorchid="#9932cc",	darkred="#8b0000",	darksalmon="#e9967a",	darkseagreen="#8fbc8f",	darkslateblue="#483d8b",	darkslategray="#2f4f4f",
	darkslategrey="#2f4f4f",	darkturquoise="#00ced1",	darkviolet="#9400d3",	deeppink="#ff1493",	deepskyblue="#00bfff",	dimgray="#696969",
	dimgrey="#696969",	dodgerblue="#1e90ff",	firebrick="#b22222",	floralwhite="#fffaf0",	forestgreen="#228b22",	fuchsia="#ff00ff",
	gainsboro="#dcdcdc",	ghostwhite="#f8f8ff",	gold="#ffd700",	goldenrod="#daa520",	gray="#808080",	grey="#808080",	green="#008000",
	greenyellow="#adff2f",	honeydew="#f0fff0",	hotpink="#ff69b4",	indianred="#cd5c5c",	indigo="#4b0082",	ivory="#fffff0",	khaki="#f0e68c",
	lavender="#e6e6fa",	lavenderblush="#fff0f5",	lawngreen="#7cfc00",	lemonchiffon="#fffacd",	lightblue="#add8e6",	lightcoral="#f08080",
	lightcyan="#e0ffff",	lightgoldenrodyellow="#fafad2",	lightgray="#d3d3d3",	lightgrey="#d3d3d3",	lightgreen="#90ee90",	lightpink="#ffb6c1",
	lightsalmon="#ffa07a",	lightseagreen="#20b2aa",	lightskyblue="#87cefa",	lightslategray="#778899",	lightslategrey="#778899",	lightsteelblue="#b0c4de",
	lightyellow="#ffffe0",	lime="#00ff00",	limegreen="#32cd32",	linen="#faf0e6",	magenta="#ff00ff",	maroon="#800000",	mediumaquamarine="#66cdaa",
	mediumblue="#0000cd",	mediumorchid="#ba55d3",	mediumpurple="#9370d8",	mediumseagreen="#3cb371",	mediumslateblue="#7b68ee",	mediumspringgreen="#00fa9a",
	mediumturquoise="#48d1cc",	mediumvioletred="#c71585",	midnightblue="#191970",	mintcream="#f5fffa",	mistyrose="#ffe4e1",	moccasin="#ffe4b5",
	navajowhite="#ffdead",	navy="#000080",	oldlace="#fdf5e6",	olive="#808000",	olivedrab="#6b8e23",	orange="#ffa500",	orangered="#ff4500",
	orchid="#da70d6",	palegoldenrod="#eee8aa",	palegreen="#98fb98",	paleturquoise="#afeeee",	palevioletred="#d87093",	papayawhip="#ffefd5",
	peachpuff="#ffdab9",	peru="#cd853f",	pink="#ffc0cb",	plum="#dda0dd",	powderblue="#b0e0e6",	purple="#800080",	red="#ff0000",	rosybrown="#bc8f8f",
	royalblue="#4169e1",	saddlebrown="#8b4513",	salmon="#fa8072",	sandybrown="#f4a460",	seagreen="#2e8b57",	seashell="#fff5ee",	sienna="#a0522d",
	silver="#c0c0c0",	skyblue="#87ceeb",	slateblue="#6a5acd",	slategray="#708090",	slategrey="#708090",	snow="#fffafa",	springgreen="#00ff7f",
	steelblue="#4682b4",	tan="#d2b48c",	teal="#008080",	thistle="#d8bfd8",	tomato="#ff6347",	turquoise="#40e0d0",	violet="#ee82ee",	wheat="#f5deb3",
	white="#ffffff",	whitesmoke="#f5f5f5",	yellow="#ffff00",	yellowgreen="#9acd32"
}

function RGBcolor (color)
	if color then
		local c ={}
		if string.sub(color,1,1) == "#" then
			color = string.sub(color,2)
			color = string.gsub(color,"%x%x",
				function(rgbComponent)
					table.insert(c,struct.pack("< i1",tonumber(rgbComponent,16)))
				end
			)
			for i=#c+1,4 do
				table.insert(c,struct.pack("< i1",0))
			end
			return table.concat(c);
		else
			color = string.lower(color)
			return RGBcolor(ctable[color])

		end
	else
		return struct.pack("< I1 I1 I1 I1",0,0,0,0)
	end
end

local  function ShortXLUnicodeString(str)
	local s = string.gsub(str,".",
		function(c)
			return struct.pack("< i2 ",string.byte(c),0)
		end)
	return struct.pack("< i1 i1 ",#str,1) .. s

end
local  function ShortXLUnicodeString(str)
	return struct.pack("< i1 i1 ",#str,0) .. str
end

local function XLUnicodeString(str)
--[[
	local s = string.gsub(str,".",
		function(c)
			return struct.pack("< i2 ",string.byte(c),0)
		end)
	return struct.pack("< i2 i1",#str,1) .. s
--]]
	str = icu.ustring.encode(icu.ustring(str), "UTF-16")
	return struct.pack("< i2 i1",#str/2,1) .. str

end
local function byteString(str)
	return struct.pack("< i2",#str) .. str;
end


local function record(self,recordType)
	self = self or {}
	function self:stream()
		return struct.pack("< i2 i2 ",recordType,self:length()) .. self:serialize()
	end
	function self:length()
		return 0
	end
	function self:serialize()
		return ""
	end

	return self

end

local function BOF (self)
	self = record(self,0x809)
	function self:length()
		return 16
	end
	function self:serialize()
		return struct.pack("< i2 i2 i2 i2 i4 i4",
			self.biffVersion or 0,
			self.typeOfFollowdata or 0,
			self.buildIdentifier or 0,
			self.buildYear or 0,
			self.fileHistoryFlags or 0,
			self.lowestExcelVersion
		)
	end
	return self
end
local function CODEPAGE(self)
	self = record(self,0x42)
	function self:length()
		return 2
	end
	function self:serialize()
		return struct.pack("< i2 ", self.codepageId or 1200) -- 1200 es el por defecto en BIFF8 = UTF-16
	end
	return self
end
local function FONT(self)
	self = record(self,0x31)
	function self:length()
		return 14 + #ShortXLUnicodeString(self.name or "Arial")
	end

	function self:serialize()
		local res
		local fontWeight
		if self.bold then
			fontWeight=700
		else
			fontWeight=400
		end
		return struct.pack("< i2 i2 i2 i2 i2 i1 i1 i1 i1 ",
				(self.size or 10) * 20,
				0, 					-- options
				self.colorIndex , 					-- colorIndex negro
				fontWeight,
				0,					-- escapementType
				0,					-- underlineType
				0,					-- fontFamily
				1,					-- characterSet 01 system default
				0					-- unused
			)  .. ShortXLUnicodeString(self.name or "Arial")
	end

	return self
end


local function XF ( self )
	self = record(self,0xe0)
	function self:length()
		return 0x14
	end
	function self:serialize()

		local l,h,l1
		l = 0;
		l = l + (self.leftLineStyle or 0 )
		l = l + ((self.rightLineStyle or 0)* (2^4))
		l = l + ((self.topLineStyle or 0)* (2^8))
		l = l + ((self.botomLineStyle or 0)* (2^12))
		l = l + ((self.leftLineColorIndex or 0)* (2^16))
		l = l + ((self.rightLineColorIndex or 0)* (2^23))
		l = l + ((self.diagonalLineFromLeftTopToRightBottom or 0) * (2^30))
		l = l + ((self.diagonalLineFromBottomLeftToRightTop or 0)* (2^31))
		h = 0;
		h = h + (self.topLineColorIndex or 0 )
		h = h + ((self.bottomLineColorIndex or 0 ) * (2^7))
		h = h + ((self.diagonalLineColorIndex or 0 ) * (2^14))
		h = h + ((self.diagonalLineStyle or 0 ) * (2^21))
		h = h + ((self.fillPattern or 0 ) * (2^26))

		l1 = 0;
		l1 = l1 + (self.patternColorIndex or 0 )
		l1 = l1 + ((self.patternBackgroundIndex or 0 ) * ((2^7)))
	--	print("fontIndex",self.fontIndex)
		return table.concat({
			struct.pack("< i2",self.fontIndex or 0),
			struct.pack("< i2",self.formatIndex or 0), -- formatIndex
			struct.pack("< i2",self.cellProtection or 1),--0xfff5), -- XF type, cell protection, and parent style
			struct.pack("< i1",0x20 + 16), -- Alignment and text break
										-- 2   Centrado horizontal
										-- 3   Right
										-- 4   Repite el texto horizontalmente
										-- 16  Alineado vertical al top

			struct.pack("< i1",0), -- Text rotation angle
			struct.pack("< i1",0), -- Indentation, shrink to cell size, and text direction
			struct.pack("< i1",self.Attributes or 8), --0xf4), -- Flags for used attribute groups
			struct.pack("< i4 i4",l,h), -- Cell border lines and background area
			struct.pack("< i2",l1)
		})
	end
	return self
end

local function STYLE (self)
	self = record(self,0x293)
	function self:length()
		return 4
	end
	function self:serialize()
		return struct.pack("< i2 i2 ",0x8010,0xff03);
	end
	return self
end

local function SHEET ( self )
	self = record(self,0x85)
	function self:length()
		return 6+#ShortXLUnicodeString(self.sheetName or "");
	end
	function self:serialize()
		return table.concat({
			struct.pack("< i4",self.streamPosition or 0),
			struct.pack("< i1",0), -- sheet state 0 visible
			struct.pack("< i1",0), -- sheet type 0 Worksheet
			ShortXLUnicodeString(self.sheetName or "")
		})
	end
	return self
end

local function EOF ( self )
	self = record(self,0xA)
	return self
end

local function NUMBER ( self )
	self = record(self,0x203)
	function self:length()
		return 14
	end
	function self:serialize()
		return table.concat({
			struct.pack("< I2 I2 i2",self.row,self.col,self.formatIndex or 0),
			struct.pack("< d",self.value)
		})
	end
	return self
end
local function BLANK ( self )
	self = record(self,0x201)
	function self:length()
		return 6
	end
	function self:serialize()
		return struct.pack("< I2 I2 i2",self.row,self.col,self.formatIndex or 0);
	end
	return self
end
local function ROW(self)

	self = record(self,0x208)
--	self = record(self,0)
	function self:length()
		return 16
	end
	function self:serialize()
		local h = self.height
		if self.height then
			self.fUnsynced =1
			self.height = self.height * 20
		else
			self.height=0x010f;
		end

		local t = 0
		t = t + (self.iOutLevel or 0)
		t = t + ((self.reserved2 or 0)* (2 ^3))
		t = t + ((self.fCollapsed or 0)* (2 ^ 4))
		t = t + ((self.fDyZero or 0) * (2 ^ 5))
		t = t + ((self.fUnsynced or 0) * (2 ^ 6) )
		t = t + ((self.fGhostDirty or 0) * (2 ^ 7))
		return table.concat({
			struct.pack("< I2 ",self.row),
			struct.pack("< I2", self.firstCol),
			struct.pack("< I2", self.lastCol+1),
			struct.pack("< I2", (self.height or 0)),
			struct.pack("< I2", 0), -- unused
			struct.pack("< I2", 0), -- unused in biff8

			struct.pack("< I2",t), -- option flags
			struct.pack("< I2",0xf) -- other options XF etc
		})

	end
	return self
end

local function RRTABID(self)
	self = record(self,0x13d)
	function self:length()
		return 2 * #(self.tabs or {})
	end
	function self:serialize()
		local res = {}
		for _,t in ipairs(self.tabs or {}) do
			table.insert(res,struct.pack("< i2",t))
		end
		return table.concat(res)
	end
	return self
end
local function LABEL ( self )
	self = record(self,0x204)
	function self:length()
		return 6 + #XLUnicodeString(self.value or "")
	end
	function self:serialize()
		return table.concat({
			struct.pack("< I2 I2 i2",self.row,self.col,self.formatIndex or 0),
			XLUnicodeString(self.value or "")
		})
	end
	return self
end
local function PALETTE( self )
	self = record(self,0x92)
	self.palette = self.palette or {}
	function self:length()

		return 2 + (56 * 4)
	end
	function self:serialize()
		local paleteStream = table.concat(self.palette)
		if #self.palette < 56 then
			paleteStream = paleteStream .. string.rep('\0\0\0\0',56-#self.palette)
		end
		return struct.pack("< i2",56) .. paleteStream
	end

	return self
end
local function WINDOW1 ( self)
	self = record(self,0x3d)
	function self:length()
		return 18
	end
	function self:serialize()
		return table.concat({

			struct.pack("< i2",self.horizontalPosition or 0 ), -- horizontal position
			struct.pack("< i2",self.verticalPosition or 0), -- vertical position
			struct.pack("< i2",self.widthOfDocumentWindow or 0), -- width of document window
			struct.pack("< i2",self.heightOfDocumentWindow or 0), -- height of document window
			struct.pack("< i2",self.optionFlags or 0 ), -- option flags
			struct.pack("< i2",self.indexToActiveWorksheet or 0 ), -- index to active worksheet
			struct.pack("< i2",self.indexToFirstVisibleWorksheet or 0), -- index to first visible tab in the worksheet tab bar
			struct.pack("< i2",self.numberOfSelectedWorksheets or 0 ), -- number of selected worksheets
			struct.pack("< i2",self.widthTheWorksheetTabBar or 0 ) -- width the worksheet tab bar
		})

	end
	return self
end
local function COLINFO ( self )
	self = record (self,0x7d)
	function self:length()
		return 12
	end
	function self:serialize()
		return table.concat({
			struct.pack("< i2 ",self.startCol or 0),
			struct.pack("< i2 ",self.endCol or self.startCol or 0),
			struct.pack("< i2 ",(self.width or 10 ) *256 ),
			struct.pack("< i2 ",0), -- XF
			struct.pack("< i2 ",0), -- options
			struct.pack("< i2 ",0) -- unused
		})
	end
	return self
end

local function WINDOW2 ( self)
	self = record(self,0x23e)
	function self:length()
		return 18
	end
	function self:serialize()

		local opt = self.optionFlags or 0x16b6
		if self.visibleGridLines ~= false then
			opt = setbit(opt,bit(2))
		else
			opt = clearbit(opt,bit(2))
		end

		return table.concat({
			struct.pack("< i2 ",opt),-- 0x16b6), --			0 2 Option flags
			struct.pack("< i2 ",self.indexToFirstVisibleRow or 0), --			2 2 Index to first visible row
			struct.pack("< i2",self.indexToFirstVisibleColumn or 0), --			4 2 Index to first visible column
			struct.pack("< i2",self.colorIndexOfGridLineColour or 0x40), --			6 2 Colour index of grid line colour (?5.74). Note that in BIFF2-BIFF5 an RGB colour is	written instead.
			struct.pack("< i2",0), --			8 2 Not used
			struct.pack("< i2",0), --			10 2 Cached magnification factor in page break preview (in percent); 0 = Default (60%)
			struct.pack("< i2",0), --			12 2 Cached magnification factor in normal view (in percent); 0 = Default (100%)
			struct.pack("< i4",0) --			14 4 Not used
		})
	end
	return self
end
local function DIMENSIONS ( self )
	self = record (self ,0x200)
	function self:length()
		return 0xe
	end
	function self:serialize()
		return table.concat({

			struct.pack("< i4",self.firstUsedRow or 0);  -- index to first used row
			struct.pack("< i4",self.lastUsedRow or 0);  -- index to last used row, increased by 1

			struct.pack("< i2",self.firstUsedCol or 0);  -- index to first used column
			struct.pack("< i2",self.lastUsedCol or 0);  -- index to last used column, increased by 1
			struct.pack("< i2",0);  -- unused
		})

	end

	return self
end
local function MERGEDCELLS(self)
	self = record (self ,0xE5)
	function self:length()
		return 10
	end
	function self:serialize()

		return table.concat({
			struct.pack("< i2",1);  -- numero de range address
			struct.pack("< i2",self.row or 0);  -- first row in range
			struct.pack("< i2",(self.row or 0) + (self.rowspan or 1)-1);  -- last row in range

			struct.pack("< i2",self.col or 0);  -- first col in range
			struct.pack("< i2",(self.col or 0) + (self.colspan or 1)-1);  -- last col in range


		})
	end

	return self
end

function workbook(self)
	self = self or {}
	local sheets = {}

	local h_fonts={}
	local fonts = {}
	local h_xf = {}
	local xf = {}

	local palette = {}
	local h_palette = {}

	local function paletteIndex(colorName)
		local builtInColours = {
			[struct.pack("< i1 i1 i1 i1",0,0,0,0)]=0;
			[struct.pack("< i1 i1 i1 i1",0xff,0xff,0xff,0)]=1;
			[struct.pack("< i1 i1 i1 i1",0xff,0,0,0)]=2;
			[struct.pack("< i1 i1 i1 i1",0,0xff,0,0)]=3;
			[struct.pack("< i1 i1 i1 i1",0,0,0xff,0)]=4;
			[struct.pack("< i1 i1 i1 i1",0xff,0xff,0,0)]=5;
			[struct.pack("< i1 i1 i1 i1",0xff,0,0xff,0)]=6;
			[struct.pack("< i1 i1 i1 i1",0,0xff,0xff,0)]=7;
		}
	
		local rgbColor = RGBcolor(colorName)
		local index = builtInColours[rgbColor]
		if not index then

			if not h_palette[rgbColor] then
				if #palette == 56 then -- maxima cantidad de entradas de la paleta
					return 0
				end
				table.insert(palette,rgbColor)
				h_palette[rgbColor]=#palette-1
				index =  #palette-1 + 8
			else
				index = h_palette[rgbColor] + 8
			end
		end
		return index
	end

	local function indexfont(font)
		local fontkey
		if not font then
			return 0
		end
		font.name = font.name or "Arial"
		font.size = font.size or 10
		font.bold = font.bold or false
		if font.weight == "bold" then
			font.bold = true
		end

		font.italic = font.italic or false
		font.colorIndex = paletteIndex(font.color or "black");
		fontkey = font.name  .. "-" .. font.size  .. "-" .. tostring((font.bold or false)) .. "-" .. tostring(font.italic or false) .. "-" .. tostring(font.color or "black")
		if not h_fonts[fontkey] then
			h_fonts[fontkey]=font
			font.index = #fonts
			if font.index >= 4 then
				font.index = font.index+1
			end
			table.insert(fonts,font)
		else
			font = h_fonts[fontkey]
		end
		return font.index
	end

	local function indexXF ( format)
		if format.foreColor then
			format.font = format.font or {}
			format.font.color = format.foreColor
		end

		if format.font then
			local xfr = {
				fontIndex=indexfont(format.font)

			};
			if format.backColor then
				xfr.patternColorIndex = paletteIndex(format.backColor);
				xfr.fillPattern = 1 -- solid

			end

			local xfKey = tostring(xfr.fontIndex) .. tostring(xfr.patternColorIndex)
			if not h_xf[xfKey] then
				h_xf[xfKey]=xfr
				xfr.index = #xf
				table.insert(xf,xfr)
			else
				xfr = h_xf[xfKey]
			end
			return xfr.index;
		else

			return 0
		end
	end


	function self:addWorksheet(worksheet)
		table.insert(sheets,worksheet)
	end
	function self:stream()
		local st = sheets
		if #st == 0 then
			st = {worksheet{name="sheet1"}}
		end
		local recarray = {}

		table.insert(recarray,BOF{
			biffVersion = 0x600;
			typeOfFollowdata = 0x5;
					buildIdentifier = 3515;
				buildYear = 1996;
				fileHistoryFlags = 0x18010;
				lowestExcelVersion = 0x306;

		})
		table.insert(recarray,
			CODEPAGE{}
		);
		local sheetstreams = {}
		local tabs = {}
		for id,sheet in ipairs(st) do
			table.insert(tabs,id)
		end

		table.insert(recarray,
			RRTABID{
				tabs = tabs
			}
		)

		table.insert(recarray,
			WINDOW1{
				horizontalPosition=0x0050,
				verticalPosition = 0xffec,
				widthOfDocumentWindow = 0x5488,
				heightOfDocumentWindow = 0x36b0,
				optionFlags = 0x38,
				indexToActiveWorksheet=0,
				indexToFirstVisibleWorksheet = 0,
				numberOfSelectedWorksheets=1,
				widthTheWorksheetTabBar=0x1f4
			}
		)
		indexXF{
			font={name="Arial",size=10,colorIndex=32767}
		}
		--[[
		for k=1,21 do
			table.insert(xf,{

					patternColorIndex=64;
					patternBackgroundIndex=130;
					cellProtection=65524;
					Attributes=252
				}
			)
		end
		--]]
		for _,sheet in ipairs(st) do
			for _,row in pairs(sheet:rows()) do
				for _,cell in pairs(row) do
					cell.formatIndex = indexXF({font=cell.font,foreColor=cell.foreColor,backColor=cell.backColor})
				end
			end
			local header = SHEET{sheetName=sheet.name}
			table.insert(sheetstreams,{header=header,stream=sheet:stream()})
		end
		table.insert(recarray,
			PALETTE{palette=palette}
		);

		for _,font in ipairs(fonts) do
			table.insert(recarray,
				FONT{
					name = font.name,
					size = font.size,
					bold = font.bold,
					colorIndex = font.colorIndex
				}
			)
		end

		for _,xfr in ipairs(xf) do
			table.insert(recarray,
				XF(xfr)
			)
		end
		--table.insert(recarray,STYLE{})

		for _,sheetstreamdata in ipairs(sheetstreams) do
			table.insert(recarray,sheetstreamdata.header)
		end


		table.insert(recarray,EOF{})
		local filepos = 0
		for _,record in ipairs(recarray) do
			filepos = filepos + (4 + record:length())
		end
		for _,sheetdata in ipairs(sheetstreams) do
			sheetdata.header.streamPosition = filepos;
			filepos = filepos + #sheetdata.stream
		end
		local res = {}
		for _,record in ipairs(recarray) do
			table.insert(res,record:stream())
		end
		for _,sheetdata in ipairs(sheetstreams) do
			table.insert(res,sheetdata.stream)
		end
		return table.concat(res)

	end


	function self:save(fileName)
		local tempName = fname
		if fileName then
			tempName = fileName
		end
		if not tempName then
			error ("Missing required filename")
		end
		local f = io.open(tempName,"wb")
		if not f then
			error ("Can't open " .. tempName .. " for writing");
		end
		f:write(self:stream())
		io.close(f)
		return true
	end
	return self
end

function worksheet(self)
	self = self or {}
	self.name = self.name or "sheet1"
	local rows = {}
	local rowHeights={}
	local columnsinfo = {}

	function self:stream()
		local res = {
			BOF{
				biffVersion = 0x600;
				typeOfFollowdata = 0x10;
				buildIdentifier = 3515;
				buildYear = 1996;
				fileHistoryFlags = 0x18010;
				lowestExcelVersion = 0x306;
			}:stream(),
			--DIMENSIONS{}:stream()
		}

		for _, columnInfo in ipairs(columnsinfo) do
			table.insert(res,COLINFO(columnInfo):stream())
		end
		local srows = {}
		for _,row in pairs(rows) do
			local trow = {}
			for _,cell in pairs(row) do
				table.insert(trow,cell)
			end
			table.sort(trow,
				function(cell1,cell2)
					return cell1.col < cell2.col
				end
			)
			table.insert(srows,trow)
		end
		table.sort(srows,
			function(row1,row2)
				return row1[1].row < row2[1].row
			end
		)

		local function insertrowblock(rowblock)
			for _,row in ipairs(rowblock) do
				-- insert ROW

				table.insert(res,ROW{
					row = row[1].row,
					firstCol = row[1].col,
					lastCol = row[#row].col,
					height = rowHeights[row[1].row]
				}:stream())
			end
			for _,row in ipairs(rowblock) do
				for _,cell in ipairs(row) do
					if type (cell.value ) == "number" then
						table.insert(res,NUMBER{row=cell.row,col=cell.col,value=cell.value,formatIndex=cell.formatIndex or 0}:stream())
					elseif type (cell.value) == "string" then
						table.insert(res,LABEL{row=cell.row,col=cell.col,value=cell.value,formatIndex=cell.formatIndex or 0}:stream())
					else
						table.insert(res,BLANK{row=cell.row,col=cell.col,formatIndex=cell.formatIndex or 0}:stream())
					end
				end
			end
			-- insert dbcell
		end
		local rowblock = {}
		for _,row in ipairs(srows) do
			table.insert(rowblock,row)
			if #rowblock == 32 then
				insertrowblock(rowblock)
				rowblock = {}
			end
		end
		insertrowblock(rowblock)
		-- MERGECELLS
		for _,row in pairs(rows) do
			for _,cell in pairs(row) do
				if (cell.rowspan or 0)> 1  or (cell.colspan or 0) > 1 then
					table.insert(res,MERGEDCELLS{row=cell.row,col = cell.col,rowspan=cell.rowspan or 1,colspan = cell.colspan or 1}:stream())
				end
			end
		end


		table.insert(res,
			WINDOW2{
				visibleGridLines=self.visibleGridLines
			}:stream()
		)
		table.insert(res,
			EOF{
			}:stream()
		)
		return table.concat(res)
	end
	function self:rows()
		return rows;
	end

	function self:columnWidth(startCol,endCol,width)
		table.insert(columnsinfo,{startCol=startCol,endCol=endCol,width=width})
	end
	function self:rowHeight(startRow,endRow,height)
		endRow = endRow or startRow
		for i = startRow,endRow do
			if not rows[i] then
				rows[i] = {}
				rows[i][0]= {row=i,col=0}
			end
			rowHeights[i] = height;
		end
	end
	function self:cell(row,col,value,font,foreColor,backColor,colspan,rowspan)
		font = font or {name="Arial",size=10}
		foreColor = foreColor or "black"
		rows[row] = rows[row] or {}
		rows[row][col] = {row=row,col=col,value=value,font = font,foreColor=foreColor,backColor = backColor,rowspan=rowspan,colspan=colspan}
	end

	return self
end
function compoundDocumentFile (self)
	self = self or {}
	local content = {}
	local msat={}
	local sat = {}
	local salloc={}
	local lastsallocSecID = 0;
	local ssat = {}
	local ssalloc = {}
	local lastssallocSecID = 0;
	local SecIdBase = 0;
	local firstSectorOfMSAT = -2;
	local totalSectorsUsedByMSAT = 0;

	local firstSectorOfDirectoryStream = 0;

	local firstSectorOfSSATStream = 0;
	local totalSectorsUsedBySSAT =0;

	local firstSectorOfSSATContainerStream = 0;
	local sizeOfSSATContainerStream=0;
	local sectorSize = 512
	local shortSectorSize = 64
	local function wsize(n)
		local c=0;
		repeat
			n = n/2
			c=c+1
		until n <= 1
		return c
	end

	function self:save(fileName)
		local tempName = self.fname
		if fileName then
			tempName = fileName
		end
		if not tempName then
			error ("Missing required filename")
		end
		local f = io.open(tempName,"wb")
		if not f then
			error ("Can't open " .. tempName .. " for writing",2);
		end
		f:write(self:stream())
		io.close(f)
		self.fname = tempName;
		return true
	end

	function self:addStream(name,stream,brinfo)
		brinfo = brinfo or {}
		for index,entry in ipairs(content) do
			if entry.name == name then
				content[index] = {name=name,stream=stream,nodeColor=brinfo.nodeColor,dirIdLeft=brinfo.dirIdLeft,dirIdRight=brinfo.dirIdRight}
				return
			end
		end
		table.insert(content,{name=name,stream=stream,nodeColor=brinfo.nodeColor,dirIdLeft=brinfo.dirIdLeft,dirIdRight=brinfo.dirIdRight})

	end

	local function findSATNextFree (search)
		local SIDFree = struct.pack("< i4 ",-1);
		local res = search.index
		while sat[res] ~= SIDFREE do
		  res  = res+1
		end

		if  not sat[res] == SIDFREE then

			res = #sat
		end


		search.index=res+1
		return res

	end
	local function findSATFirstFree(search)
		search.index = 1
		return findSATNextFree(search)
	end


	local function allocSATStream(streamData)
		local first = lastsallocSecID
		local bytesLeft = #streamData

		if #streamData == 0 then
			return -2
		end
		for i=0,first do
			if not sat[i] then
				sat[i] = struct.pack("< i4 ",-1)
			end
		end
		while bytesLeft > 0 do
			lastsallocSecID=lastsallocSecID+1
			bytesLeft = bytesLeft - sectorSize
			if bytesLeft > 0 then
				table.insert(sat,struct.pack("< i4 ",lastsallocSecID))
			else
				table.insert(sat,struct.pack("< i4 ",-2))
			end
		end
		table.insert(salloc,streamData)
		if bytesLeft < 0 then
			table.insert(salloc,string.rep('\0',-bytesLeft))
		end
		return first

	end

	local function allocSSATStream(streamData)
		local bytesLeft = #streamData
		local first = lastssallocSecID
		if #streamData == 0 then
			return -2
		end

		while bytesLeft > 0 do
			lastssallocSecID = lastssallocSecID+1
			bytesLeft = bytesLeft - shortSectorSize
			if bytesLeft > 0 then
				table.insert(ssat,struct.pack("< i4 ",lastssallocSecID))
			else
				table.insert(ssat,struct.pack("< i4 ",-2))
			end
		end
		table.insert(ssalloc,streamData)
		if bytesLeft < 0 then
			table.insert(ssalloc,string.rep('\0',-bytesLeft))
		end
		return first;
	end
	local function allocStream(streamData,forceSAT)
		if #streamData > 4096 or forceSAT then
			return allocSATStream(streamData)
		else
			return allocSSATStream(streamData)
		end

	end

	local function getDirectoryEntryStream(entry)
		local t = {}
		local entryName = string.gsub(entry.name or "",".",
			function(c)
				return struct.pack("< i2 ",string.byte(c),0)
			end)
		local entryNameDataLen = 	#entryName+2;
		entryName = entryName .. string.rep('\0',shortSectorSize-#entryName)
		table.insert(t,entryName)
		table.insert(t,struct.pack("< i2 ",(entryNameDataLen)))
		table.insert(t,struct.pack("i1 ",entry.typeOfEntry or 0))
		table.insert(t,struct.pack("i1 ",entry.nodeColor or 0))
		table.insert(t,struct.pack("< i4 ",entry.dirIdLeft or -1))
		table.insert(t,struct.pack("< i4 ",entry.dirIdRight or -1))
		table.insert(t,struct.pack("< i4 ",entry.dirIdRoot or -1))
		table.insert(t,string.rep('\0',16)) -- UID
		table.insert(t,string.rep('\0',4)) -- USer flags
		table.insert(t,string.rep('\0',8)) -- Timestamp of creation of  this entry
		table.insert(t,string.rep('\0',8)) -- Timestamp of last modification of  this entry
		table.insert(t,struct.pack("< i4 ",entry.SecID or 0))
		table.insert(t,struct.pack("< i4 ",entry.streamSize or 0))
		table.insert(t,string.rep('\0',4)) -- unused
		return table.concat(t)
	end
	local function getRootDirectoryStream()
		local t = {}
		local nentr,nfill
		table.insert(t,getDirectoryEntryStream({name="Root Entry",typeOfEntry=5,dirIdRoot=1,SecID=firstSectorOfSSATContainerStream,streamSize=sizeOfSSATContainerStream}))
		nentr=1
		for _,streamData in ipairs(content) do
			table.insert(t,getDirectoryEntryStream({name=streamData.name,typeOfEntry=2,streamSize=#streamData.stream,SecID = streamData.SecID,dirIdRoot=streamData.dirIdRoot,dirIdLeft=streamData.dirIdLeft,dirIdRight=streamData.dirIdRight,nodeColor=streamData.nodeColor}))
			nentr=nentr+1
		end
		-- calcula las entradas que deben llenarse con unused
		-- basado en un tamao de sector de 512 bytes
		local entriesBySector = sectorSize / 128

		nfill = entriesBySector-math.fmod(nentr,4)
		if nfill < entriesBySector then
			for c=1,nfill do
				table.insert(t,getDirectoryEntryStream({}))
			end
		end
		return table.concat(t)
	end
	local function getNumOfRootSectors()
		local t = {}
		local nentr,nfill
		nentr = 1+#content
		nfill =  4-math.fmod(nentr,4)
		if nfill < 4 then
			nentr = nentr + nfill
		end
		return (nentr * 128)/sectorSize;
	end
	local function getNumOfSATSectors()

		local allocSize = getNumOfRootSectors()*sectorSize;
		local sallocSize = 0;
		local f
		for _,streamData in ipairs(content) do

			local t = #streamData.stream
			if t > 4096 then
				f = sectorSize - math.fmod(t,sectorSize)
				if f < sectorSize then
					t = t + f
				end
				allocSize = allocSize + t
			else
				f = shortSectorSize - math.fmod(t,shortSectorSize)
				if f < shortSectorSize then
					t = t + f
				end
				sallocSize = sallocSize + t
			end
		end
		f = sectorSize - math.fmod(sallocSize,sectorSize)
		if f < sectorSize then
			allocSize = allocSize + f
		end
		local sectors = (allocSize / sectorSize )
		local uall = sectors / (sectorSize/128)
		local nuall = math.floor(uall)
		if nuall < uall then
			nuall = nuall +1
		end
		return nuall
	end

	local function getHeaderStream()
		local out = {}
		table.insert(out,string.char(0xD0,0xCF,0x11,0xE0,0xA1,0xB1,0x1A,0xE1))  -- Compound document file identifier: D0H CFH 11H E0H A1H B1H 1AH E1H
		table.insert(out,string.rep('\0',16)) 									-- Unique identifier (UID) of this file (not of interest in the following, may be all 0)
		table.insert(out,struct.pack("< i2",0x3e))								-- Revision number of the file format (most used is 003EH)
		table.insert(out,struct.pack("< i2",0x3))								-- Version number of the file format (most used is 0003H)
		table.insert(out,struct.pack("< i2",0xfffe))							-- Byte order identifier (?4.2): FEH FFH = Little-Endian
																				--  FFH FEH = Big-Endian

		table.insert(out,struct.pack("< i2",wsize(sectorSize)))								-- Size of a sector in the compound document file (?3.1) in power-of-two (ssz), real sector
																				-- size is sec_size = 2ssz bytes (minimum value is 7 which means 128 bytes, most used
																				-- value is 9 which means 512 bytes)

																				-- Size of a short-sector in the short-stream container stream (?6.1) in power-of-two (sssz),
																				-- real short-sector size is short_sec_size = 2sssz bytes (maximum value is sector size
		table.insert(out,struct.pack("< i2",wsize(shortSectorSize)))								-- ssz, see above, most used value is 6 which means 64 bytes)
		table.insert(out,string.rep('\0',10))								    --  10 bytes unused


		table.insert(out,struct.pack("< i4",totalSectorsUsedBySAT))				-- total sectors used by SAT


		table.insert(out,struct.pack("< i4",firstSectorOfDirectoryStream))		-- SectID of first sector of directory stream
		table.insert(out,struct.pack("< i4",0))									-- unused
		table.insert(out,struct.pack("< i4",4096))									-- Minimum size of a standard stream (in bytes, minimum allowed and most used size is 4096

																				-- bytes), streams with an actual size smaller than (and not equal to) this value are stored as
																				-- short-streams (?6)


		table.insert(out,struct.pack("< i4",firstSectorOfSSATStream))			-- SecID of first sector of the short-sector allocation table (?6.2), or 2 (End Of Chain

		table.insert(out,struct.pack("< i4",totalSectorsUsedBySSAT))									-- Total number of sectors used for the short-sector allocation table (?6.2)

		table.insert(out,struct.pack("< i4",firstSectorOfMSAT))		-- SecID of first sector of the master sector allocation table (?5.1), or 2 (End Of Chain
		table.insert(out,struct.pack("< i4",totalSectorsUsedByMSAT))																		-- SecID, ?3.1) if no additional sectors used


		local n																		-- First part of the master sector allocation table (?5.1) containing 109 SecIDs
		for _n,SecID in ipairs(msat) do
			n=_n
			table.insert(out,SecID)
			if _n == 109 then
				break
			end
		end
		-- si es una msat totalmente contenida en el header rellena todo con free hasta la posicion 109


		for sl=109-n,1,-1 do
			table.insert(out,struct.pack("< i4 ",0xffffffff))
		end


		return table.concat(out)
	end
	local function getmsatextendedStream()
		local n = 110
		local out = {}
		local c = 0
		local prev_eof
		local firstSECId=lastsallocSecID;
		for n=110,#msat do
			if c == 128 then
				firstSECId=firstSECId+1
				table.insert(out,struct.pack("< i4 ",firstSECId))
				table.insert(out,msat[n])
				c=0
			else
				table.insert(out,msat[n])
			end
			c=c+1
		end
		if c > 0 then
			c=c+1
			for c=c,127 do
				table.insert(out,struct.pack("< i4 ",-1))
			end
			if c < 128 then
				table.insert(out,struct.pack("< i4 ",-2))
			end
		end
		return table.concat(out)
	end
	local function allocsat(satstream)

		for c=#satstream/sectorSize,1,-1 do

			table.insert(msat,struct.pack("< i4 ",lastsallocSecID))
			lastsallocSecID = lastsallocSecID+1
		end
	end

	function self:stream()
		local res = {}
		-- inicializacion
		msat={}
		sat = {}
		salloc={}
		lastsallocSecID = 0;
		ssat = {}
		ssalloc = {}
		lastssallocSecID = 0;
		firstSectorOfMSAT = -2;
		totalSectorsUsedByMSAT = 0;
		firstSectorOfDirectoryStream=0;


		-- aloja todos los streams del usuario
		for _,fileData in ipairs(content) do
			fileData.SecID = allocStream(fileData.stream)
		end
		local t;
		-- aloja el ssat container stream
		local ssatContainerStream = 	table.concat(ssalloc)
		t = sectorSize - math.fmod(#ssatContainerStream,sectorSize)
		if t < sectorSize then
			ssatContainerStream = ssatContainerStream .. string.rep('\0',t)
		end
		sizeOfSSATContainerStream = #ssatContainerStream;
		firstSectorOfSSATContainerStream = allocStream(ssatContainerStream,true)


		-- aloja el ssat stream


		local ssat_stream = table.concat(ssat)

		t = sectorSize - math.fmod(#ssat_stream,sectorSize)
		if t < sectorSize then
			ssat_stream = ssat_stream .. string.rep(struct.pack('< i4 ',-1),t/4)
		end
		totalSectorsUsedBySSAT	= #ssat_stream / sectorSize

		firstSectorOfSSATStream = allocStream(ssat_stream,true)

		-- obtuene el root y lo aloja en la sat
		firstSectorOfDirectoryStream=allocStream(getRootDirectoryStream(),true)


		local satSectorCounts = function()
			local totalSectorsUsedBySAT
			totalSectorsUsedBySAT = (#sat*4)/sectorSize
			if math.floor(totalSectorsUsedBySAT) ~= totalSectorsUsedBySAT then
				totalSectorsUsedBySAT = math.floor(totalSectorsUsedBySAT)+1
			else
				totalSectorsUsedBySAT = math.floor(totalSectorsUsedBySAT)
			end
			return totalSectorsUsedBySAT
		end


		local satMarks = 0;
		while satMarks < satSectorCounts() do
			for c=satSectorCounts()-satMarks,1,-1 do
				table.insert(sat,struct.pack("< i4",-3))
				satMarks = satMarks+1
			end
		end


		local sat_stream = table.concat(sat)
		t = sectorSize - math.fmod(#sat_stream,sectorSize)
		if t < sectorSize then
			sat_stream = sat_stream .. string.rep(struct.pack('< i4 ',-1),t/4)
		end


		totalSectorsUsedBySAT = satSectorCounts();


		allocsat(sat_stream);

		local msat_stream = getmsatextendedStream(0)
		if #msat_stream>0 then
			firstSectorOfMSAT=lastsallocSecID
			totalSectorsUsedByMSAT = #msat_stream/512
		end


		table.insert(res,getHeaderStream())
		table.insert(res,table.concat(salloc))
		table.insert(res,root_stream)
		table.insert(res,sat_stream);
		if #msat_stream>0 then
			table.insert(res,msat_stream)
		end
		return table.concat(res)
	end

	return self
end

function excelDocument(self)
	self = workbook(self)
	local stream = self.stream

	function self:stream()
		local cdf = compoundDocumentFile{}
		cdf:addStream("Workbook",stream(self))
		return cdf:stream();
	end

	return self
end

