module(...,package.seeall)

local config,messages,application
local config = require (tweety_modules_prefix .. "config");
local application = require (tweety_modules_prefix .. "application");
local messages = require (tweety_modules_prefix .. "messages");
local flowrun = require ("flowrun")
local tweetyio = require ( tweety_modules_prefix .. "tweetyio" );
local sidereal = require ("sidereal")
local http = require("socket.http")
local manage = require "reporter.manage"
local cassandra = require "lua_cassandra"

-- LogDebug("Modulo cargado");

function url_encode(str)
  if (str) then
    str = string.gsub (str, "\n", "\r\n")
    str = string.gsub (str, "([^%w %-%_%.%~])",
        function (c) return string.format ("%%%02X", string.byte(c)) end)
    str = string.gsub (str, " ", "+")
  end
  return str	
end

function repositoryURL()
	local host = "http://" .. config.repository[1].host
	if config.repository[1].port and tonumber(config.repository[1].port) ~= 80 then
		host = host .. ":" .. config.repository[1].port
	end
	return host .. "/inconcert/apps/repository";
end


function downloadFile(name,part)
	local response_body={}
	local result, code, headers, status =http.request{
		url = repositoryURL() .. "/get/?id=" .. url_encode(name) .."&section=1&part=1",
		method = "GET",
		headers = {
			["Content-Type"] =  "text/plain"
		},
		sink = ltn12.sink.table(response_body)
	}
	if tonumber(code) == 200 then
		return true,table.concat(response_body)
	else
		LogDebug("downloadFile Error %s",tostring(code))
		return false,messages.messageKey('repository_download_error',{status=status,result=result,httpcode=tonumber(code)});
	end
end


function uploadFile(name,mimeType,fmt,content)
	local l = #content;
	LogDebug("Upload file %s size = %d",name,l);

	local result, code, headers, status = http.request {
		url = repositoryURL() .. "/add/?id=" .. url_encode(name) .. "&section=1&format=" .. url_encode(fmt) .. "&contentType=" .. url_encode(mimeType) .. "&fileName=" .. url_encode(name),
		method = "POST",
		headers = {
			["Content-Type"] =  "text/plain",
			["Content-Length"] = l
		},
		source = ltn12.source.string(content)
	}
	if tonumber(code) == 200 then
		return true
	else
		LogDebug("Error %s",tostring(code))
		return false,messages.messageKey('repository_upload_error',{status=status,result=result,httpcode=tonumber(code)});
	end

end

function deleteRepositoryFile(name)
	LogDebug("Deleting file %s",name);
	local response_body={}
	local result, code, headers, status = http.request {
		url = repositoryURL() .. "/delete/?id=" .. url_encode(name) .. "&section=1",
		method = "GET",
		headers = {
			["Content-Type"] =  "text/plain",
		},
		sink = ltn12.sink.table(response_body)		
	}
	LogDebug("code %s response=%s status=%s",tostring(code),table.concat(response_body),tostring(status))
	if tonumber(code) == 200 and table.concat(response_body) == "true" then
		return true
	else
		return false,messages.messageKey('repository_delete_error',{status=status,result=result,httpcode=tonumber(code)});
	end
	
end
local controlData
local controlDataIsDirty=false;
function fileExist(vccName,flowName,fileName)
	local status
	LogDebug("Check file existence %s",fileName);
	if not controlData then
		status,controlData = downloadFile("flowcontrol_" .. flowName .. "_at_" .. vccName)
		if not status then
			controlData = {}
		else
			LogDebug("CONTROL DATA %s",controlData);
			controlData = json.decode(controlData)
		end
	end	
	return controlData[fileName]~=nil
end
local function scanDimensions (appName,ctrl,dhash)
	dhash = dhash or {}
	if ctrl.type == "ui.flowactivity.setInteractionData" and ctrl.vardata and ctrl.vardata.interactionVarlist and (ctrl.vardata.mode == "report" or ctrl.vardata.mode == "both") then
		
		for _,var in ipairs(ctrl.vardata.interactionVarlist) do
			if var.name then
				dhash[var.name]= {
					scope="worflow",
					type=var.type=="integer" and "number" or "string",
					name=var.name,
					projectId=appName,
					requiredFieldName=appName .. '.__SID',
					requiredFieldValue=appName,
					sourceName=appName
				};
			end
		end
		
	end	
	if ctrl.childs then
		for k,v in ipairs(ctrl.childs) do 
			scanDimensions(appName,v,dhash)
		end
	end
	return dhash
end

local function deleteDimensions(vccName,appName,formData)
	
	if formData then
		local summarizerKS = cassandra:CassandraConnection("SummarizerData")		
		local dhash = scanDimensions(appName,formData)
		for name,dm in pairs(dhash) do
			LogDebug("deleteDimension %s, %s",vccName,appName .. "." .. name);
			manage.deleteDimension(summarizerKS,vccName,appName .. "." .. name);
			
		end
		
	end
end

function createDimensions(vccName,appName,formData)
	if formData then
		local summarizerKS = cassandra:CassandraConnection("SummarizerData")		
		local dhash = scanDimensions(appName,formData)
		for name,dm in pairs(dhash) do
			LogDebug("setDimension %s, %s,%s",vccName,appName .. "." .. name,json.encode(dm,true));
			manage.setDimension(summarizerKS, vccName,appName .. "." .. name,dm);
		end
	end
	
end
function configure(app,formName,formData,oldFormData)
	LogDebug("=========== FLOW CONFIGURE %s",formName)
	local designerApp = application.new{vccName = "system"}	
	local status,result = designerApp:initialize("designer");
	controlData=nil
	controlDataIsDirty=false;
	local run={}
	if status then
		deleteDimensions(app:getVCCName(),app:getName(),oldFormData)
		createDimensions(app:getVCCName(),app:getName(),formData)
		
		local res = {
			activities={};
			draw=formData;
			run=run
		}
		local addActivityData;
		addActivityData = function (ctrl)
			local t = string.sub(ctrl.type,4,15);
			local n = string.sub(ctrl.type,17);
			if t == "flowactivity" then
				if not res.activities[n] then
					status,result = designerApp:readResource( n .. ".activity.json",true);
					if status then
						res.activities[n] = json.decode(result.content);
					end
				end
			end
		end
		local mapActivities
		local actmap ={}
		local start;
		mapActivities = function (ctrl)
			if ctrl.beginactivity then
				start = ctrl
			end
			local t = string.sub(ctrl.type,4,15);
			local n = string.sub(ctrl.type,17);
			actmap[ctrl.id] = ctrl;
			ctrl.exec = n;
			ctrl.line=nil;
			if ctrl.childs then
				for k,v in ipairs(ctrl.childs) do 
					mapActivities(v)
				end
			end
		end
		
		mapActivities(formData);
		local addLine
		addLine =function (ctrl)
			if ctrl.line then
				return ctrl.line
			end
			table.insert(run,ctrl)
			addActivityData(ctrl);
			ctrl.line = #run;
			local l = #run
			local t=0
			for _,cnx in pairs(formData.connections or {}) do
				if cnx.from.child == ctrl.id then
					t = t+1
					local goto=addLine(actmap[cnx.to.child])
					table.insert(run,l+1,{
						transition=true;
						result= cnx.outputName,
						goto=goto
					});
			
					for i=l+1,#run do
						if run[i].transition  then 
							if run[i].goto and run[i].goto > l then
								run[i].goto = run[i].goto+1 
							end
						else 
							actmap[run[i].id].line =actmap[run[i].id].line+1
						end
					end
					
				end
			end
			if t == 0 then
				table.insert(run,l+1,{
					transition=true;
					result= "__no_transition__"
				});
		
				for i=l+1,#run do
					if run[i].transition  then 
						if run[i].goto and run[i].goto > l then
							run[i].goto = run[i].goto+1 
						end
					else 
						actmap[run[i].id].line =actmap[run[i].id].line+1
					end
				end
			end
			return ctrl.line
		end
		if start then
			addLine(start)
		end
		-- Upload de prompts
		LogDebug("Uploading ");
		local resources = {}
		for k,v in pairs(actmap) do

			local function setupPrompt(prompt)
				if prompt.resourceFile then
					if not fileExist(app:getVCCName(),app:getName(),prompt.resourceFile) then
						local status, result = app:readResource(prompt.resourceFile,true)
						if status then
							status,result = uploadFile(prompt.resourceFile,"wav","wav",result.content);
							
							if not status then
								return status,result
							end
							controlData[prompt.resourceFile]=true;
							controlDataIsDirty=true;
							resources[prompt.resourceFile] = true;
							return status,result
						else
							return status,result
						end
					else
						resources[prompt.resourceFile] = true;
					end
				end
				return true
			end
		
			local act = actmap[k];
			local function scanPrompts(data)
				
				if data.Prompt or data.prompt then
					status,result = setupPrompt(data.Prompt or data.prompt )
					if not status then
						return status,result
					end
				elseif data.Prompts or data.prompts then
					for _,p in ipairs(data.Prompts or data.prompts ) do
						status,result = setupPrompt(p)
						if not status then
							return status,result
						end
					end
				end
			end
			
			scanPrompts(act)
			for k,v in pairs(act) do
				if k ~= "Prompts" and k~= "prompts" and type(v) == "table" then
					LogDebug("Scan prompts en la propiedad %s",k)
					scanPrompts(v)	
				end
			end
		end
		-- borrado de los que no se usan
		LogDebug("Cleanup resources");
		
		local l = {}
		for k,v in pairs(app:listResources()) do l[k]=v end
		
		for k,v in ipairs(l) do
			LogDebug("Test for references to %s",v.src);
			if string.gsub(v.src,"~.+",function() return "" end) == formName and not resources[v.src] then
				LogDebug("Deleting")
				status,result = deleteRepositoryFile(v.src);
				if not status and result.httpcode ~= 200 then
					return status,result
				end
				controlData = controlData or {}
				
				controlData[v.src]=nil
				controlDataIsDirty=true;
				status,result = app:deleteResource(v.src)				
				if not status then
					return status,result
				end
			else
				LogDebug("Preserved")
			end
		end
		if controlDataIsDirty then
			uploadFile("flowcontrol_" ..app:getName() .. "_at_" .. app:getVCCName(),"json","json",json.encode(controlData));
		end
		
	--	LogDebug("Updating flow %s",json.encode(res,true))

		local function send_auth(client_redis, pwd)
			if not pwd or type(pwd) ~= 'string' or pwd == "" then return true end
			local authresult = client_redis:auth(pwd)
			return authresult == "OK"
		end			
		
		local redis = sidereal.connect(config.redis_host,config.redis_port)
		redis = send_auth(redis, os.getenv("REDIS_PWD")) and redis or nil
		
		if not flowrun.redisFlowSource{redisClient=redis,vccName=app:getVCCName()}:store(app:getName(),res) then
			return false,messages.messageKey("database_conection_error")
		end
		
		for k,ctrl in pairs(actmap) do
--			LogDebug("Clean %s",ctrl.id)
			ctrl.line = nil;
			ctrl.exec = nil;
		end
	end
	return status,result
end

function deconfigure(app,formName,formData)
	LogDebug("=========== FLOW DECONFIGURE %s",formName)
	deleteDimensions(app:getVCCName(),app:getName(),formData)
	return true
end


