local OS = os.getenv("OS");
local BUILD_NUMBER = os.getenv("BUILD_NUMBER") or ""
local tmppath = "./"
local gitcmd = "git"
local zipcmd = "zip"
local mkdircmd = "mkdir"
local pathSeparator = "/"
local copycmd = "cp "

local rmdircmd = "rmdir -p --ignore-fail-on-non-empty"
local inconcertPrefix ="Web/inConcert/apps/webdesigner"
local exclude = {"scpusers/system/testsuite","rsvg","doc","hotfixBuilder.lua"}
local outputPrefix="../../"
if string.match(os.getenv("OS") or "","Windows") then
	copycmd = "copy"
	rmdircmd = "rmdir /Q /S"
	pathSeparator = "\\"
end

ox = os.execute
os.execute = function ( command)
	print("[",command,"]")
	return ox(command)
end


local function prepareDirectoryStructure(files)
	print("generating Inconcert directory structure");
	local paths={}


	for _,filename in ipairs(files) do
		local p={}
		filename = inconcertPrefix .."/"  .. filename
		local parent =paths
		string.gsub(filename,"([%a_]+)/",
			function(sdir)
				parent[sdir] = parent[sdir] or {}
				parent = parent[sdir]
			end
		)
	end
	local mkdir
	mkdir=function (name, tree)
		os.execute( mkdircmd .. " " ..name)

		for k,v in pairs(tree) do
			mkdir ( name .. pathSeparator ..  k,v)
		end
	end
	for k,v in pairs(paths) do
		os.execute(rmdircmd .. " " .. k);
		mkdir(k,v)
	end
end

local function normalizePath (path)
	path = string.gsub(path,"/",pathSeparator)
	return path;
end

local function fcopy ( frompath, topath)
	os.execute(copycmd .. " " .. normalizePath(frompath) .. " " .. normalizePath(topath));
end

local function trim(s)
	return (string.gsub(s, "^%s*(.-)%s*$", "%1")) -- trim
end

local function getComments(fromTag)
	print("Retrieving comments from " ..  fromTag)
	local tmpFile = tmppath .. "git_comments"
	os.execute( gitcmd .. " --no-pager log " .. fromTag .. "..master >" .. tmpFile )
	local f = io.open(tmpFile,"rb")
	local res = f:read("*a");
	local t ={}
	local t2 ={}
	res = string.gsub(res,"#[^\r\n]+",
		function(c)
			if string.match(c,"#%d+") then
				table.insert(t,"fix " .. c)
			else
				table.insert(t2,"" .. trim(string.sub(c,2)))
			end
		end)
	io.close(f);
	table.insert(t,"")
	for k,v in ipairs(t2) do
		table.insert(t,v)
	end
	return table.concat(t,"\n")
end



local function buildPatch ( fromTag,outputName,fileList )
	print("building patch ")
	local tmpFile = tmppath .. "files"
	local fullfilelist = {}
	os.remove("inConcertWeb.zip");
	os.remove(outputPrefix .. outputName .. ".zip");
	if #fileList > 0 then
		f = io.open(tmpFile,"w")
		prepareDirectoryStructure(fileList);
		for _,filename in ipairs(fileList) do
			local fullFile = inconcertPrefix .. "/" .. filename
			table.insert(fullfilelist,fullFile);
			fcopy(filename,fullFile)
			f:write(fullFile .. "\n")
		end
		local maxLength=0;
		for _,filename in ipairs(fullfilelist) do
			if #filename > maxLength then maxLength= #filename end
		end
		for idx,filename in ipairs(fullfilelist) do
			fullfilelist[idx] = filename .. string.rep(' ',maxLength + 4 - #filename) .. " BUILD " .. BUILD_NUMBER .. " (GIT)";
		end
		io.close(f)
		os.execute(zipcmd  .. " " .. "inConcertWeb.zip " .. "-@<" .. tmpFile );
		os.remove(tmpFile)
		f = io.open("database_update.sql","wb");
		local sql = [[
---------------------------------------------------------------------------------------------------------------------------------------------

/*
#--------------------------------------#
#  	 HOTFIX TABLE UPDATE	 	  #
#--------------------------------------#
*/

USE MMPRODAT
GO

IF NOT exists (select * from sysobjects where name='HotFix' and xtype='U')
CREATE TABLE [dbo].[HotFix] (
	[Id] [varchar] (50) NOT NULL,
	[Description] [varchar] (200),
	[TimeStamp] [datetime] NOT NULL
) ON [PRIMARY]

GO
IF NOT EXISTS ( SELECT * FROM HotFix WHERE Id='{{HOTFIXNAME}}')
BEGIN
	INSERT INTO HotFix VALUES('{{HOTFIXNAME}}', 'WebDesigner corrections.', getutcdate())
END
GO
	]]

		sql = string.gsub(sql,"{{HOTFIXNAME}}",outputName)
		sql = string.gsub(sql,"\13","")
		sql = string.gsub(sql,"\10","\r\n")

		f:write(sql)
		io.close(f);
		f = io.open("Readme.txt","wb");
		txt = [[
Descripci�n:
====================================================================================================
Correcciones sobre WebDesigner.

{{COMMENTS}}


Como aplicarlo
====================================================================================================

* Descomprimir el contenido del archivo  {{HOTFIXNAME}}.zip  a un directorio temporal.
* Descomprimir el archivo inConcertWeb.zip en la carpeta %ProgramFiles(x86)%\InConcert
* Cuando pregunte sobre-escribir indica que si.
* Reiniciar el servicio Inconcert WebHandler
* Abrir el archivo database_update.sql con query analizer conectandose contra su servidor de base de datos y ejecutarlo


Archivos
====================================================================================================
{{FILES}}

database_update.sql
	]]

		txt = string.gsub(txt,"{{HOTFIXNAME}}",outputName);

		txt = string.gsub(txt,"{{FILES}}",table.concat(fullfilelist,"\n"))
		txt = string.gsub(txt,"{{COMMENTS}}",getComments(fromTag))

		txt = string.gsub(txt,"\13","")
		txt = string.gsub(txt,"\10","\r\n")


		f:write(txt)
		io.close(f);
		os.execute(zipcmd .. " " .. outputPrefix .. outputName .. ".zip inConcertWeb.zip Readme.txt database_update.sql")
		os.remove("inConcertWeb.zip");

	end

end
local function getmodifiedFiles()
	print("Generating list of modified files");
	local tmpFile = tmppath .. "git_status"
	os.execute( gitcmd .. " status > " .. tmpFile )
	local names ={}
	for k in io.lines(tmpFile) do
		local e = false;
		if string.match(k,"#[^:]+modified:[\t ]+[%a]+") or string.match(k,"#[^:]+new file:[\t ]+[%a]+") then

			k = string.gsub(k,"#[^:]+:[\t ]+","")
			for _,excludePath in ipairs(exclude) do
				if string.sub(k,1,#excludePath) == excludePath then
					e = true
				end
			end
			if not e then
				table.insert(names,k)
			end
		end
	end
	os.remove(tmpFile)
	return names;
end
local function getTags()
	print("Retrieving current tag list ")
	local tmpFile = tmppath .. "git_tags"
	os.execute( gitcmd .. " --no-pager log --decorate=full | grep 'tag' > " .. tmpFile )
	local tags ={}
	for k in io.lines(tmpFile) do
		string.gsub(k,"%refs/tags/([%w%._-]+)[ ,)]",
			function(tagName)
				table.insert(tags,tagName);
			end
		)
	end
	os.remove(tmpFile)
	return tags;
end

local function sync()
	print("sync local repository")
	os.execute( gitcmd .. " checkout master --quiet  --force ")
	os.execute( gitcmd .. " pull --quiet ")
	os.execute( gitcmd .. " checkout master --quiet --force" )
end
local function checkout( tag )
	print("checkout tag " ..tag)

	os.execute( gitcmd .. " checkout --quiet " .. tag )
end
local function patch ( tag )
	print ( "Apply patch with commits from tag " ..  (tag or "master"));
	os.execute( gitcmd .. " merge --quiet --no-commit --no-ff " .. (tag or "master"))
end
local function tag ( newTagName)
	os.execute( gitcmd .. " checkout master --quiet  --force ")
	os.execute( gitcmd .. " tag -d " .. newTagName )
	os.execute( gitcmd .. " tag -a " .. newTagName  .. " -m \"Automatic tag created by jenkins\"")
	os.execute( gitcmd .. " push origin refs/tags/" ..newTagName .. ":refs/tags/" .. newTagName);

end

local function generateHotfix(hotfixName,fromTag)
	local tags
	local found=false
--	sync();
	tags = getTags();
	if fromTag then
		fromTag = string.upper(fromTag)
		for _,tagName in ipairs(tags) do
			if string.upper(fromTag) == string.upper(tagName) then
				found = true
				fromTag = tagName;
				break;
			end
		end
		if not found then
			print("Required tag " .. fromTag .. " not found")
			os.exit(1)
		end
	else
		fromTag = tags[1]
	end
	checkout(fromTag);
	patch();
	local files = getmodifiedFiles();
	if #files>0 then
		buildPatch(fromTag,hotfixName ,files)
		tag(hotfixName);
	end

end
--print(getComments("HOTFIX_201206204"))

if #arg == 1 or #arg == 2 then

	generateHotfix(arg[1],arg[2]);
else
	print ("usage: " .. arg[0] .. " PATCHNAME [FROMTAG] ")
end



