module(..., package.seeall)
local inconcert_base = require (tweety_modules_prefix .. "icc_base_ds")

require "LuaOrm.Date"
local config = require (tweety_modules_prefix .. "config")
local runtime = require (tweety_modules_prefix .. "runtime")
local messages = require (tweety_modules_prefix .. "messages")
local api = require (tweety_modules_prefix .. "webhandler_api")
local dsdialercontact = require (tweety_modules_prefix .. "icc_contact_ds")
local utils = require (tweety_modules_prefix .. "utils")

function new(self)

	local function empty(value)
		return not value or value == ""
	end
	
	local function Array2Hash(data)
		local hash = {}
		for _,argument in ipairs(data) do
			hash[argument.name] = argument.value or argument.orgivalue
		end
		return hash
	end
	
	local allegroTypes = {
		CALL = true,
		VOICEMAIL = true
	}

	------------------- ALLEGRO

	local m_hasAllegro = nil
	function self:hasAllegro()
		if m_hasAllegro == nil then
			m_hasAllegro = api.HasAllegroIntegration() or false
		end
		return m_hasAllegro
	end

	local function sendAllegroDispositionCode(interaction)
		local disposition = {
			interactionId		= interaction.Id,
			processId			= interaction.processId,
			batchId				= interaction.batchId,
			contactId			= interaction.DialerContact,
			contactName			= interaction.ContactName,
			businessCode		= interaction.ResultCode,
			contactCode			= interaction.ContactCode or 0
		}
		local status = api.SetAllegroDisposition(disposition)
		if not status then
			return false, messages.messageKey('database_conection_error',{errText = "error setting allegro disposition"})
		end
		return true
	end

	local function processAllegroSchedulePhone(interaction)
		-- get phone timezone
		local status, phone
		if interaction.RescheduledToPhone then
			LogInfo("Attempt to recognize alternative phone %s ", interaction.RescheduledToPhone)
			status, phone = inconcert_base.recognizePhone(interaction.RescheduledToPhone, true)
		elseif interaction.ContactAddress then
			LogInfo("Attempt to recognize ContactAddress %s ", interaction.ContactAddress)
			if string.sub(interaction.ContactAddress,1,1) ~= '+' then
				interaction.ContactAddress = '+' ..  interaction.ContactAddress
			end
			status, phone = inconcert_base.recognizePhone(interaction.ContactAddress, true)
		elseif interaction.CallerId then
			LogInfo("Attempt to recognize CallerId %s", interaction.CallerId)
			if string.sub(interaction.CallerId,1,1) ~= '+' then
				interaction.CallerId = '+' ..  interaction.CallerId
			end
			status, phone = inconcert_base.recognizePhone(interaction.CallerId, true)
		end
		--then process date
		if status then
			local isoTime
			local _,_,y,m,d=string.find(interaction.RescheduledToDate, "(%d+)-(%d+)-(%d+)")
			local _,_,h,M = string.find(interaction.RescheduledToTime, "(%d+):(%d+)")
			isoTime = y .. "-" .. m .. "-" .. d  .. " " .. h .. ":" .. M .. ":00.000"
			local d
			pcall(function()
				d = LuaOrm.Date():newFromDifferentTimezone(isoTime, phone.timeZone)
				interaction.RescheduledToDate = d:year() .. "-" .. d:month() .. "-"  .. d:day()
				interaction.RescheduledToTime = d:hour() .. ":" .. d:minute()
			end)
			return phone
		end
	end

	local function addAllegroPhoneToContact(phone, contactId)
		LogInfo("addPhoneToContact %s %s-%s-%s,%s",
			contactId, phone.countryId, phone.areaId, phone.phoneNumber, phone.phoneExtension)
		local csave = false;
		local data = {}
		local hdata={}
		--get contact
		local contacts = dsdialercontact.new({
			description= {
				columns = {
					{	name= "contactId",type = "string"},
					{	name= "name",type = "string"},
					{	name= "phones",type = "phonelist"}
				},
				store={
					inputParams={}
				}
			}
		})
		contacts.description.name = datasourceName
		local status, rs = contacts:get({
			{ name = "contactId", value = contactId }
		},{})
		if not status then
			return status, rs
		end
		if #rs.rows ~= 1 then
			return false,messages.messageKey('database_error',
				{ description = string.format("Unable to retrieve contact [%s]", contactId) })
		end
		--add phone
		for idx, name in ipairs(rs.colnames) do
			local p = { name = name, value=rs.rows[1][idx] }
			table.insert(data,p)
			hdata[name] = p
		end
		--find in contact phones
		local found = false
		for _, mphone in ipairs(hdata.phones) do
			if 	mphone.phoneNumber == phone.phoneNumber and
				mphone.countryId == phone.countryId and
				mphone.areaId == phone.areaId and
				mphone.phoneExtension == phone.phoneExtension 
			then
				if mphone.phoneType ~= phone.phoneType then
					mphone.phoneType = phone.phoneType
					bsave = true
				end
				found = true
				break
			end
		end
		--add phone
		if not found then
			table.insert(hdata.phones.value,phone);
		end
		if not found or bsave then
			return contacts:update(data)
		end
		return true
	end

	local function rescheduleAllegroContactToCampaign(interaction, phone)
		local result= {}
		local t
		local status = true
		LogInfo("rescheduleContactToCampaign")
		--process phone
		if not phone then
			--invalid phone
			if interaction.RescheduledToPhone then
				result = messages.messageKey('cant_reschedule_contact_by_incorrect_phone_number',
					{ number = interaction.RescheduledToPhone })
				result.field = "RescheduledToPhone"
			elseif interaction.ContactAddress then
				result = messages.messageKey('cant_reschedule_contact_by_incorrect_contact_address',
					{ number = interaction.ContactAddress })
				result.field = "ContactAddress"
			else
				result = messages.messageKey('cant_reschedule_contact_by_incorrect_caller_id',
					{ number = interaction.CallerId })
				result.field = "CallerId"
				result.field_id = "CallerId"
			end
			-- agregamos el campo para que pueda ser procesado por el contro
			result.level=3;
			--empacamos dentro de un error general 
			result = messages.messageKey('fields_validation_error',{
				level=3,
				count=1,
				field_errors = {
					[result['field']] = result
				}
			})
			return false, result
		end
		phone.phoneType = interaction.RescheduledToPhoneType or "CELLULAR"
		status, result = addAllegroPhoneToContact(phone, interaction.DialerContact)
		if not status then
			return status, result
		end
		--reschedule
		local _, _, year, month, day = string.find(interaction.RescheduledToDate, "(%d+)-(%d+)-(%d+)")
		local _, _, hour, minute = string.find(interaction.RescheduledToTime, "(%d+):(%d+)")
		local phoneNumber = phone.phoneNumber
		if phoneNumber ~= "" and phone.phoneExtension ~= "" then
			phoneNumber = phoneNumber .. "," .. phone.phoneExtension
		end
		local data = {
			campaignId = interaction.RescheduledToCampaign or interaction.Campaign,
			processId = interaction.processId,
			batchId = interaction.batchId,
			contactId = interaction.DialerContact,
			agent = interaction.RescheduledToAgent,
			year = year,
			month = month,
			day = day,
			hour = hour,
			minute = minute,
			phoneCountry = phone.countryId,
			phoneArea = phone.areaId,
			phoneType = phone.phoneType,
			phoneNumber = phoneNumber
		}
		status = api.RescheduleAllegroContactToCampaign(data)
		if not status then
			return false, messages.messageKey('cant_reschedule_contact')
		end
		return true
	end

	local function rescheduleAllegroContactToPhone(interaction, phone)
		local status, result
		--validate phone
		if not phone then
			-- agregamos el campo para que pueda ser procesado por el contro
			result = messages.messageKey('cant_reschedule_contact_by_incorrect_phone_number',
				{ number = interaction.RescheduledToPhone })
			result.field = "RescheduledToPhone"
			result.field_id = "RescheduledToPhone"
			result.level=3
			--empacamos dentro de un error general 
			result = messages.messageKey('fields_validation_error',{
				level=3,
				count=1,
				field_errors={
					RescheduledToPhone=result
				}
			})
			return false, result
		end
		phone.phoneType = interaction.RescheduledToPhoneType or "CELLULAR"
		status, result = addAllegroPhoneToContact(phone, interaction.DialerContact)
		if not status then
			return status, result
		end
		--reschedule
		local _, _, year, month, day = string.find(interaction.RescheduledToDate, "(%d+)-(%d+)-(%d+)")
		local _, _, hour, minute = string.find(interaction.RescheduledToTime, "(%d+):(%d+)")
		local phoneNumber = phone.phoneNumber
		if phoneNumber ~= "" and phone.phoneExtension ~= "" then
			phoneNumber = phoneNumber .. "," .. phone.phoneExtension
		end
		local data = {
			processId = interaction.processId,
			batchId = interaction.batchId,
			contactId = interaction.DialerContact,
			agent = interaction.RescheduledToAgent,
			year = year,
			month = month,
			day = day,
			hour = hour,
			minute = minute,
			phoneCountry = phone.countryId,
			phoneArea = phone.areaId,
			phoneType = phone.phoneType,
			phoneNumber = phoneNumber
		}
		status = api.RescheduleAllegroContactToPhone(data)
		if not status then
			return false, messages.messageKey('cant_reschedule_contact')
		end
		return true
	end

	local function rescheduleAllegroContact(interaction)
		LogInfo("rescheduleContact")
		local _, _, year, month, day = string.find(interaction.RescheduledToDate, "(%d+)-(%d+)-(%d+)")
		local _, _, hour, minute = string.find(interaction.RescheduledToTime, "(%d+):(%d+)")
		local data = {
			processId = interaction.processId,
			batchId = interaction.batchId,
			contactId = interaction.DialerContact,
			agent = interaction.RescheduledToAgent,
			year = year,
			month = month,
			day = day,
			hour = hour,
			minute = minute
		}
		status = api.RescheduleAllegroContact(data)
		if not status then
			return false, messages.messageKey('cant_reschedule_contact')
		end
		return true
	end

	local function scheduleAllegroInteraction(interaction, hdata)
		--process phone
		local phone = processAllegroSchedulePhone(interaction)
		--reschedule
		local status, result
		if not interaction.processId or not interaction.batchId or interaction.RescheduledToCampaign then
			status, result = rescheduleAllegroContactToCampaign(interaction, phone)
		elseif interaction.RescheduledToPhone then
			status, result = rescheduleAllegroContactToPhone(interaction, phone)
		else
			status, result = rescheduleAllegroContact(interaction)
		end
		if not status then
			return status, result
		end
		--result
		return status, result
	end

	------------------- END ALLEGRO

	local function datePart(part)
		local partNumber = tonumber(part or "0")
		return (partNumber < 10 and "0" or "") .. partNumber
	end

	local function getRescheduleIsoDate(interaction)
		local _,_,y,m,d=string.find(interaction.RescheduledToDate, "(%d+)-(%d+)-(%d+)")
		local _,_,h,M = string.find(interaction.RescheduledToTime, "(%d+):(%d+)")
		return y .. "-" .. datePart(m) .. "-" .. datePart(d)  .. " " ..
			datePart(h) .. ":" .. datePart(M) .. ":00.000"
	end

	local function getRescheduleTimestamp(interaction)
		local isoDate = getRescheduleIsoDate(interaction)
		return LuaOrm.Date(isoDate):time()
	end

	local function rescheduleTimestampToTimezone(interaction, timezone)
		local isoDate = getRescheduleIsoDate(interaction)
		pcall(function()
			local d = LuaOrm.Date():newFromDifferentTimezone(isoDate, timezone)
			interaction.RescheduledToDate = d:year() .. "-" .. d:month() .. "-"  .. d:day()
			interaction.RescheduledToTime = d:hour() .. ":" .. d:minute()
		end)
	end

	local function processSchedulePhone(interaction)
		-- get phone timezone
		local status, phone
		if interaction.RescheduledToPhone then
			LogInfo("Attempt to recognize alternative phone %s ", interaction.RescheduledToPhone)
			status, phone = inconcert_base.recognizePhone(interaction.RescheduledToPhone)
		elseif interaction.ContactAddress then
			LogInfo("Attempt to recognize ContactAddress %s ", interaction.ContactAddress)
			if string.sub(interaction.ContactAddress,1,1) ~= '+' then
				interaction.ContactAddress = '+' ..  interaction.ContactAddress
			end
			status, phone = inconcert_base.recognizePhone(interaction.ContactAddress)
		elseif interaction.CallerId then
			LogInfo("Attempt to recognize CallerId %s", interaction.CallerId)
			if string.sub(interaction.CallerId,1,1) ~= '+' then
				interaction.CallerId = '+' ..  interaction.CallerId
			end
			status, phone = inconcert_base.recognizePhone(interaction.CallerId)
		end
		--then process date
		if status then
			rescheduleTimestampToTimezone(interaction, phone.timeZone)
			return phone
		end
	end

	local function rescheduleContactToCampaign(interaction, phone)
		local result= {}
		local t
		local status = true
		--process phone
		if not phone then
			--invalid phone
			if interaction.RescheduledToPhone then
				result = messages.messageKey('cant_reschedule_contact_by_incorrect_phone_number',
					{ number = interaction.RescheduledToPhone })
				result.field = "RescheduledToPhone"
			elseif interaction.ContactAddress then
				result = messages.messageKey('cant_reschedule_contact_by_incorrect_contact_address',
					{ number = interaction.ContactAddress })
				result.field = "ContactAddress"
			else
				result = messages.messageKey('cant_reschedule_contact_by_incorrect_caller_id',
					{ number = interaction.CallerId })
				result.field = "CallerId"
				result.field_id = "CallerId"
			end
			-- agregamos el campo para que pueda ser procesado por el contro
			result.level=3;
			--empacamos dentro de un error general 
			result = messages.messageKey('fields_validation_error',{
				level=3,
				count=1,
				field_errors = {
					[result['field']] = result
				}
			})
			return false, result
		end
		--reschedule
		local data = {
			campaignId 		= interaction.RescheduledToCampaign or interaction.Campaign,
			processId 		= interaction.processId,
			batchId 		= interaction.batchId,
			contactId 		= interaction.DialerContact,
			configuration 	= {
				restartRetries 	= true,
				agent 			= interaction.RescheduledToAgent,
				scheduleDate	= getRescheduleTimestamp(interaction),
				address 		= {
					Channel			= "CALL",
					Type			= "Phone",
					Kind			= interaction.RescheduledToPhoneType or "CELLULAR",
					ZipPhone		= "",
					Extension		= phone.phoneExtension,
					Status			= "Enabled",
					Country			= phone.countryId,
					Area			= phone.areaId,
					LocalNumber		= phone.localNumber,
					Number			= phone.phoneNumber,
					SanitizedId		= phone.phoneNumber,
					displayable		= phone.displayable,
				}
			}
		}
		LogInfo("rescheduleContactToCampaign %s", json.encode(data))
		status = api.RescheduleDialerContact(data)
		if not status then
			return false, messages.messageKey('cant_reschedule_contact')
		end
		return true
	end

	local function rescheduleContactToPhone(interaction, phone)
		local status, result
		--validate phone
		if not phone then
			-- agregamos el campo para que pueda ser procesado por el contro
			result = messages.messageKey('cant_reschedule_contact_by_incorrect_phone_number',
				{ number = interaction.RescheduledToPhone })
			result.field = "RescheduledToPhone"
			result.field_id = "RescheduledToPhone"
			result.level=3
			--empacamos dentro de un error general 
			result = messages.messageKey('fields_validation_error',{
				level=3,
				count=1,
				field_errors={
					RescheduledToPhone=result
				}
			})
			return false, result
		end
		--reschedule
		local data = {
			processId 		= interaction.processId,
			batchId 		= interaction.batchId,
			contactId 		= interaction.DialerContact,
			configuration 	= {
				restartRetries 	= true,
				agent 			= interaction.RescheduledToAgent,
				scheduleDate	= getRescheduleTimestamp(interaction),
				address 		= {
					Channel			= "CALL",
					Type			= "Phone",
					Kind			= interaction.RescheduledToPhoneType or "CELLULAR",
					ZipPhone		= "",
					Extension		= phone.phoneExtension,
					Status			= "Enabled",
					Country			= phone.countryId,
					Area			= phone.areaId,
					LocalNumber		= phone.localNumber,
					Number			= phone.phoneNumber,
					SanitizedId		= phone.phoneNumber,
					displayable		= phone.displayable,
				}
			}
		}
		LogInfo("rescheduleContactToPhone %s", json.encode(data))
		status = api.RescheduleDialerContact(data)
		if not status then
			return false, messages.messageKey('cant_reschedule_contact')
		end
		return true
	end

	local function rescheduleContact(interaction)
		local data = {
			campaignId 		= interaction.RescheduledToCampaign or interaction.Campaign,
			processId 		= interaction.processId,
			batchId 		= interaction.batchId,
			contactId 		= interaction.DialerContact,
			configuration 	= {
				restartRetries 	= true,
				agent 			= interaction.RescheduledToAgent,
				scheduleDate	= getRescheduleTimestamp(interaction),
			}
		}
		LogInfo("rescheduleContact %s", json.encode(data))
		status = api.RescheduleDialerContact(data)
		if not status then
			return false, messages.messageKey('cant_reschedule_contact')
		end
		return true
	end

	local function scheduleInteraction(interaction, hdata)
		if self:hasAllegro() then
			return scheduleAllegroInteraction(interaction, hdata)
		end
		--process phone
		local phone = processSchedulePhone(interaction)
		--reschedule
		local status, result
		if not interaction.processId or not interaction.batchId or interaction.RescheduledToCampaign then
			status, result = rescheduleContactToCampaign(interaction, phone)
		elseif interaction.RescheduledToPhone then
			status, result = rescheduleContactToPhone(interaction, phone)
		else
			status, result = rescheduleContact(interaction)
		end
		if not status then
			return status, result
		end
		--result
		return status, result
	end
	
	local function sendDispositionCode(interaction)
		if self:hasAllegro() and allegroTypes[interaction.Type] then
			return sendAllegroDispositionCode(interaction)
		end
		local disposition = {
			name			= interaction.ResultCodeDescription,
			treePath		= interaction.ResultTreePath
		}
		interaction.CurrentId = api.GetDatasourceCurrentInteraction(interaction.Id, interaction.Type)
		local status = api.SetDisposition(interaction.CurrentId, disposition, interaction.ResultComment or "")
		if not status then
			return false, messages.messageKey('database_conection_error',{errText = "error setting disposition"})
		end
		return true
	end

	local function setContact(interaction)
		interaction.CurrentId = api.GetDatasourceCurrentInteraction(interaction.Id, interaction.Type)
		local status = api.SetInteractionContact(interaction.CurrentId, interaction.Contact, interaction.ContactName)
		if not status then
			return false, messages.messageKey('database_conection_error',{errText = "error setting contact"})
		end
		return true
	end
	
	function self:update(data)
		--validate data
		local hdata={}
		for _,param in ipairs(data) do
			hdata[param.name]=param;
		end
		local interaction = Array2Hash(data)
		if not interaction.Id then
			return false, messages.messageKey('missing_keys' ,{missingKeyName='Id'})
		end
		local status, result
		--if contact was changed then set it
		if not empty(interaction.Contact) and hdata.Contact.value ~= hdata.Contact.origvalue then
			status, result = setContact(interaction)
			if not status then
				return status, result
			end
		end
		--send disposition
		status, result = sendDispositionCode(interaction)
		if not status then
			return status, result
		end
		--then reschedule
		if interaction.schedule and interaction.DialerContact then
			status, result = scheduleInteraction(interaction, hdata)
			if not status then
				return status, result
			end
		end
		--ok
		return status, result
	end

	local function getContactAddress(interaction, params)
		local addressTable = interaction.interactionData and interaction.interactionData.completeAddress
		if not addressTable or addressTable.Type ~= "Phone" then
			return ContactAddress
		end

		return utils.FormatPhone(addressTable, params.phoneFormat)
	end

	local INTERACTION_COLUMNS = {
		{ 
			id = "Id", 
			type = "string",
			get = "id"
		},
		{ 
			id = "VirtualCC", 
			type = "string",
			get = "VCC"
		},
		{ 
			id = "Campaign", 
			type = "string",
			get = function(interaction)
				return interaction.campaign:match("(.+)@.+")
			end
		},
		{ 
			id = "Account", 
			type = "string",
			get = "account"
		},
		{ 
			id = "AttentionLevel", 
			type = "string",
			get = function(interaction)
				return interaction.attention_level and interaction.attention_level.id or nil
			end
		},
		{ 
			id = "Type", 
			type = "string",
			get = "type"
		},
		{ 
			id = "WorkitemType", 
			type = "string",
			get = function(interaction)
				--another channels
				if interaction.type ~= "CALL" then
					return interaction.type
				end
				--inbound
				if interaction.initiative == "INBOUND" then
					return "ENTDIRECTA"
				end
				--dialer
				if interaction.interactionData.dialer ~= nil then
					return "SALDIRECTA"
				end
				--manual
				return "TPOQUECALL"
			end
		},
		{ 
			id = "Initiative", 
			type = "string",
			get = "initiative"
		},
		{ 
			id = "Sections", 
			type = "integer",
			get = function(interaction)
				local sections = 1
				for _, step in pairs(interaction.steps) do
					if step.action == "TRANSFER TAKEN" or step.action == "REOPENED" then
						sections = sections + 1
					end
				end
				return sections
			end
		},
		{ 
			id = "Agent", 
			type = "string",
			get = function(interaction)
				local agent = nil
				if #interaction.assignedAgents > 0 then
					agent = interaction.assignedAgents[1]
				elseif #interaction.reservedAgents > 0 then
					agent = interaction.reservedAgents[1]
				elseif not empty(interaction.requiredAgent) then
					agent = interaction.requiredAgent
				elseif not empty(interaction.interactionData.enderAgent) then
					agent = interaction.interactionData.enderAgent
				end
				return agent and agent:match("(.+)@.+") or nil
			end
		},
		{ 
			id = "State", 
			type = "string",
			get = "state"
		},
		{ 
			id = "Subtype", 
			type = "string",
			interactionTypes = {
				FACEBOOK = true,
				INSTAGRAM = true,
				YOUTUBE = true,
				APPSTORE = true,
				GOOGLEPLAY = true,
				TWITTER = true
			},
			get = "subtype",
			data = "interactionData"
		},
		{ 
			id = "PostId", 
			type = "string",
			get = "postId",
			data = "interactionData"
		},
		{ 
			id = "ProcessFlow", 
			type = "string",
			get = "processFlow"
		},
		{ 
			id = "CreationDate", 
			type = "datetime",
			get = "creationTimestamp",
			data = "analysisTimes",
			timestamp = true
		},
		{ 
			id = "StartDate", 
			type = "datetime",
			get = "startTimestamp",
			timestamp = true
		},
		{ 
			id = "EndDate", 
			type = "datetime",
			get = "lastCloseTimestamp",
			data = "analysisTimes",
			timestamp = true
		},
		{ 
			id = "LastUpdate", 
			type = "datetime",
			get = "timestamp",
			timestamp = true
		},
		{ 
			id = "Contact", 
			type = "string",
			get = "contact"
		},
		{ 
			id = "ContactAddress", 
			type = "string",
			get = function(interaction, params)
				return getContactAddress(interaction, params)
			end
		},
		{ 
			id = "ContactName", 
			type = "string",
			get = "contactName",
			data = "interactionData"
		},
		{
			id = "Transfer", 
			type = "string",
			get = function(interaction)
				if not interaction.transferInfo then
					return nil
				end
				return interaction.transferInfo.target_data and
					interaction.transferInfo.target_data.type or "campaign"
			end
		},
		{ 
			id = "TransferrerCampaign", 
			type = "string",
			get = function(interaction)
				return interaction.transferInfo and interaction.transferInfo.origin_data and
					interaction.transferInfo.origin_data.from_campaign and 
					interaction.transferInfo.origin_data.from_campaign.id or nil
			end
		},
		{ 
			id = "TransferredCampaign", 
			type = "string",
			get = function(interaction)
				if not interaction.transferInfo then
					return nil
				end
				return interaction.transferInfo and interaction.transferInfo.target_data and
					interaction.transferInfo.target_data.to_campaign or interaction.campaign
			end
		},
		{ 
			id = "TransferrerAgent", 
			type = "string",
			get = function(interaction)
				return interaction.transferInfo and interaction.transferInfo.origin_data and
					interaction.transferInfo.origin_data.from_agent or nil
			end
		},
		{ 
			id = "TransferredAgent", 
			type = "string",
			get = function(interaction)
				return interaction.transferInfo and interaction.transferInfo.target_data and
					interaction.transferInfo.target_data.to_member or nil
			end
		},
		{ 
			id = "TransferrerAttLevel", 
			type = "string",
			get = function(interaction)
				return interaction.transferInfo and interaction.transferInfo.origin_data and
					interaction.transferInfo.origin_data.from_attention_level or nil
			end
		},
		{ 
			id = "TransferredAttLevel", 
			type = "string",
			get = function(interaction)
				return interaction.transferInfo and interaction.transferInfo.target_data and
					interaction.transferInfo.target_data.to_attention_level or nil
			end
		},
		{ 
			id = "Dnis", 
			type = "string",
			interactionTypes = {
				CALL = true,
				VOICEMAIL = true
			},
			get = "DNIS",
			data = "interactionData"
		},
		{ 
			id = "CallerId", 
			type = "string",
			interactionTypes = {
				CALL = true,
				VOICEMAIL = true
			},
			get = function(interaction, params)
				if interaction.initiative == "INBOUND" then
					return getContactAddress(interaction, params)
				else
					return nil
				end
			end
		},
		{ 
			id = "CalledId", 
			type = "string",
			interactionTypes = {
				CALL = true,
				VOICEMAIL = true
			},
			get = function(interaction, params)
				if interaction.initiative == "INBOUND" then
					return nil
				else
					return getContactAddress(interaction, params)
				end
			end
		},
		{ 
			id = "IsAppointment", 
			type = "boolean",
			interactionTypes = {
				CALL = true,
				VOICEMAIL = true
			},
			get = "isAppointment",
			data = "interactionData"
		},
		{ 
			id = "processId", 
			type = "string",
			interactionTypes = {
				CALL = true,
				VOICEMAIL = true
			},
			get = function(interaction)
				return interaction.interactionData.dialer and interaction.interactionData.dialer.outboundProcessId or nil
			end
		},
		{ 
			id = "batchId", 
			type = "string",
			interactionTypes = {
				CALL = true,
				VOICEMAIL = true
			},
			get = function(interaction)
				return interaction.interactionData.dialer and interaction.interactionData.dialer.batchId or nil
			end
		},
		{ 
			id = "DialerContact", 
			type = "string",
			interactionTypes = {
				CALL = true,
				VOICEMAIL = true
			},
			get = function(interaction)
				return interaction.interactionData.dialer and interaction.interactionData.dialer.contactId or nil
			end
		},
		{ 
			id = "FacebookPageId", 
			type = "string",
			interactionTypes = {
				FACEBOOK = true
			},
			get = "page",
			data = "interactionData"
		},
		{ 
			id = "ManagementResult", 
			type = "string",
			get = "dispositionCode"
		},
		--hidden
		{ 
			id = "Events", 
			type = "array",
			get = "events"
		},
		{ 
			id = "Steps", 
			type = "array",
			get = "steps"
		},
		{ 
			id = "Raw", 
			type = "object"
		},
	}

	local function makeResultSet()
		local result = {
			colnames = {},
			coltypes = {},
			rows = {}
		}
		for _, column in pairs(INTERACTION_COLUMNS) do
			table.insert(result.colnames, column.id)
			table.insert(result.coltypes, column.type)
		end
		return result
	end

	local function getTimestampDatetime(timestamp)
		local date = LuaOrm.Date(timestamp)
		return date:toString("%Y-%m-%d %H:%M:%S.000")
	end

	local function getColumnValue(column, interaction, params)
		if column.id == "Raw" then
			return interaction
		end
		if column.interactionTypes and not column.interactionTypes[interaction.type] then
			return json.null
		end
		local value = nil
		if type(column.get) == "function" then
			value = column.get(interaction, params)
		else
			local data = interaction
			if column.data then
				data = interaction[column.data]
			end
			if data then
				value = data[column.get]
			end
		end
		if column.timestamp and value then
			value = getTimestampDatetime(value)
		end
		if value == nil then
			return json.null
		end
		return value
	end

	local function makeInteractionRow(interaction, params)
		local result = {}
		for _, column in pairs(INTERACTION_COLUMNS) do
			table.insert(result, getColumnValue(column, interaction, params))
		end
		return result
	end	

	function self:get(data)
		local result = makeResultSet()
		local data_d = Array2Hash(data)
		local interaction = api.GetInteraction(data_d.Id, true)
		local params = {
			phoneFormat = utils.GetPhoneFormat()
		}
		if interaction then
			local interactionRow = makeInteractionRow(interaction, params)
			table.insert(result.rows, interactionRow)
		end
		return true, result
	end
	return self
end

