module(..., package.seeall)

local lua_cassandra = require "lua_cassandra"

local m_permissions


--[[
	Retrieves all the available permissions for all the users
	{
		role = {
			permission = 1,
			permission = 1
		}
	}
]]--
function GetPermissionList()
	local results = {}
	
	if (m_permissions) then
		return m_permissions
	end
	
	local keyspace = lua_cassandra:CassandraConnection("Agent")
	
	local permissions = keyspace:GetRangeSlices("RolePermissions",  --column family
										   "",  --supercolumn
										   "", --startkey
										   "", --endkey
												{ --predicate
												column_names = {},
												slice_range = { 
															start = "", 
															finish = "", 
															reversed = false, 
															count = 100}
												},
											500 --total keys
										) 
	if (permissions) then
		for roleId, permissionRow in permissions:Iterator() do
			local permission = {}
			for k, v in ipairs(permissionRow) do
				
				permission[v.name] = v.value
			end
			results[roleId] = permission
		end
	end
	keyspace:Close()
	m_permissions = results
	return results
end

--returns true if the specified permission is enabled for the role
function IsPermissionEnabledForRole(role, permission)
	local permissions = GetPermissionList()
	return (permissions[role] and permissions[role][permission] == '1')
end


function new(applicationName)
	local self = {}
	function self:HasPermissionTo( action)
		local role = inconcert.authUser.userGroup or "member"
		local r = IsPermissionEnabledForRole(role, action)
		return r;
	end
	function self:CheckPermission( action, domain, functionToCall)
		return function(web, ...)
			if self:HasPermissionTo(action, domain) then
				return functionToCall(web, ...)
			end
			web.status = "403"
			return "forbidden"
		end
	end
	
	return self
end
