module(..., package.seeall)
require (tweety_modules_prefix .. "application")
local config = require (tweety_modules_prefix .. "config")
local http = require("socket.http")
local ltn12 = require("ltn12")
local json = require "json"
local url = require("socket.url")
local messages = require (tweety_modules_prefix .. "messages")
local api = require (tweety_modules_prefix .. "webhandler_api")

require "md5"


local function empty(value)
	return not value or value == ""
end

local function GetBaseUrl(server, port)
	return string.format([[http://%s:%s/inconcert/apps/]], server, port)
end

local function LoginUser(server, port, username, callcenterId, password, token)
	local textToEncode = ""
	local cookie
	
	textToEncode = string.lower(username) .. password
	local hash = md5.sumhexa( md5.sumhexa(textToEncode) .. token.loginToken)
		
	local params = "hash=" .. hash .. "&username=" .. url.escape(username) .. "&sessionid=" .. token.sessionId .. "&callcenter=" .. callcenterId .. "&app=" .. url.escape("agent")
		
	local headers = {
		['Content-Type'] = 'application/x-www-form-urlencoded; charset=utf-8'
	}
	local result, response = Post(server, port, "login/login", params, headers)
	if (result) then
		local data = json.decode(response)
		cookie = "SID=".. data.authToken
		LogDebug("Logged in with cookie: "..cookie)
		return cookie
	else
		LogDebug("Error logging in: %s",response)
	end	
	return false
end
	
function Login(server, port, user, vcc, password)
	LogDebug("Logging in as %s@%s", user, vcc)
	local err
	local response, tokenResponse = Get(server, port, "login/getToken")
	if (response) then
		local response = json.decode(tokenResponse)
		return LoginUser(server, port, user, vcc, password, response)
	else
		err = "Error getting token on Login: "..tokenResponse
	end
	return nil, err
end

function Get(server, port, url, headers, cookie)
	local complete_url = GetBaseUrl(server, port) .. url
	LogInfo("COMPLETE URL %s",complete_url)
	local response_body = {}
	headers = headers or {}
	if (cookie and string.len(cookie)) then
		headers["Cookie"] = cookie
	end
	socket.http.TIMEOUT = config.sockethttp_timeout
	local b, c, h = socket.http.request{
		url = complete_url,
		method = "GET",
		headers = headers,
		sink = ltn12.sink.table(response_body)
	}
	local response = table.concat(response_body,'')
	if (c == 200) then
		return true, response
	else
		LogError ("error getting url %s  result code: %d\nbody:%s",url,c,response)
		return false, response ..  " " .. tostring(c)
	end

end

function Post(server, port, url, body, headers, cookie)
	local complete_url = GetBaseUrl(server, port) .. url
	local response_body = {}
	headers = headers or {}
	headers["content-length"] = string.len(body)
	
	if (cookie and string.len(cookie)) then
		headers["Cookie"] = cookie
	end
	socket.http.TIMEOUT = config.sockethttp_timeout
	local b, c, h = socket.http.request{
		url = complete_url,
		method = "POST",
		headers = headers,
		source = ltn12.source.string(body),
		sink = ltn12.sink.table(response_body)
	}
	local response = table.concat(response_body,'')
	if (c == 200) then
		return true, response
	else
--		error ("Error posting request, result code:"..c .. "\nbody:"..response)
		return false, response .. " " .. tostring(c)
	end
end
function toxml_string(value)
	local t = type(value)
	if t == "nil" then
		return ""
	elseif t == "string" then
		return value
	else
		return tostring(value)
	end
end

function GetSingleNodeValue(xml, path)
	local node = xml:select(path)
	if type(node) ~= "table" then
		return
	end
	if #node > 1 then
		LogError("Single node select returned more than one node %s [%s]", path,node:print())

	else
		local v = node[1]:text()
		if v == "" then
			return nil
		else
			return v
		end
	end
end
function parsePhone ( phoneNumber )
	local country,area,number,ext
	c,_,country,area,number,ext = string.find(phoneNumber,"(%d*)-(%d*)-(%d+),(%d*)")
	if c then
		return country,area,number,ext
	end
	c,_,country,number,ext= string.find(phoneNumber,"+(%d%d%d)(%d+),(%d*)")
	if c then
		return country,area,number,ext
	end

	c,_,country,area,number= string.find(phoneNumber,"(%d*)-(%d*)-(%d+)")
	if c then
		return country,area,number,ext
	end

	c,_,country,number= string.find(phoneNumber,"+(%d%d%d)(%d+)")
	if c then
		return country,area,number,ext
	end
	c,_	,number,ext= string.find(phoneNumber,"(%d+),(%d*)")
	if c then
		return country,area,number,ext
	end
	c,_,number= string.find(phoneNumber,"(%d+)")
	if c then
		return country,area,number,ext
	end
	return "","",phoneNumber,""
end
	
function recognizePhone(phoneNumber, isAllegro)
	if isAllegro then
		return recognizeAllegroPhone(phoneNumber)
	end

	local status, result
	local country, area, number
	if type(phoneNumber) == "string" then
		--country,area,number,ext = parsePhone(phoneNumber)
		number = string.gsub(phoneNumber,"[- ()]","")
	elseif type(phoneNumber) == "table" then
		country = phoneNumber.countryId
		area = phoneNumber.areaId
		number = phoneNumber.globalNumber or phoneNumber.phoneNumber
		if number and string.sub(number, 1, 1) == "+" then
			area = ""
			country = ""
		end
	end

	LogInfo("Call to recognize phone [%s] ([%s])[%s]-[%s]", json.encode(phoneNumber), country, area, number)
	
	local request = {
		country = not empty(country) and country or nil,
		area = not empty(area) and area or nil,
		number = not empty(number) and number or nil,
	}
	
	local phone = api.RecognizePhone(request)
	if not phone then
		return false, messages.messageKey('database_conection_error',{errText="error recognizing phone"})
	end

	result = {
		countryId = phone.countryCode,
		areaId = phone.areaCode,
		phoneNumber = "+" .. phone.dialGlobal,
		localNumber = phone.dialLocal,
		displayable = phone.displayable,
		timeZone = {
			Id = phone.timezone.id,
			offset = tonumber(phone.timezone.TimeZone),
			minutes = tonumber(phone.timezone.TimeZoneMinutes),
			dst = phone.timezone.DaylightSavingTime == "1",
		}
	}
	LogInfo("RecognizePhone Result %s", json.encode(result))
	return true, result
end

------------------- ALLEGRO
	
function recognizeAllegroPhone(phoneNumber)
	local status, result
	local country,area,number,ext
	if type(phoneNumber) == "string" then
		--country,area,number,ext = parsePhone(phoneNumber)
		number = string.gsub(phoneNumber,"[- ()]","")
	elseif type(phoneNumber) == "table" then
		country = phoneNumber.countryId
		area = phoneNumber.areaId
		ext = phoneNumber.phoneExtension
		number = phoneNumber.phoneNumber
		if ext and #ext > 0 then
			number = number .. "," .. ext
		end
		if number and string.sub(number,1,1) =="+" then
			area = ""
			country=""
		end
			
	end

	LogInfo("Call to recognize phone [%s] ([%s])[%s]-[%s]", phoneNumber, country, area, number)
	
	local request = {
		country = not empty(country) and country or nil,
		area = not empty(area) and area or nil,
		number = not empty(number) and number or nil,
	}
	
	local phone = api.RecognizeAllegroPhone(request)
	if not phone then
		return false, messages.messageKey('database_conection_error',{errText="error recognizing phone"})
	end

	local toboolean = function(str)
		if str == "true" or str == "1" or str == "TRUE" or str == "True" then
			return true
		else
			return false
		end
	end
	
	result = {
		countryId = phone.countryId,
		areaId = phone.areaId,
		phoneNumber = phone.number,
		phoneExtension = phone.extension,
		ZIP = phone.ZIP,
		status = phone.status,
		phoneType = phone.type,
		importName = phone.importName,
		timeZone = {
			Id = phone.timezone.Id,
			Description = phone.timezone.Description,
			offset = phone.timezone.TimeZone or 0,
			minutes = phone.timezone.TimeZoneMinutes or 0,
			dst = phone.timezone.DaylightSavingTime == 1,
		}
	}
	LogInfo("RecognizePhone Result " ..
			"\ncountryId=%s" ..
			"\nareaId=%s" ..
			"\nphoneNumber = %s" ..
			"\nphoneExtension = %s" ..
			"\nZIP = %s" ..
			"\nstatus = %s" ..
			"\nphoneType = %s" ..
			"\nimportName = %s" ..
			"\nTimeone data" ..
			"\n\tId = %s" ..
			"\n\tDescription = %s" ..
			"\n\toffset = %s" ..
			"\n\tminutes = %s" ..
			"\n\tdst = %s",
			tostring(result.countryId),
			tostring(result.areaId),
			tostring(result.phoneNumber),
			tostring(result.phoneExtension),
			tostring(result.ZIP),
			tostring(result.status),
			tostring(result.phoneType),
			tostring(result.importName),
			tostring(result.timeZone.Id),
			tostring(result.timeZone.Description),
			tostring(result.timeZone.offset),
			tostring(result.timeZone.minutes),
			tostring(result.timeZone.dst)
	)
	return true, result
end

------------------- END ALLEGRO
