module(..., package.seeall)
require (tweety_modules_prefix .. "sqlerror")
local runtime = require (tweety_modules_prefix .. "runtime")
local config = require (tweety_modules_prefix .. "config")
local messages = require (tweety_modules_prefix .. "messages")
--print("cargando mongods");
--local uuid = require "uuid";
local dsbase = require (tweety_modules_prefix .. "datasourceBase")

--local mongo = require "luamongo"
mongo = {
	Connection = {
		New = function()
			return {
				connect=function()
				end,
				get_indexes=function()
					return {has_more=function() end }
				end,
				ensure_index=function()
					return true
				end
			}
		end
	}
}
local md5 = require "md5";
local csvutils = require (tweety_modules_prefix .. "csvutils");
local dbconn


local lwuid_count = 0;
local lwuid_pid = string.gsub(tostring({}),"table: ","")
local lwuid_mcid = string.sub(md5.sumhexa(os.getenv("COMPUTERNAME") or os.getenv("HOSTNAME") or tostring(os.time())),1,8);
math.randomseed( os.time() )
function lwuid ()
	lwuid_count=lwuid_count+1
	if lwuid_count == 1e4 then
		lwuid_count=0
	end
	local res = {
		tostring(os.time()),
		string.format("%05d",lwuid_count),
		lwuid_mcid,
		lwuid_pid,
		string.format("%05d",math.random(1e4)),

	}
	return table.concat(res,"-")
end



function new(self)
	self = dsbase.datasourceBase(self);
	local function initialize()
		local status,result
		if not dbconn then
			dbconn = mongo.Connection.New();
			LogDebug ("Connecting to mongodb at %s",config.mongodb_connection_string);
			status,result = pcall(dbconn.connect,dbconn,config.mongodb_connection_string);
			if not status then
				LogDebug ( "Failed %s",mongodb_connection_string);
				dbconn = nil; -- gc collected
				return false,messages.messageKey('database_conection_error',{description=result})
			end
		end
		return true;
	end
	local function nativeError(err)
		if err.code == 11000 then	-- este tiene informacion de que indice fue violado hay que obtener info de indice para saber que campo
			return messages.messageKey('duplicated_key',{nativeError=err});
		elseif err.code == 11001 then	-- en este mongo no da informacion de que indice fallo.
			return messages.messageKey('duplicated_key',{nativeError=err});
		else
			return messages.messageKey('unknown',{nativeError=err});
		end
	end
	local function formatValue(value,paramInfo)
--		LogInfo("value %s type %s name %s ",value,paramInfo.type,paramInfo.name);
		if value ~= nil then
			if 	paramInfo.type == "integer" or
				paramInfo.type == "numeric" then
					return tonumber(value)
			elseif paramInfo.type == "string" or
					paramInfo.type == "datetime" then
					return tostring(value)
			elseif	paramInfo.type == "date" then
					return string.sub(tostring(value),1,10)
			else
					return value
			end
		end
	end
	local function setupRowId(reg)
		reg._id = lwuid();
		return reg._id;
	end
	
	local function getNamespace()
		if config.mongodb_one_db_by_table then
			return config.mongodb_namespace_prefix .. (self.inconcert_authUser or {virtualContactCenter=self.vccName}).virtualContactCenter  .. "_" .. self.description.tableName .. ".data";

		else
			return config.mongodb_namespace_prefix .. (self.inconcert_authUser or {virtualContactCenter=self.vccName}).virtualContactCenter .. "." .. self.description.tableName

		end
	end


	function self:update(data)
		local status,result = initialize();
		if status then
			local hdata={}
			for _,param in ipairs(data) do
				hdata[param.name]=param;
			end
			if hdata._id and hdata._id.value ~= nil then
				local regq = {_id=hdata._id.value}
				local reg={}
				local p;
				local cursor,err;
				-- formamos un registro con que hacer el query

				for _,param in ipairs(self.description.update.inputParams) do
					-- ALGO PASA QUE EL QUERY COMPLETO NO COINCIDE APARENTEMENTE EN LOS DATOS CON TILDES
					if param.name ~= "_id" then
--						p = hdata[param.name]
--						if p then
--							regq[param.name]=formatValue(p.origvalue,param);
--						end
					end
				end

				--primero obtenemos el registro

				cursor,result = dbconn:query(getNamespace(),regq,1)
				if cursor then
					if cursor:has_more() then
						reg = cursor:next();
						for _,param in ipairs(self.description.update.inputParams) do
							p = hdata[param.name]
							if p and p.value ~= p.origvalue then
								reg[param.name]=formatValue(p.value,param);
							end


						end

						status,result = dbconn:update(getNamespace(),regq,reg);

						if not status then
							return false,messages.messageKey('database_conection_error')
						end
						local lastError = dbconn:get_last_error();
						if lastError.code then
							return false,nativeError(lastError);
						end

						if lastError.n == 1 then
							return true;
						else
							return false,messages.messageKey('unknown_error',{query=regq,data=reg})
						end
					else
						return false,messages.messageKey('data_syncing_error',{query=regq})
					end
				else
					return false,messages.messageKey('database_conection_error',{description=result})
				end
			else
				return false,messages.messageKey('missing_keys')
			end
		end
		return status,result
	end

	function self:truncate(data)
		local status,result = initialize();
		if status then
			status,result = dbconn:remove(getNamespace(),{});
			if not status then
				return false,messages.messageKey('database_conection_error',{description=result})
			end
		end
		return status,result
	end

	function self:delete(data)
		local status,result = initialize();

		if status then
			local hdata={}
			for _,param in ipairs(data) do
				hdata[param.name]=param;
			end
			if hdata._id and hdata._id.value ~= nil and hdata._id.value ~= "" then
				status,result = dbconn:remove(getNamespace(),{_id=hdata._id.value});
				if not status then
					return false,messages.messageKey('database_conection_error',{description=result})
				end
				return true;
			else
				return false,messages.messageKey('missing_keys')
			end
		end
		return status,result
	end
	local function build_filter_order(name)
		local filter
		local t = {}
		for _,fl in ipairs(self.description.filters) do
			if fl.name == name then
				filter = fl
				break
			end
		end
		if filter then
			for _,ordercolumn in ipairs(filter.order) do
				if ordercolumn.column and ordercolumn.order then
					local s={}
					local dir

					if ordercolumn.order == "ASC" then
						dir=1
					else
						dir=-1
					end
					s[ordercolumn.column]=dir
					table.insert(t,s)
				end
			end

		end
		if #t==0 then
			table.insert(t,{_id=1})
		end
		local j={}
		for _,e in ipairs(t) do
			for k,v in pairs(e) do
				table.insert(j,json.encode(k) .. ":" .. json.encode(v))
			end
		end
		local t = {}
		mt = getmetatable(t) or {}
		mt.__toJson = function(self)
			return "{" .. table.concat(j,",") .. "}"		
		end
		setmetatable(t,mt)
		return t;
	end
	local parse2bson
	parse2bson = function(t)
		local andexpr={}
		local orexpr={}
		local mapop={}
		local operators = {
			['>']='$gt',
			['>=']='$gte',
			['<']='$lt',
			['<=']='$lte',
		}

		if t._type == "op" then
			local o = operators[t.optype]
			if t.optype == "=" then

				if andexpr[parse2bson(t.left)] then
					error("grammar error 20 for field " .. parse2bson(t.left))
				end
				andexpr[parse2bson(t.left)]=parse2bson(t.right)
			elseif o then
				local column = parse2bson(t.left)

				local tp = type(andexpr[column])
				if tp == "nil" then
					andexpr[column] = {}
				elseif tp ~= "table" then
					error("grammar error 21 for field " .. column)
				end
				if andexpr[column][o] then
					error("grammar  error 22 for field " .. column)
				end
				andexpr[column][o]=parse2bson(t.right)
			elseif t.optype == "like" then

				if andexpr[parse2bson(t.left)] then
					error("grammar error 23 for field " .. parse2bson(t.left))
				end
				local likeExpr = parse2bson(t.right)

				if string.sub(likeExpr,0,1) == "%" and string.sub(likeExpr,#likeExpr)== "%" then
					andexpr[parse2bson(t.left)]={['$regex']=string.sub(likeExpr,2,#likeExpr-1),['$options']="i"};
				elseif string.sub(likeExpr,#likeExpr) == "%" then
					andexpr[parse2bson(t.left)]={['$regex']="^" .. string.sub(likeExpr,2,#likeExpr-1),['$options']="i"};
				end

			elseif t.optype == "and" then
				for k,v in pairs(parse2bson(t.left)) do
					if type(v) == "table" and type(andexpr[k]) == "table" then
						for j,i in pairs(v) do
							andexpr[k][j]=i
						end
					else
						andexpr[k]=v
					end

				end
				for k,v in pairs(parse2bson(t.right)) do
					if type(v) == "table" and type(andexpr[k]) == "table" then
						for j,i in pairs(v) do
							andexpr[k][j]=i
						end
					else
						andexpr[k]=v
					end

				end

			elseif t.optype == "or" then
				table.insert(orexpr,parse2bson(t.left))
				table.insert(orexpr,parse2bson(t.right))
			end
		elseif t._type == "const" then
			return t.value
		elseif t._type == "keyname" then
			return t.value
		elseif t._type == "number" then
			return tonumber(t.value)
		elseif t._type == "param" then
			return t.value
		elseif t._type == "like" then
			return t.value
		end

		if #orexpr == 0 then
			result = andexpr
		else
			local c=0
			for k,v in pairs(andexpr) do
				c=c+1
			end
			if c> 0 then
				table.insert(orexpr,andexpr);
			end
			local t = {['$or']=orexpr}
			result = t
		end

		return result;
	end
	function self:filter_criteria_string(filterName,params)

		local function filterTostring ( filter)
			local operators = {
				['>']=' > ',
				['>=']=' >= ',
				['<']=' < ',
				['<=']=' <= ',
				['==']=' = ',
				['!=']=' <> ',
				['%']=' LIKE '
			}
			local t = {}
			for idx,segment in ipairs(filter.criteria) do
				table.insert(t,segment.open)
				op = operators[segment.comparator] or segment.comparator;
				table.insert(t, '\"' .. segment.column  ..  '\"' .. op  .. tostring(segment.expr) )
				table.insert(t,segment.close);
				if idx < #filter.criteria then
					table.insert(t,segment.combinator)
				end
			end
			return table.concat(t," ");

		end
		local filter = self:get_filter(filterName,params,function(param,value,likeMode)

			if value == false or value == true then
				value = tostring(value)
			end
			if param.type == "string" then
				if likeMode then
					value = "'%" .. string.gsub(tostring(value), "'", "''") .. "%'"
				else
					value = "'" .. string.gsub(tostring(value), "'", "''") .. "'"
				end

			elseif param.type == "datetime"  then
				if likeMode then
					value = "'%" .. string.gsub(tostring(value), "'", "''") .. "%'"
				else
					value = "'" .. string.gsub(tostring(value), "'", "''") .. "'"
				end
			elseif param.type == "date" then
				if likeMode then
					value = "'%" .. string.gsub(string.sub(tostring(value),1,10), "'", "''") .. "%'"
				else
					value = "'" .. string.gsub(string.sub(tostring(value),1,10), "'", "''") .. "'"
				end

			elseif param.type == "integer" or param.type == "numeric" then
				if likeMode then
					value = "'%" .. string.gsub(tostring(value), "'", "''") .. "%'"
				else
					value = tonumber(value)
				end
			end
			return value

		end)
		return filterTostring(filter);
	end

	local function build_filter_criteria(filter_ref,params)
		local r


		local query =self:filter_criteria_string(filter_ref,params)
		if query then
			if query ~= "" then
				res,query = self:parse_query(query)
				if res then
					r,query = pcall(parse2bson,query)
					if r then
						return query
					end
				end
			else
				return {}
			end
		end
	end

	function self:cursor(queryData)
		local error_iterator = {
			eof = function()
				return true
			end
		}
		local status,result = initialize();
		local cursor,err
		local retrieve_keys -- estas son las columnas que quiero traer para optimizar y no traer todo lo que encuentre
		local query = {}
		LogDebug("mongodbs CURSOR %s",json.encode(queryData,true));
		if queryData then
--			query = mongo.Query.New();

			if queryData.orderby then
				local t = {}
				for _,col in ipairs(queryData.orderby) do
					local s = {}
					if col.order and col.column then
						if col.order == "ASC" then
							s[col.column]= 1
						else
							s[col.column]= -1
						end
						table.insert(t,s)
					end
					
				end
				local j={}
				for _,e in ipairs(t) do
					for k,v in pairs(e) do
						table.insert(j,json.encode(k) .. ":" .. json.encode(v))
					end
				end
				t = {}
				mt = getmetatable(t) or {}
				mt.__toJson = function(self)
					return "{" .. table.concat(j,",") .. "}"		
				end
				setmetatable(t,mt)
				query['$orderby']=  t
			end
			filter = build_filter_criteria(queryData.filter,queryData.parameters or {})
			if not filter then
				return error_iterator
			end
			query['$query']= filter;

		end
		if status then
			--LogDebug("namespace " .. getNamespace())
			LogDebug("%s CURSOR QUERY %s",self.description.name,json.encode(query));
			cursor,err = dbconn:query(getNamespace(),json.encode(query),0,0);
			if not cursor then
				return false,messages.messageKey("database_error");
			end
			return true,{
				eof = function()
					return not cursor:has_more();
				end,
				next = function()
					return cursor:next()
				end
			}
		else
			return false,error_iterator
		end
	end
	function self:get(data,context)
		local ts = os.clock();
		local tbinsert = table.insert;
		local status,result = initialize();
		local err
		if status then
			local filter={};
			local order
			local retrieve_keys={}
			local reg;
			local hdata={}
			for _,param in ipairs(data) do
				hdata[param.name]=param;
			end
			local colnames = {};
			--local c = 0
			local resulset = {colnames={"_id"},coltypes={},rows={}};
			local res={};
			for _,column in ipairs(self.description.columns)  do
				table.insert(resulset.colnames,column.name);
				retrieve_keys[column.name]=1;
			end
			retrieve_keys._id = 1;
			filter = build_filter_criteria(context.filter,data);
			if not filter then
				return false,messages.messageKey('unknown_error',{description='Incorrect filter expresion'})
			end
			order = build_filter_order(context.filter);
			--LogDebug("QUERY [%s]",json.encode(filter));
			context.limit = context.limit or 200
			if context.limit == -1 then
			   context.limit=0
			end
			LogDebug("GET QUERY [%s]",json.encode({['$query']=filter,['$orderby']=order}));
			cursor,err = dbconn:query(getNamespace(),json.encode({['$query']=filter,['$orderby']=order}),context.limit,0,retrieve_keys);
			if not cursor then
				return false,messages.messageKey('database_conection_error',{description=err})
			end
			
			-- si el contexto trae countRecords es porque se trata de un conteo nada mas
			resulset.count = 0;
			if not context.countRecords then

				while cursor:has_more() do

					local row = {}
					reg = cursor:next();
					for _,colname in ipairs(resulset.colnames) do
						local v = reg[colname]
						if v == nil then v = json.null end
						tbinsert(row,v);
					end
					tbinsert(resulset.rows,row);

				end

				resulset.count = #resulset.rows;
			else
				resulset.count = cursor:itcount();
			end
			return true,resulset
		end
		return status,result
	end

	function self:configure(dsdata)
		local cursor,err,nsindexes,inx,c
		local status,result = initialize();
		local hindex = {}
		local addIndexes = {}
		local dropIndexes = {}
		local function addIndex(inx)
			local key = {}
			local dir
			local empty = true
			for _,column in ipairs(inx.columns) do
				if column.column then

					if column.order == "ASC" then
						dir = 1
					else
						dir = -1
					end
					key[column.column]=dir
					empty = false;
				end
			end
			if not empty then

				table.insert(addIndexes,{name=inx.name,unique=inx.unique,key=key,dropDups=inx.unique})
			end
		end
		if status then

			cursor,err = dbconn:get_indexes(getNamespace());
			if not cursor then
				return false,messages.messageKey('database_conection_error',{description=err})
			end
			--print(cursor:itcount());
			while cursor:has_more() do
				inx = cursor:next();
				hindex[inx.name]=inx;
			end
			for _,inx in ipairs(dsdata.description.indexes) do
				local n = hindex[inx.name]
				if n then -- existe el indice
					if (n.unique==1 and inx.unique) or ((n.unique==0 or not n.unique) and not inx.unique) then

						local keys = {}					-- copiamos las claves del indice a algo temporal
						for k,v in pairs(n.key) do
							keys[k] = v
						end

						for _,column in ipairs(inx.columns) do
							if column.column and keys[column.column] and ((column.order == "ASC" and keys[column.column]==1) or (column.order == "DESC" and keys[column.column]==-1)) then
								keys[column.column]= nil
							else				-- la clave no coincide, no existe o su orden es diferente
								table.insert(dropIndexes,n)
								addIndex(inx);
								break;
							end
						end
						c=0
						for _,key in pairs(keys) do
							c=c+1
						end
						if c>0 then		-- tiene mas claves que lo que se necesita , lo regeneramos
							table.insert(dropIndexes,n)
							addIndex(inx);
							break;
						end
						hindex[inx.name]=nil; -- procesado, lo quitamos, el que quede hay que borrarlo de la bd a menos que sea el _id
					else	-- no existe hay que crearlo
						addIndex(inx)
					end
				else	-- no coincide el unique flag , regeneramos
					table.insert(dropIndexes,n)
					addIndex(inx);
				end
			end

			for _,inx in pairs(hindex) do	-- todos los indices creados que quedaron y no son _id se borran
				if inx.name ~= "_id_" then

					table.insert(dropIndexes,inx)
				end
			end
			-- recorremos dropIndexes y borramos los indices
			for _,inx in ipairs(dropIndexes) do
				--print("DROP INDEX ",inx.name);
				status,err = dbconn:drop_index_by_name(getNamespace(),inx.name)
				if not status then
					return false,messages.messageKey('database_conection_error',{description=err})
				end
			end
			--
			for _,inx in ipairs(addIndexes) do
				local dropDups
				if inx.unique then
					dropDrups=true
				end
				--print("BUILD INDEX ",inx.name);
				status,err = dbconn:ensure_index(getNamespace(),inx.key,inx.unique,inx.name,true,true);
				if not status then
					return false,messages.messageKey('database_conection_error',{description=err})
				end

			end

		end
		return status,result
	end

	function self:save(data)
		local status,result = initialize();

		if status then
			local hdata={}
			local reg = {}
			local p;
			for _,param in ipairs(data) do
				hdata[param.name]=param;
			end
			for _,param in ipairs(self.description.store.inputParams) do
				p = hdata[param.name]
				if p then
					reg[p.name]=formatValue(p.value,param);
				end
			end
			local _id = setupRowId(reg);

			if dbconn:insert(getNamespace(),reg) then
				result = dbconn:get_last_error();
				if result.code then
					return false,nativeError(result);
				end
				if hdata._rowid then
					return true,{colnames={'_rowid','_id'},rows={{hdata._rowid.value,_id}}}
				else
					return true,{colnames={'_id'},rows={{_id}}}
				end
			else
				return false,messages.messageKey('database_conection_error')
			end

		end
		return status,result
	end
	return self
end
if not inconcert then
  return {
    new = new
  }
end
