module(..., package.seeall)
local runtime = require (tweety_modules_prefix .. "runtime")
local config = require (tweety_modules_prefix .. "config")
local messages = require (tweety_modules_prefix .. "messages")
local dsbase = require (tweety_modules_prefix .. "datasourceBase")
local md5 = require "md5";
local cassandra = require "lua_cassandra"
local reporter  = require "reporter"
local manage = require "reporter.manage"

local ir = require "lua_iragentlink"
local summarizerKS;
local irLink;
function new(self)
	self = dsbase.datasourceBase(self);
	local function Array2Hash(data)
		local hash = {}
		for _,argument in ipairs(data) do
			hash[argument.name] = argument.value -- or argument.origvalue
		end
		return hash
	end
	local initialized=false
	local function initialize()
		LogDebug("SUMMARIZER DS Initialize CLUSTER %s",config.cluster);
		summarizerKS = cassandra:CassandraConnection("SummarizerData")
		irLink = ir.IRAgentLink(config.cluster or "inconcert")
		return true;

	end

	function self:delete(data)
		local status,result = initialize();
		if status then
			local hdata={}
			for _,param in ipairs(data) do
				hdata[param.name]=param;
			end
		end
		return status,result
	end
	function self:get(data,context)
		local tbinsert = table.insert;
		local status,result = initialize();
		if status then
			local resulset = {colnames={},coltypes={},rows={}};
			return true,resulset
		end
		return status,result
	end
	function self:deconfigure()
		local status,result
		local status,result = initialize();
		if not status then
			return status,result
		end

		local vccName = runtime.getCurrentVCCName();
		-- borra todas las dimensiones que tiene
		for _,param in ipairs(self.description.columns) do
			if not param.hidden or param.name == "ID" or param.name == "timeStamp"  then
				LogDebug("deleteDimension %s, %s",vccName,self.appName .. "." .. self.name .. "." .. param.name);
				manage.deleteDimension(summarizerKS,vccName,self.appName .. "." .. self.name .. "." .. param.name);
			end
		end
		return true,nil
	end

	function self:configure(dsdata)

		local status,result = initialize();
		if status then
			local vccName = runtime.getCurrentVCCName();
			-- borra todas las dimensiones que tiene
			for _,param in ipairs(self.description.columns) do
				if not param.hidden or param.name == "ID" or param.name == "timeStamp" then
					LogDebug("deleteDimension %s, %s",vccName,self.appName .. "." .. self.name .. "." .. param.name);
					manage.deleteDimension(summarizerKS,vccName,self.appName .. "." .. self.name .. "." .. param.name);
				end
			end
			
			for _,param in ipairs(dsdata.description.columns) do
				if not param.hidden then
					local dimension = {
						scope=string.gsub(dsdata.description.dataType or "dscci","^ds","",1),
						type=param.type=="integer" and "number" or param.type,
						name=param.name,
						projectId=self.appName,
						requiredFieldName=string.gsub(dsdata.description.dataType or "dscci","^ds","",1) .. '.__SID',
						requiredFieldValue=self.appName .. "_" .. dsdata.name,
						sourceName=dsdata.name
					};
					LogDebug("setDimension %s, %s,%s",vccName,self.appName .. "." .. self.name .. "." .. param.name,json.encode(dimension,true));
					manage.setDimension(summarizerKS, vccName,self.appName .. "." .. self.name .. "." .. param.name,dimension);
				end
			end
		end
		return status,result
	end

	function self:save(data)
		local status,result = initialize();
		if status then

			for _,param in ipairs(data) do
				if param.name == "VCC" or param.name == "VCC@VCC" then
					param.value = (self.inconcert_authUser or {virtualContactCenter=self.vccName}).virtualContactCenter;
				end
			end
		
			-- Solo los parametros que correponden a columnas llegan al summarizer
			local hcolumns = {}
			for _,col in ipairs(self.description.columns) do
				hcolumns[col.name]=col
			end
		
			local fields={}
			local d = Array2Hash(data);
			local ID = d.ID;
			local timeStamp = d.timeStamp;
			d.ID=nil;
			d.timeStamp = nil;
			for name,value in pairs(d) do
				if hcolumns[name] then
					local colType = hcolumns[name].type
					local searchable = hcolumns[name].searchable
					table.insert(fields,{name=name,value=value,type=colType,searchable=searchable})
					data[name]=value;
				end
			end
			for name,col in pairs(hcolumns) do
				if col.type == "boolean" and data[name] == nil and not col.hidden then
					data[name]=false
					local searchable = hcolumns[name].searchable
					table.insert(fields,{name=name,value=false,type=col.type,searchable=searchable})
				end
			end
			
			if ID and timeStamp then
				local data={}
				data.data={};
				data.data[self.description.dataType or "cci"]={
					__SID = self.appName .. "_" .. self.name;
					fields=fields
				}
				status,result = pcall(
					function()
						LogDebug("call SetDataForReporting ID=%s dataType=%s data=%s",ID,self.description.dataType or "cci",json.encode(data))
						local result = irLink:SetDataForReporting(ID,self.description.dataType or "cci",data);
						if result.result ~= "OK" then
							error(result.reason)
						end
					end
				);
				if not status then
					return status,messages.messageKey('database_error',{description=result})
				end
			else
				return false,messages.messageKey('missing_keys')
			end
		end

		return status,result
	end

	function self:update(data)
		return self:save(data)
	end

	return self
end
if not inconcert then
  return {
    new = new
  }
end
