require "orbit"
require "inconcert.orbit.application"
module(..., package.seeall, orbit.new, inconcert.orbit.application.Decorate)

local cassandra = require "lua_cassandra"
local ir = require "lua_iragentlink"
local reporter  = require "reporter"
local authFilter;
local i18nlib;
local htmlRender;
local config = relativeRequire("config",...)
local utils = relativeRequire("utils",...)
local m_helpers = require "helpers"

if config.inConcert6 then
	require("inconcert.translators")
	authFilter = (require "authfilter")("webdesigner")
	i18nlib = require "i18nlib"
	htmlRender = require"inconcert.render".new("webdesigner", {})
end

local m_appLocation = select(2, ...)
local model;
assert(m_appLocation, "should be running under inconcertWebHandler")
module(..., package.seeall)
local args = {...}

local configControl

-- App  control table for the appReadyFilter
-- It validates the configuration received from the config server
app_status = {
	actualStatus = "waiting",

	CheckDependencies = function (newConfig, checkList)
		for _, dep in ipairs (checkList) do
			if newConfig[dep] == nil or newConfig:DiffState(dep) == "deleted" then
				LogError("Dependency '%s' is not met", dep)
				app_status.actualStatus = "invalid"
				error("Unmet dependency "..dep)
			end
		end
	end,

	--Actually this is the only function called from the appReadyFilter
	CheckStatus = function ()
		local config = require (tweety_modules_prefix .. "config")
		local hasChanged, newConfig = configControl:ConfigurationHasChanged()
		if not hasChanged and app_status.actualStatus ~= "invalid" then
			return app_status.actualStatus
		end

		newConfig = newConfig or app_status.current
		app_status.CheckDependencies(newConfig, {"irLinks","cassandra","redis","repository"})

		app_status.current = newConfig

		--Configure IR links
		if newConfig:DiffState("irLinks") == "updated" or newConfig:DiffState("irLinks") == "added" then
			local links = {}
			for key, value, state in newConfig["irLinks"]:childs() do
				if state ~= "deleted" then
					table.insert(links, value)
				end
			end
			LogInfo("Adding new webdesigner links for cluster %s", inconcert.environment.apps.webdesigner.cluster)
			local irLink = ir.IRAgentLink(inconcert.environment.apps.webdesigner.cluster)
			LogInfo("irLinks %s", json.encode(links, true))
			irLink:DiscoverIRs(links)
		end

		--Configure Cassandra
		for key, value, state in newConfig["cassandra"]:childs() do
			if state ~= "deleted" then
				LogInfo("Connecting to cassandra on %s:%s", value.host, tostring(value.port))
				cassandra.seed(value.host, value.port)
				reporter.ConnectCassandra(value.host, value.port)
				reporter.SetGlobalTimer(inconcert.environment.apps.webdesigner.cluster)
			end
		end
		-- Configure redis
		config.redis_host = newConfig["redis"]["host"]
		config.redis_port = tonumber(newConfig["redis"]["port"])
		LogInfo("CONFIG REDIS %s %s",tostring(config.redis_host),tostring(config.redis_port));

		--config.repository_host = newConfig["repository"]["ip"]
		--config.repository_port = tonumber(newConfig["repository"]["port"])
		config.repository = {}
		for key, value, state in newConfig["repository"]:childs() do
			if state ~= "deleted" then
				table.insert(config.repository,{host=value.ip,port=value.port});
			end
		end
		for k,v in ipairs(config.repository) do
			LogInfo("CONFIG REPOSITORY %s %s",tostring(v.host),tostring(v.port));
		end

		--LogInfo("CONFIG REPOSITORY %s %s",tostring(config.repository_host),tostring(config.repository_port));
		
		-- Mysql config
		config.mysql = newConfig["mysql"]

		app_status.actualStatus = "ready"
		return "ready"
	end
}

local function ReplyJSON(web, data)
	assert(type(data) == "table", "Parameter data needs to be a table")
	web.headers["Content-Type"] = "application/json"
	return json.encode(data)
end

local function Authenticate(web)
	local username = web.input.username
	local vcc = web.input.callcenter
	local password = web.input.pwd
	
	if not(username and vcc and password) then
		return ReplyJSON(web, {
			status = false,
			message = "invalid request",
		})
	end
	
	local options = {password = password, web = web}
	if vcc:match("%.") then
		--it's a mail address
		options.mail = username.."@"..vcc
	else
		options.username = username
		options.vcc = vcc
	end

	local result, authToken = authFilter:AuthenticateUser(options)
	if result then
		local authUser = authFilter:GetUserFromToken(authToken)
		InitializeSession(authUser)

		return ReplyJSON(web, {
			status = true,
			message = "Login succeeded",
			authToken = authToken
		})
	else
		return ReplyJSON(web, {
			status = false,
			message = authToken, --authToken is the error message
		})
	end
end

--[[
	Recorre todos los vcc tratando de encontrar la aplicacion 
	cuando la encuentra obtiene las credenciales para toda la aplicacion de la propiedad _wholeapp.
	Esta puede ser remplazada por credenciales asignadas mediante el nombre del formulario.
	En caso de haber argumentos en el objeto _matcharguments ellos se comparan uno a uno 
	con los pasados en la url y si uno de ellos coincide se aceptan las credenciales.
	Si no hay entradas en _matcharguments se aceptan las credeciales obtenidas.
	
		_wholeapp = {userName = 'admin'},
		
		formConsultas = {userName='pepeconsulta'}
		
		_matcharguments ={
			token=093023904239042903402390
		}
	
	Retorna la entrada con las credenciales obtenidas a las que le adiciona el virtualContactCenter
	y un token guid.
	
--]]
function CustomCredentialFn (wsapi_env)
	local url = wsapi_env.PATH_INFO;
	local _,c = string.gsub(url,"/",function() end) -- esto me llama hasta cuando ya tengo la cookie si viene con mas de una / no le damos bola
													-- ya que es porque ya vengo caminando luego de logonearme buscando imagenes etc
	if c == 1 then
		LogDebug("[INIT.CustomCredentialFn] %s q=%s", url,wsapi_env.QUERY_STRING)

		local stringUrlParameters
		--[[
			callback que invoca la rocka helpers por cada parametro parseado.
				De momento se ha utilizado unicamente cuando el parametro es uuid en las urls de survey;
				cuando llega este parametro, se reemplazan todos los parametros por los que se han almacenado en redis.
				Es el caso de los short parameters en la url.
		]]--
		local function TransformValuesFn (name, value)
			if name:lower() == "uuid" then
				local redis = utils.GetRedisClient()
				-- Invoca a la funcion de la rocka helpers. El valor que se encuentra bajo esta key lo ha invocado el IR utilizando la misma rocka. (MakeShortURL)
				stringUrlParameters = m_helpers.request.GetLongParamsUrlById(value, redis)
			end
			return name, value
		end

		local args = utils.ParseQueryString(wsapi_env.QUERY_STRING or "", TransformValuesFn)
		if type(stringUrlParameters) == "string" then
			-- debo procesar todos los parametros nuevamente, ya que los mismos han sido reemplazados por completo (caso de survey -> short parameters en la url)
			wsapi_env.QUERY_STRING = stringUrlParameters
			args = utils.ParseQueryString(wsapi_env.QUERY_STRING or "")
		end

		local function unescape (s)
			s = string.gsub(s, "+", " ")
			s = string.gsub(s, "%%(%x%x)", function (h)
				return string.char(tonumber(h, 16))
			  end)
			return s
		end
		local function parseUrl(url)
			url = unescape(url or ""):gsub('"', ""):gsub("'", ""):gsub("<", ""):gsub(">", "")
			local m_appName,m_formName
			string.gsub(url,"/([^/%.]+)",function(appName)
				m_appName = appName
			end)
			string.gsub(url,"/([^/%.]+)%.([^/]+)",function(appName,formName)
				m_appName = appName
				m_formName = formName
			end)
			return m_appName,m_formName
		end
		local appName,formName = parseUrl(url);
		LogDebug("AppName = %s FormName = %s",appName,formName);
		local tweetyio = require(tweety_modules_prefix .. "tweetyio");
		local iter, dir_obj = tweetyio.dir(config.usersPath)
		local dir_entry = iter(dir_obj)

		while dir_entry do
			if dir_entry == "." then
				dir_entry = "system"
			end
			if dir_entry ~= "." and dir_entry ~= ".." then
				local credentials
				LogDebug("Scanning vcc %s",dir_entry);
				local app = application.new{hostVCC = dir_entry}
				if appName then
					status,result = app:initialize(appName)
					if status then
						LogDebug("App %s founded",appName);
						local authfilter = app:getSettings().authfilter or {}
						local matcharguments = authfilter._matcharguments or {}
						credentials = authfilter._wholeapp;
						if formName then
							credentials = credentials or authfilter[formName]
						end
						if credentials then
							local rejected = false
							if #matcharguments > 0 then
								rejected=true
								for _,arg in ipairs(matcharguments) do
									if args[arg.name] == arg.value then
										rejected=false
										break
									end
								end
							end
							if not rejected then
								credentials.virtualContactCenter = dir_entry;
								LogDebug("---> Retorna VCC %s ",credentials.virtualContactCenter)
								credentials.timezone = {
									id="UTC",
									offset=0
								}
								return credentials, "token"
							end
						end
					end
				end
			end
			dir_entry = iter(dir_obj)
		end
	end
end

--Login APIs
function RenderLoginFormWSAPI(wsapi_env)
	--magic! invoke 'RenderLoginForm' as if the request came through Orbit
	local web = orbit.make_web_object(_M, wsapi_env)
	return web.status, web.headers, RenderLoginForm(web)
end

---
-- Shows the 'login' form
--
function RenderLoginForm (web)
	local env = {}
	env.required_login_script = "/inconcert/lib/webtoolkit/webtoolkit.base64.js"
	-- Last Used Language ID or english by default
	local languageCode = utils.DetectLanguage(web)
	local translator = i18nlib.Translator{domain = "login", language = languageCode, codepage="UTF-8", transliterate = false}
	env._T = function(t)
		if type(t) == "table" then
			return translator:Text(unpack(t))
		else
			return translator:Text(t)
		end
	end
	local fh = io.open(m_appLocation.."/pages/login.html");
	if not fh then
		-- TODO: generar alguna cosa para mostrar este error
		-- y salir
	end
	template = fh:read("*a");
	io.close(fh);
	return cosmo.fill(template,{})
end

local function Logout(web)
	if inconcert.authUser then
		FinalizeSession()
		authFilter:Logout(web)
	end
	return ""
end

function Initialize(appcfg)
	LogInfo("Starting application in " .. m_appLocation)


	config.initialize(appcfg or {},m_appLocation);

	configControl = LuaConfigurationControl.Client {
		type = "WEBDESIGNER",
		address = inconcert.environment.apps.webdesigner.config_server.ip,
		port = inconcert.environment.apps.webdesigner.config_server.port,
		node_address = inconcert.environment.apps.webdesigner.config_server.myIp,
		node_port = inconcert.environment.apps.webdesigner.config_server.myPort,
	}
	configControl:AddServiceDependency("InteractionRouter")
	configControl:AddServiceDependency("Repository")

	configControl:TriggerNodeReady()
--[[
	inconcert = inconcert or {}
	inconcert.authUser = inconcert.authUser or {
		timezone={offset=0},
		userName='default',
		userGroup='default',
		virtualContactCenter = 'system'
	};
]]

	_G['tweety_modules_prefix'] = config.tweety_modules_prefix;
	_G['tweety_permisions_actionId'] =  config.tweety_permisions_actionId
	_G['tweety_folder']=  config.tweety_folder;

	local ok,permisionModule,designerAppPermission

	if config.checkPermisions then
		if not config.inConcert6 then
			_M.model = nil
		--- Requires Genericos de la aplicacion ---
			model = require (_G['tweety_modules_prefix'] .. "model")
		-- se necesitan los permisos de la aplicaci�n 'designer'
			ok, permissionModule = pcall(require, "shared.permissions")
		else
			ok,permissionModule = pcall (require,_G['tweety_modules_prefix'] .. "icc6_permissions")
		end
	end
	--------------------------------------------------------------
	--si el m�dulo shared.permission existe lo cargo, 
	--	sino emulo el objeto que retorna �ste m�dulo, pero la validaci�n no se realiza.
	if ok then
		LogInfo("Designer permissions loading.")
		designerAppPermission = permissionModule.new("designer")
		if model then
			model:BuildModels()
		end
	else
		LogWarning("Designer 'permission' has not been loading because there is no module permits.")
		designerAppPermission = {
			HasPermissionTo = function()
				return true
			end,
			CheckPermission = function(self, action, domain, functionToCall)
				return function(web, ...)
					return functionToCall(web, ...)
				end
			end
		}
	end

	--requires e inicializaci�n
	for i, name in ipairs{"bgrender", "runtime", "datasource", "designer", "uploader"} do
		local md = require(tweety_modules_prefix  .. name)
		if md.setViewEnvironment then
			md.setViewEnvironment({designerAppPermission = designerAppPermission})
		end
		LogInfo("Starting module %s",name)
		md.initialize(_M, nil, m_appLocation)
	end

	return true
end

--
-- InicializeSession y FinalizeSession asumo que es cuando se usa el filtro de login
function InitializeSession (authUser)
	inconcert.authUser = inconcert.authUser or authUser
	LogInfo("InitializeSession - Logged user detail:  userName: (%s) - userGroup: (%s) - virtualContactCenter: (%s)", 
		authUser.userName, authUser.userGroup, authUser.virtualContactCenter)
	if config.checkPermisions and model then
		model:LoadUserPermissions()
	end
end

--
-- Function that gets called when the webserver finalizes a user's session.
function FinalizeSession (authUser)
	LogDebug("FinalizeSession")

	-- if the user does not especify an authUser, we use inconcert.authUser as default.
	local authUser = authUser or inconcert.authUser
	--pcall(function()
		local tweetyio = require (tweety_modules_prefix .. "tweetyio")
		local session = tweetyio.session()
		LogDebug("%s",json.encode(session))
		session:release()
		tweetyio.discardsession()

	--end)
end

--///////////////////////////////////////////////////////////////////////////////////
--
function serve_file(app_module)
	return function (web)
		local filename = web.path_info;
		--LogDebug("[serve_file] filename: %s",filename)
		-- Para algunas librerias seteo el expired para que se cacheen.
		if string.find(filename, "emoji-picker",0,true) or string.find(filename, "jquery-ui-1.12.1",0,true) then
			web.headers["Expires"]=os.date("!%a, %d %b %Y %H:%M:%S GMT",os.time()+(3600*24*365));
		end
		return _M:serve_static(web, m_appLocation.."/"..filename)
	end
end

function serve_js_file_with_cache(app_module)
	return function (web)
		local filename = web.path_info;
		--LogDebug("[serve_js_file_with_cache] filename: %s",filename)
		web.headers["Expires"]=os.date("!%a, %d %b %Y %H:%M:%S GMT",os.time()+(3600*24*365));
		web.headers["Content-Type"] = "text/javascript; charset=utf-8";
		web.headers["scriptCharset"]="utf-8"

		local exstension = filename:sub(filename:len() - 1):upper()
		local isJsFile = exstension == "JS"

		local res
		if isJsFile then
			local languageId, webLanguageCode = utils.GetLanguageFromWebInfo(web)
			local _T, _L = utils.MakeTextTranslator(languageId, webLanguageCode)
			local env = {_T = _T, _L = _L}

			local fileNameToRender = "/apps/webdesigner" .. filename
			LogInfo("[serve_js_file_with_cache] ---> Translate Javascript File: '%s'", fileNameToRender)

			res = htmlRender:ScriptLp(fileNameToRender, web, env)
		else
			res = _M:serve_static(web, m_appLocation.."/"..filename)
		end

		return res
	end
end

function serve_resource_image_file(app_module)
	return function (web,imageName)
		web.headers["Expires"]=os.date("!%a, %d %b %Y %H:%M:%S GMT",os.time()+(3600*72));
		return _M:serve_static(web, m_appLocation.."/resources/images/"..imageName)
	end
end

-- statics
_M:dispatch_get(serve_file(app_module),
--	"/clientlib/.+", 
	"/lib/.+",
	"/themes/.+",
	"/images/.+",
	"/scripts/.+",
	"/styles/.+",
	"/resources/images/.+",
	"/resources/sounds/.+",
	"/imgs/.+"
)

_M:dispatch_get(serve_js_file_with_cache(app_module),
	"/clientlib/.+"
)

_M:dispatch_get(serve_resource_image_file(app_module),
	"/reportview/resources/images/(.+)"
)



_M:dispatch_post(Authenticate, "/login/authenticate/")
_M:dispatch_post(Logout, "/login/logout")
--_M:dispatch_post(CleanupSessions, "/cleanup_sessions")
