module(..., package.seeall)

local messages = require (tweety_modules_prefix .. "messages")
local application = require (tweety_modules_prefix .. "application")
local runtime = require (tweety_modules_prefix .. "runtime")
local sqlconnector = require (tweety_modules_prefix .. "sqlconnector")
local config = require (tweety_modules_prefix .. "config");
local cassandra = require "lua_cassandra"
local m_helpers = require "helpers"

local function loadDataSourceBuiltin()
	local mysql_config = config.mysql
	if not mysql_config or not mysql_config.ndb_nodes then
		return
	end
	
	local ndb_nodes_table = {}
	for key, value, state in mysql_config["ndb_nodes"]:childs() do
		if state ~= "deleted" then
			table.insert(ndb_nodes_table, {host = value.host })
		end
	end
	-- validar que existe al menos un host
	if ndb_nodes_table[1] and mysql_config.ndb_nodes[1].host then
		return {
			ndb_nodes = ndb_nodes_table,
			Username = mysql_config.user or "root",
			Password = mysql_config.password or "",
			Port = mysql_config.port or 3306,
			Type = "mysql"
		}
	end
end

--Parametro testConections: Para probar todas las conexiones antes 
local ConnectionData = function (dataPrivate, testConections)
	--LogDebug("[INSTROSPECTOR.ConnectionData] --> dataPrivate '%s'", json.encode(dataPrivate))
	--0 Si no es bultin el dbms, pues conecto con la config pasada por parametro
	if dataPrivate.dbms ~= "builtin" then
		return sqlconnector.connect(dataPrivate)
	end

	-- DMBS builtin, busco parametros de conexion configurados
	--1 valido que existan datos del config de host
	local dataSource = loadDataSourceBuiltin()
	if not dataSource then
		return
	end

	--2 Agrego al obj data los datos para bultin configurados
	dataPrivate.userName = dataSource.Username
	dataPrivate.userPassword = dataSource.Password
	dataPrivate.port = dataSource.Port
	dataPrivate.dbms = dataSource.Type

	if dataPrivate.databaseName then
		dataPrivate.databaseName = runtime.getCurrentVCCName().. "_" .. dataPrivate.databaseName
		local connection = cassandra:CassandraConnection("Agent")
		local dbUserId = connection:GetColumnValue(dataPrivate.databaseName, "Databases", "UserId")
		if dbUserId then
			dataPrivate.userName = dbUserId
		else
			return nil, "User Id for database name not found"
		end
	end
	-- obtengo el primer nodo que este disponible
	local ndb_nodes =  dataSource.ndb_nodes
	local connectionData
	for pos, value in ipairs(ndb_nodes) do
		dataPrivate.serverAddress = value.host
		connectionData = sqlconnector.connect(dataPrivate)
		if not connectionData and testConections then
			return nil
		elseif connectionData and not testConections then
			return connectionData
		end
		--Si no conecta, desconecto y pruebo con el sigt nodo. Si es testConections
		-- y es la validacion del ultimo nodo, retorno obj de conexion
		if pos ~= #ndb_nodes then
			sqlconnector.closeConnection(connectionData)
		end
	end
	return connectionData
end


local function mssql_list(web)
	--parse data
	status, data = pcall(json.decode, web.input["post_data"])
	if not status then
		LogError("instrospect:mssql_list invalid request: %s", web.input["post_data"])
		return json.encode({ 
			status = false, 
			result = messages.messageKeyCode('argument_error',{ 
				module = "introspect.lua", func= "mssql_table", param="[POST]"}) 
		})
	end
	--make connection
	local connection, err = ConnectionData(data)
	if not connection then
		return json.encode({ status = false, result = messages.messageKey('database_conection_error') })
	end
	--run query
	local status, result = sqlconnector.listObjects(connection)
	local response
	if status then
		response = { status = true, result = result.rows }
	else
		response = { status = false, result = sqlconnector.processQueryError(connection, result) }
	end
	--close connection
	sqlconnector.closeConnection(connection)
	--return
	return json.encode(response)
end

local function mssql_table(web, tableName)
	--parse data
	status, data = pcall(json.decode, web.input["post_data"])
	if not status then
		LogError("instrospect:mssql_table invalid request: %s", web.input["post_data"])
		return json.encode({ 
			status = false, 
			result = messages.messageKeyCode('argument_error',{ 
				module = "introspect.lua", func= "mssql_table", param="[POST]"}) 
		})
	end
	--make connection
	local connection, err = ConnectionData(data)
	if not connection then
		return json.encode({ status = false, result = messages.messageKey('database_conection_error') })
	end
	--run query
	local status, result = sqlconnector.tableDefinition(connection, tableName)
	local response
	if status then
		response = { status = true, result = result.rows }
	else
		response = { status = false, result = sqlconnector.processQueryError(connection, result) }
	end
	--close connection
	sqlconnector.closeConnection(connection)
	--return
	return json.encode(response)
end

local function mssql_proc(web, procName)
	--parse data
	status, data = pcall(json.decode, web.input["post_data"])
	if not status then
		LogError("instrospect:mssql_proc invalid request: %s", web.input["post_data"])
		return json.encode({ 
			status = false, 
			result = messages.messageKeyCode('argument_error',{ 
				module = "introspect.lua", func= "mssql_table", param="[POST]"}) 
		})
	end
	--make connection
	local connection, err = ConnectionData(data)
	if not connection then
		return json.encode({ status = false, result = messages.messageKey('database_conection_error') })
	end
	--run query
	local status, result = sqlconnector.spDefinition(connection, procName)
	local response
	if status then
		response = { status = true, result = result.rows }
	else
		response = { status = false, result = sqlconnector.processQueryError(connection, result) }
	end
	--close connection
	sqlconnector.closeConnection(connection)
	--return
	return json.encode(response)
end

local function mssql_proc_columns (web, procName)
	LogDebug("[INSTROSPECTOR.mssql_proc_columns] --> MS SQL proc columns SP '%s'", procName)
--	LogDebug("[INSTROSPECTOR.mssql_proc_columns] --> MS SQL proc columns post_data '%s'", web.input["post_data"])

	--parse data
	status, data = pcall(json.decode, web.input["post_data"])
	if not status then
		LogError("instrospect:mssql_proc_columns invalid request: %s", web.input["post_data"])
		return json.encode({ 
			status = false, 
			result = messages.messageKeyCode('argument_error',{
				module = "introspect.lua", func= "mssql_table", param="[POST]"
			}) 
		})
	end
	--make connection
	local connection, err = ConnectionData(data.private)
	if not connection then
		return json.encode({ status = false, result = messages.messageKey('database_conection_error') })
	end
	--make query
	local query
	local dbms = (connection.data.dbms or ""):lower()
	--LogDebug("[INSTROSPECTOR:mssql_proc_columns] --> dbms: '%s'", dbms)
	if dbms == "sqlserver" then
		query = "EXEC [%s]"
		query = string.format(query, procName)
		for index, param in ipairs(data.params) do
			query = query .. string.format("%s ''", index > 1 and "," or "", tostring(param))
		end
	elseif dbms == "mysql" then
		query = "CALL %s"
		query = string.format(query, procName)
		local params = ""
		for index, param in ipairs(data.params) do
			params = params .. string.format("%s ''", index > 1 and "," or "", tostring(param))
		end
		if #params > 0 then
			query = query .. string.format(" (%s)", params)
		end
	else
		error("Unsuported DBMS " .. dbms)
	end

	LogDebug("[INSTROSPECTOR.mssql_proc_columns] --> DMBS '%s' proc columns query '%s'", dbms, query)
	--run query
	local status, result = sqlconnector.query(connection, query, { columnsOnly = true })
	local response
	if status then
		response = { status = true, result = result.columns }
	else
		response = { status = false, result = sqlconnector.processQueryError(connection, result) }
	end
	--close connection
	sqlconnector.closeConnection(connection)
	--return
	return json.encode(response)
end

local function mssql_builtin_config (web)

	local post_data = json.decode(web.input["post_data"])
	if not post_data.vcc  then
		return json.encode({status = false, result = "NOT_FOUND"})
	end

	local keyspace = cassandra:CassandraConnection("Agent")

	local databases = keyspace:GetRangeSlices("Databases",  --column family
											   "",  --supercolumn
											   "", --startkey
											   "", --endkey
													{ 
													column_names = {},
													slice_range = { 
																start = "", 
																finish = "", 
																reversed = false, 
																count = 100}
													},
												1000 --total keys
									)
			local retDatabases = {}
			if  databases  then
				for dbId, database in databases:Iterator() do
					if next(database) then
						local split = m_helpers.utils.SplitString(dbId, "_")
						if #split > 0 and  split[1] == post_data.vcc then
							local id = split[2]
							local text = id
							for k, v in ipairs(database) do
								if v.name == "Name" then
									text = v.value
								end
							end
							table.insert(retDatabases, {id = id, text = text})
						end
					end
				end				
			end

		local result = {
			Databases = retDatabases,
		} 

	keyspace:Close()
	return  json.encode({status = true, result = "OK", data = result})
end

function initialize(_M, prefix,appLocation)
	prefix = prefix or ""
	_M:dispatch_post(mssql_list, prefix .. "/introspect/mssql/lista");
	_M:dispatch_post(mssql_table, prefix .. "/introspect/mssql/table/([^/]+)");
	_M:dispatch_post(mssql_proc, prefix .. "/introspect/mssql/proc/([^/]+)");
	_M:dispatch_post(mssql_proc_columns, prefix .. "/introspect/mssql/proc_columns/([^/]+)");
	_M:dispatch_post(mssql_builtin_config, prefix .. "/introspect/builtin_config");
end
