
let icApi;

function makeSuccessResponse(data, funcName) {
    return {
        data: data,
        message: (funcName || "") + " succeeded",
        code: 200,
        internalCode: 1
    }
}
function makeFailedResponse(data, funcName) {
    return {
        data: data,
        message: (funcName || "") + " failed",
        code: 500,
        internalCode: 0
    }
}

function Baragent(callbackSuccess, callbackError) {
    var m_callbackSuccess = callbackSuccess;
    var m_callbackError = callbackError;
    let voidSuccess = () => { };
    let voidError = () => { };

    let _notImplemented = (mMethodName) => {
        console.warn("Attempt to call a method not implemented in the Baragent interface:" + mMethodName);
    }


    ///////////////////////////////////////////////////////////////////////////////////////////

    this.getCampaign = function (campaignName, callbackSuccess = voidSuccess, callbackError) {
        icApi.GetCampaign(campaignName,
            function (result) {
                callbackSuccess(result);
            }
        );
    }
    //Public Method GetInteractionId

    //TODO: Error response

    this.GetInteractionId = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        icApi.GetInteraction(interactionId,
            (result) => {
                callbackSuccess(makeSuccessResponse({ interactionId: (result || {}).id }, "GetInteractionId"));
            }
        );

    };



    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method GetInteractionState

    // En todos los proyectos revisados solo importa si interactionState = "" porque significa que un preview fue cancelado.

    this.GetInteractionState = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {


        icApi.GetInteraction(interactionId,
            (result) => {
                let vState = (result || {}).state;
                // EMULAMOS LO ESPERADO POR EL BLENDING DE ALLEGRO, SI ES UN PREVIEW CANCELADO EL ESTADO ES BLANCO
                if (result.interactionData &&
                    result.interactionData.dialer &&
                    result.interactionData.dialer.hasPreview &&
                    result.interactionData.dialer.interactionCode === "PREVIEW_CANCELED")
                    vState = "";
                callbackSuccess(makeSuccessResponse({ interactionState: vState }, "GetInteractionState"));
            }
        );

    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method Wrapup
    this.Wrapup = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        this.GetInteractionId(interactionId, (result) => {
            interactionId = result.data.interactionId;
            let sdk_disposition = window.icc.__icc_sdk_last_disposition;
            if (sdk_disposition) {
                let ds = sdk_disposition.datasource;
                if (ds) {
                    if (ds.mode !== "browse") {
                        this.GetInteractionState(interactionId || ds.row().Id, (state) => {
                            if (state.data.interactionState === "WRAPUP") {
                                ds.executeAction("save", {},
                                    function (status, result) {
                                        if (status) {
                                            callbackSuccess(makeSuccessResponse({}, "Wrapup"));
                                        } else {
                                            callbackError(makeFailedResponse(result, "Wrapup"))
                                        }
                                    }
                                );
                            } else {
                                callbackError(makeFailedResponse(state, "Wrapup"))
                            }
                        }, callbackError)
                    } else {
                        callbackSuccess(makeSuccessResponse({}, "Wrapup"));
                    }
                } else {
                    callbackError(makeFailedResponse({}, "Wrapup"))
                }
            } else {
                this.GetInteractionState(interactionId || ds.row().Id, (state) => {
                    if (state.data.interactionState === "WRAPUP") {
                        var request = {
                            id : interactionId || ds.row().Id,
                            comment : null
                        };
                        icApi.ArchiveInteraction(request,(data)=>{
                            if(data.result == "FAIL"){
                                callbackError(makeFailedResponse(data.reason || {}, "Wrapup"))
                            }else{
                                callbackSuccess(makeSuccessResponse(data, "Wrapup"));
                            }

                        })
                    }else {
                        callbackError(makeFailedResponse(state, "Wrapup"))
                    }
                })
            }

        }, callbackError)
    }

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method Redial
    // NOTE: phone  no soportado
    this.Redial = function (interactionId, phone, callbackSuccess = voidSuccess, callbackError = voidError) {
        this.GetInteractionId(interactionId, (result) => {
            interactionId = result.data.interactionId;
            icApi.RedialCall({ id: interactionId }, (result) => {
                if (result.result == "OK")
                    this.GetInteractionId(interactionId, (result) => {
                        callbackSuccess(makeSuccessResponse({ interactionId: result.data.interactionId !== interactionId ? result.data.interactionId : "" }, "Redial"));
                    }, callbackError)
                else
                    callbackError(makeFailedResponse(result, "Redial"))
            });
        }, callbackError)
    };
    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method Transfer
    this.Transfer = function (interactionId, phone, value, autocommit, callbackSuccess = voidSuccess, callbackError = voidError) {
        if (value == "INTERNO") {
            let request = {
                id: interactionId,
                number: phone
            };

            icApi.TransferInteractionToNumber(request, (result) => {
                if (result.result == "OK") {
                    callbackSuccess(makeSuccessResponse({ result: result.result }, "Transfer"));
                } else {
                    callbackError(makeFailedResponse(result, "Transfer"));
                }
            });

        } else {
            let request = {
                id: interactionId,
                member: phone
            };
            icApi.TransferInteractionToMember(request, (result) => {
                if (result.result == "OK") {
                    callbackSuccess(makeSuccessResponse({ result: result.result }, "Transfer"));
                } else {
                    callbackError(makeFailedResponse(result, "Transfer"));
                }
            })
        }
    };
    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method MakeCall
    this.MakeCall = function (campaign, phone, iml, callbackSuccess = voidSuccess, callbackError = voidError) {
        var request = {
            type: "CALL",
            campaign: campaign || null,
            account: null,
            address: phone || null,
            addressType: "phone",
            contactId: null,
            isExternal: false,
            autosend: true,
            message: icApi.MakeTextMessage({ text: '' })
        };
        icApi.CreateOutgoingInteraction(request, function (result) {
            if (result.result == "OK") {
                callbackSuccess(makeSuccessResponse({ result: result.result }, "MakeCall"));
            } else {
                callbackError(makeFailedResponse(result, "MakeCall"));
            }
        })
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method Answer
    this.Answer = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        icApi.AnswerCall({ id: interactionId }, (result) => {
            callbackSuccess(makeSuccessResponse(result, "Answer"));
        })
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method Hangup
    this.Hangup = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        icApi.HangupInteraction({ id: interactionId }, (result) => {
            callbackSuccess(makeSuccessResponse(result, "Hangup"));
        })
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method Hold
    this.Hold = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        icApi.HoldCall({ id: interactionId }, (result) => {
            callbackSuccess(makeSuccessResponse(result, "Hold"));
        })
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method Mute
    this.Mute = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        icApi.MuteCall({ id: interactionId }, (result) => {
            callbackSuccess(makeSuccessResponse(result, "Mute"));
        })
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method Ignore
    this.Ignore = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("Ignore");
    };




    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method ManualReschedule
    this.ManualReschedule = function (phone, date, hour, iml, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("ManualReschedule");
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method Take
    this.Take = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("Take");
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method TakeFirst
    this.TakeFirst = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("TakeFirst");
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method Hold By Id
    this.HoldById = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("HoldById");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method Switch
    this.Switch = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("Switch");
    };



    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method TransferCommit
    this.TransferCommit = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("TransferCommit");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method TransferCancel
    this.TransferCancel = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("TransferCancel");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method Conference
    this.Conference = function (interactionId, phone, value, autocommit, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("Conference");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method ConferenceCommit
    this.ConferenceCommit = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("ConferenceCommit");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method ConferenceCancel
    this.ConferenceCommit = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("ConferenceCommit");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method Forward
    this.Forward = function (interactionId, campaign, iml, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("Forward");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method TransferPopup
    this.TransferPopup = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("TransferPopup");
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method ConferencePopup
    this.ConferencePopup = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("ConferencePopup");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method GetPhoneState
    this.GetPhoneState = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("GetPhoneState");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method AcceptPreview
    this.AcceptPreview = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("AcceptPreview");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method CancelPreview
    this.CancelPreview = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("CancelPreview");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method GetState
    this.GetState = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("GetState");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method ChangeState
    this.ChangeState = function (state, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("ChangeState");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method SetbuttonEnabled
    this.SetbuttonEnabled = function (idButton, state, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("SetbuttonEnabled");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method SetbuttonVisible
    this.SetbuttonVisible = function (idButton, state, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("SetbuttonVisible");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method recording
    this.Recording = function (interactionId, action, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("Recording");
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method PlayAudio
    this.PlayAudio = function (streamserver, repositoryserver, interactionId, section, part, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("PlayAudio");
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method StartScreenRecording
    this.StartScreenRecording = function (screenserver, port, section, cleanTemp, interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("StartScreenRecording");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method StopScreenRecording
    this.StopScreenRecording = function () {
        _notImplemented("StopScreenRecording");
    };
    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method LogoutAgent
    this.LogoutAgent = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("StopScreenRecording");
    };
    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method SendDigits
    this.SendDigits = function (interactionId, digits, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("SendDigits");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method GetBarAgentStatus
    this.GetBarAgentStatus = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("GetBarAgentStatus");
    };
    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method GetBarAgentDestinations
    this.GetBarAgentDestinations = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("GetBarAgentDestinations");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method GetAgentStates
    this.GetAgentStates = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("GetAgentStates");
    };
    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method MakeChat
    this.MakeChat = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("MakeChat");
    };

    /////////////////////////////////////////////////////////////////////////////////////////// 
    //Public Method GetInteractionAttributeByID 
    this.GetInteractionAttributeByID = function (interactionId, attribute, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("GetInteractionAttributeByID");
    };

    /////////////////////////////////////////////////////////////////////////////////////////// 
    //Public Method TakeById 
    this.TakeById = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("TakeById");
    };

    /////////////////////////////////////////////////////////////////////////////////////////// 
    //Public Method SelectActiveInteraction 
    this.SelectActiveInteraction = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("SelectActiveInteraction");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method TransferPopup
    this.RePlayVoiceMail = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("RePlayVoiceMail");
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method CallAbandoned
    this.CallAbandoned = function (interactionid, campaign, phone, agent, contactid, contactname, schedule, sDate, sHour, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("CallAbandoned");
    };
}



///////////////////////////////////////////////////////////////////////////////////////////
//Public Object OutBoundEngine

function OutBoundEngine(callbackSuccess, callbackError) {
    let voidSuccess = () => { };
    let voidError = () => { }
    let _notImplemented = (mMethodName) => {
        console.warn("Attempt to call a method not implemented in the OutboundEngine interface:" + mMethodName);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method FinishContact
    this.FinishContact = function (processId, batchId, contactId, callbackSuccess = voidSuccess, callbackError = voidError) {

        let request = {
            id: batchId,
            contactId: contactId,
            processId: processId
        };
        icApi.FinishOutboundContact(request,
            (result) => {
                if (result.result === "OK") {
                    callbackSuccess(makeSuccessResponse(result, "FinishContact"));
                } else {
                    callbackError(makeFailedResponse(result, "FinishContact"));
                }
            })
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method RemoveContact
    this.RemoveContact = function (processId, batchId, contactId, callbackSuccess = voidSuccess, callbackError = voidError) {

        let request = {
            id: batchId,
            contactId: contactId,
            processId: processId
        };
        icApi.RemoveOutboundContact(request,
            (result) => {
                if (result.result === "OK") {
                    callbackSuccess(makeSuccessResponse(result, "RemoveContact"));
                } else {
                    callbackError(makeFailedResponse(result, "RemoveContact"));
                }
            })
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method RecycleContact
    this.RecycleContact = function (processId, batchId, contactId, agent, callbackSuccess = voidSuccess, callbackError = voidError) {
        let request = {
            id: batchId,
            contactId: contactId,
            processId: processId,
            configuration: { restartRetries: true }
        };
        icApi.RecycleOutboundContact(request,
            (result) => {
                if (result.result === "OK") {
                    callbackSuccess(makeSuccessResponse(result, "RecycleContact"));
                } else {
                    callbackError(makeFailedResponse(result, "RecycleContact"));
                }
            })

    };
    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method ChangePriority
    this.ChangePriority = function (processId, batchId, contactId, newPriority, callbackSuccess = voidSuccess, callbackError = voidError) {
        let request = {
            id: batchId,
            contactId: contactId,
            processId: processId,
            priority: newPriority
        };
        icApi.ChangePriorityOutboundContact(request,
            (result) => {
                if (result.result === "OK") {
                    callbackSuccess(makeSuccessResponse(result, "ChangePriorityOutboundContact"));
                } else {
                    callbackError(makeFailedResponse(result, "ChangePriorityOutboundContact"));
                }
            });
    };
    //
    // /usr/lib/inconcert/webdesigner/inconcert/apps/webdesigner/lib/allegro# rsync malibu:/home/fapenia/Documents/devel/inconcert/agent/inconcert/lib/allegro/i6BridgeSDK.js .
    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method ReScheduleContact
    this.ReScheduleContact = function (processId, batchId, contactId, agent, rescheduleDate, outOfBatchAllowed, IgnoreRules, callbackSuccess = voidSuccess, callbackError = voidError) {
        if (rescheduleDate instanceof Date) {
            rescheduleDate = icc.utils.date2ISO(rescheduleDate)
        }
        let request = {
            id: batchId,
            batchId: batchId,
            contactId: contactId,
            processId: processId,
            configuration: {
                scheduleDate: rescheduleDate,
                agent: agent
            }
        };
        icApi.RescheduleOutboundContact(request,
            (result) => {
                if (result.result === "OK") {
                    callbackSuccess(makeSuccessResponse(result, "ReScheduleContact"));
                } else {
                    callbackError(makeFailedResponse(result, "ReScheduleContact"));
                }
            })
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method RecognizePhone
    this.RecognizePhone = function (country, area, number, extension, callbackSuccess = voidSuccess, callbackError = voidError) {
        let request = {
            type: "PHONE",
            number: number,
            country: country,
            area: area,
            extension: extension
        }
        icApi.ValidateAddress(request,
            (result) => {
                callbackSuccess(makeSuccessResponse(result, "RecognizePhone"));
            });
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method CancelContact
    this.CancelContact = function (processId, batchId, contactId, callbackSuccess = voidSuccess, callbackError = voidError) {

        let request = {
            id: '',
            batchId: batchId,
            contactId: contactId,
            processId: processId
        };
        icApi.CancelOutboundContact(request,
            (result) => {
                if (result.result === "OK") {
                    callbackSuccess(makeSuccessResponse(result, "CancelContact"));
                } else {
                    callbackError(makeFailedResponse(result, "CancelContact"));
                }
            })
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method AddContactToBatch
    this.AddContactToBatch = function (processId, batchId, contactId, agentId, outofbatchallowed, IgnoreRules, contactDate, callbackSuccess = voidSuccess, callbackError = voidError, isProgressiveType) {
        /*
        if (contactDate instanceof Date) {
            contactDate = icc.utils.date2ISO(contactDate)
        }
        */
        let request = {
            id: '',
            contactId: contactId,
            processId: processId,
            batchId: batchId,
            configuration: {
                agent: agentId,
                contactData: {}
            }
        };
        icApi.AddContactToOutboundBatch(request,
            (result) => {
                if (result.result === "OK") {
                    callbackSuccess(makeSuccessResponse(result, "AddContactToBatch"));
                } else {
                    callbackError(makeFailedResponse(result, "AddContactToBatch"));
                }
            })
    };
    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method PauseContact
    this.PauseContact = function (processId, batchId, contactId, callbackSuccess = voidSuccess, callbackError = voidError) {
        let request = {
            id: '',
            batchId: batchId,
            contactId: contactId,
            processId: processId
        };
        icApi.PauseOutboundContact(request,
            (result) => {
                if (result.result === "OK") {
                    callbackSuccess(makeSuccessResponse(result, "PauseContact"));
                } else {
                    callbackError(makeFailedResponse(result, "PauseContact"));
                }
            })
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method ResumeContact
    this.ResumeContact = function (processId, batchId, contactId, callbackSuccess = voidSuccess, callbackError = voidError) {
        let request = {
            id: '',
            batchId: batchId,
            contactId: contactId,
            processId: processId
        };
        icApi.ResumeOutboundContact(request,
            (result) => {
                if (result.result === "OK") {
                    callbackSuccess(makeSuccessResponse(result, "ResumeContact"));
                } else {
                    callbackError(makeFailedResponse(result, "ResumeContact"));
                }
            })
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method AddContact
    this.AddContact = function (contactId, name, accountOfficer, accountGroup, category, campaign, importation, isVip, isClient, lastManagementResult, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("AddContact");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method AddContactValues
    this.AddContactValues = function (contactId, contactDataList, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("AddContactValues");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method AddContactSkills
    this.AddContactSkills = function (contactId, contactSkillList, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("AddContactSkills");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method AddPhone
    this.AddPhone = function (contactId, phoneType, countryId, areaId, phoneNumber, ZIP, phoneExtension, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("AddPhone");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method AssignAgent
    this.AssignAgent = function (processId, batchId, contactId, agentId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("AssignAgent");
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method FindContactInBatch
    this.FindContactInBatch = function (processId, batchId, contactId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("FindContactInBatch");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method CancelPhone
    this.CancelPhone = function (contactId, countryId, areaId, phoneNumber, phoneExtension, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("CancelPhone");
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method DisablePhone
    this.DisablePhone = function (contactId, countryId, areaId, phoneNumber, phoneExtension, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("DisablePhone");
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method EnablePhone
    this.EnablePhone = function (contactId, countryId, areaId, phoneNumber, phoneExtension, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("EnablePhone");
    };





    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method ReScheduleContact
    this.ReScheduleContactToPhone = function (processId, batchId, contactId, agent, rescheduleDate, outOfBatchAllowed, IgnoreRules, phoneCountry, phoneArea, phoneNumber, phoneExtension, phoneType, ZIP, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("ReScheduleContactToPhone");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method ReScheduleContactCampaign
    this.ReScheduleContactCampaign = function (campaignId, contactId, agent, rescheduleDate, batchId, outOfBatchAllowed, IgnoreRules, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("ReScheduleContactCampaign");
    };




    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method ExtendBatch
    this.ExtendBatch = function (processId, batchId, endDate, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("ExtendBatch");
    }

    //DELETE BATCH
    this.DeleteBatch = function (processId, batchId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("DeleteBatch");
    };

    this.ExpireBatch = function (processId, batchId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("ExpireBatch");
    };

    this.StopProcess = function (processId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("StopProcess");
    };

    this.StartProcess = function (processId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("StartProcess");
    };

    this.StatusProcess = function (processId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("StatusProcess");
    };




    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method RetrieveContact
    this.RetrieveContact = function (contactId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("RecognizePhone");
    };


    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method SetDispositionCode
    this.SetDispositionCode = function (processId, batchId, contactId, interactionId, contactCode, businessCode, contactName, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("SetDispositionCode");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method StartContactImport
    this.StartContactImport = function (divForm, inputFile, importId, dbProvider, formatId, duplicateCheck, duplicateSolver, vipImport, clientImport) {
        _notImplemented("StartContactImport");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method StartContactImportFromfile
    this.StartContactImportFromFile = function (file, importId, formatId, DBProvider, duplicateCheck, duplicateSolver, isVipImport, areClientsImport, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("StartContactImportFromFile");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method DeleteContactImport
    this.DeleteContactImport = function (importationId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("DeleteContactImport");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method GetImportStatus
    this.GetImportStatus = function (importId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("GetImportStatus");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method GetImportDetail
    this.GetImportDetail = function (importId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("GetImportDetail");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method CreateFormatImport
    this.CreateFormatImport = function (formatId, description, formatImportFieldList, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("CreateFormatImport");
    };
    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method CreateBatchFromFilter
    this.CreateBatchFromFilter = function (filterId, processId, batchId, startDate, endDate, excludeDuplicates, orderField, priorityBase, inverseOrder, priority, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("CreateBatchFromFilter");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method CreateFilter
    this.CreateFilter = function (filterId, filterValues, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("CreateFilter");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method CreateBatchFromImport
    this.CreateBatchFromImport = function (importId, processId, batchId, startDate, endDate, excludeDuplicates, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("CreateBatchFromImport");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method SearchContacts
    this.SearchContacts = function (predicate, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("SearchContacts");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method GetAllOutboundCampaings
    this.GetAllOutboundCampaings = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("GetAllOutboundCampaings");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method GetImportFormatFields
    this.GetImportFormats = function (callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("GetImportFormats");
    };

    ///////////////////////////////////////////////////////////////////////////////////////////
    //Public Method GetHistoricalInteraction
    this.GetHistoricalInteraction = function (interactionId, callbackSuccess = voidSuccess, callbackError = voidError) {
        _notImplemented("GetHistoricalInteraction");
    };

    if (typeof (callbackSuccess) === "function") {
        g_callbackSuccess = callbackSuccess;
    }

    if (typeof (callbackError) === "function") {
        g_callbackError = callbackError;
    }

}
(function () {
    window.addEventListener("load", () => {
        icApi = new inConcertAgentApi(window.location.hostname);
    });
})();


