/* global hermes */

/************************ API iframe ************************/

/**
* Constructor
*/

function inConcertAgentApiIframe() {
	this.__initialize();
}

/************************ General Use ************************/

inConcertAgentApiIframe.prototype.__getQuerystringParam = function(name) {
	name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
	var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
		results = regex.exec(location.search);
	return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
};

inConcertAgentApiIframe.prototype.__S4 = function() {
	/*jshint bitwise:false*/
	return (((1+Math.random())*0x10000)|0).toString(16).substring(1);
};

inConcertAgentApiIframe.prototype.GUID = function() {
	var S4 = this.__S4;
	return (S4() + S4() + S4() + S4() + S4() + S4() + S4() + S4()).toUpperCase();
};

inConcertAgentApiIframe.prototype.__noop = function() {
	
};

/************************ Internal ************************/

inConcertAgentApiIframe.prototype.__initialize = function() {
	this.iframeToken = this.__getQuerystringParam("token");
	this.__listenHermesEvents();
	this.__listenIframeEvents();
	this.__sendIframeMessage("iframe_ready");
};

inConcertAgentApiIframe.prototype.__hermesIncomingEventName = function() {
	return "inconcert_agent_api_incoming";
};

inConcertAgentApiIframe.prototype.__hermesOutgoingEventName = function() {
	return "inconcert_agent_api_outgoing";
};

inConcertAgentApiIframe.prototype.__callbackPrefix = function() {
	return "callback_";
};

inConcertAgentApiIframe.prototype.__listenHermesEvents = function() {
	var self = this;
	
	hermes.on(self.__hermesIncomingEventName(), function(eventData) {
		if (!eventData || !eventData.length || eventData.length < 2) {
			return;
		}
		var tokens = eventData[0];
		if (tokens && tokens !== self.iframeToken && !tokens[self.iframeToken]) {
			return;
		}
		var eventName = eventData[1];
		var data = eventData.length > 2? eventData[2] : null;
		var callbackId = eventData.length > 3? eventData[3] : null;
		self.__incomingEvent(eventName, data, callbackId);
	});
	
	hermes.on(self.__callbackPrefix() + self.__hermesOutgoingEventName(), function(eventData) {
		if (!eventData || !eventData.length || eventData.length < 4 || eventData[0] !== self.iframeToken) {
			return;
		}
		var eventName = eventData[1];
		var data = eventData[2];
		var callbackId = eventData[3];
		self.__fireOutgoingMethodCallback(eventName, data, callbackId);
	});
};

inConcertAgentApiIframe.prototype.__sendHermesMessage = function(topic, event, data, callbackId) {
	try {
		hermes.send(topic, [this.iframeToken, event, data, callbackId], true);
		return true;
	}
	catch(e) {
		console.error("Error sending message to hermes", event, e);
		return false;
	}
};

inConcertAgentApiIframe.prototype.__outgoingEvent = function(event, data, callbackId) {
	return this.__sendHermesMessage(this.__hermesOutgoingEventName(), event, data, callbackId);
};

inConcertAgentApiIframe.prototype.__fireIncomingMethodCallback = function(event, data, callbackId) {
	return this.__sendHermesMessage(this.__callbackPrefix() + this.__hermesIncomingEventName(), event, data, callbackId);
};

inConcertAgentApiIframe.prototype.__listenIframeEvents = function() {
	var self = this;
	window.addEventListener("message", function(event) {
		try {
			if (!event.data || !event.data.length || event.data.length < 2 || event.data[0] !== self.iframeToken) {
				return;
			}
			var eventName = event.data[1];
			var data = event.data.length > 2? event.data[2] : null;
			var callbackId = event.data.length > 3? event.data[3] : null;
			if (eventName.indexOf(self.__callbackPrefix()) === 0) {
				//callback
				eventName = eventName.replace(self.__callbackPrefix(), "");
				self.__fireIncomingMethodCallback(eventName, data, callbackId);
			}
			else {
				//new incoming event
				if (self["__" + eventName]) {
					self["__" + eventName](eventName, data, callbackId);
				}
				else {
					self.__outgoingEvent(eventName, data, callbackId);
				}
			}
		}
		catch(e) {
			console.error("Error processing event", event, e);
		}		
	});
};

inConcertAgentApiIframe.prototype.__sendIframeMessage = function(event, data, callbackId) {
	try {
		parent.postMessage([this.iframeToken, event, data, callbackId], "*");
		return true;
	}
	catch(e) {
		console.error("Error sending message to iframe", event, e);
		return false;
	}
};

inConcertAgentApiIframe.prototype.__incomingEvent = function(event, data, callbackId) {
	return this.__sendIframeMessage(event, data, callbackId);
};

inConcertAgentApiIframe.prototype.__fireOutgoingMethodCallback = function(event, data, callbackId) {
	return this.__sendIframeMessage(this.__callbackPrefix() + event, data, callbackId);
};

inConcertAgentApiIframe.prototype.__PostFile = function(uri, file, callback, onProgress) {
	callback = callback || this.__noop;
	onProgress = onProgress || this.__noop;
	let xhr = new XMLHttpRequest();
	xhr.open("POST", uri, true);
	xhr.onload = function(e)  {
		callback(e.target.status === 200, e.target.responseText);
	};

	// show the uploaded percentage
	xhr.upload.onprogress = function( e ) {
		let percentage = "";
		if ( e && !isNaN( e.loaded ) && !isNaN( e.total ) ) {
			percentage = Math.floor( e.loaded / e.total * 100 ) + "%";
		}
		onProgress(percentage);
	};

	xhr.send( file );
};

inConcertAgentApiIframe.prototype.__UploadFile = function(event, data, callbackId) {
	var self = this;
	var file = data.file;
	var fileData = {
		id : self.GUID(),
		name : file.name || "",
		contentType : file.type || ""
	};
	var name_a = fileData.name.split(".");
	fileData.extension = name_a.length > 1? name_a[name_a.length - 1] : "";
	
	var uri = window.location.origin + "/inconcert/apps/repository/new/?id=" +
		encodeURIComponent(fileData.id) + "&section=-1&part=-1&contentType=" + encodeURIComponent(fileData.contentType) +
		"&format=" + encodeURIComponent(fileData.extension) + "&fileName=" + encodeURIComponent(fileData.name);
	self.__PostFile(uri, file, function(status, text) {
		var result = status? { result : "OK", file : fileData } : 
			{ result : "FAIL", reason : "error uploading file" };
		self.__fireOutgoingMethodCallback(event, result, callbackId);
	});
};

var iframeApi = new inConcertAgentApiIframe();
