/*jshint bitwise:true, browser:true, curly:false, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, strict: false, esversion: 11 */
/*jshint quotmark:false*/
/*global $*/

/************************ API ************************/

/**
* Constructor
*
* @param  {string}		domain						-Required. API domain. Example: "i6.inconcert.cc"
* @param  {string}		accessToken					-Optional. Api user access token.
*/
function inConcertApi(domain, accessToken) {
	this.domain = domain;
	this.accessToken = accessToken;
	this.__makeAsync();
}

inConcertApi.prototype.__maxFileSize = function() {
	return 64;
};

inConcertApi.prototype.__S4 = function() {
	/*jshint bitwise:false*/
	return (((1+Math.random())*0x10000)|0).toString(16).substring(1);
};

inConcertApi.prototype.GUID = function() {
	var S4 = this.__S4;
	return (S4() + S4() + S4() + S4() + S4() + S4() + S4() + S4()).toUpperCase();
};

inConcertApi.prototype.__fileExtension = function(fileName) {
	fileName = fileName || "";
	var match = fileName.match(/[^.]*(.)$/);
	if ( !match ) {
		return "";
	}
	var extension = match[0];
	if (extension === fileName) {
		extension = "";
	}
	return extension || "";
};

inConcertApi.prototype.__makeQueryString = function(data) {
	var qs = "";
	$.each(data, function(k, v) {
		qs += (qs? "&" : "?") + k + "=" + encodeURIComponent(v);
	});
	return qs;
};

inConcertApi.prototype.__get = function(path, callback) {
	var self = this;
	$.ajax({
		url : window.location.protocol + "//" + self.domain + "/inconcert/api/" + path,
		type : "GET",
		beforeSend: function(request) {
			request.setRequestHeader("Authorization", self.accessToken);
		},
		success: function(response) {
			callback(true, response);
		},
		error: function(response) {
			var result = response.responseJSON || { 
				status : false, 
				reason : response.responseText || response.statusText || "unknown error" 
			};
			callback(false, result);
		}
	});
};

inConcertApi.prototype.__post = function(path, body, contentType, callback) {
	var self = this;
	$.ajax({
		url : window.location.protocol + "//" + self.domain + "/inconcert/api/" + path,
		type : "POST",
		contentType: contentType,
		data : body,
		beforeSend: function(request) {
			request.setRequestHeader("Authorization", self.accessToken);
		},
		success: function(response) {
			callback(true, response);
		},
		error: function(response) {
			var result = response.responseJSON || { 
				status : false, 
				reason : response.responseText || "unknown error" 
			};
			callback(false, result);
		}
	});
};

inConcertApi.prototype.__postJson = function(path, data, callback) {
	this.__post(path, JSON.stringify(data), "application/json", callback);
};

inConcertApi.prototype.__uploadFile = function(path, file, callback, progress) {
	progress = progress || function() {};
	var self = this;
	var url = window.location.protocol + "//" + self.domain + "/inconcert/api/" + path;
	var xhr = new XMLHttpRequest();
	xhr.open("POST", url, true);
	xhr.setRequestHeader("Authorization", self.accessToken);
	xhr.onload = function(e) {
		var response;
		try {
			response = JSON.parse(xhr.responseText);
		}
		catch {
			response = {
				status : false,
				reason : xhr.responseText || "unknown error" 
			};
		}
		callback(e.target.status === 200, response);
	};

	// show the uploaded percentage
	xhr.upload.onprogress = function(e) {
		var percentage = 0;
		if ( e && !isNaN( e.loaded ) && !isNaN( e.total ) ) {
			percentage = Math.floor( e.loaded / e.total * 100 );
		}
		progress(percentage);
	};
	xhr.send(file);
};

inConcertApi.prototype.__makeAsync = function() {
	this.async = Object.keys(inConcertApi.prototype).reduce((async, methodName) => {
		const asyncMethod = this.__makeAsyncMethod(methodName);
		if (asyncMethod) async[methodName] = asyncMethod;
		return async;
	}, {});
};

inConcertApi.prototype.__makeAsyncMethod = function(methodName) {
	const self = this;
	const method = self[methodName];
	if (methodName.indexOf("_") === 0 || typeof(method) !== "function")
		return;

	return async function() {
		let args = arguments;
		const unsetArgs = method.length - args.length - 1;
		if (unsetArgs > 0) args = [...args, ...Array(unsetArgs)];

		return new Promise((resolve, _reject) => {
			method.apply(self, [...args, (status, data) => resolve([status, data])]);
		});
	};
};

/**
* Login
*
* @param  {string}		user						-Required. User id. Example: apiuser@myvcc
* @param  {string}		password					-Required. User password.
* @param  {Function}	callback(status)			-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.Login = function(user, password, callback) {
	var self = this;
	$.ajax({
		url : window.location.protocol + "//" + self.domain + "/inconcert/api/login/",
		type : "POST",
		contentType: "application/json",
		data : JSON.stringify({ user : user, password : password }),
		success: function(response) {
			self.accessToken = response.token;
			callback(true);
		},
		error: function() {
			callback(false);
		}
	});
};

// /************************ API DEFINITION ************************/

/**
*CCI_Get
*
* @param  {string}		campaignId					-Required. Example: campaign1
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.campaign
*/
inConcertApi.prototype.API_definition = function(type, callback) {
	this.__get("definition/" + encodeURIComponent(type), callback);
};


// /************************ REPORTS ************************/


// /**
// *CCI_Get
// *
// * @param  {string}		campaignId					-Required. Example: campaign1
// * @param  {Function}	callback(status, data)		-Required. Callback function.
// * @returns {boolean}	status
// * @returns {object}		data
// * @returns {object}		data.campaign
// */
// inConcertApi.prototype.CCI_Get = function(campaignId, callback) {
// 	this.__get("cci/get/" + encodeURIComponent(campaignId), callback);
// };

/**
*CCI_Execute
*
* @param  {string}		campaignId					-Required. Example: campaign1
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.campaign
*/
inConcertApi.prototype.ReportsExecute = function(rptId, request, callback) {
	let url = "reports/execute/" + rptId + "/";
	this.__postJson(url , request, callback);
};

/************************ Batch Management ************************/

/**
* Get batch
*
* @param  {string}		request.campaign				-Required. 
* @param  {string}		request.account					-Required this or account_name. 
* @param  {string}		request.account_name			-Required this or account. Account name.
* @param  {string}		request.batchId					-Required.
* @param  {Function}	callback(status, data)			-Required. Callback function.
* @returns {object}		data.batch
* @returns {boolean}	status
*/
inConcertApi.prototype.GetBatch = function(request, callback) {
	this.__postJson("batch_management/get_batch/", request, callback);
};

/**
* Get batch address
*
* @param  {string}		request.campaign				-Required. 
* @param  {string}		request.account					-Required this or account_name. 
* @param  {string}		request.account_name			-Required this or account. Account name.
* @param  {string}		request.batchId					-Required.
* @param  {string}		request.address					-Required.
* @param  {Function}	callback(status, data)			-Required. Callback function.
* @returns {object}		data.address
* @returns {boolean}	status
*/
inConcertApi.prototype.GetBatchAddress = function(request, callback) {
	this.__postJson("batch_management/get_batch_address/", request, callback);
};

/**
* Get batch addresses
*
* @param  {string}		request.campaign				-Required. 
* @param  {string}		request.account					-Required this or account_name. 
* @param  {string}		request.account_name			-Required this or account. Account name.
* @param  {string}		request.batchId					-Required.
* @param  {array<string>} request.addresses				-Optional. Addresses list.
* @param  {Function}	callback(status, data)			-Required. Callback function.
* @returns {object}		data.addresses
* @returns {boolean}	status
*/
inConcertApi.prototype.GetBatchAddresses = function(request, callback) {
	this.__postJson("batch_management/get_batch_addresses/", request, callback);
};

/**
* Get Batch Addresses Status
* 
* @param  {string}		request.account					-Required 
* @param  {string}		request.batchId					-Required.
* @param  {string}		request.limit					-Optional.
* @param  {string}		request.numberPage				-Optional.
* @param  {Function}	callback(status, data)			-Required. Callback function.
* @returns {array}		data.rows						-A list of addresses.
* @returns {integer}	data.total_rows					-Total addresses number.
* @returns {boolean}	status
*/
inConcertApi.prototype.GetDetailBatchAddress = function(request, callback) {
	this.__postJson("batch_management/detail_address_batch/", request, callback);
};

/**
* Close batch
* 
* @param  {string}		request.account					-Required 
* @param  {string}		request.batchId					-Required.
* @param  {string}		request.campaign				-Required.
* @param  {string}		request.type					-Required.
* @param  {Function}	callback(status, data)			-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.CloseBatch = function(request, callback) {
	this.__postJson("batch_management/close_batch/", request, callback);
};


/**
* Search batch addresses
*
* @param  {string}		request.campaign				-Required. 
* @param  {string}		request.account					-Required this or account_name. 
* @param  {string}		request.account_name			-Required this or account. Account name.
* @param  {string}		request.batchId					-Required.
* @param  {array<string>} request.addresses				-Optional. Addresses list.
* @param  {object}		request.query					-Optional.
* @param  {string}		request.query.status			-Optional.
* @param  {int}			request.query.from_status_timestamp	-Optional.
* @param  {int}			request.query.to_status_timestamp	-Optional.
* @param  {integer}		request.limit					-Optional. Rows limit. Default: 1000.
* @param  {integer}		request.numberPage				-Optional. Rows page.
* @param  {string}		request.sort					-Optional. Default: address.
* @param  {boolean}		request.desc					-Optional.
* @param  {Function}	callback(status, data)			-Required. Callback function.
* @returns {array}		data.rows						-A list of addresses.
* @returns {integer}	data.total_rows					-Total addresses number.
* @returns {integer}	data.total_unfiltered_rows		-Total unfiltered addresses number.
*/
inConcertApi.prototype.SearchBatchAddresses = function(request, callback) {
	this.__postJson("batch_management/search_batch_addresses/", request, callback);
};

/**
* Search batches
*
* @param  {string}		request.view					-Optional. ALL (default), ACTIVE, SCHEDULED, ACTIVE_SCHEDULED, ARCHIVE
* @param  {object}		request.query					-Optional. 
* @param  {string}		request.query.type				-Optional. Channel type. Example: "WHATSAPP"
* @param  {string}		request.query.campaign			-Optional. 
* @param  {string}		request.query.account			-Optional this or account_name. 
* @param  {string}		request.query.account_name		-Optional this or account. Account name.
* @param  {string}		request.query.content			-Optional.
* @param  {integer}		request.from					-Optional. From row number. Default: 0.
* @param  {integer}		request.rows					-Optional. Page size. Default: 20.
* @param  {string}		request.sort					-Optional. Sort by row: publish_timestamp (default), last_message_timestamp
* @param  {boolean}		request.desc					-Optional. Sort desc. Default: false.
* @param  {Function}	callback(status, data)			-Required. Callback function.
* @returns {object}		data.batch
* @returns {boolean}	status
*/
inConcertApi.prototype.SearchBatches = function(request, callback) {
	this.__postJson("batch_management/search/", request, callback);
};

/**
* Create batch
*
* @param  {string}		request.type					-Required. Channel type. Example: "WHATSAPP"
* @param  {string}		request.campaign				-Required. 
* @param  {string}		request.account					-Required this or account_name. 
* @param  {string}		request.account_name			-Required this or account. Account name.
* @param  {string}		request.batchId					-Required.
* @param  {object}		request.message					-Optional. Example: { text : "my custom message" }
* @param  {string}		request.reportingGroup			-Optional. Address data reporting group. If is empty then address data won't be added to reports.
* @param  {array}		request.addresses				-Optional.
* @param  {object}		request.addresses[i]			
* @param  {string}		request.addresses[i].address	-Required.
* @param  {string}		request.addresses[i].name		-Optional. Address contact name.
* @param  {object}		request.addresses[i].message	-Optional. Custom address message (overrites batch message).
* @param  {object}		request.addresses[i].data		-Optional. Key value message data. Example: { user_id : "1234" }
* @param  {object}		request.addresses[i].agentData	-Optional. Key value data to be shown in agent view. Example: { user_country : "USA" }
* @param  {boolean}		request.async					-Optional. If true then don't wait for result
* @param  {integer}		request.timeout					-Optional. Timeout waiting for result. Default: 3600
* @param  {boolean}		request.validateActiveInt	    -Optional. Validate if the address has active interactions
* @param  {Function}	callback(status, data)			-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.CreateBatch = function(request, callback) {
	this.__postJson("batch_management/create_batch/", request, callback);
};

/**
* Add address to batch
*
* @param  {string}		request.type				-Required. Channel type. Example: "WHATSAPP"
* @param  {string}		request.campaign			-Required. 
* @param  {string}		request.account				-Required this or account_name. 
* @param  {string}		request.account_name		-Required this or account. Account name.
* @param  {string}		request.batchId				-Required.
* @param  {string}		request.address				-Required. Message address.
* @param  {string}		request.name				-Required. Message address contact name.
* @param  {object}		request.data				-Optional. Key value message data. Example: { user_id : "1234" }
* @param  {string}		request.reportingGroup		-Optional. Address data reporting group. If is empty then address data won't be added to reports.
* @param  {object}		request.agentData			-Optional. Key value data to be shown in agent view. Example: { user_country : "USA" }
* @param  {object}		request.message				-Optional. Custom message. Example: { text : "my custom message" }
* @param  {boolean}		request.async				-Optional. If true then don't wait for result
* @param  {integer}		request.timeout				-Optional. Timeout waiting for result. Default: 3600
* @param  {boolean}		request.validateActiveInt	-Optional. Validate if the address has active interactions
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.AddAddressToBatch = function(request, callback) {
	this.__postJson("batch_management/add_address_to_batch/", request, callback);
};

/**
* Send messages to multiple addresses for an existing batch or creates it.
*
* @param  {string}		request.type				-Required. Channel type. Example: "WHATSAPP"
* @param  {string}		request.campaign			-Required. 
* @param  {string}		request.account				-Required.
* @param  {string}		request.batchId				-Required.
* @param  {array}		request.addresses			-Required
* @param  {object}		request.addresses[i]			
* @param  {string}		request.addresses[i].address	-Required.
* @param  {string}		request.addresses[i].name		-Optional. Address contact name.
* @param  {object}		request.addresses[i].message	-Optional. Custom address message (overrites batch message).
* @param  {object}		request.addresses[i].data		-Optional. Key value message data. Example: { user_id : "1234" }
* @param  {object}		request.addresses[i].agentData	-Optional. Key value data to be shown in agent view. Example: { user_country : "USA" }
* @param  {object}		request.message					-Optional. Message to create a batch. Example: { text : "my custom message" }
* @param  {string}		request.message.msgtype			-Optional. Message type. Example "template"
* @param  {string}		request.message.text			-Optional. Text to send
* @param  {string}		request.message.template		-Optional. template data if the msgtype == "template"
* @param  {string}		request.message.body			-Optional. Body data. If the message type is not 'template'. For email for example
* @param  {string}		request.message.subject			-Optional. Subject. If the message type is not 'template'. For email for example
* @param  {boolean}		request.async					-Optional. If true then don't wait for result
* @param  {integer}		request.timeout					-Optional. Timeout waiting for result. Default: 3600
* @param  {boolean}		request.validateActiveInt		-Optional. Validate if the address has active interactions
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.AddAddressesToBatch = function(request, callback) {
	this.__postJson("batch_management/add_addresses_to_batch/", request, callback);
};

/**
* Makes a text message
*
* @param  {object}		request					-Required.
* @param  {string}		request.text			-Required. Message text.
* @return {object}		message					-Created message.
*/
inConcertApi.prototype.MakeTextMessage = function(request) {
	return {
		msgtype : "text",
		text : request.text || ""
	};
};

/**
* Get account template messages
*
* @param  {string}		request.type				-Required. Channel type. Example: "WHATSAPP"
* @param  {string}		request.campaign			-Required. 
* @param  {string}		request.account				-Required this or account_name. 
* @param  {string}		request.account_name		-Required this or account. Account name.
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.messages
*/
inConcertApi.prototype.GetAccountTemplateMessages = function(request, callback) {
	this.__postJson("campaigns/get_account_template_messages/", request, callback);
};

/**
* Makes a template message
*
* @param  {object}		request					-Required.
* @param  {string}		request.groupId			-Required. Template group.
* @param  {string}		request.id				-Required. Template id.
* @param  {string}		request.text			-Required. Template text.
* @param  {string}		request.language		-Required. Template language.
* @param  {array}		request.parameters		-Optional. Template parameters array.
* @return {object}		message					-Created message.
*/
inConcertApi.prototype.MakeTemplateMessage = function(request) {
	request = request || {};
	return {
		msgtype : "template",
		template : {
			text : request.text,
			group : request.groupId,
			id : request.id,
			language : request.language,
			parameters: request.parameters || [],
			hsm_buttons : request.hsm_buttons || []
		}
	};
};


/************************ VCCs ************************/

/**
* Get vcc
*
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.vcc
*/
inConcertApi.prototype.GetVCC = function(callback) {
	this.__get("vcc/get/", callback);
};

/************************ Groups ************************/

/**
* Get groups
*
* @param  {string}		permisssion					-Optional. Example: interaction
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.groups
*/
inConcertApi.prototype.GetGroups = function(permisssion, callback) {
	this.__get("groups/get/" + encodeURIComponent(permisssion), callback);
};

inConcertApi.prototype.GetGroupsExportInfo = function(callback) {
	this.__get("groups/get_export_info/", callback);
};

/************************ Users ************************/

/**
* Search vcc users
*
* @param  {object}		request
* @param  {object}		request.query				-Optional. Example: { Id : "my_user" }
* @param  {array}		request.columns				-Optional. Example: [ "Id", "Name" ]
* @param  {integer}		request.limit				-Optional. Rows limit.
* @param  {integer}		request.numberPage			-Optional. Rows page.
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {array}		data.rows					-A list of users.
* @returns {integer}	data.total_rows				-Total users number.
* @returns {integer}	data.total_unfiltered_rows	-Total unfiltered users number.
*/
inConcertApi.prototype.SearchUsers = function(request, callback) {
	this.__postJson("users/search/", request, callback);
};

/**
* Get user
*
* @param  {string}		userId						-Required. Example: user1
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.user
*/
inConcertApi.prototype.GetUser = function(userId, callback) {
	this.__get("users/get/" + encodeURIComponent(userId), callback);
};

inConcertApi.prototype.GetUsersExportInfo = function(callback) {
	this.__get("users/get_export_info/", callback);
};

/**
* Save user
*
* @param  {object}		user										-Required.
* @param  {boolean}		user.isNew									-Optional. Indicates if is a new user.
* @param  {boolean}		user.Password								-Required when is new.
* @param  {string}		user.Id										-Required.
* @param  {string}		user.Name									-Required.
* @param  {string}		user.FirstSurname							-Required.
* @param  {string}		user.LastName								-Optional.
* @param  {string}		user.LastSurname							-Optional.
* @param  {string}		user.Mail									-Optional.
* @param  {string}		user.TimeZoneId								-Required. Example: "UYT"
* @param  {string}		user.LanguageId								-Optional. Example: "en"
* @param  {string}		user.countryCode							-Required. Example: "598"
* @param  {string}		user.areaCode								-Optional.
* @param  {string}		user.UserGroup								-Required. Example: "agent"
* @param  {string}		user.WorkgroupId							-Optional.
* @param  {string}		user.DefaultCampaign						-Required when has interaction permisssions. Example: mycampaign@myvcc
* @param  {string}		user.initialState							-Optional. Example: "BREAK"
* @param  {object}		user.CompanyInformation						-Optional.
* @param  {string}		user.CompanyInformation.Location			-Optional.
* @param  {string}		user.CompanyInformation.Position			-Optional.
* @param  {string}		user.CompanyInformation.Organization		-Optional.
* @param  {object}		user.CustomInformation						-Optional.
* @param  {string}		user.CustomInformation.custom_agent_info_1	-Optional.
* @param  {string}		user.CustomInformation.custom_agent_info_2	-Optional.
* @param  {string}		user.CustomInformation.custom_agent_info_3	-Optional.
* @param  {string}		user.CustomInformation.custom_agent_info_4	-Optional.
* @param  {string}		user.CustomInformation.custom_agent_info_5	-Optional.
* @param  {array}		user.skills									-Optional. Example: [ { skill : "myskyll", weight : 10 } ]
* @param  {array}		user.groups									-Required when has interaction permisssions. Example: [ { id : "mycampaign@myvcc", role : "agent" } ]
* @param  {object}		user.custom_permissions						-Optional.
* @param  {string}		user.custom_permissions.has_override		-Optional. "1" or "0".
* @param  {object}		user.custom_permissions.apps				-Optional. Example: { settings : true }
* @param  {object}		user.custom_permissions.apps				-Optional. Example: { settings : { changenick : true } }
* @param  {object}		user.AllegroIntegration						-Optional. Agents with an assigned telephony peer.
* @param  {object}		user.AllegroIntegration.peer				-Optional. Agents with an assigned telephony peer.
* @param  {object}		user.AllegroIntegration.peer.service		-Required. Peer service.
* @param  {object}		user.AllegroIntegration.peer.peer			-Required. Peer number.
* @param  {object}		user.AllegroIntegration.peer.technology		-Required. Peer technology.
* @param  {Function}	callback(status, data)						-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.SaveUser = function(user, callback) {
	this.__postJson("users/save/", user, callback);
};

/**
* Activates user
*
* @param  {string}		userId						-Required. Example: user1
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.ActivateUser = function(userId, callback) {
	this.__postJson("users/activate/", { memberId : userId }, callback);
};

/**
* Deactivates user
*
* @param  {string}		userId						-Required. Example: user1
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.DeactivateUser = function(userId, callback) {
	this.__postJson("users/deactivate/", { memberId : userId }, callback);
};

/**
* Get available telephony peers
*
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {array}		data.peers
* @returns {object}		data.peers[i]
* @returns {string}		data.peers[i].service
* @returns {string}		data.peers[i].peer
* @returns {string}		data.peers[i].technology
*/
inConcertApi.prototype.GetAvailableTelephonyPeers = function(callback) {
	this.__get("users/get_available_telephony_peers/", callback);
};


/**
* Users States Detail Definitions

	API metodos por usuarios por estados (history)

	En el array "usersDynamicDetailDefinitions" debemos definir los diferentes canales, con el nombre de la funcion que queda expuesta en la clase JavaScript del API "fnName",
		como tambien el sufijo de la "url" que impactara contra el servidor.
*/

let usersDynamicDetailDefinitions = [
	//realtime
	{fnName: "UsersRealTimeLoggedIn"     , url: "users/realtime/logged_in/"},
	//states
	{fnName: "UsersStatesQueryByCampaign", url: "users/states/by_campaign/"},
	{fnName: "UsersStatesQueryByAgent"   , url: "users/states/by_agent/"   }
];

usersDynamicDetailDefinitions.forEach(element => {
	let fnName = element.fnName;
	let url = element.url;
	inConcertApi.prototype[fnName] = function(request, callback) {
		this.__postJson(url , request, callback);
	};
});


/************************ Campaigns ************************/

/**
* Search vcc campaigns
*
* @param  {object}		request
* @param  {object}		request.query				-Optional. Example: { Id : "my_campaign" }
* @param  {array}		request.columns				-Optional. Example: [ "Id", "Name" ]
* @param  {integer}		request.limit				-Optional. Rows limit.
* @param  {integer}		request.numberPage			-Optional. Rows page.
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {array}		data.rows					-A list of campaigns.
* @returns {integer}	data.total_rows				-Total campaigns number.
* @returns {integer}	data.total_unfiltered_rows	-Total unfiltered campaigns number.
*/
inConcertApi.prototype.SearchCampaigns = function(request, callback) {
	this.__postJson("campaigns/search/", request, callback);
};

/**
* Get campaign
*
* @param  {string}		campaignId					-Required. Example: campaign1
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.campaign
*/
inConcertApi.prototype.GetCampaign = function(campaignId, callback) {
	this.__get("campaigns/get/" + encodeURIComponent(campaignId), callback);
};

/**
* Get campaign dispositions
*
* @param  {string}		campaignId					-Required. Example: campaign1
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.dipositions
*/
inConcertApi.prototype.GetCampaignDispositions = function(campaignId, callback) {
	this.__get("campaigns/get_dispositions/" + encodeURIComponent(campaignId), callback);
};

/**
* Get vcc or campaign configured channels
*
* @param  {object}		request
* @param  {string}		request.campaign			-Optional. Filter by campaign. If not then all vcc campaigns.
* @param  {boolean}		request.enabled				-Enabled accounts only. Default: false.
* @param  {Function}	callback(channels)			-Required. Callback function.
* @returns {object}		channels					-A list of channels.
* @returns {object}		channels[channel]			-A list of campaign channels.
* @returns {array}		channels[channel][campaign] -A list of campaign channel accounts.
*/
inConcertApi.prototype.GetConfiguredChannels = function(request, callback) {
	this.__postJson("campaigns/get_configured_channels/", request, function(_, result) {
		callback(result.channels || {});
	});
};

/**
* Get campaign Members
* @param  {object}		request
* @param  {object}		request.campaigns				-Optional. Example: {campaigns: {"my_campaign1", "my_campaign2"} } 
**/
inConcertApi.prototype.GetCampaignMembers = function(request, callback) {
	this.__postJson("campaigns/get_campaign_members/", request, callback);
};

/**
* Get Campaigns With Speech Integrations
*	(las campaÃ±as que utilicen un determinado formato de evaluacion)
*
* @param  {string}			evaluationFormatId       -Required. Example: elformatodeevaluacion1
* @param  {Function}		callback(status, data)   -Required. Callback function.
* @returns {boolean}		status
* @returns {object}			data
* @returns {string}			data.vcc
* @returns {number}			data.totalCampaigns
* @returns {array string}	data.campaigns
* @returns {string}			data.evaluationFormatId
* @returns {string}			data.occRelationId
**/
inConcertApi.prototype.GetCampaignsWithSpeechIntegrations = function(evaluationFormatId, callback) {
	this.__get("campaigns/get_campaigns_with_speech_integrations/" + encodeURIComponent(evaluationFormatId), callback);
};

/************************ Interactions ************************/

/**
* Get interaction
*
* @param  {string}		interactionId				-Required.
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.interaction
*/
inConcertApi.prototype.GetInteraction = function(interactionId, callback) {
	this.__get("interactions/get/" + encodeURIComponent(interactionId), callback);
};
/**
* @param  {string}		id				-Required.
*/
inConcertApi.prototype.GetCompositeInteraction = function(id, callback) {
	this.__get("interactions/composite/get/" + encodeURIComponent(id), callback);
};

/**
* Get interaction media list
*
* @param  {string}		interactionId				-Required.
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {array}		data.media
* @returns {object}		data.media[i]
* @returns {string}		data.media[i].creator		-Required: client|agent|bot
* @returns {string}		data.media[i].agent			-creator=agent: sender agent
* @returns {string}		data.media[i].source		-Required: repository|url
* @returns {string}		data.media[i].name			-Optional: file name
* @returns {string}		data.media[i].format		-Optional: file format
* @returns {string}		data.media[i].type			-Optional: file type (image, audio, video, file)
* @returns {string}		data.media[i].id			-source=repository: file id
* @returns {string}		data.media[i].section		-source=repository: file section
* @returns {string}		data.media[i].part			-source=repository: file part
* @returns {string}		data.media[i].url			-source=url: file url
*/
inConcertApi.prototype.GetInteractionMediaList = function(interactionId, callback) {
	this.__get("interactions/media/list/" + encodeURIComponent(interactionId), callback);
};

/**
* Get interaction media repository file
*
* @param  {object}		request
* @param  {string}		request.interaction			-Required. Media interaction id.
* @param  {string}		request.id					-Required.
* @param  {integer}		request.section				-Optional. Default: -1.
* @param  {integer}		request.part				-Optional. Default: -1.
* @param  {Function}	callback(status, data)		-Required. Callback function.
*/
inConcertApi.prototype.GetInteractionMediaFile = function(request, callback) {
	var url = "interactions/media/get/" + this.__makeQueryString({
		interaction : request.interaction,
		id : request.id,
		section : (request.section !== null && request.section !== undefined && request.section !== "")? request.section : -1,
		part : (request.part !== null && request.part !== undefined)? request.part : -1
	});
	this.__get(url, callback);
};

/**
* Get Queue Interactions Info (total queue & estimated time) for now, only works for CALL
*
* @param  {object}		request
* @param  {string}		request.vcc					-Required.
* @param  {string}		request.type			    -Required.
* @param  {string}		request.campaign			-Required.
* @param  {string}		request.accountId		    -Optional. Default null
* @param  {string}		request.attLevelId			-Optional. Default null
* @param  {Function}	callback(status, data)		-Required. Callback function.
*/
inConcertApi.prototype.GetWaitingIntInfo = function(request, callback) {
	var url = "interactions/waitingint/get/info/" + this.__makeQueryString({
		vcc        : request.vcc,
		type       : request.type,
		campaign   : request.campaign,
		accountId  : (request.accountId  !== null && request.accountId  !== undefined) ? request.accountId  : "",
		attLevelId : (request.attLevelId !== null && request.attLevelId !== undefined) ? request.attLevelId : ""
	});
	this.__get(url, callback);
};

// /**
// *CCI_Get
// *
// * @param  {string}		campaignId					-Required. Example: campaign1
// * @param  {Function}	callback(status, data)		-Required. Callback function.
// * @returns {boolean}	status
// * @returns {object}		data
// * @returns {object}		data.campaign
// */
// inConcertApi.prototype.CCI_Get = function(campaignId, callback) {
// 	this.__get("cci/get/" + encodeURIComponent(campaignId), callback);
// };

/**
*interactions find
*
* @param  {string}		campaignId					-Required. Example: campaign1
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.campaign
*/
inConcertApi.prototype.interactions_find = function(request, callback) {
	this.__postJson("interactions/find/", request, callback);
//	this.__postJson("interactions/execute_otro/", request, callback);
};


/**
* Conversations Detail Definitions

	API metodos por interaccion de tipo "live" y "archive"

	En el array "conversationsDynamicDetailDefinitions" debemos definir los diferentes canales, con el nombre de la funcion que queda expuesta en la clase JavaScript del API "fnName",
		como tambien el sufijo de la "url" que impactara contra el servidor.
*/

let conversationsDynamicDetailDefinitions = [
// archive
	//facebook
	{fnName: "FacebookArchiveQueryByCampaign", url: "interactions/facebook/archive/by_campaign/" },
	{fnName: "FacebookArchiveQueryByAgent"   , url: "interactions/facebook/archive/by_agent/"    },
	//mail
	{fnName: "MailArchiveQueryByCampaign"    , url: "interactions/mail/archive/by_campaign/"     },
	{fnName: "MailArchiveQueryByAgent"       , url: "interactions/mail/archive/by_agent/"        },
	//twitter
	{fnName: "TwitterArchiveQueryByCampaign" , url: "interactions/twitter/archive/by_campaign/"  },
	{fnName: "TwitterArchiveQueryByAgent"    , url: "interactions/twitter/archive/by_agent/"     },
	//call
	{fnName: "CallArchiveQueryByCampaign"    , url: "interactions/call/archive/by_campaign/"  },
	{fnName: "CallArchiveQueryByAgent"       , url: "interactions/call/archive/by_agent/"     },
	//chat
	{fnName: "ChatArchiveQueryByCampaign"    , url: "interactions/chat/archive/by_campaign/"  },
	{fnName: "ChatArchiveQueryByAgent"       , url: "interactions/chat/archive/by_agent/"     },
	//Instagram
	{fnName: "InstagramArchiveQueryByCampaign"    , url: "interactions/instagram/archive/by_campaign/"  },
	{fnName: "InstagramArchiveQueryByAgent"       , url: "interactions/instagram/archive/by_agent/"     },
	//Whatsapp
	{fnName: "WhatsappArchiveQueryByCampaign"    , url: "interactions/whatsapp/archive/by_campaign/"  },
	{fnName: "WhatsappArchiveQueryByAgent"       , url: "interactions/whatsapp/archive/by_agent/"     },
	//Webcontacts
	{fnName: "WebcontactArchiveQueryByCampaign"    , url: "interactions/webcontact/archive/by_campaign/"  },
	{fnName: "WebcontactArchiveQueryByAgent"       , url: "interactions/webcontact/archive/by_agent/"     },
	//Voicemails
	{fnName: "VoicemailArchiveQueryByCampaign"    , url: "interactions/voicemail/archive/by_campaign/"  },
	{fnName: "VoicemailArchiveQueryByAgent"       , url: "interactions/voicemail/archive/by_agent/"     },
// live
	{fnName: "FacebookLiveQuery"             , url: "interactions/facebook/live/"                },
	{fnName: "MailLiveQuery"                 , url: "interactions/mail/live/"                    },
	{fnName: "TwitterLiveQuery"              , url: "interactions/twitter/live/"                 },
	{fnName: "CallLiveQuery"                 , url: "interactions/call/live/"                 },
	{fnName: "ChatLiveQuery"                 , url: "interactions/chat/live/"                 },
	{fnName: "InstagramLiveQuery"            , url: "interactions/instagram/live/"                 },
	{fnName: "WhatsappLiveQuery"             , url: "interactions/whatsapp/live/"                 },
	{fnName: "WebcontactLiveQuery"            , url: "interactions/webcontact/live/"                 },
	{fnName: "VoicemailLiveQuery"            , url: "interactions/voicemail/live/"                 }
];

conversationsDynamicDetailDefinitions.forEach(element => {
	let fnName = element.fnName;
	let url = element.url;
	inConcertApi.prototype[fnName] = function(request, callback) {
		this.__postJson(url , request, callback);
	};
});

/************************ Load Interactions Data ************************/

inConcertApi.prototype.loadInteractionsData = function(request, callback) {
const url = "interactions/load/";
this.__postJson(url , request, callback);
};

/************************ Repository ************************/

/**
* Upload file
*
* @param  {object}		request
* @param  {string}		request.id					-Optional. If not then a guid will be assigned and returned in callback.
* @param  {integer}		request.section				-Optional. Default: -1.
* @param  {file}		request.file				-Required.
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @param  {Function}	callback(percentage)		-Optional. Progress function.
*/
inConcertApi.prototype.AddFileToRepository = function(request, callback, progress) {
	var maxSize = this.__maxFileSize();
	if (request.file.size > maxSize * 1024 * 1024) {
		return callback(false, {
			status : false,
			reason : "file size must be less than " + maxSize + "MB"
		});
	}
	var id = request.id || this.GUID();
	var url = "repository/new/" + this.__makeQueryString({
		id : id,
		section : (request.section !== null && request.section !== undefined)? request.section : -1,
		contentType : request.file.type,
		format : this.__fileExtension(request.file.name) || "",
		fileName : request.file.name
	});
	this.__uploadFile(url, request.file, 
	function(status, result) {
		if (status) {
			callback(true, { id : id });
		}
		else {
			callback(false, result);
		}
	}, progress);
};

/**
* Upload content
*
* @param  {object}		request
* @param  {string}		request.id					-Optional. If not then a guid will be assigned and returned in callback.
* @param  {integer}		request.section				-Optional. Default: -1.
* @param  {string}		request.name				-Required. File name.
* @param  {string}		request.content				-Required. File content.
* @param  {string}		request.contentType			-Required. File content type.
* @param  {string}		request.format				-Optional. File format. Example: png.
* @param  {Function}	callback(status, data)		-Required. Callback function.
*/
inConcertApi.prototype.AddContentToRepository = function(request, callback) {
	var id = request.id || this.GUID();
	var url = "repository/new/" + this.__makeQueryString({
		id : id,
		section : (request.section !== null && request.section !== undefined)? request.section : -1,
		contentType : request.contentType,
		format : request.format || "",
		fileName : request.name
	});
	this.__post(url, request.content, request.contentType, function(status, result) {
		if (status) {
			callback(true, { id : id });
		}
		else {
			callback(false, result);
		}
	});
};

/**
* Get file
*
* @param  {object}		request
* @param  {string}		request.id					-Required.
* @param  {integer}		request.section				-Optional. Default: -1.
* @param  {Function}	callback(status, data)		-Required. Callback function.
*/
inConcertApi.prototype.GetFileFromRepository = function(request, callback) {
	var url = "repository/download/" + this.__makeQueryString({
		id : request.id,
		section : (request.section !== null && request.section !== undefined)? request.section : -1
	});
	this.__get(url, callback);
};

/**
* Get file
*
* @param  {object}		request
* @param  {string}		request.id					-Required.
* @param  {integer}		request.section				-Optional. Default: -1.
* @param  {Function}	callback(status, data)		-Required. Callback function.
*/
inConcertApi.prototype.ExistsFileInRepository = function(request, callback) {
	var url = "repository/is_found/" + this.__makeQueryString({
		id : request.id,
		section : (request.section !== null && request.section !== undefined)? request.section : -1
	});
	this.__get(url, callback);
};

/**
* Delete file
*
* @param  {object}		request
* @param  {string}		request.id					-Required.
* @param  {integer}		request.section				-Optional. Default: -1.
* @param  {Function}	callback(status, data)		-Required. Callback function.
*/
inConcertApi.prototype.DeleteFileFromRepository = function(request, callback) {
	this.__postJson("repository/erase/", request, callback);
};

/**
* Delete files
*
* @param  {object}		request
* @param  {array}		request.files				-Required.
* @param  {string}		request.files[].id			-Required.
* @param  {integer}		request.files[].section		-Optional. Default: -1.
* @param  {Function}	callback(status, data)		-Required. Callback function.
*/
inConcertApi.prototype.DeleteFilesFromRepository = function(request, callback) {
	this.__postJson("repository/batch_delete/", request, callback);
};

/************************ Webcontact ************************/

/**
* Start webcontact
*
* @param  {string}		request.campaign						-Required. 
* @param  {string}		request.account							-Required this or account_name. 
* @param  {string}		request.account_name					-Required this or account. Account name.
* @param  {string}		request.interactionId					-Optional. Interaction id. If not a guid will be assigned.
* @param  {object}		request.contact							-Optional.
* @param  {string}		request.contact.externalId				-Optional. Contact external id.
* @param  {string}		request.contact.name					-Optional. Contact name.
* @param  {array}		request.contact.addresses				-Optional. Contact addresses.
* @param  {string}		request.contact.addresses[i].type		-Required. Address type: MAIL, CELLULAR, HOME, OFFICE, TWITTER
* @param  {string}		request.contact.addresses[i].address	-Required. Address id (like mymail@mail.com or a phone number)
* @param  {string}		request.reportingGroup					-Optional. Reporting group. If is empty then reporting data won't be added to reports.
* @param  {array}		request.reportingData					-Optional. Data to be used in reports.
* @param  {string}		request.reportingData[i].name			-Required. Reporting item name.
* @param  {string}		request.reportingData[i].value			-Required. Reporting item value.
* @param  {array}		request.agentData						-Optional. Data to be used in agent integration.
* @param  {string}		request.agentData[i].name				-Required. Agent data item name.
* @param  {string}		request.agentData[i].value				-Required. Agent data item value.
* @param  {string}		request.agentData[i].displayName		-Optional. Agent data item display name (if not name is shown).
* @param  {boolean}		request.agentData[i].hidden				-Optional. Indicates if item is hidden in agent view.
* @param  {Function}	callback(statusagentData data)			-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.StartWebcontact = function(request, callback) {
	this.__postJson("webcontact/start/", request, callback);
};

/************************ Mail ************************/

/**
* Start mail
*
* @param  {string}		request.campaign						-Required. 
* @param  {string}		request.account							-Required this or account_name. 
* @param  {string}		request.account_name					-Required this or account. Account name.
* @param  {string}		request.interactionId					-Optional. Interaction id. If not a guid will be assigned.
* @param  {object}		request.head							-Required.
* @param  {string}		request.head.fromEMail					-Required. Email from address.
* @param  {string}		request.head.fromName					-Optional. Email from name.
* @param  {array}		request.head.cc							-Optional. Email CC array.
* @param  {array}		request.head.cc[i]						-Optional. Email CC item address.
* @param  {string}		request.head.subject					-Required. Email Subject
* @param  {string}		request.body							-Required. Email Body. Can contains html tags. Inline images can be:
                                                                              Image in request.attachments referenced by its position in it (src="attach_num:position")
																			  Image previously uploaded to repository with AddFileToRepository API (src="cid:fileId"
																			  Image referenced by a public URL (src= "https://...")
* @param  {array}		request.attachments						-Optional. Email Attachments. Array of File elements or undefined.
* @param  {Function}	callback(statusagentData data)			-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.StartMail = function(request, callback) {
	request.body = request.body.replace(/\\([\s\S])|(")/g, "\\$1$2").replace(/(?:\r\n|\r|\n)/g, '<br>');
	var attachmentsMetadata = [];
	if(request.attachments && request.attachments.length > 0)
	{
		Array.from(request.attachments).forEach(attachment => {
			var self = this;
			var requestFile = {};
			requestFile.file = attachment;
			requestFile.id = this.GUID();
			requestFile.section = "-1";
			this.AddFileToRepository(requestFile, function(status, data) { 
				if(status)
				{
					var attachmentMetadata = {};
					attachmentMetadata.name = attachment.name;
					attachmentMetadata.type = attachment.type;
					attachmentMetadata.id = data.id;
					attachmentMetadata["content-disposition"] = "attachment";
					attachmentMetadata.section = requestFile.section;
					attachmentsMetadata.push(attachmentMetadata);
					if(attachmentsMetadata.length === request.attachments.length)
					{
						request.attachments = attachmentsMetadata;
						self.__postJson("mail/start/", request, callback);
					}
				}
				else
				{
					callback({status:status, data:data});
				}
			});
		});
	}
	else
	{
		request.attachments = attachmentsMetadata;
		this.__postJson("mail/start/", request, callback);
	}
};

/************************ Whatsapp ************************/

/**
* Start Whatsapp
*
* @param  {string}		request.campaign						-Required. 
* @param  {string}		request.account							-Required this or account_name. 
* @param  {string}		request.account_name					-Required this or account. Account name.
* @param  {string}		request.interactionId					-Optional. Interaction id. If not a guid will be assigned.
* @param  {string}		request.name							-Required. Whatsapp Contact Name.
* @param  {string}		request.address							-Required. Whatsapp Address.
* @param  {string}		request.text							-Optional. Whatsapp Text message.
* @param  {array}		request.attachments						-Optional. Whatsapp Attachments. Array of File elements or undefined.
* @param  {Function}	callback(statusagentData data)			-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.StartWhatsapp = function(request, callback) {
	var attachmentsMetadata = [];
	if(request.attachments && request.attachments.length > 0)
	{
		Array.from(request.attachments).forEach(attachment => {
			var self = this;
			var requestFile = {};
			requestFile.file = attachment;
			requestFile.id = this.GUID();
			requestFile.section = "-1";
			this.AddFileToRepository(requestFile, function(status, data) { 
				if(status)
				{
					var attachmentMetadata = {};
					attachmentMetadata.name = attachment.name;
					attachmentMetadata.type = attachment.type;
					attachmentMetadata.id = data.id;
					attachmentMetadata["content-disposition"] = "attachment";
					attachmentsMetadata.push(attachmentMetadata);
					if(attachmentsMetadata.length === request.attachments.length)
					{
						request.attachments = attachmentsMetadata;
						self.__postJson("whatsapp/start/", request, callback);
					}
				}
				else
				{
					callback({status:status, data:data});
				}
			});
		});
	}
	else
	{
		request.attachments = attachmentsMetadata;
		this.__postJson("whatsapp/start/", request, callback);
	}
};


/************************ Vcc ************************/

// Get available Channels and States
inConcertApi.prototype.GetStates = function(callback) {
	this.__get("vcc/get/states/", callback);
};

inConcertApi.prototype.GetChannels = function(callback) {
	this.__get("vcc/get/channels/", callback);
};

/**
* Get Available Agents & slots by campaign@vcc
*
* @param  {object}		request
* @param  {string}		request.vcc					-Required.
* @param  {string}		request.campaign			-Required.
* @param  {string}		request.channel			    -Optional. Default null
* @param  {string}		request.idAttLevel			-Optional. Default null
* @param  {Function}	callback(status, data)		-Required. Callback function.
*/
inConcertApi.prototype.GetAvailableAgents = function(request, callback) {
	var url = "vcc/get/available_agents/" + this.__makeQueryString({
		vcc        : request.vcc,
		campaign   : request.campaign,
		channel    : (request.channel    !== null && request.channel    !== undefined) ? request.channel    : "",
		idAttLevel : (request.idAttLevel !== null && request.idAttLevel !== undefined) ? request.idAttLevel : ""
	});
	this.__get(url, callback);
};


/************************ Outbound Engine Process ************************/

/**
* Get process
*
* @param  {string}		processId					-Required. Example: process1@vcc
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.process
*/
inConcertApi.prototype.GetOutboundProcess = function(processId, callback) {
	this.__get("outbound_engine/process/get/" + encodeURIComponent(processId), callback);
};

/**
* Start process
*
* @param  {string}		processId					-Required. Example: process1@vcc
* @param  {Function}	callback(status, data)		-Required. Callback function.
*/
inConcertApi.prototype.StartOutboundProcess = function(processId, callback) {
	this.__postJson("outbound_engine/process/start/" + encodeURIComponent(processId), {}, callback);
};

/**
* Stop process
*
* @param  {string}		processId					-Required. Example: process1@vcc
* @param  {Function}	callback(status, data)		-Required. Callback function.
*/
inConcertApi.prototype.StopOutboundProcess = function(processId, callback) {
	this.__postJson("outbound_engine/process/stop/" + encodeURIComponent(processId), {}, callback);
};

/**
* Set manual rate available agents
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.  
* @param  {integer}		request.agents							-Required. Number of available agents.
* @param  {Function}	callback(status, data)					-Required. Callback function.
*/
inConcertApi.prototype.SetManualRateAvailableAgents = function(request, callback) {
	this.__postJson("outbound_engine/process/set_manual_rate_available_agents/", request, callback);
};


/************************ Outbound Engine Batch ************************/

/**
* Get batch action status
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.  
* @param  {string}		request.actionId						-Required. Id returned by any batch action.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.status
*/
inConcertApi.prototype.GetBatchActionStatus = function(request, callback) {
	this.__postJson("outbound_engine/batches/action_status/", request, callback);
};

/**
* Create Batch
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.  
* @param  {object}		request.configuration					-Required.
* @param  {string}		request.configuration.name				-Required.
* @param  {string}		request.configuration.startDate			-Optional.
* @param  {string}		request.configuration.endDate			-Optional.
* @param  {number}		request.configuration.batchPriority		-Optional.
* @param  {number}		request.configuration.priority			-Optional.
* @param  {number}		request.configuration.percentage		-Optional.
* @param  {string}		request.configuration.scheduleDate		-Optional.
* @param  {string}		request.configuration.agent				-Optional.
* @param  {string}		request.configuration.engineType		-Optional.
* @param  {string}		request.configuration.channel			-Optional.
* @param  {string}		request.configuration.kind				-Optional.
* @param  {boolean}		request.configuration.removeOldAddresses-Optional. Removes old contact adddress keeping just the addresses sent by this request
* @param  {object}		request.scope							-Required.
* @param  {string}		request.scope.contactId					-Optional.
* @param  {array}		request.scope.contacts					-Optional.
* @param  {string}		request.scope.contacts[].id				-Required.
* @param  {string}		request.scope.contacts[].scheduleDate	-Optional. Overrides configuration.
* @param  {string}		request.scope.contacts[].agent			-Optional. Overrides configuration.
* @param  {string}		request.scope.contacts[].engineType		-Optional. Overrides configuration.
* @param  {string}		request.scope.contacts[].channel		-Optional. Overrides configuration.
* @param  {string}		request.scope.contacts[].kind			-Optional. Overrides configuration.
* @param  {object}		request.scope.contacts[].address		-Optional. Overrides configuration.
* @param  {string}		request.scope.contacts[].contactData	-Optional. If present contact is stored as part of the operation. See StoreOutboundContacts for details.
* @param  {object}		request.scope.conditions				-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.CreateOutboundBatch = function(request, callback) {
	this.__postJson("outbound_engine/batches/create/", request, callback);
};

/**
* Update Batch
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.  
* @param  {string}		request.batchId							-Required.
* @param  {object}		request.configuration					-Required.
* @param  {string}		request.configuration.startDate			-Optional.
* @param  {string}		request.configuration.endDate			-Optional.
* @param  {number}		request.configuration.batchPriority		-Optional.
* @param  {number}		request.configuration.basePriority		-Optional.
* @param  {number}		request.configuration.percentage		-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.UpdateOutboundBatch = function(request, callback) {
	this.__postJson("outbound_engine/batches/update/", request, callback);
};

/**
* Destroy Batch
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.  
* @param  {string}		request.batchId							-Required.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.DestroyOutboundBatch = function(request, callback) {
	this.__postJson("outbound_engine/batches/destroy/", request, callback);
};

/**
* Add Contacts to Batch
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.
* @param  {string}		request.batchId							-Required.
* @param  {object}		request.configuration					-Required.
* @param  {string}		request.configuration.scheduleDate		-Optional.
* @param  {number}		request.configuration.priority			-Optional.
* @param  {string}		request.configuration.agent				-Optional.
* @param  {string}		request.configuration.engineType		-Optional.
* @param  {string}		request.configuration.channel			-Optional.
* @param  {string}		request.configuration.kind				-Optional.
* @param  {object}		request.configuration.address			-Optional.
* @param  {string}		request.configuration.address.Type		-Required. Example: "Phone"
* @param  {string}		request.configuration.address.Kind		-Optional. Default: "HOME"
* @param  {array}		request.configuration.address.Channels	-Optional. Default: ["CALL"]
* @param  {string}		request.configuration.address.Status	-Optional. Enabled|Disabled
* @param  {string}		request.configuration.address.Number	-Required.
* @param  {boolean}		request.configuration.removeOldAddresses-Optional. Removes old contact adddress keeping just the addresses sent by this request
* @param  {string}		request.configuration.contactData		-Optional. If present contact is stored as part of the operation. See StoreOutboundContacts for details.
* @param  {object}		request.scope							-Required.
* @param  {string}		request.scope.contactId					-Optional.
* @param  {array}		request.scope.contacts					-Optional.
* @param  {string}		request.scope.contacts[].id				-Required.
* @param  {string}		request.scope.contacts[].scheduleDate	-Optional. Overrides configuration.
* @param  {string}		request.scope.contacts[].agent			-Optional. Overrides configuration.
* @param  {string}		request.scope.contacts[].engineType		-Optional. Overrides configuration.
* @param  {string}		request.scope.contacts[].channel		-Optional. Overrides configuration.
* @param  {string}		request.scope.contacts[].kind			-Optional. Overrides configuration.
* @param  {object}		request.scope.contacts[].address		-Optional. Overrides configuration.
* @param  {string}		request.scope.contacts[].contactData	-Optional. If present contact is stored as part of the operation. See StoreOutboundContacts for details.
* @param  {object}		request.scope.conditions				-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.AddContactsToOutboundBatch = function(request, callback) {
	this.__postJson("outbound_engine/batches/add_contacts/", request, callback);
};

/**
* Pause Contacts
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.
* @param  {object}		request.scope							-Required.
* @param  {string}		request.scope.batchId					-Optional.
* @param  {string}		request.scope.contactId					-Optional.
* @param  {array}		request.scope.contacts					-Optional.
* @param  {string}		request.scope.contacts[].id				-Required.
* @param  {object}		request.scope.conditions				-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.PauseOutboundContacts = function(request, callback) {
	this.__postJson("outbound_engine/batches/pause/", request, callback);
};

/**
* Resume Contacts
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.
* @param  {object}		request.scope							-Required.
* @param  {string}		request.scope.batchId					-Optional.
* @param  {string}		request.scope.contactId					-Optional.
* @param  {array}		request.scope.contacts					-Optional.
* @param  {string}		request.scope.contacts[].id				-Required.
* @param  {object}		request.scope.conditions				-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.ResumeOutboundContacts = function(request, callback) {
	this.__postJson("outbound_engine/batches/resume/", request, callback);
};

/**
* Cancel Contacts
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.
* @param  {object}		request.scope							-Required.
* @param  {string}		request.scope.batchId					-Optional.
* @param  {string}		request.scope.contactId					-Optional.
* @param  {array}		request.scope.contacts					-Optional.
* @param  {string}		request.scope.contacts[].id				-Required.
* @param  {object}		request.scope.conditions				-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.CancelOutboundContacts = function(request, callback) {
	this.__postJson("outbound_engine/batches/cancel/", request, callback);
};

/**
* Finish Contacts
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.
* @param  {object}		request.scope							-Required.
* @param  {string}		request.scope.batchId					-Optional.
* @param  {string}		request.scope.contactId					-Optional.
* @param  {array}		request.scope.contacts					-Optional.
* @param  {string}		request.scope.contacts[].id				-Required.
* @param  {object}		request.scope.conditions				-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.FinishOutboundContacts = function(request, callback) {
	this.__postJson("outbound_engine/batches/finish/", request, callback);
};

/**
* Remove Contacts
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.
* @param  {object}		request.scope							-Required.
* @param  {string}		request.scope.batchId					-Optional.
* @param  {string}		request.scope.contactId					-Optional.
* @param  {array}		request.scope.contacts					-Optional.
* @param  {string}		request.scope.contacts[].id				-Required.
* @param  {object}		request.scope.conditions				-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.RemoveOutboundContacts = function(request, callback) {
	this.__postJson("outbound_engine/batches/remove/", request, callback);
};

/**
* Recycle Contacts
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.
* @param  {object}		request.configuration					-Required.
* @param  {boolean}		request.configuration.restartRetries	-Optional.
* @param  {string}		request.configuration.scheduleDate		-Optional.
* @param  {string}		request.configuration.agent				-Optional.
* @param  {object}		request.scope							-Required.
* @param  {string}		request.scope.batchId					-Optional.
* @param  {string}		request.scope.contactId					-Optional.
* @param  {array}		request.scope.contacts					-Optional.
* @param  {string}		request.scope.contacts[].id				-Required.
* @param  {object}		request.scope.conditions				-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.RecycleOutboundContacts = function(request, callback) {
	this.__postJson("outbound_engine/batches/recycle/", request, callback);
};

/**
* Schedule Contacts
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.
* @param  {object}		request.configuration					-Required.
* @param  {boolean}		request.configuration.restartRetries	-Optional.
* @param  {string}		request.configuration.scheduleDate		-Optional.
* @param  {string}		request.configuration.agent				-Optional.
* @param  {string}		request.configuration.channel			-Optional.
* @param  {string}		request.configuration.kind				-Optional.
* @param  {object}		request.configuration.address			-Optional.
* @param  {string}		request.configuration.address.Type		-Required. Example: "Phone"
* @param  {string}		request.configuration.address.Kind		-Optional. Default: "HOME"
* @param  {array}		request.configuration.address.Channels	-Optional. Default: ["CALL"]
* @param  {string}		request.configuration.address.Status	-Optional. Enabled|Disabled
* @param  {string}		request.configuration.address.Number	-Required.
* @param  {object}		request.scope							-Required.
* @param  {string}		request.scope.batchId					-Optional.
* @param  {string}		request.scope.contactId					-Optional.
* @param  {array}		request.scope.contacts					-Optional.
* @param  {string}		request.scope.contacts[].id				-Required.
* @param  {string}		request.scope.contacts[].scheduleDate	-Optional. Overrides configuration.
* @param  {string}		request.scope.contacts[].agent			-Optional. Overrides configuration.
* @param  {string}		request.scope.contacts[].engineType		-Optional. Overrides configuration.
* @param  {string}		request.scope.contacts[].channel		-Optional. Overrides configuration.
* @param  {string}		request.scope.contacts[].kind			-Optional. Overrides configuration.
* @param  {object}		request.scope.contacts[].address		-Optional. Overrides configuration.
* @param  {object}		request.scope.conditions				-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.ScheduleOutboundContacts = function(request, callback) {
	this.__postJson("outbound_engine/batches/schedule/", request, callback);
};

/**
* Change Priority Contacts
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.
* @param  {number}		request.priority						-Optional.
* @param  {object}		request.scope							-Required.
* @param  {string}		request.scope.batchId					-Optional.
* @param  {string}		request.scope.contactId					-Optional.
* @param  {array}		request.scope.contacts					-Optional.
* @param  {string}		request.scope.contacts[].id				-Required.
* @param  {object}		request.scope.conditions				-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.ChangePriorityOutboundContacts = function(request, callback) {
	this.__postJson("outbound_engine/batches/change_priority/", request, callback);
};

/**
* Set Engine Type Contacts
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.
* @param  {string}		request.engineType						-Optional.
* @param  {object}		request.scope							-Required.
* @param  {string}		request.scope.batchId					-Optional.
* @param  {string}		request.scope.contactId					-Optional.
* @param  {array}		request.scope.contacts					-Optional.
* @param  {string}		request.scope.contacts[].id				-Required.
* @param  {object}		request.scope.conditions				-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.SetEngineTypeOutboundContacts = function(request, callback) {
	this.__postJson("outbound_engine/batches/set_engine_type/", request, callback);
};

/**
* Set Agent Contacts
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.
* @param  {string}		request.agent							-Required.
* @param  {object}		request.scope							-Required.
* @param  {string}		request.scope.batchId					-Optional.
* @param  {string}		request.scope.contactId					-Optional.
* @param  {array}		request.scope.contacts					-Optional.
* @param  {string}		request.scope.contacts[].id				-Required.
* @param  {object}		request.scope.conditions				-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.SetAgentOutboundContacts = function(request, callback) {
	this.__postJson("outbound_engine/batches/set_agent/", request, callback);
};

/**
* Get outbound engine action status
*
* @param  {string}		request.aciontId						-Required. 
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.GetOutboundEngineActionStatus = function(request, callback) {
	this.__postJson("outbound_engine/batches/action_status/", request, callback);
};


/************************ Outbound Engine Contact ************************/

/**
* Get Contact
*
* @param  {string}		contactId					-Required
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.contact
*/
inConcertApi.prototype.GetOutboundContact = function(contactId, callback) {
	this.__get("outbound_engine/contacts/get/" + encodeURIComponent(contactId), callback);
};

/**
* Get Contact by Name
*
* @param  {string}		contactName					-Required.
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.contact
*/
inConcertApi.prototype.GetOutboundContactByName = function(name, callback) {
	this.__postJson("outbound_engine/contacts/get_by_name/", { name }, callback);
};

/**
* Get Contact by Address
*
* @param  {string}		addressType					-Required. Example: Phone
* @param  {string}		address						-Required.
* @param  {Function}	callback(status, data)		-Required. Callback function.
* @returns {boolean}	status
* @returns {object}		data
* @returns {object}		data.contact
*/
inConcertApi.prototype.GetOutboundContactByAddress = function(addressType, address, callback) {
	this.__postJson(
		"outbound_engine/contacts/get_by_address/", { addressType, address }, callback
	);
};

/**
* List Contacts
*
* @param  {array<string>}	request.contactIds					-Required. 
* @param  {boolean}			request.fullData					-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {object}		data.contacts
*/
inConcertApi.prototype.ListOutboundContacts = function(request, callback) {
	this.__postJson("outbound_engine/contacts/list/", request, callback);
};

/**
* Search Contacts
*
* @param  {string}		request.processId						-Optional. 
* @param  {string}		request.campaignId						-Optional.
* @param  {string}		request.batchId							-Optional.
* @param  {string}		request.skillId							-Optional.
* @param  {number}		request.size							-Optional.
* @param  {object}		request.conditions						-Optional.
* @param  {string}		request.pit								-Optional. Pagination
* @param  {string}		request.searchAfter						-Optional. Pagination
* @param  {boolean}		request.keepOpen						-Optional. Pagination
* @param  {object}		request.sortTerms						-Optional.
* @param  {string}		request.sortTerms[].name				-Required.
* @param  {string}		request.sortTerms[].order				-Optional. asc|desc
* @param  {array<string>} request.fields						-Optional. Retrieve specific fields
* @param  {boolean}		request.fullData						-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {object}		data.contacts
*/
inConcertApi.prototype.SearchOutboundContacts = function(request, callback) {
	this.__postJson("outbound_engine/contacts/search/", request, callback);
};

/**
* Summary Contacts
*
* @param  {string}		request.processId						-Optional. 
* @param  {string}		request.campaignId						-Optional.
* @param  {string}		request.batchId							-Optional.
* @param  {string}		request.skillId							-Optional.
* @param  {object}		request.conditions						-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {object}		data.summary
*/
inConcertApi.prototype.SummaryOutboundContacts = function(request, callback) {
	this.__postJson("outbound_engine/contacts/summary/", request, callback);
};

/**
* Store Contacts
*
* @param  {boolean}		request.mix										-Optional. Mix or replace arrays (skills, addresses...)
* @param  {array}		request.contacts								-Required. 
* @param  {string}		request.contacts[].Id							-Optional.
* @param  {string}		request.contacts[].Name							-Optional.
* @param  {string}		request.contacts[].Campaign						-Optional.
* @param  {string}		request.contacts[].Category						-Optional.
* @param  {string}		request.contacts[].AccountOfficer				-Optional.
* @param  {string}		request.contacts[].ExternalId					-Optional.
* @param  {string}		request.contacts[].ImportId						-Optional.
* @param  {string}		request.contacts[].AccountGroup					-Optional.
* @param  {bollean}		request.contacts[].IsClient						-Optional.
* @param  {string}		request.contacts[].ManualScheduleDate			-Optional.
* @param  {string}		request.contacts[].Agent						-Optional.
* @param  {string}		request.contacts[].Addresses					-Optional.
* @param  {string}		request.contacts[].Addresses[].Type				-Required. Example: "Phone"
* @param  {string}		request.contacts[].Addresses[].Kind				-Optional. Default: "HOME"
* @param  {array}		request.contacts[].Addresses[].Channels			-Optional. Default: ["CALL"]
* @param  {string}		request.contacts[].Addresses[].Status			-Optional. Enabled|Disabled
* @param  {string}		request.contacts[].Addresses[].Number			-Required.
* @param  {string}		request.contacts[].Skills						-Optional.
* @param  {string}		request.contacts[].Skills[].SkillId				-Required.
* @param  {string}		request.contacts[].Skills[].Mode				-Optional. SORT|REQUIRED|PREFERRED
* @param  {number}		request.contacts[].Skills[].Value				-Required.
* @param  {string}		request.contacts[].NameValuesSimple				-Optional.
* @param  {string}		request.contacts[].NameValuesSimple[].Name		-Required.
* @param  {string}		request.contacts[].NameValuesSimple[].Value		-Required.
* @param  {string}		request.contacts[].NameValuesSearchDate			-Optional.
* @param  {string}		request.contacts[].NameValuesSearchDate[].Name	-Required.
* @param  {string}		request.contacts[].NameValuesSearchDate[].Value	-Required.
* @param  {string}		request.contacts[].NameValuesSearchNumeric		-Optional.
* @param  {string}		request.contacts[].NameValuesSearchNumeric[].Name	-Required.
* @param  {number}		request.contacts[].NameValuesSearchNumeric[].Value	-Required.
* @param  {string}		request.contacts[].NameValuesSearchText			-Optional.
* @param  {string}		request.contacts[].NameValuesSearchText[].Name	-Required.
* @param  {string}		request.contacts[].NameValuesSearchText[].Value	-Required.
* @param  {Function}	callback(status, data)							-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.StoreOutboundContacts = function(request, callback) {
	this.__postJson("outbound_engine/contacts/store/", request, callback);
};

/**
* Store Contact addresses
*
* @param  {array}		request.mix										-Optional. Mix or replace arrays (skills, addresses...)
* @param  {array}		request.contactId								-Required. 
* @param  {string}		request.addresses								-Required.
* @param  {string}		request.addresses[].Type						-Required. Example: "Phone"
* @param  {string}		request.addresses[].Kind						-Optional. Default: "HOME"
* @param  {array}		request.addresses[].Channels					-Optional. Default: ["CALL"]
* @param  {string}		request.addresses[].Status						-Optional. Enabled|Disabled
* @param  {string}		request.addresses[].Number						-Required.
* @param  {Function}	callback(status, data)							-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.SetContactAddresses = function(request, callback) {
	this.__postJson("outbound_engine/contacts/set_addresses/", request, callback);
};

/**
* Store Contact Values
*
* @param  {array}		request.mix										-Optional. Mix or replace arrays (skills, addresses...)
* @param  {array}		request.contactId								-Required. 
* @param  {string}		request.simple									-Optional.
* @param  {string}		request.simple[].Name							-Required.
* @param  {string}		request.simple[].Value							-Required.
* @param  {string}		request.date									-Optional.
* @param  {string}		request.date[].Name								-Required.
* @param  {string}		request.date[].Value							-Required.
* @param  {string}		request.numeric									-Optional.
* @param  {string}		request.numeric[].Name							-Required.
* @param  {number}		request.numeric[].Value							-Required.
* @param  {string}		request.text									-Optional.
* @param  {string}		request.text[].Name								-Required.
* @param  {string}		request.text[].Value							-Required.
* @param  {Function}	callback(status, data)							-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.SetContactValues = function(request, callback) {
	this.__postJson("outbound_engine/contacts/set_values/", request, callback);
};

/**
* Store Contact Skills
*
* @param  {array}		request.mix										-Optional. Mix or replace arrays (skills, addresses...)
* @param  {array}		request.contactId								-Required. 
* @param  {string}		request.skills									-Required.
* @param  {string}		request.skills[].SkillId						-Required.
* @param  {string}		request.skills[].Mode							-Optional. SORT|REQUIRED|PREFERRED
* @param  {number}		request.skills[].Value							-Required.
* @param  {Function}	callback(status, data)							-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.SetContactSkills = function(request, callback) {
	this.__postJson("outbound_engine/contacts/set_skills/", request, callback);
};

/**
* Store Contact addresses
*
* @param  {array}		request.contactId								-Required. 
* @param  {string}		request.addressType								-Required. Example: "Phone"
* @param  {string}		request.address									-Required.
* @param  {Function}	callback(status, data)							-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.RemoveContactAddress = function(request, callback) {
	this.__postJson("outbound_engine/contacts/remove_address/", request, callback);
};

/**
* Enable Contact addresses
*
* @param  {array}		request.contactId								-Required. 
* @param  {string}		request.addressType								-Required. Example: "Phone"
* @param  {string}		request.address									-Required.
* @param  {Function}	callback(status, data)							-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.EnableContactAddress = function(request, callback) {
	this.__postJson("outbound_engine/contacts/enable_address/", request, callback);
};

/**
* Disable Contact addresses
*
* @param  {array}		request.contactId								-Required. 
* @param  {string}		request.addressType								-Required. Example: "Phone"
* @param  {string}		request.address									-Required.
* @param  {Function}	callback(status, data)							-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.DisableContactAddress = function(request, callback) {
	this.__postJson("outbound_engine/contacts/disable_address/", request, callback);
};

/**
* Reschedule Contact
*
* @param  {string}		request.processId						-Required this or campaign id. 
* @param  {string}		request.campaignId						-Required this or process id.
* @param  {string}		request.batchId							-Required.
* @param  {string}		request.contactId						-Required.
* @param  {object}		request.configuration					-Required.
* @param  {boolean}		request.configuration.restartRetries	-Optional.
* @param  {string}		request.configuration.scheduleDate		-Optional.
* @param  {string}		request.configuration.agent				-Optional.
* @param  {string}		request.configuration.channel			-Optional.
* @param  {string}		request.configuration.kind				-Optional.
* @param  {object}		request.configuration.address			-Optional.
* @param  {string}		request.configuration.address.Type		-Required. Example: "Phone"
* @param  {string}		request.configuration.address.Kind		-Optional. Default: "HOME"
* @param  {array}		request.configuration.address.Channels	-Optional. Default: ["CALL"]
* @param  {string}		request.configuration.address.Status	-Optional. Enabled|Disabled
* @param  {string}		request.configuration.address.Number	-Required.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.RescheduleOutboundContact = function(request, callback) {
	this.__postJson("outbound_engine/contacts/reschedule/", request, callback);
};

/**
* Get Contact Results
*
* @param  {string}		request.contactId						-Required.
* @param  {string}		request.processId						-Optional. 
* @param  {string}		request.campaignId						-Optional.
* @param  {string}		request.batchId							-Optional.
* @param  {string}		request.contactId						-Optional.
* @param  {object}		request.conditions						-Optional.
* @param  {object}		request.sortTerms						-Optional.
* @param  {string}		request.sortTerms[].name				-Required.
* @param  {string}		request.sortTerms[].order				-Optional. asc|desc
* @param  {number}		request.size							-Optional.
* @param  {number}		request.offset							-Optional.
* @param  {Function}	callback(status, data)					-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.GetOutboundContactResults = function(request, callback) {
	this.__postJson("outbound_engine/contacts/get_results/", request, callback);
};

/**
* Validate Address
*
* @param  {string}		request.type						-Required. Example: "Phone"
* @param  {string}		request.number						-Required.
* @param  {string}		request.country						-Optional.
* @param  {string}		request.area						-Optional.
* @param  {Function}	callback(status, data)				-Required. Callback function.
* @returns {boolean}	status
*/
inConcertApi.prototype.ValidateAddress = function(request, callback) {
	this.__postJson("outbound_engine/contacts/validate_address/", request, callback);
};
