/* jshint smarttabs:true, quotmark:double, esversion: 11 */
/* global $, inConcert */


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";
	const FORMAT = inConcert.Utils.format;
	const _T = inConcert.i18n.getString;
	const REQUEST_TIMEOUT_MS = 300000; // 5 minutes

	function SharedFilesAPI(params)
	{
		if(typeof(params) !== "object") {
			throw "SharedFilesAPI.Constructor - expected an object with configuration";
		}
		if(typeof(params.uri ) !== "string") {
			throw "SharedFilesAPI.Constructor - base URI is required";
		}

		this.uri = params.uri + "/shared_files";
		this.cached_cultures = null;
	}

	if (typeof inConcert.SharedFilesAPI !== "function") {
		inConcert.SharedFilesAPI = SharedFilesAPI;
	}

/********************************************************************************************************
	FUNCIONES GENERICAS
********************************************************************************************************/

	/*
	
	*/
	SharedFilesAPI.prototype.__Get = function(uri, callback)
	{
		if (typeof(uri) !== "string") {
			throw "SharedFilesAPI.prototype.__Get: uri variable must be string type.";
		}
		if (typeof(callback) !== "function") {
			throw "SharedFilesAPI.prototype.__Get: callback variable must be function type.";
		}

		var request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: REQUEST_TIMEOUT_MS,
			success: function (response)    {callback(true, response);},
			error  : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	SharedFilesAPI.prototype.__Post = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw "SharedFilesAPI.prototype.__Post: uri variable must be string type.";
		}
		if (typeof(callback) !== "function") {
			throw "SharedFilesAPI.prototype.__Post: callback variable must be function type.";
		}
		if (typeof(data) !== "object") {
			throw "SharedFilesAPI.prototype.__Post: data variable must be object type.";
		}

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

/********************************************************************************************************

********************************************************************************************************/
	/*
	
	*/
	SharedFilesAPI.prototype.getUsedSpace = function(callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/shared_files/get_used_space/
		 */
		let uri = FORMAT("{0}/get_used_space/",
			this.uri
		);

		var request_handler = this.__Get(uri, function(success, result) {
			if (success) {
				callback(true, result.used_space);
				return;
			}
			// error
			callback(false, result);
		});
		return request_handler;
	};

	/*
	
	*/
	SharedFilesAPI.prototype.getFile = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/shared_files/get/F2089C37B7C6B13E58D6A9315ED91702
		 */
		let uri = FORMAT("{0}/get/{1}",
			this.uri,
			encodeURIComponent(params.fileId || "")
		);

		var request_handler = this.__Get(uri, function(success, file) {
			if (success) {
				callback(true, file);
				return;
			}
			// error
			callback(false, file);
		});
		return request_handler;
	};

	/*
	
	*/
	SharedFilesAPI.prototype.saveFile = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/shared_files/save/
		 */
		let uri = FORMAT("{0}/save/", this.uri);

		if (data.Id) {
			data = { [data.Id] : data };
		}
		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	SharedFilesAPI.prototype.validateFile = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/shared_files/save/
		 */
		let uri = FORMAT("{0}/validate/", this.uri);

		if (data.Id) {
			data = { [data.Id] : data };
		}
		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	SharedFilesAPI.prototype.deleteFiles = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/shared_files/delete/
		 */
		let uri = FORMAT("{0}/delete/", this.uri);

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	SharedFilesAPI.prototype.searchFiles = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/shared_files/search/
		 */
		let uri = FORMAT("{0}/search/", this.uri);

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	SharedFilesAPI.prototype.getFilesByIds = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/shared_files/get_files_by_ids/
		 */
		let uri = FORMAT("{0}/get_files_by_ids/", this.uri);

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	SharedFilesAPI.prototype.downloadFile = function(fileId, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/shared_files/download/
		 */
		let uri = FORMAT("{0}/download/{1}", this.uri, fileId);
		window.open(uri, "_blank");
		callback();
	};

	/*

	*/
	SharedFilesAPI.prototype.exportFile = function(fileId, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/shared_files/export/
		 */
		let uri = FORMAT("{0}/export/{1}", this.uri, fileId);
		window.open(uri, "_blank");
		callback();
	};

	/*
	
	*/
	SharedFilesAPI.prototype.canExport = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/shared_files/can_export/
		 */
		let uri = FORMAT("{0}/can_export/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	SharedFilesAPI.prototype.endedCompressedExport = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/shared_files/ended_compressed_export/
		 */
		let uri = FORMAT("{0}/ended_compressed_export/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	SharedFilesAPI.prototype.exportFilesToCompressed = function(data, callback = $.noop)
	{
		let uri = FORMAT("{0}/export_to_compressed/", this.uri);
		$.fileDownload(uri, {
			httpMethod: "GET",
			cookieName: "exported_started",
			data      : {
				data: JSON.stringify(data)
			}
		}).done(function() {
			callback(true);
		}).fail(function(response) {
			callback(false, response);
		});
	};

	/*
	
	*/
	SharedFilesAPI.prototype.startExportFilesToCompressed = function(files, callback = $.noop)
	{
		const self = this;
		let files_info = {
			files: files
		};

		const bytes_to_mb = function(bytes) {
			let mb = parseInt((bytes / 1024) / 1024, 10);
			return mb;
		};
		const make_err_msg = function (err) {
			switch(err.code) {
				case "EXPORT EXCEEDS THE LIMIT OF FILES SUPPORTED":
					err.message = FORMAT(_T("The export must not exceed {0} files. The current export contains {1} files."), err.maxNumberOfFilesInExport, err.totalRows);
				break;
				case "EXPORT EXCEEDS THE COMPRESS FILE SIZE LIMIT SUPPORTED":
					err.message = FORMAT(_T("The export ZIP must not exceed {0} MB. The current export ZIP is of {1} MB."), bytes_to_mb(err.maxFileSizeBytes), bytes_to_mb(err.fileSizeBytes));
				break;
				case "SHARED FILE NOT FOUND":
					err.message = FORMAT(_T("Shared file {0} not found."), err.file);
				break;
				case "REPOSITORY FILE NOT FOUND":
					err.message = FORMAT(_T("Shared file {0} not exist in Repository with Id {1}."), err.file, err.repositoryId);
				break;
				case "UNABLE TO COMPRESS FILES TO EXPORT":
					err.message = _T("Unable to compress files to export.");
				break;
				case "THERE ANOTHER PROCESS BATCH IN PROGRESS":
					err.message = _T("Unable to complete action. There is another batch process in progress.");
				break;
				default:
					err.message = _T("Unable to export files.");
			}
			return err;
		};

		this.canExport(files_info, function(success, response) {
			if (success) {
				let zip_file_info = response;
				self.exportFilesToCompressed(zip_file_info, function(success, err) {
					if (!success) {
						callback(success, err);
						return;
					}
					self.endedCompressedExport(zip_file_info, callback);
				});
				return;
			}
			let err = make_err_msg(response);
			callback(false, err);
		});
	};

}());
