module(..., package.seeall)

local dsbase = require (tweety_modules_prefix .. "datasourceBase")
local config = require (tweety_modules_prefix .. "config")
local sqlconnector = require (tweety_modules_prefix .. "sqlconnector")
local cassandra = require "lua_cassandra"
function msSQLDatasource(self)
	self = dsbase.datasourceBase(self)

	local function loadDataSourceBuiltin()
		local mysql_config = config.mysql
		if not mysql_config or not mysql_config.ndb_nodes then
			return
		end
		
		local ndb_nodes_table = {}
		for key, value, state in mysql_config["ndb_nodes"]:childs() do
			if state ~= "deleted" then
				table.insert(ndb_nodes_table, {host = value.host })
			end
		end
		-- validar que existe al menos un host
		if ndb_nodes_table[1] and mysql_config.ndb_nodes[1].host then
			return {
				ndb_nodes = ndb_nodes_table,
				Username = mysql_config.user or "root",
				Password = mysql_config.password or "",
				Port = mysql_config.port or 3306,
				Type = "mysql"
			}
		end
	end
	
	local getConnection = function (private)
	
		-- Si no es bultin el dbms, pues conecto con la config pasada por parametro
		if private.dbms ~= "builtin" then
			return sqlconnector.connect(private)
		end
	
		-- DMBS builtin, busco parametros de conexion configurados
		--1 valido que existan datos del config de host
		local dataSource = loadDataSourceBuiltin()
		if not dataSource then
			return
		end
	
		--2 Agrego al obj data los datos para bultin configurados
		private.userName = dataSource.Username
		private.userPassword = dataSource.Password
		private.port = dataSource.Port
		private.dbms = dataSource.Type
		if private.databaseName then
			private.databaseName = self.vccName .. "_" .. private.databaseName
			local connection = cassandra:CassandraConnection("Agent")
			local dbUserId = connection:GetColumnValue(private.databaseName, "Databases", "UserId")
			if dbUserId then
				private.userName = dbUserId
			else
				return nil, "User Id for database name not found"
			end
		end
		-- obtengo el primer nodo que este disponible
		local ndb_nodes =  dataSource.ndb_nodes
		local connectionData
		for pos, value in ipairs(ndb_nodes) do
			private.serverAddress = value.host
			connectionData = sqlconnector.connect(private)
			if connectionData then
				return connectionData
			end
			--Si no conecta, desconecto y pruebo con el sigt nodo. Si es testConections
			-- y es la validacion del ultimo nodo, retorno obj de conexion
			if pos ~= #ndb_nodes then
				sqlconnector.closeConnection(connectionData)
			end
		end
		return connectionData
	end

	function self:getDriverType()
		return self.private.dbms or ""
	end

	function self:getConnection()
		--LogDebug("[msSQLDatasource:getConnection] self.private: %s", json.encode(self.private))
		return getConnection(self.private)
	end

	function self:closeConnection(connection)
		sqlconnector.closeConnection(connection)
	end

	function self:query(connection, query, options)
		return sqlconnector.query(connection, query, options)
	end

	--[[
		Esta funcion es utilizada solo para MySQL ya que el LOCK / UNLOCK de un query NO lo puede realizar en el mismo paso como si lo realiza SQL SERVER.
			Referencia: https://stackoverflow.com/questions/917640/any-way-to-select-without-causing-locking-in-mysql 
	]]--
	function self:queryNoLock(connection, query, options)
		local queryMySQLNoLock = self:noLock()
		local noLockStatus, noLockResult = self:query(connection, queryMySQLNoLock, {maybeOperation = true})
		if not noLockResult then
			LogWarning("[msSQLDatasource:queryNoLock] MySQL - queryMySQLNoLock: %s, noLockStatus: '%s' noLockResult: '%s'", queryMySQLNoLock, noLockStatus, noLockResult)
		end

		LogInfo("[msSQLDatasource:queryNoLock] MySQL query: %s", query)
		local queryStatus, queryResult = self:query(connection, query, options)

		if noLockResult then
			local queryMySQLResetLock = self:resetLock()
			local resetLockStatus, resetLockResult = self:query(connection, queryMySQLResetLock, {maybeOperation = true})
			if not resetLockStatus then
				LogWarning("[msSQLDatasource:queryNoLock] MySQL - queryMySQLResetLock: %s, resetLockStatus: '%s' resetLockResult: '%s'", queryMySQLResetLock, resetLockStatus, resetLockResult)
			end
		end

		return queryStatus, queryResult
	end

	function self:operation(connection, query)
		return sqlconnector.operation(connection, query)
	end

	function self:tableFormat(value)
		return sqlconnector.tableFormat(self.private.dbms, value)
	end

	function self:columnFormat(value)
		return sqlconnector.columnFormat(self.private.dbms, value)
	end

	function self:fieldFormat(fieldInfo, value, likeMode)
		if not self.m_fieldFormat then
			self.m_fieldFormat = sqlconnector.getFieldFormatFunction(self.private.dbms)
		end
		return self.m_fieldFormat(fieldInfo, value, likeMode)
	end

	function self:processQueryError(connection, err, params, data)
		return sqlconnector.processQueryError(connection, err, params, data)
	end

	function self:getScopeIdentity(connection)
		return sqlconnector.getScopeIdentity(connection)
	end

	function self:noLock()
		return sqlconnector.noLock(self.private.dbms)
	end

	--[[
		Solo para MySQL
	]]--
	function self:resetLock()
		return sqlconnector.resetLock(self.private.dbms)
	end

	function self:makeResultSet(result)
		local resultset = {
			status = true,
			colnames = {},
			coltypes = {},
			rows = result.rows
		}
		for _, column in pairs(result.columns) do
			table.insert(resultset.colnames, column.ColumnName)
			table.insert(resultset.coltypes, column.typeName)
		end
		return resultset
	end

	return self
end


