module(..., package.seeall)

local messages = require (tweety_modules_prefix .. "messages");
local msSQLBase = require (tweety_modules_prefix .. "msSQLDatasource")

function new(self)
	self = msSQLBase.msSQLDatasource(self)

	local function fieldFormat(fieldInfo, value)
		if value == nil then
			value = "default"
		else
			value = self:fieldFormat(fieldInfo, value, false)
		end
		return value
	end

	function self:processOperationResult(description, data, hdata, resultset)
		--add _rowid and get only first row
		table.insert(resultset.colnames, '_rowid')
		if #resultset.rows == 0 then
			return true, resultset
		end
		local row = resultset.rows[1]
		resultset.rows = { row }
		local _rowid = tonumber(hdata._rowid.value)
		if _rowid then
			table.insert(row, _rowid)
		end
		local spResult, spMsg
		--check sp status from row values
		for idx, colname in ipairs(resultset.colnames) do
			if		( colname == "status" and not hdata.status ) or
					( colname == "result" and not hdata.result )
			then
				spResult = row[idx]
			elseif  ( colname == "message" and not hdata.message ) or
					( colname == "sp_message" and not hdata.sp_message ) or
					( colname == "msg" and not hdata.msg ) or
					( colname == "description" and not hdata.description ) 
			then
				spMsg = row[idx]
			end
		end
		if spResult == 0 or spReult == false then
			return false, messages.messageKey('storedprocedure_error', { description = spMsg })
		end
		return true, resultset
	end

	function self:runSp (description, data, options)
		options = options or {}
		local operation = options.operation or false
		local origValue = options.origValue or false
		--process data
		local hdata = {}
		for _,param in ipairs(data) do
			hdata[param.name]= param
		end
		--get params
		local params = {}
		for _, param in ipairs(description.inputParams) do
			local p = hdata[param.name]
			if p then
				local pValue = p.value
				if origValue and not pValue then
					pValue = p.origvalue
				end
				local formatedValue = fieldFormat(param, pValue)
				table.insert(params, formatedValue)
			elseif param.defaultValue then
				table.insert(params, param.defaultValue)
			end
		end

		--open connection
		local connection = self:getConnection()
		if not connection then
			return false, messages.messageKey('database_conection_error')
		end

		--make query
		local query
		local dbms = (connection.data.dbms or ""):lower()
		--LogDebug("[MSQLPROC:runSp] --> dbms: '%s'", dbms)
		if dbms == "sqlserver" then
			query = string.format("EXEC [%s] %s",
				description.spName,
				table.concat(params, ", ")
			)
		elseif dbms == "mysql" then
			query = "CALL %s"
			query = string.format("CALL %s", description.spName)
			if #params > 0 then
				query = query .. string.format(" (%s);", table.concat(params, ", "))
			end
		else
			error("Unsuported DBMS " .. dbms)
		end

		--make query

		LogInfo("[MSQLPROC.runSp] --> MSQL Running SP '%s', query: '%s'.", description.spName, query)

		--run query
		local status, result = self:query(connection, query, { rowAsArray = true, maybeOperation = operation })
		--close connection
		self:closeConnection(connection)
		--process result
		if not status then
			return false, self:processQueryError(connection, result, description.inputParams, data)
		end
		if type(result) == "number" then
			--operation
			return true
		end
		--makes resultset
		local resultset = self:makeResultSet(result)
		if operation then
			return self:processOperationResult(description, data, hdata, resultset)
		end
		return true, resultset
	end
	
	function self:save(data)
		return self:runSp(self.description.store, data, { operation = true })
	end
	
	function self:update(data)
		return self:runSp(self.description.update, data, { operation = true, origValue = true })
	end
	
	function self:delete(data)
		return self:runSp(self.description.delete, data, { operation = true })
	end
	
	function self:get(data,context)
		return self:runSp(self.description.retrieve, data)
	end
	
	return self
end
