module(..., package.seeall)

local messages = require (tweety_modules_prefix .. "messages")
local csvutils = require (tweety_modules_prefix .. "csvutils")
local msSQLBase = require (tweety_modules_prefix .. "msSQLDatasource")

function new(self)
	--LogDebug("[msqltable:new] creating.")

	self = msSQLBase.msSQLDatasource(self)

	function self:save (data)
		--LogDebug("[msqltable:save] 1 - saving.")

		--process data
		local hdata = {}
		for _,param in ipairs(data) do
			hdata[param.name]= param
		end
		----1. fields
		local columns = {}
		local fields = {}
		local identityName
		for _, param in ipairs(self.description.store.inputParams) do
			local p = hdata[param.name]
			if p and not param.identity then
				local formatedValue = self:fieldFormat(param, p.value, false)
				local formatedColumn = self:columnFormat(param.name)
				table.insert(columns, formatedColumn)
				table.insert(fields, formatedValue)
			elseif param.identitiy then
				identityName = param.name
			end
		end
		if #fields == 0 then
			return true -- nothing to insert
		end
		--make query
		local formatedTableName = self:tableFormat(self.description.store.tableName)
		local query = string.format("INSERT INTO %s (%s) VALUES (%s)",
			formatedTableName,
			table.concat(columns, ", "),
			table.concat(fields, ", ")
		)
		--open connection
		local connection = self:getConnection()
		if not connection then
			return false, messages.messageKey('database_conection_error')
		end
		--run query
		LogInfo("[msqltable:save] %s", query)
		local status, count = self:operation(connection, query)
		--get scope identitiy
		local identity = nil
		if status and identityName then
			local statusId, identityValue = self:getScopeIdentity(connection)
			if status then
				identity = { colnames = { identityName, '_rowid' }, rows = { { identityValue, hdata._rowid.value } } }
			else
				statusId = false
				count = identityValue
			end
		end
		--close connection
		self:closeConnection(connection)
		--process result
		if status then
			return true, identity
		else
			return false, self:processQueryError(connection, count, self.description.store.inputParams, data)
		end
	end

	function self:update (data)
		--LogDebug("[msqltable:update] 1 - updating.")
		--LogDebug("[msqltable:update] 2 - self.description.update.inputParams: %s.", json.encode(self.description.update.inputParams))
		--LogDebug("[msqltable:update] 3 - data: %s.", json.encode(data))

		--process data
		local hdata = {}
		for _,param in ipairs(data) do
			hdata[param.name]= param
		end
		----1. fields
		local fields = {}
		for _, param in ipairs(self.description.update.inputParams) do
			local p = hdata[param.name]
			if p and not param.key and not param.identity and p.value ~= p.origvalue then
				local formatedValue = self:fieldFormat(param, p.value, false)
				local formatedColumn = self:columnFormat(param.name)
				table.insert(fields, string.format("%s = %s", formatedColumn, formatedValue))
			end
		end
		--LogDebug("[msqltable:update] 4 - fields: %s.", json.encode(fields))
		if #fields == 0 then
			LogInfo("[msqltable:update] It is not updated table '%s', there are no fields.", self.description.update.tableName)
			return true -- nothing to update
		end
		----2. keys
		local filters = {}
		for _, param in ipairs(self.description.update.inputParams) do
			local p = hdata[param.name]
			if p and (param.key or param.identity) then
				local formatedValue = self:fieldFormat(param, p.origvalue or p.value, false)
				local formatedColumn = self:columnFormat(param.name)
				table.insert(filters, string.format("%s = %s", formatedColumn, formatedValue))
			end
		end
		--LogDebug("[msqltable:update] 5 - filters: %s.", json.encode(filters))
		if #filters == 0 then
			local err = messages.messageKey('missing_keys')
			LogError("[msqltable:update] Error updating table '%s'. Missing Primary Key. Err: %s.", self.description.update.tableName, json.encode(err or {}))
			return false, messages.messageKey('missing_keys')
		end
		--make query
		local formatedTableName = self:tableFormat(self.description.update.tableName)
		local query = string.format("UPDATE %s SET %s WHERE %s", 
			formatedTableName,
			table.concat(fields,", "),
			table.concat(filters," AND ")
		)
		--open connection
		local connection = self:getConnection()
		if not connection then
			local err = messages.messageKey('database_conection_error')
			LogError("[msqltable:update] Error updating table '%s'. Unable to connect. Err: %s.", self.description.update.tableName, json.encode(err or {}))
			return false, err
		end
		--run query
		LogInfo("[msqltable:update] 5 - %s", query)
		local status, count = self:operation(connection, query)
		--close connection
		self:closeConnection(connection)
		--process result
		if status then
			-- ok
			return true
		end
		-- error
		local err = self:processQueryError(connection, count, self.description.update.inputParams, data)
		LogError("[msqltable:update] Error updating table '%s', executing query. Err: %s.", self.description.update.tableName, json.encode(err or {}))
		return false, err
	end

	function self:delete (data)
		--LogDebug("[msqltable:delete] 1 - deleting.")

		--process data
		local hdata = {}
		for _,param in ipairs(data) do
			hdata[param.name]= param
		end
		----1. keys
		local filters = {}
		for _, param in ipairs(self.description.delete.inputParams) do
			local p = hdata[param.name]
			if p and (param.key or param.identity) then
				local formatedValue = self:fieldFormat(param, p.value, false)
				local formatedColumn = self:columnFormat(param.name)
				table.insert(filters, string.format("%s = %s", formatedColumn, formatedValue))
			end
		end
		if #filters == 0 then
			return false, messages.messageKey('missing_keys')
		end
		--make query
		local formatedTableName = self:tableFormat(self.description.delete.tableName)
		local query = string.format("DELETE FROM %s WHERE %s", 
			formatedTableName,
			table.concat(filters," AND ")
		)
		--open connection
		local connection = self:getConnection()
		if not connection then
			return false, messages.messageKey('database_conection_error')
		end
		--run query
		LogInfo("[msqltable:delete] %s", query)
		local status, count = self:operation(connection, query)
		--close connection
		self:closeConnection(connection)
		--process result
		if status then
			return true
		else
			return false, self:processQueryError(connection, count, self.description.delete.inputParams, data)
		end
	end
	
	function self:filter_criteria_string(filterName, params)
		local function filterTostring ( filter)
			local fieldFilterTemplate = ""
			local driverType = self:getDriverType():lower()
			if driverType == "sqlserver" then
				fieldFilterTemplate = '\"%s\"%s%s'
			elseif driverType == "mysql" or driverType == "builtin" then
				fieldFilterTemplate = '%s%s%s'
			else
				error("Unsuported DBMS driverType " .. driverType)
			end
			local operators = {
				['>']  =' > '   ,
				['>='] =' >= '  ,
				['<']  =' < '   ,
				['<='] =' <= '  ,
				['=='] =' = '   ,
				['!='] =' <> '  ,
				['%']  =' LIKE '
			}
			local t = {}
			for idx,segment in ipairs(filter.criteria) do
				table.insert(t,segment.open)
				local operator = operators[segment.comparator] or segment.comparator
				local fieldFilter = string.format(fieldFilterTemplate, segment.column, operator , tostring(segment.expr))
				table.insert(t,  fieldFilter)
				table.insert(t, segment.close)
				if idx < #filter.criteria then
					table.insert(t,segment.combinator)
				end
			end
			return table.concat(t," ")
		end

		local filter = self:get_filter(filterName,params,
			function(fieldInfo,value,likeMode)
				return self:fieldFormat(fieldInfo, value, likeMode)
			end
		)

		local stringFilter = filterTostring(filter)
		LogDebug("[msqltable:filter_criteria_string] stringFilter: '%s'", stringFilter)
		return stringFilter
	end

	function self:filter_order (name)
		local filter
		local res = {}
		local empty = true
		local chash = {}
		for _,column in ipairs(self.description.columns)  do
			chash[column.name]=column
		end
		for _,fl in ipairs(self.description.filters) do
			if fl.name == name then
				filter = fl
				break
			end
		end
		if filter then
			for _,ordercolumn in ipairs(filter.order) do
				if ordercolumn.column and ordercolumn.order then
					local col = chash[ordercolumn.column]
					local dir
					if ordercolumn.order == "ASC" then
						dir = ' ASC '
					else
						dir = ' DESC '
					end
					if col and col.type == "integer" then
						res["cast( " .. ordercolumn.column .. " as int)"] = dir
					elseif col and col.type == "numeric" then
						res["cast( " .. ordercolumn.column .. " as numeric)"] = dir
					else
						res[ordercolumn.column] = dir
					end
					empty = false
				end
			end
		end
		local t = {}
		for k, v in pairs(res) do
			table.insert(t, k .. v)
		end
		return table.concat(t, ',')
	end

	function self:get (data,context)
		--LogDebug("[msqltable:get] 1 - getting.")

		--make query
		context.limit = context.limit or 100
		local filter = self:filter_criteria_string(context.filter, data, true)
		if not filter then
			return false, messages.messageKey('unknown_error', { description = 'Incorrect filter expresion' })
		end

		local function QueryFn (connection, query)
			LogInfo("[msqltable:get] %s", query)
			local status, result = self:query(connection, query, {rowAsArray = true})
			return status, result
		end

		local formatedTableName = self:tableFormat(self.description.retrieve.tableName)
		local where = filter ~= "" and string.format("WHERE %s", filter) or ""
		local order = self:filter_order(context.filter)
		local orderBy = order and order ~= "" and string.format("ORDER BY %s", order) or ""
		local query
		local driverType = self:getDriverType():lower()

		if driverType == "sqlserver" then
			query = string.format("SELECT TOP %d * FROM %s %s %s %s",
				context.limit,
				formatedTableName,
				self:noLock(),
				where,
				orderBy
			)
		elseif driverType == "mysql" or driverType == "builtin" then
			-- SELECT * FROM Persons where FirstName = 'de Mello' order by LastName  LIMIT 3;
			query = string.format("SELECT * FROM %s %s %s LIMIT %d",
				formatedTableName,
				where,
				orderBy,
				context.limit
			)
			QueryFn = function (connection, query)
				LogInfo("[msqltable:get] MySQL %s", query)
				local status, result = self:queryNoLock(connection, query, {rowAsArray = true})
				return status, result
			end
		else
			error("Unsuported DBMS driverType " .. driverType)
		end
		--open connection
		local connection = self:getConnection()
		if not connection then
			return false, messages.messageKey('database_conection_error')
		end

		--run query
		local status, result = QueryFn(connection, query)

		--close connection
		self:closeConnection(connection)
		--process result
		if status then
			local resultset = self:makeResultSet(result)
			return true, resultset
		else
			return false, self:processQueryError(connection, result, self.description.retrieve.inputParams, data)
		end
	end

	return self
end
