module(...,package.seeall)

local function getTextWidth(str,font,size,bold)
	local fonts = {
		sans={	['!'] = 0.118, ['"'] = 0.284, ['#'] = 0.577, ['$'] = 0.512, ['%'] = 0.832, ['&'] = 0.650, ['('] = 0.256, [')'] = 0.256,
				['*'] = 0.349, ['+'] = 0.511, [','] = 0.115, ['-'] = 0.292, ['.'] = 0.108, ['/'] = 0.300, ['0'] = 0.505, ['1'] = 0.285,
				['2'] = 0.513, ['3'] = 0.507, ['4'] = 0.535, ['5'] = 0.513, ['6'] = 0.511, ['7'] = 0.501, ['8'] = 0.510, ['9'] = 0.509,
				[':'] = 0.108, [';'] = 0.115, ['='] = 0.511, ['?'] = 0.499, ['@'] = 1.000, ['A'] = 0.724, ['B'] = 0.584, ['C'] = 0.684,
				['D'] = 0.640, ['E'] = 0.578, ['F'] = 0.522, ['G'] = 0.716, ['H'] = 0.607, ['I'] = 0.102, ['J'] = 0.428, ['K'] = 0.640,
				['L'] = 0.484, ['M'] = 0.739, ['N'] = 0.610, ['O'] = 0.740, ['P'] = 0.591, ['Q'] = 0.755, ['R'] = 0.682, ['S'] = 0.616,
				['T'] = 0.614, ['U'] = 0.609, ['V'] = 0.708, ['W'] = 0.995, ['X'] = 0.710, ['Y'] = 0.710, ['Z'] = 0.612, ['['] = 0.210,
				[']'] = 0.210, ['_'] = 0.292, ['a'] = 0.516, ['b'] = 0.486, ['c'] = 0.488, ['d'] = 0.486, ['e'] = 0.517, ['f'] = 0.328,
				['g'] = 0.494, ['h'] = 0.457, ['i'] = 0.095, ['j'] = 0.215, ['k'] = 0.465, ['l'] = 0.095, ['m'] = 0.760, ['n'] = 0.456,
				['o'] = 0.525, ['p'] = 0.487, ['q'] = 0.486, ['r'] = 0.305, ['s'] = 0.466, ['t'] = 0.274, ['u'] = 0.455, ['v'] = 0.514,
				['w'] = 0.769, ['x'] = 0.525, ['y'] = 0.514, ['z'] = 0.496, [''] = 0.118, [''] = 0.610, [''] = 0.456, [' '] = 0.5
		}
	}
	local font = fonts.sans
	local w = 0
	local wwidth=11;
	string.gsub(str,".",
		function(c)
			w = w + (wwidth * (font[c] or 0.5))
		end
	)
	return w
end

local function wrap(str, limit)
	limit = limit or 200
	local here = 0
	local n=0;
	local res = str:gsub("(%s+)()(%S+)()",
		function(sp, st, word, fi)
			local t = string.sub(str,here,fi)
			if getTextWidth(t)>limit then
				here = st
				n = n+1
				return "\n"..word
			end
		end
	)
	return res,n+1
end
local function widget(self)
	self = self or {}
	function self:getHeight()
		return (self.height or self:getContentHeight() );
	end
	function self:getWidth()
		return (self.width or self:getContentWidth());
	end
	return self
end
function panel (self)
	self = widget(self)
	self.type = "ui.panel";
	self.childs = self.childs or {}
	self.layoutMode = self.layoutMode or "vertical"
	function self:horizontalLayout()
		local left = self.pl or 0;
		local top = self.pt or 0;
		local bottom = self:getHeight() - (self.pb or 0)

		for _,c in ipairs(self.childs) do
			c.x = left;
			if c.sideBottom then
				c.y = bottom - c:getHeight();
			else
				c.y = top;
			end

			left = left + c:getWidth() + (self.hs or 0)
			c.height = c:getHeight();
			c.width = c:getWidth();
			if c.layout then
				c:layout()
			end
		end
	end
	function self:verticalLayout()

		local top = self.pt or 0;
		local left = self.pl  or 0
		local right = self.right or self:getWidth();
		local bottom = self:getHeight() - (self.pb or 0)
		
		for _,c in ipairs(self.childs) do
			c.x = c.x or left;

			if not c.y then
				c.y = top;
				if c.sideBottom then
					c.y = bottom - c:getHeight();
					bottom = c.y - (self.vs or 0)
				else
					c.y = top;
					top = top + c:getHeight() + (self.vs or 0)
					
				end
				c.height = c:getHeight();
				if c.fillWidth then
					c.width = right - c.x
				else
					c.width = c:getWidth();
				end

			end

			if c.layout then
				c:layout()
			end
		end
	end

	function self:layout()
		if self.layoutMode == "vertical" then
			self:verticalLayout()
		elseif self.layoutMode == "horizontal" then
			self:horizontalLayout()
		end
	end


	function self:getContentWidth()
		local maxWidth = 0;
		if self.layoutMode == "vertical" then
			local t
			for _,c in ipairs(self.childs) do
				t = c:getWidth();
				if t> maxWidth then
					maxWidth = t
				end

			end
		else
			for _,c in ipairs(self.childs) do
				maxWidth = maxWidth + c:getWidth() + (self.hs or 0)
			end
		end
		if maxWidth > 0 then
			maxWidth = maxWidth + (self.pl or 0 ) + (self.pr or 0)
		end
		return maxWidth;
	end
	function self:getContentHeight()
		local maxHeight = 0;
		if self.layoutMode == "horizontal" then
			local t
			for _,c in ipairs(self.childs) do
				t = c:getHeight();
				if t> maxHeight then
					maxHeight = t
				end

			end
		else
			for _,c in ipairs(self.childs) do
				maxHeight = maxHeight + c:getHeight() + (self.vs or 0)
			end
		end
		if maxHeight > 0 then
			maxHeight = maxHeight + (self.pt or 0 ) + (self.pb or 0)
--[[			
			if self.borderTop then
				maxHeight=maxHeight+(self.borderWidth or 0)
			end
			if self.borderBottom then
				maxHeight=maxHeight+(self.borderWidth or 0)
			end
			if not self.borderTop and not self.borderBottom and not self.borderLeft and not self.borderRight then
				maxHeight=maxHeight+((self.borderWidth or 0)*1)
			end
--]]			
		end
		return maxHeight;

	end



	return self
end
function label ( self)
	self = widget(self)
	self.type = "ui.label"
	self.caption = self.caption or "";
	function self:getContentWidth()
		return getTextWidth(tostring(self.caption or ''));
	end
	function self:getContentHeight()
		if self.multiLine then
			local str,n = wrap(tostring(self.caption or ''),self:getWidth())
			return n*14
		else
			return 16;
		end
	end
	return self
end
function hline ( self)
	self = widget(self)
	self.type = "ui.hline"
	function self:getContentHeight()
		return self.weight or 1
	end
	return self
end
function vline ( self)
	self = widget(self)
	self.type = "ui.vline"
	function self:getContentWidth()
		return self.weight or 1
	end
	return self
end
function image ( self)
	self = widget(self)
	self.type = "ui.image"
	return self
end



