module(..., package.seeall,inconcert.orbit.application.Decorate)

local application = require (tweety_modules_prefix .. "application")
local messages = require (tweety_modules_prefix .. "messages")
local config = require (tweety_modules_prefix .. "config")
local http = require("socket.http")
local i6_campaigns = require(tweety_modules_prefix .. "models.campaigns")
local ir = require "lua_iragentlink"
local captcha = require (tweety_modules_prefix .. "captcha"); 
local md5 = require "md5";
local utils = require (tweety_modules_prefix .. "utils")
require "uuid"
require "LuaOrm.Date"
---
-- LuaSec require must be synchronized because OpenSSL initialization is not thread-safe.
-- Multiple VMs requiring LuaSec at the same time will crash.
--
local Synchronize = require "Synchronize"
local https = Synchronize("luasec_mutex", function()
	return require ("ssl.https")
end)

local rex = require("rex_pcre")
local tweetyio = require(tweety_modules_prefix .. "tweetyio");
local AsyncTasks = require ("AsyncTasks");
local m_appLocation
local cosmo = require ("cosmo")
local function url_encode(str)
  if (str) then
    str = string.gsub (str, "\n", "\r\n")
    str = string.gsub (str, "([^%w ])",
        function (c) return string.format ("%%%02X", string.byte(c)) end)
    str = string.gsub (str, " ", "+")
  end
  return str	
end
local function trim(s)
	if type(s) == "string" then
		return (string.gsub(s, "^%s*(.-)%s*$", "%1")) -- trim
	else
 		return s
	end
end

function getCurrentVCCName()
	--LogDebug("getCurrentVCCName %s", json.encode(inconcert.authUser));
	if inconcert.authUser and inconcert.authUser.virtualContactCenter then
		return string.lower(inconcert.authUser.virtualContactCenter);
	end
end

function cache ( handler) 
	return function (web,...)
		web.headers["Expires"]=os.date("!%a, %d %b %Y %H:%M:%S GMT",os.time()+(3600*72));
		return handler(web,...)
	end
end
function nocache ( handler) 
	return function (web,...)
		_M.response.DontCache(web)
		return handler(web,...)
	end
end

local function getArgumentsByGET (web)
	local queryStr = web.vars.QUERY_STRING;
	return utils.ParseQueryString(queryStr)
end

local function getArgumentsByPOST(web)
	local  arguments = {}
	local gsub = string.gsub
	local c
	local _ = web.POST['nada'];
	for k,v in pairs(web.POST)	do
		c = arguments;
		local vname = gsub(k,"([^%.]+)%.",function(v)
			c[v]=c[v] or {}
			c=c[v]
			return ""
		end)
		if #vname then
			c[vname]=v
		end
	end
	return arguments;
end

function get_info_from_token(token)
	local campaigns = i6_campaigns.getAll();
	for campaignId,campaign in pairs(campaigns) do
		for _,account in ipairs(campaign.accounts) do
			local website = account.webcallConfig ~= "" and account.webcallConfig or account
			if type(website) == "string" then
				website = json.decode(website)
			end
			if website.Token == token then
				LogDebug("TOKEN FOUND")
				 return {
					campaignAtVCC=campaignId,
					TweetyAppName=website.TweetyAppName,
					containerId=website.containerId,
					snippet_parameters= website.snippet_parameters,
					VCC=campaign.VCC
				}
			end
		end
	end
end
function click_to_contact_form(web)
	local args = getArgumentsByGET(web);
	-- con el token debo obtener el id del proyecto 
	LogInfo("URL CONTACTFORM %s",json.encode({
			SERVER_NAME=web.vars.SERVER_NAME,
			SERVER_PORT=web.vars.SERVER_PORT,
			SERVER_PROTOCOL=web.vars.SERVER_PROTOCOL,
			HTTPS=web.vars.HTTPS
		},true));
	local token_data = get_info_from_token(args.token);
	if token_data then
		local app = application.new{vccName = token_data.VCC}	
		local status,result = app:initialize(token_data.TweetyAppName)
		if status then
			local authfilter = app:getSettings().authfilter or {}
			local matcharguments = authfilter._matcharguments or {}
			credentials = authfilter._wholeapp;
			if credentials then
				local rejected = false
				if #matcharguments > 0 then
					rejected=true
					for _,arg in ipairs(matcharguments) do
						if args[arg.name] == arg.value then
							rejected=false
							break
						end
					end
				end
				if not rejected then
					theApp=app;
				end
			end
		end
	end
	if theApp then
		local popupSize = theApp:getPopupSize();
		local fh = io.open(m_appLocation.."/resources/go_to_contact_form.js");
		if not fh then
			return _M.not_found(web)
		end
		template = fh:read("*a");
		io.close(fh);
		local protocol
		if web.vars.HTTPS == "on" or web.vars.HTTPS=="ON" then
			protocol="https"
		else
			protocol="http"
		end
		return cosmo.fill(template,{
			popupWidth=popupSize.width,
			popupHeight=popupSize.height,
			hostname="i6.inconcert.cc",--web.vars.SERVER_NAME,
			token=args.token,
			protocol=protocol,
			hostport=80,--web.vars.SERVER_PORT,
			container_id=token_data.containerId,
			campaignAtVCC=token_data.campaignAtVCC,
			projectid=token_data.TweetyAppName,
			snippet_parameters=token_data.snippet_parameters
		})
	else
		return _M.not_found(web)
	end
end

local function getLibType(appName, formName, appClass, arguments, oldChat)
	if formName == "___start_minimized" then
		return "minimizedchat"
	elseif (appClass == "chatcampaign" or appClass == "webcallcampaign") and not oldChat then
		return "chat"
	else
		return "default"
	end
end

local function getApiLibs(appName, formName, appClass, arguments, minLibs, oldChat)
	if formName == "___start_minimized" then
		return ""
	elseif (appClass == "chatcampaign" or appClass == "webcallcampaign" or appClass == "contactform") and not oldChat then
		return cosmo.fill([[
	<!-- OCC APIS -->
	<script language="javascript" src="lib/api/api$min.js"></script>
		]], { min = minLibs })
	else
		return cosmo.fill([[
	<!-- OCC APIS -->
	<script language="javascript" src="lib/api/api$min.js"></script>
	<script language="javascript" src="lib/api/agent_api$min.js"></script>
	<!-- ALLEGRO APIS -->
	<script language="javascript" src="lib/allegro/inConcertSDK$min.js"></script>
	<script language="javascript" src="lib/allegro/inConcertSOA$min.js"></script>
		]], { min = minLibs })
	end
end

local function getDesignerLibs(appName, formName, appClass, arguments, minLibs, oldChat)
	if formName == "___start_minimized" then
		return ""
	else
		return cosmo.fill([[
	<link rel="stylesheet" type="text/css" href="themes/inconcert.designer/theme$min.css" />
	<script language="javascript" src="themes/inconcert.designer$min.js"> </script>
		]], { min = minLibs })
	end
end

local function MustIncludeCaptchaLibs (appName, formName)
	appName = appName or ""
	formName = formName or ""

	if appName:lower() == "designer" then
		LogInfo("[RUNTIME.MustIncludeCaptchaLibs] Must include Captcha Libs for Designer.")
		return true
	end

	local vccName = getCurrentVCCName()
	if formName:lower() == "___start_minimized" then
		LogInfo("[RUNTIME.MustIncludeCaptchaLibs] 1 - NOT must include Captcha Libs for minimized formName: '%s', app '%s' vcc: '%s'.", formName, appName, vccName)
		return false
	end

	local app = application.new{vccName = vccName}
	local status, result = app:initialize(appName)

	--LogDebug("[RUNTIME.MustIncludeCaptchaLibs] ---> formName: '%s', app '%s' vcc: '%s'. Status: '%s', result: '%s'.", formName, appName, vccName, status, result)

	if not status then
		LogWarning("[RUNTIME.MustIncludeCaptchaLibs] 2 - NOT must include Captcha Libs for formName: '%s', app '%s' vcc: '%s'. Error retrieve App. Status: '%s', result: '%s'.", formName, appName, vccName, status, result)
		return false
	end

	local hasCaptchaConfigured = app:HasCaptchaConfigured(appName)
	if hasCaptchaConfigured then
		LogWarning("[RUNTIME.MustIncludeCaptchaLibs] + The App '%s' in Vcc '%s' HAVE CAPTCHA CONFIGURED.", appName, vccName)
		return true
	end

	LogInfo("[RUNTIME.MustIncludeCaptchaLibs] - The App '%s' in Vcc '%s' does not have CAPTCHA configured.", appName, vccName)
	return false
end

local function getCaptchaLibs(appName, formName, appClass, arguments, oldChat)
	LogDebug("[RUNTIME.getCaptchaLibs] ---> appName: '%s' formName '%s' appClass '%s' arguments '%s' oldChat '%s'", appName, formName, appClass, arguments, oldChat)
	if not MustIncludeCaptchaLibs(appName, formName) then
		-- No debe incluir CAPTCHA
		return ""
	end
	-- SI, tiene que incluir CAPTCHA
	return [[
<script src="https://www.google.com/recaptcha/api.js?onload=onloadCallback&render=explicit" async defer></script>
	]]
end

local function getAppStyles(appName, formName, appClass, arguments, minLibs, oldChat)
	if formName == "___start_minimized" then
		return cosmo.fill([[
	<link rel="stylesheet" type="text/css" href="styles/webchat/webchat$min.css" />
	<link rel="stylesheet" type="text/css" href="styles/webchat/skin-2021/webchat-fonts$min.css" />
	<link rel="stylesheet" type="text/css" href="styles/webchat/skin-2021/i6-chat-icons/i6-chat-icons$min.css" />
		]], { min = minLibs })
	elseif (appClass == "chatcampaign" or appClass == "webcallcampaign") and not oldChat then
		return cosmo.fill([[
	<link rel="stylesheet" type="text/css" href="styles/mobileForm$min.css" />
	<link rel="stylesheet" type="text/css" href="styles/webchat/webchat$min.css" />
	<link rel="stylesheet" type="text/css" href="styles/webchat/skin-2021/webchat-fonts$min.css" />
	<link rel="stylesheet" type="text/css" href="styles/webchat/skin-2021/i6-chat-icons/i6-chat-icons$min.css" />
	<link rel="stylesheet" type="text/css" href="styles/webcall/videocall$min.css" />
	<link rel="stylesheet" type="text/css" href="styles/webcall/occ-videocall-icons$min.css" />
		]], { min = minLibs })
	elseif appClass == "contactform" then
		return cosmo.fill([[
	<link rel="stylesheet" type="text/css" href="styles/mobileForm$min.css" />
	<link rel="stylesheet" type="text/css" href="styles/webcontact/webcontact$min.css" />
		]], { min = minLibs })
	else
		return cosmo.fill([[
	<link rel="stylesheet" type="text/css" href="styles/mobileForm$min.css" />
	<link rel="stylesheet" type="text/css" href="styles/webchat/webchat$min.css" />
	<link rel="stylesheet" type="text/css" href="styles/webchat/skin-2021/webchat-fonts$min.css" />
	<link rel="stylesheet" type="text/css" href="styles/webchat/skin-2021/i6-chat-icons/i6-chat-icons$min.css" />
	<link rel="stylesheet" type="text/css" href="styles/webcall/videocall$min.css" />
	<link rel="stylesheet" type="text/css" href="styles/webcall/occ-videocall-icons$min.css" />
	<link rel="stylesheet" type="text/css" href="styles/webcontact/webcontact$min.css" />
		]], { min = minLibs })
	end
end

local REFERENCES_FILTERS = {
	minimizedchat = {
		"___helpers.js",
		"styles.css",
		"minimizedchat-"
	}
}

local function hasReferencesLib(reference, appName, formName, appClass, arguments, minLibs, oldChat)
	--1. filter reference
	local filters = nil
	if formName == "___start_minimized" then
		filters = REFERENCES_FILTERS.minimizedchat
	end
	if filters then
		local found = false
		for _, filter in pairs(filters) do
			if string.match(reference, filter) then
				found = true
				break
			end
		end
		if not found then
			return false
		end
	end
	return true
end

local function getReferencesLib(reference, appName, formName, appClass, arguments, minLibs, oldChat)
	if not hasReferencesLib(reference, appName, formName, appClass, arguments, minLibs, oldChat) then
		return nil
	end
	return reference
end

local function getReferencesLibs(references, appName, formName, appClass, arguments, minLibs, oldChat)
	local filteredReferences = {}
	for _, reference in pairs(references) do
		local filteredReference = getReferencesLib(reference, appName, formName, appClass, arguments, minLibs, oldChat)
		if filteredReference then
			table.insert(filteredReferences, filteredReference)
		end
	end
	return filteredReferences
end

local function getMessagesLib(appName, formName, arguments, appClass, locale, oldChat)
	if formName == "___start_minimized" then
		return ""
	else
		return cosmo.fill([[
	<script language="javascript" src="generated/icc-ui-messages$locale.js"></script>
		]], { locale = locale })
	end
end

local function isOldChat(appClass, catalog)
	if appClass == "chatcampaign" then
		for _, view in pairs(catalog.views or {}) do
			if view.name == "___chat" or view.class == "___chat" then
				return true
			end
		end
	end
	return false
end

function application_render_form(web,appName,formName,arguments)
	appName = appName:gsub('"', ""):gsub("'", ""):gsub("<", ""):gsub(">", "")
	formName = formName:gsub('"', ""):gsub("'", ""):gsub("<", ""):gsub(">", "")
	if arguments and arguments.token then
		arguments.token = arguments.token:gsub('"', ""):gsub("'", ""):gsub("<", ""):gsub(">", "")
	end
	local vccName = getCurrentVCCName()

	LogDebug("[RUNTIME:application_render_form] ==> Cargando aplicacion '%s' form '%s' vcc '%s'", appName, formName, vccName)

	local app = application.new{vccName = vccName}
	local token = arguments.token or md5.sumhexa(vccName .. appName );
	local isDev = arguments.dev == "true" or arguments.debug == "true"
	tweetyio.set_token_data(token,{vccName=vccName});
	local references={}
	
	local status,result = app:initialize(appName)
	if not status then
		return "Invalid application name";
	end
	_M.response.DontCache(web)

	local catalog = app.getCatalog()
	local appClass = catalog.appClass
	local oldChat = isOldChat(appClass, catalog)

	local template =  [[<script language="javascript" src="$vccName/$appName/resources/$filename"> </script>]]
	for _,res in ipairs(app:listResources("text/javascript")) do
		table.insert(references,cosmo.fill(template,{vccName=app.vccName,filename=res.src,appName=appName}))
	end
	template = [[<link rel="stylesheet" href="$vccName/$appName/resources/$filename" type="text/css"/>]]
	for _,res in ipairs(app:listResources("text/css")) do
		table.insert(references,cosmo.fill(template,{vccName=app.vccName,filename=res.src,appName=appName}))
	end
	local refs = catalog.refs or "";
	refs = refs .. "\n";

	string.gsub(refs,"([^\n]+)[\n]",function(line)
		LogDebug("Line " .. line);
		if line:sub(-4) == ".css" then
			table.insert(references,cosmo.fill([[<link rel="stylesheet" href="$url" type="text/css"/>]],{url=line}))
		elseif line:sub(-3) == ".js" then
			table.insert(references,cosmo.fill([[<script language="javascript" src="$url"> </script>]],{url=line}))
		end
	end)

	local fh = io.open(m_appLocation.."/singleform.html");
	if not fh then
		-- TODO: generar alguna cosa para mostrar este error
		-- y salir
	end
	template = fh:read("*a");
	io.close(fh);
	local settings = app:getSettings();
	settings.tweety_folder = tweety_folder;
	
	settings.authfilter = nil;
	
	settings.UserGroup = inconcert.authUser.UserGroup;
	settings.VccGroup = inconcert.authUser.VccGroup;
	settings.platform = config.platform;
	settings.inConcert6 = config.inConcert6;
	settings.serverTimeUTC = LuaOrm.Date():toString();
	settings.serverTimeLocal = os.date("%Y-%m-%d %H:%M:%S");
	settings.appVersion = app.getVersion();
	settings.authToken = token;
	settings.phoneFormat = utils.GetPhoneFormat()
	local mobile_meta;
	settings.isMobile = nil ~= rex.match(web.vars.HTTP_USER_AGENT,"Mobile|iP(hone|od|ad)|Android|BlackBerry|IEMobile|Kindle|NetFront|Silk-Accelerated|(hpw|web)OS|Fennec|Minimo|Opera M(obi|ini)|Blazer|Dolfin|Dolphin|Skyfire|Zune");
	if settings.isMobile or arguments.mobile then
		mobile_meta=[[
			<meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1">
		]]
	end
	local protocol;
	if web.vars.HTTPS == "on" or web.vars.HTTPS=="ON" then
		protocol="https"
	else
		protocol="http"
	end
	--LogDebug("--> template: %s", template)
	local plainArguments = {}
	for k, v in pairs(arguments) do
		--LogDebug("  arguments-> name: %s, type: %s, value: %s", k, type(v), v)
		if type(v) ~= "table" then
			plainArguments[k:gsub("<", ""):gsub(">", "")] = v:gsub("<", ""):gsub(">", "")
		else
			plainArguments[k] = v
		end
	end
	-- Para logeuar los datos a enviar al lado del cliente.
	----for name, value in pairs(plainArguments) do LogDebug(" plainArguments-> name: %s, type: %s, value: %s", name, type(value), value) end

	local minLibs = not isDev and ".min" or ""
	local libtype = getLibType(appName, formName, appClass, arguments, oldChat)
	local apiLibs = getApiLibs(appName, formName, appClass, arguments, minLibs, oldChat)
	local designerLibs = getDesignerLibs(appName, formName, appClass, arguments, minLibs, oldChat)
	local captchaLibs = getCaptchaLibs(appName, formName, appClass, arguments, minLibs, oldChat)
	local appStyles = getAppStyles(appName, formName, appClass, arguments, minLibs, oldChat)
	local filteredReferences = getReferencesLibs(references, appName, formName, appClass, arguments, minLibs, oldChat)
	local locale = "$locale"--app:getLocale()
	local messagesLib = getMessagesLib(appName, formName, appClass, arguments, locale, oldChat)
	
	return cosmo.fill(template, {
		appName = appName,
		formName = formName,
		vccName = app.vccName,
		userName = inconcert.authUser.userName,
		arguments = json.encode({ arguments = plainArguments }),
		settings = json.encode(settings),
		mobile_meta = mobile_meta or "<meta/>",
		references = table.concat(filteredReferences, "\r\n"),
		weblibs = config.weblibs,
		protocol = protocol,
		min = minLibs,
		libtype = libtype,
		apiLibs = apiLibs,
		designerLibs = designerLibs,
		captchaLibs = captchaLibs,
		appStyles = appStyles,
		messagesLib = messagesLib
	})
end

local function application_render_defaultform(web,appName,arguments)
	appName = appName:gsub('"', ""):gsub("'", ""):gsub("<", ""):gsub(">", "")
	local app = application.new{vccName = getCurrentVCCName()}
	LogDebug("Cargando aplicacion %s vcc %s ",appName,getCurrentVCCName())

	local status,result
	status,result = app:initialize(appName);
	if status then
	
		return application_render_form(web,appName,app:getDefaultView(),arguments);
	else
		return "Invalid application name";
		-- TODO : generar alguna cosa para mostrar este error
	end
end

-- genera un javascript que contiene los comandos para crear los controles y mapear eventos del usuario en runtime

function application_build_form(web,tokenId,appName,formName)
	local status,err
	vccName = getCurrentVCCName() or tweetyio.get_token_data(tokenId).vccName;
	LogDebug("application_build_form vcc=%s app=%s form=%s",vccName,appName,formName);

	web.headers["Content-Type"] = "text/javascript; charset=utf-8";
	web.headers["scriptCharset"]="utf-8"
	local app = application.new{
		vccName =vccName
	}
	status,err = app:initialize(appName)
	if status then
		if app:getSettings().cacheTime and app:getSettings().cacheTime ~= 0 then
			web.headers["Expires"]=os.date("!%a, %d %b %Y %H:%M:%S GMT",os.time()+(3600*app:getSettings().cacheTime));
		else
			_M.response.DontCache(web)
		end
		return app:buildForm(formName);
	else
		LogDebug("No pudo iniciar app ".. appName);
		-- TODO : generar alguna cosa para mostrar este error
	end
end

-- retorna la configuracion del formulario

function application_get_chat_first_form(web, vccName, tokenId,appName)
	local status,err
	LogDebug("application_get_chat_first_form vcc=%s app=%s",vccName,appName);
	
	web.headers["Content-Type"] = "text/javascript; charset=utf-8";
	web.headers["scriptCharset"]="utf-8"
	local app = application.new{
		vccName = vccName
	}
	status,err = app:initialize(appName)
	local body
	if status then
		if app:getSettings().cacheTime and app:getSettings().cacheTime ~= 0 then
			web.headers["Expires"]=os.date("!%a, %d %b %Y %H:%M:%S GMT",os.time()+(3600*app:getSettings().cacheTime));
		else
			_M.response.DontCache(web)		
		end
		body = app:getChatFirstForm();
	else
		LogDebug("No pudo iniciar app ".. appName);
		body = json.encode({ status = false, err = "can't initialize app " .. appName })
	end

	-- optional callback, when consumed with jsonp
	local query = getArgumentsByGET(web)
	if query.callback then
		body = query.callback .. "(" .. body .. ")"
	end
	return body
end


-- Carga el workspace del usuario y devuelve la lista de aplicaciones que este contiene.

---------   RESOURCES ------------------

function read_resource(web,...)
	local status,err
	local res
	local arg={...}
	local vccName,appName,fileName
	if #arg == 2 then
		appName=arg[1]
		fileName=arg[2]
		vccName = getCurrentVCCName()
	else
		vccName=arg[1]
		appName=arg[2]
		fileName=arg[3]
	end
	local app = application.new{vccName=vccName}
	status,err = app:initialize(appName)
	if status then

		status,res = app:readResource(fileName);
		-- TODO:Controlar status y generar algo
		web:content_type(res.mimeType)
		
		if res.mimeType == "text/javascript" then
			web.headers["Content-Type"] = res.mimeType .. "; charset=utf-8";
			web.headers["scriptCharset"]="utf-8"
		end
		if app:getSettings().cacheTime then
			web.headers["Expires"]=os.date("!%a, %d %b %Y %H:%M:%S GMT",os.time()+(3600*app:getSettings().cacheTime));
		else
			_M.response.DontCache(web)		
		end
		
		return res.content
	end
	return "No esta che";--_M.not_found(web) -- TODO: devolver un 404 ?
end
function read_datasource(web,tokenId,appName,datasourceName)
	local status,result
	local vccName = getCurrentVCCName() or tweetyio.get_token_data(tokenId).vccName;
	local app = application.new{vccName=vccName}
	status,result = app:initialize(appName)
	if status then
		if app:getSettings().cacheTime then
			web.headers["Expires"]=os.date("!%a, %d %b %Y %H:%M:%S GMT",os.time()+(3600*app:getSettings().cacheTime));
		else
			_M.response.DontCache(web)		
		end
	
		status,result = app:getDatasourceDef(datasourceName);
	end
	return json.encode({status=status,result=result})
end


function create_server_process(web)
	local inputdata = web.input["post_data"];
	inputdata = json.decode(inputdata);
	local newId = uuid.new()
	LogDebug("Creating id for server process: %s", newId)
	local session = tweetyio.session();
	session.tweetyprocess = session.tweetyprocess or {}
	session.tweetyprocess[newId] = {isCompleted=false,messages={},inputdata=inputdata,processCompletion=0}
	session:release();
	return json.encode({status=true,result={processId=newId}})
end

function save_postsurvey_data(web)
	local irLink = ir.IRAgentLink(config.cluster or "inconcert")
	local status,result,data
	status,data = utils.check_json_params(web.input["post_data"] or "{}")
	if not status then
		return json.encode({status=false,result=data})
	end
	local interactionID = data.interactionID;
	data.interactionID = nil;
	status,result = pcall(
		function()
			LogDebug("call SetDataForReporting interactionID=%s dataType=%s data=%s",interactionID,"",json.encode(data))
			local result = irLink:SetDataForReporting(interactionID,"",data);
			if result.result ~= "OK" then
				error(result.reason)
			end
		end
	);
	if not status then
		result=messages.messageKey('database_error',{description=result})
	else
		result = nil;
	end
	return json.encode({status=status,result=result})
end

function save_contact_form(web)
	local status
	local data = web.input["post_data"];
	
	local function createInteraction(token, data)
		local irLink = ir.IRAgentLink(config.cluster or "inconcert")
		local new_interaction_id = uuid.new()
		local result = irLink:NewWebContact({
			token			= token,
			interactionID	= new_interaction_id,
			data			= data,
		})
		if result.result == "OK" then
			return true, { interactionId = new_interaction_id }
		else
			LogDebug("IR response %s", json.encode(result))
			return false, result.reason
		end
	end

	LogDebug("APPOINTMENT DATA " .. data)
	status,data = utils.check_json_params(data)
	if not status then
		return json.encode({status=false,result=data})
	end

	-- verificacion de captcha
	local app = application.new{hostVCC = data.context.vcc}	
	local status,result = app:initialize(data.context.app)
	if status then
		status,result = captcha.verify(app,data,web.vars.REMOTE_ADDR)
		if status then
			local function scanAddresses(data,out)
				local scan
				local function scanEmail(data,out)
					if data.type == "MAIL" and data.value and trim(data.value) ~= "" then
						table.insert(out,{type="MAIL",address=data.value})
					end
				end
				local function scanPhone(data,out)
					if (data.type == "PHONE" or data.type == "CELLULAR" or data.type == "HOME" or data.type == "OFFICE") and data.value and trim(data.value) ~= "" then 
						table.insert(out,{
							type = data.type,
							address = data.value,
							countryCode = (data.original_values and data.original_values.countryCode) or nil,
							areaCode = (data.original_values and data.original_values.areaCode) or nil
						})
					end	
				end
				local function scanTwitter(data,out)
					if data.type == "TWITTER" and data.value and trim(data.value) ~= "" then
						table.insert(out,{type="TWITTER",address=data.value})
					end	
				end
				scan = function ( d )
					if type ( d ) == "table" then
						scanEmail(d,out)
						scanPhone(d,out)
						scanTwitter(d,out)
						for _,e in pairs(d) do
							scan(e)
						end
					end
				end
				for _,d in pairs(data) do
					scan(d)
				end
			end
			local function scanContactData(data,out)
				local scan
				scan = function ( d )
					if type ( d ) == "table" then
						if d.type == "NAME" and d.value and trim(d.value) ~= "" then
							out.name = d.value
						elseif d.type == "ID" and d.value and trim(d.value) ~= "" then
							out.contactId = d.value
						elseif d.type == "REASON" and d.value and trim(d.value) ~= "" then
							out.reason = d.value
						end
						for _,e in pairs(d) do
							scan(e)
						end

					end
				end
				for _,d in pairs(data) do
					scan(d)
				end
			end
			
			local addresses = {}
			local cdata = {}
			scanAddresses(data,addresses)
			scanContactData(data,cdata);
			
			data.name = cdata.name;
			local params={
				__SID=data.context.app,
				fields={}
			}
			
			for _,field in ipairs(data.params or {}) do
				if field.type == "CUSTOM"  then
					field.value = field.value or "";
					table.insert(params.fields,{name=field.name,displayName=field.alias,value=field.value})
				end
			end
			local contactform={
				__SID=data.context.app,
				fields={}
			}
			for _,field in ipairs(data.Submit or {}) do
				if field.type == "CUSTOM" then
					field.value = field.value or "" 
					table.insert(contactform.fields,{name=field.name,displayName=field.alias,value=field.value})
				end
			end
			if #params.fields == 0 then
				params = nil
			end
			if #contactform.fields == 0 then
				contactform = nil
			end
			if #addresses  == 0 then
				addresses = nil
			end
			
			status,result = createInteraction(data.context.token,{
				__SID=data.context.app,
				contact={
					name=cdata.name,
					externalId=cdata.contactId,
					addresses=addresses,
				},
				data={
					params=params,
					contactform=contactform
				}	
			});
			if not status then
				result = messages.messageKey('cant_save_contact_form',{reason=result})
			end
		end
	end
	return json.encode({status=status,result=result})
end

function check_survey_validity(web)
	local args = getArgumentsByGET(web)
	LogDebug("[runtime::check_survey_validity] Arguments %s",json.encode(args,true))
	local result
	local existPreviousData
	local sentDate
	local expireTime
	local status = false
	if args.interactionId then
		local irLink = ir.IRAgentLink(config.cluster or "inconcert")
		local details = irLink:GetInteractionDetail{
			proxyType="AGENT",
			id=args.interactionId
		}
		--LogDebug("[runtime::check_survey_validity] INTERACTION %s",json.encode(details,true))
		-- chequeo si no se recibieron datos previamente.
		if details.interaction then
			local interData = details.interaction
			status=true
			existPreviousData = interData.data and interData.data.reportingData and interData.data.reportingData.survey
			sentDate = interData.interactionData and interData.interactionData.finalizationData and interData.interactionData.finalizationData.sendSurveyData
			expireTime = interData.interactionData and interData.interactionData.finalizationData and interData.interactionData.finalizationData.sendSurveyExpireTime
			local now = os.time()
			LogDebug("[runtime::check_survey_validity] existPreviousData %s - now: %s - sentDate: %s - expireTime: %s",json.encode(existPreviousData),now, sentDate, expireTime)
			if existPreviousData then
				result={existPreviousData=existPreviousData}
			else
				if sentDate and expireTime and expireTime > 0 then
					local surveyExpired = (sentDate + expireTime) < now
					if surveyExpired then
						result={surveyExpired=surveyExpired}
					end
				end
			end
		else
			result = messages.messageKey('invalid_request')
		end
		
	else
		result=messages.messageKey('invalid_request')
	end
	
	return json.encode({status=status,result=result})
end

function save_survey_form(web)
	local status
	local data = web.input["post_data"];
	status,data = utils.check_json_params(data)
	if not status then
		return json.encode({status=false,result=data})
	end
	local irLink = ir.IRAgentLink(config.cluster or "inconcert")
	
	-- verificacion de captcha
	local app = application.new{hostVCC = data.context.vcc}	
	local status,result = app:initialize(data.context.app)
	if status then
		local interactionId = data.context.interactionId;
		local campaignAtVCC = data.context.campaignAtVCC;
		status,result = captcha.verify(app,data,web.vars.REMOTE_ADDR)
		if status then
			local function scanAddresses(data,out)
				local scan
				local function scanEmail(data,out)
					if data.type == "MAIL" and data.value and trim(data.value) ~= "" then
						table.insert(out,{type="MAIL",address=data.value})
					end
				end
				local function scanPhone(data,out)
					if (data.type == "PHONE" or data.type == "CELLULAR" or data.type == "HOME" or data.type == "OFFICE") and data.value and trim(data.value) ~= "" then 
						table.insert(out,{
							type = data.type,
							address = data.value,
							countryCode = (data.original_values and data.original_values.countryCode) or nil,
							areaCode = (data.original_values and data.original_values.areaCode) or nil
						})
					end	
				end
				local function scanTwitter(data,out)
					if data.type == "TWITTER" and data.value and trim(data.value) ~= "" then
						table.insert(out,{type="TWITTER",address=data.value})
					end	
				end
				scan = function ( d )
					if type ( d ) == "table" then
						scanEmail(d,out)
						scanPhone(d,out)
						scanTwitter(d,out)
						for _,e in pairs(d) do
							scan(e)
						end
					end
				end
				for _,d in pairs(data) do
					scan(d)
				end
			end
			local function scanContactData(data,out)
				local scan
				scan = function ( d )
					if type ( d ) == "table" then
						if d.type == "NAME" and d.value and trim(d.value) ~= "" then
							out.name = d.value
						elseif d.type == "ID" and d.value and trim(d.value) ~= "" then
							out.contactId = d.value
						elseif d.type == "REASON" and d.value and trim(d.value) ~= "" then
							out.reason = d.value
						end
						for _,e in pairs(d) do
							scan(e)
						end

					end
				end
				for _,d in pairs(data) do
					scan(d)
				end
			end
			
			local addresses = {}
			local cdata = {}
			scanAddresses(data,addresses)
			LogInfo("PRUEBA %s %s", json.encode(data), json.encode(addresses))
			scanContactData(data,cdata);
			
			data.name = cdata.name;
			local params={
				__SID=data.context.app,
				fields={}
			}
			
			for _,field in ipairs(data.params or {}) do
				if field.type == "CUSTOM" and field.value and trim(field.value) ~= "" then
					table.insert(params.fields,{name=field.name,displayName=field.alias,value=field.value})
				end
			end
			local surveyform={
				__SID=data.context.app,
				fields={}
			}
			for _,field in ipairs(data['Survey Screen'] or {}) do
				if field.type == "CUSTOM" and field.value and trim(field.value) ~= "" then
					table.insert(surveyform.fields,{name=field.name,displayName=field.alias,value=field.value})
				end
			end
			if #params.fields == 0 then
				params = nil
			end
			if #surveyform.fields == 0 then
				surveyform = nil
			end
			if #addresses  == 0 then
				addresses = nil
			end
			local contact 
			if cdata.name or cdata.contactId or addresses then
				contact = {
					name=cdata.name,
					externalId=cdata.contactId,
					addresses=addresses,
				}
			end
			
			status,result = pcall(
				function()
					local idata = {
						campaignAtVCC=campaignAtVCC,
						contact=contact,
						data={
							params=param,
							survey=surveyform
						}
					}

					LogDebug("call SetDataForReporting interactionID=%s dataType=%s data=%s",interactionId,"",json.encode(idata))
					local result = irLink:SetDataForReporting(interactionId,"",idata);
					if result.result ~= "OK" then
						error(result.reason)
					end
				end
			);
			if not status then
				result = messages.messageKey('cant_save_survey_form',{reason=result})
			end
			
		end
	end
	return json.encode({status=status,result=result})
end


function ajax_call(web,data)
	local status,result
	data = data or web.input["post_data"];
	LogInfo("ajax_call invoqued ");
	socket.http.TIMEOUT = config.sockethttp_timeout;
	https.TIMEOUT = config.sockethttp_timeout;
	
	local function Get(url, headers, cookie)
		local response_body = {}
		LogInfo ("HTTPGet invoqued %s ",url);
		
		headers = headers or {}
		if (cookie and string.len(cookie)) then
			headers["Cookie"] = cookie
		end
		
		local b, c, h = socket.http.request{
			url = url,
			method = "GET",
			headers = headers,
			sink = ltn12.sink.table(response_body)
		}
		local response = table.concat(response_body,'')
		if (tonumber(c) >= 200) then
			return true, response
		else
			LogDebug ("error getting url %s  result code: %s\nbody:%s",url,tostring(c),response);
			return false, response
		end

	end

	local function Post(url, body, headers, cookie,method) -- method is optional and can be POST or PUT
		LogInfo ("HTTPPost invoqued %s ",url);
	
		local response_body = {}
		headers = headers or {}
		if type (body) == "table" then
			body = json.encode(body);
		end
		headers["content-length"] = string.len(body)
		
		if (cookie and string.len(cookie)) then
			headers["Cookie"] = cookie
		end
		local b, c, h = socket.http.request{
			url = url,
			method = method or "POST",
			headers = headers,
			source = ltn12.source.string(body),
			sink = ltn12.sink.table(response_body)
		}
		local response = table.concat(response_body,'')
		if (tonumber(c) == 200) then
			return true, response
		else
			LogDebug ("error getting url %s  result code: %s\nbody:%s",url,tostring(c),response);

			return false, response
		end
	end
	local function HTTPSGet(url, headers, cookie) 
		LogInfo ("HTTPSGet invoqued %s ",url);
	
		local response_body = {}
		headers = headers or {}
		if (cookie and string.len(cookie)) then
			headers["Cookie"] = cookie
		end
		local b, c, h = https.request{
			url = url,
			method = "GET",
			headers = headers,
			sink = ltn12.sink.table(response_body),
			--protocol = "tlsv1",
			options = "all",
			verify = "none"
			
		}
		local response = table.concat(response_body,'')
		if (tonumber(c) and tonumber(c) >= 200 and tonumber(c)<300) then
			return true, response
		else
			LogDebug ("error getting url %s  result code: %s\nbody:%s",url,tostring(c),response);
			return false, response
		end

	end
	
	local function HTTPSPost(url, body, headers, cookie, method) -- method is optional and can be POST or PUT
		LogInfo ("HTTPSPost invoqued %s ",url);
		local response_body = {}
		headers = headers or {}
		if type (body) == "table" then
			body = json.encode(body);
		end

		headers["content-length"] = string.len(body)

		if (cookie and string.len(cookie)) then
			headers["Cookie"] = cookie
		end
		local b, c, h = https.request{
			url = url,
			method = method or "POST",
			headers = headers,
			source = ltn12.source.string(body),
			sink = ltn12.sink.table(response_body),
			--protocol = "tlsv1",
			options = "all",
			verify = "none"
		}
		local response = table.concat(response_body,'')
		if (tonumber(c) and tonumber(c) >= 200 and tonumber(c)<300) then
			return true, response
		else
			LogDebug ("error getting url %s  result code: %s\nbody:%s",url,tostring(c),response);
			return false, response
		end
	end
	
	if data then
		local status,response
		if type (data) == "string"  then
			LogInfo ("DATA: %s",data);
			data = json.decode(data);
		end
		local params = data.parameters or {}
		socket.http.TIMEOUT = params.timeout or config.sockethttp_timeout;
		https.TIMEOUT = params.timeout or config.sockethttp_timeout;
		socket.BLOCKSIZE = params.blocksize or 2048;
		if data.method == "POST" or data.method == "PUT" then
			if string.sub(data.url,1,8) == "https://" then
				status, response = HTTPSPost(data.url,data.data,data.headers,nil,data.method)
			else
				status, response = Post(data.url,data.data,data.headers,nil,data.method)
			end
		elseif data.method == "GET" then
			if string.sub(data.url,1,8) == "https://" then
				status, response = HTTPSGet (data.url,data.headers)
			else
				status, response = Get (data.url,data.headers)
			end
		end
		LogDebug("ajax_call end of invocation");
		socket.http.TIMEOUT = config.sockethttp_timeout;
		https.TIMEOUT = config.sockethttp_timeout;
		socket.BLOCKSIZE = 2048;
		if status then
			return json.encode({status=true,result=response})
		else
			return json.encode({status=false,result=messages.messageKey('ajax_call_error',{description=response})})
		end
	else
		return json.encode({status=false,result=messages.messageKey('ajax_call_error',{description='Invalid call'})})
	end
	
end

function get_I6_campaign_dispositions(web)

	local data = web.input["post_data"];
	data = json.decode(data);
	local token = inconcert.authUser.authToken
	if token and string.len(token)>0 then
		token  = "SID=" .. token
	else
		return false,messages.messageKey('inconcert_autentication_error',{errText=err});
	end
	return ajax_call(web,{
		url="http://" .. config.integration_address .. ":" .. config.integration_port .. "/inconcert/apps/agent/campaigns/get_campaign_info/" .. url_encode(data.campaignId),
		method="POST",
		data="",
		headers={
			Cookie=token
		}
	});
end
function check_server_process(web)
	local status,result
	local data = web.input["post_data"];
	if data then
		data = json.decode(data);
	end
	local processId = data.processId;
	local session = tweetyio.session();
	session.tweetyprocess = session.tweetyprocess or {}
	local process = session.tweetyprocess[processId]
	if process then
		LogDebug(json.encode(process.messages or {}));
		local msgs
		local pdata
		process.stop = data.stop;
		if process.data then
			pdata = process.data or {};
			if data then
				for k,v in pairs(data) do
					pdata[k]=v
					
				end
				process.data = pdata;
				data = pdata;
			end
		end
		if process.messages then
			msgs = process.messages;
			process.messages = {};
		end
		

		session:release();
		return json.encode({status=true,result={processCompletion=process.processCompletion,isCompleted=process.isCompleted,haveError=process.haveError,messages=msgs,data = data}})
	else
		session:release();
		return json.encode({status=false,result=messages.messageKey('argument_error',{module="runtime.lua",func="check_server_process",param="processId",value=processId})});
	end
end

function application_start_report(web,appName,processId)
	local status,result
	LogDebug("Start report")
	local app = application.new{vccName=getCurrentVCCName()}
	status,result = app:initialize(appName);
	if status then
		status,result = app:startReportProcess(processId);
	end
	return json.encode({status=status,result=result})
end
function application_start_report_async(web,appName,processId)
	--local cookie,value = inconcert.authUser:GetHttpCredentials():match("([^=]-)=(.+)");
	local cookie = "AUTH_SID_WEBDESIGNER"
	local value = inconcert.authUser.authToken;
	LogDebug("Cookie = %s value = %s",cookie,value)
	local task = AsyncTasks{
		url = config.myself .. "/inconcert/apps/webdesigner/buildreport/" ..  appName .. "/"..processId,
		cookies = {
			[cookie] = value
		}
	}
	task:Add("webdesigner");
	return json.encode({status=true})
end

function application_report_info(web,appName,reportName)
	local status,result
	local app = application.new{vccName=getCurrentVCCName()}
	status,result = app:initialize(appName);
	if status then
		status,result = app:getReportInfo(reportName);
	end
	return json.encode({status=status,result=result})
end
function application_view_report(web,arguments)
	local status,result
	web.headers["Content-Type"] = "text/html; charset=utf-8";
	local app = application.new{vccName=getCurrentVCCName()}
	status,result = app:initialize(arguments.app);
	if status then
		return app:renderReportHTML(arguments.id,tonumber(arguments.pg),tonumber(arguments.pf),tonumber(arguments.prn));
	else
		return "";
	end
end
function application_xlsexport_report(web,arguments)
	local status,result
	web.headers["Content-Type"] = "application/vnd.ms-excel; charset=utf-8";
	local app = application.new{vccName=getCurrentVCCName()}
	status,result = app:initialize(arguments.app);
	if status then
		return app:renderReportXLS(arguments.id,tonumber(arguments.pg),tonumber(arguments.pf));
	else
		return "";
	end
end
function gen_messages(web,locale_code)
	web:content_type("text/javascript")
	locale_code = locale_code or ""
	local res = {}
	for k,msg in pairs(messages.codes) do 
		res[k]={
			description = msg.description,
			code = msg.code,
			level = msg.level
		};
	end
	return string.format(
	[[
		icc.ui.messages={};
		icc.ui.messages.codes = %s
		icc.ui.messages.messageKey = function (key,values){
			var res = icc.ui.messages.codes[key];
			if(res === undefined){
				res = icc.ui.messages.codes.unknown;
			}
			res = icc.utils.clone(res);
			res.className='nativeError';
			if(values !== undefined){
				for ( var k in values){
					if(values[k]!==undefined){
						res[k] = values[k];
					}
				}
			}
			if(res.description !== undefined){
				res.description=res.description.replace(new RegExp("{(\\w+)}","g"),
					function(match, value) {
						return res[value] !==undefined?res[value]:""
					}
				)
			}

			return res;
		};
		icc.ui.messages.messageCode = function (code,values){

			var res;
			for (var k in  icc.ui.messages.codes){
				if (icc.ui.messages.codes[k] === code){
					res = icc.ui.messages.codes[k];
					break;
				}
			}
			if(res === undefined){
				res = icc.ui.messages.codes.unknown;
			}
			res = icc.utils.clone(res);
			res.className='nativeError';
			if(values !== undefined){
				for ( var k in values){
					if(values[k]!==undefined){
						res[k] = values[k];
					}
				}
			}
			if(res.description !== undefined){
				res.description=res.description.replace(new RegExp("{(\\w+)}","g"),
					function(match, value) {
						return res[value] !==undefined?res[value]:""
					}
				)
			}
			return res;
		};
		icc.ui.message = icc.ui.messages.messageKey
	]],json.encode(res));
end

local m_designerAppPermission = nil
function setViewEnvironment(env)
	assert(env and env.designerAppPermission)
	m_designerAppPermission = env.designerAppPermission
end

local function validate_captcha(web)
	local status,result,data
	status,data = utils.check_json_params(web.input["post_data"] or "{}")
	if not status then
		return json.encode({status=false,result=data})
	end
	status,result = captcha.validateCaptcha(data.response, web.vars.REMOTE_ADDR)
	return json.encode({status=status,result=result})
end

function initialize(_M, prefix,appLocation)
	prefix = prefix or ""
	m_appLocation = appLocation
	
	_M:dispatch_get(gen_messages,"/generated/icc%-ui%-messages([^/]*)%.js")

	_M:dispatch_post(
		function(web)
			return ajax_call(web)
		end,
		"/runtime/ajax_call"
	)
	
	_M:dispatch_post(get_I6_campaign_dispositions,"/runtime/get_i6_campaign_dispositions");
	_M:dispatch_post(save_contact_form,"/runtime/save_contact_form");
	_M:dispatch_post(save_postsurvey_data,"/runtime/save_postsurvey_data");
	_M:dispatch_post(save_survey_form,"/runtime/save_survey_form");
	_M:dispatch_get(check_survey_validity,"/runtime/check_survey_validity");
	
	
	_M:dispatch_get(
		function(web, appName)
			if appName:lower() == "designer" and not m_designerAppPermission:HasPermissionTo(tweety_permisions_actionId, "entities") then
				LogWarning("403 forbidden - Action: '%s', domain: '%s' on application: '%s'", "entities", tweety_permisions_actionId, "designer")
				web.status = "403"
				return "forbidden"
			end
			return application_render_defaultform(web,appName,getArgumentsByGET(web))
		end,"/([^/%.]+)")
		_M:dispatch_get(
		
		function(web)
			if not m_designerAppPermission:HasPermissionTo(tweety_permisions_actionId, "entities") then
				LogWarning("403 forbidden - Action: '%s', domain: '%s' on application: '%s'", "entities", tweety_permisions_actionId, "designer")
				web.status = "403"
				return "forbidden"
			end
			return application_render_defaultform(web,"designer",getArgumentsByGET(web))
		end,"/")
			
	_M:dispatch_post(
		
		function(web,appName)
			if appName:lower() == "designer" and not m_designerAppPermission:HasPermissionTo(tweety_permisions_actionId, "entities") then
				LogWarning("403 Forbidden - Action: '%s', domain: '%s' on application: '%s'", "entities", tweety_permisions_actionId, "designer")
				web.status = "403"
				return "forbidden"
			end
			return application_render_defaultform(web,appName,getArgumentsByPOST(web))
		end,"/([^/%.]+)")

	_M:dispatch_get(
		function(web,appName,formName)
			return application_render_form(web,appName,formName,getArgumentsByGET(web))
		end, "/([^/%.]+)%.([^/]+)")

	_M:dispatch_post(function(web,appName,formName)
			return application_render_form(web,appName,formName,getArgumentsByPOST(web))
		end,"/([^/%.]+)%.([^/]+)")

	_M:dispatch_post(application_start_report,"/buildreport/([^/]+)/([^/]+)")

	_M:dispatch_post(application_start_report_async,"/buildreport_async/([^/]+)/([^/]+)");

	_M:dispatch_get(application_report_info,"/runtime/reportinfo/([^/]+)/([^/]+)")
	_M:dispatch_get(click_to_contact_form,"/runtime/click_to_contact_form")
	
	_M:dispatch_post(check_server_process,"/runtime/check_process")
	_M:dispatch_post(create_server_process,"/runtime/create_process")
	_M:dispatch_get(
		function(web)
			return application_view_report(web,getArgumentsByGET(web))
		end,"/reportview/")
	_M:dispatch_get(
		function(web)
			return application_xlsexport_report(web,getArgumentsByGET(web))
		end,"/reportxls/([^/%.]+)%.xls")
		
	_M:dispatch_get(application_build_form, "/buildform/([^/]+)/([^/]+)/([^/]+)")
		
	_M:dispatch_get(application_get_chat_first_form, "/get_chat_first_form/([^/]+)/([^/]+)/([^/]+)")

	_M:dispatch_get(read_resource, "/([^/]+)/([^/]+)/resources/([^/]+)");
	
	_M:dispatch_get(read_datasource, "/([^/]+)/([^/]+)/datasource/([^/]+)");
	_M:dispatch_get(read_datasource, "/([^/]+)/([^/]+)/datasource_cti/([^/]+)");
	_M:dispatch_get(function(web) return "" end, "/alive/");

	_M:dispatch_post(validate_captcha,"/runtime/validate_captcha");
end
