
module("Datasource field type validation")

test("Operacion save no permitida por campo sin valor que no permite nulos",
	function(){
		expect(8);
		stop();
		loadForm("frmTest14",function(status,form,controls){
			start();
			ok(status,"Carga del formulario frmTest14");
			var ds = form.environment.datasources['dsTest7'];
			ok(ds != undefined,"dsTest7 está presente en el formulario.");
			
			var serverCall=false;
			var rows_populated_event = false;
			var ready_event = false;

			var url=""
			var requestParams = {content:[]}

 			if(ds){
			    ds.ajax = function(request){
				    url = request.url;
				    serverCall = true;
				    requestParams = JSON.parse(request.data);
				    request.success({"status":true})
			    }

			}
			stop();
			form.onReady(function(){
			    start();	
			    
			    ok(ds.isFieldEnabled("field1") == true,"isFieldEnabled retorna true");
			    ok(ds.isFieldRequired("field1") == true,"isFieldRequired retorna true");
			    ok(ds.isFieldEditable("field1") == true,"isFieldEditable retorna true");
			    ok(ds.save() == false,"save retorna false");
			    ok(ds.lastError.code == 1030,"lastError code es 1030 (fields_validation_error)");
			    ok(ds.lastError.field_errors['field1'] && ds.lastError.field_errors['field1'].code == 1020,"Error del campo field1 es 1020 (missing_field)")
			});
		})
	}
);

test("Operacion save permitida para campo que no permite nulos, tiene valor inicial en blank y no tiene control asignado",
	function(){
		expect(10)
		stop();
		loadForm("frmTest15",function(status,form,controls){
			start();
			ok(status,"Carga del formulario frmTest15");
			var ds = form.environment.datasources['dsTest8']
			ok(ds != undefined,"dsTest8 está presente en el formulario");
			var serverCall=false;
			var rows_populated_event = false;
			var ready_event = false;
			var url=""
			var requestParams = {content:[]};
 			if(ds){
			    ds.ajax = function(request){
				    url = request.url;
				    serverCall = true;
				    requestParams = JSON.parse(request.data);
				    request.success({"status":true})
			    }

			}
			
			stop();
			form.onReady(function(){
			    start();	
			    ok(ds.row().field1 === '',"Row field1 tiene el valor inicial esperado");
			    ok(ds.isFieldEnabled("field1") == true,"isFieldEnabled retorna true");
			    ok(ds.isFieldRequired("field1") == true,"isFieldRequired retorna true");
			    ok(ds.isFieldEditable("field1") == true,"isFieldEditable retorna true");
			    ok(ds.save() == true,"save retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case '_rowid':
					    ok(param.value === 1,"request param "+param.name + " Tiene el valor (1) esperado");	
				    break;
				    case 'field1':
					    ok(param.value === '',"request param "+param.name + " Tiene el valor blank esperado");
				    break;

				    default:
					    ok(param.value == undefined,"request param "+param.name + " No tiene valor");
				    break;

				}
			    });

			});
		})
	}
);

test("Operacion save no permitida por campo que no permite nulos, con valor inicial en blank y con control asignado",
	function(){
		expect(9);
		stop();
		loadForm("frmTest16",function(status,form,controls){
			start();
		    ok(status,"Carga del formulario frmTest16");
			var ds = form.environment.datasources['dsTest8']
		    ok(ds != undefined,"dsTest7 está presente en el formulario");
			
			var serverCall=false;
			var rows_populated_event = false;
			var ready_event = false;

			var url=""
			var requestParams = {content:[]}

 			if(ds){
			    ds.ajax = function(request){
				    url = request.url;
				    serverCall = true;
				    requestParams = JSON.parse(request.data);
				    request.success({"status":true})
			    }

			}
			stop();
			form.onReady(function(){
			    start();	
			    ok(ds.row().field1 === '',"Row field1 tiene el valor inicial esperado");
			    ok(ds.isFieldEnabled("field1") == true,"isFieldEnabled retorna true");
			    ok(ds.isFieldRequired("field1") == true,"isFieldRequired retorna true");
			    ok(ds.isFieldEditable("field1") == true,"isFieldEditable retorna true");
			    ok(ds.save() == false,"save retorna false");
			    ok(ds.lastError.code == 1030,"lastError code es 1030 (fields_validation_error)");
			    ok(ds.lastError.field_errors['field1'] && ds.lastError.field_errors['field1'].code == 1020,"Error del campo field1 es 1020 (missing_field)")
			});
		})
	}
);

test("Operacion save permitida para campo que no permite nulos, tiene valor inicial y tiene control asignado",
	function(){
		expect(10);
		stop();
		loadForm("frmTest17",function(status,form,controls){
			start();
		    ok(status,"Carga del formulario frmTest17");
			var ds = form.environment.datasources['dsTest9']
		    ok(ds != undefined,"dsTest9 está presente en el formulario");
			
			var serverCall=false;
			var rows_populated_event = false;
			var ready_event = false;

			var url=""
			var requestParams = {content:[]}

 			if(ds){
			    ds.ajax = function(request){
				    url = request.url;
				    serverCall = true;
				    requestParams = JSON.parse(request.data);
				    request.success({"status":true})
			    }

			}
			stop();
			form.onReady(function(){
			    start();	
			    
			    ok(ds.isFieldEnabled("field1") == true,"isFieldEnabled retorna true");
			    ok(ds.isFieldRequired("field1") == true,"isFieldRequired retorna true");
			    ok(ds.isFieldEditable("field1") == true,"isFieldEditable retorna true");
			    ok(ds.row().field1 == 'valor inicial',"Row field1 tiene el valor inicial esperado");
			    ok(ds.save() == true,"save retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case '_rowid':
					    ok(param.value === 1,"request param "+param.name + " Tiene el valor (1) esperado");	
				    break;
				    case 'field1':
					    ok(param.value === 'valor inicial',"request param "+param.name + " Tiene el valor esperado");
				    break;

				    default:
					    ok(param.value == undefined,"request param "+param.name + " No tiene valor");
				    break;

				}
			    });

			});
		})
	}
);
test("Operacion save no permitida para campo que no permite nulos, tiene valor inicial y control asignado, datos borrados por api",
	function(){
		
		expect(10);
		stop();
		
		loadForm("frmTest17",function(status,form,controls){
			start();
			ok(status,"Carga del formulario frmTest17");
			
			var ds = form.environment.datasources['dsTest9']
			ok(ds != undefined,"dsTest9 está presente en el formulario");
			
			var serverCall=false;
			var rows_populated_event = false;
			var ready_event = false;

			var url=""
			var requestParams = {content:[]}

 			if(ds){
			    ds.ajax = function(request){
				    url = request.url;
				    serverCall = true;
				    requestParams = JSON.parse(request.data);
				    request.success({"status":true})
			    }

			}
			stop();
			form.onReady(function(){
			    start();	
			    
			    ok(ds.isFieldEnabled("field1") == true,"isFieldEnabled retorna true");
			    ok(ds.isFieldRequired("field1") == true,"isFieldRequired retorna true");
			    ok(ds.isFieldEditable("field1") == true,"isFieldEditable retorna true");
			    ok(ds.row().field1 === 'valor inicial',"Row field1 tiene el valor inicial esperado");
			    ds.set('field1',undefined);
			    ok(ds.row().field1 === undefined,"Row field1 tiene el valor undefined esperado luego del 'set'");

			    ok(ds.save() == false,"save retorna false");
			    ok(ds.lastError.code == 1030,"lastError code es 1030 (fields_validation_error)");
			    ok(ds.lastError.field_errors['field1'] && ds.lastError.field_errors['field1'].code == 1020,"Error del campo field1 es 1020 (missing_field)")

			});
		})
	}
);

test("Validacion de campo tipo 'integer' ",
	function(){
		expect(33);
		stop();
		loadForm("frmTest18",function(status,form,controls){
			start();
			ok(status,"Carga del formulario frmTest19");
			var ds = form.environment.datasources['dsTest10']
			ok(ds != undefined,"dsTest10 está presente en el formulario");
			var serverCall=false;
			var rows_populated_event = false;
			var ready_event = false;

			var url=""
			var requestParams = {content:[]}

 			if(ds){
			    ds.ajax = function(request){
				    url = request.url;
				    serverCall = true;
				    requestParams = JSON.parse(request.data);
				    request.success({"status":true})
			    }

			}
			stop();
			form.onReady(function(){
			    start();	
			    ok(ds.mode == "browse","dsTest10 esta en modo browse");
			    ds.appendNew();
			    ds.set('integerField1','234');	
			    ok(ds.row().integerField1 === 234,"Asignacion de un string '234' al campo con conversion correcta a integer");
			    ok(ds.save() == true,"save retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'integerField1':
					    ok(param.value === 234,"request param "+param.name + " Tiene el valor 234 esperado");
				    break;
				}
			    });
			    
			    ds.appendNew();

			    ds.set('integerField1','-234');	
			    ok(ds.row().integerField1 === -234,"Asignacion de un string '-234' al campo con conversion correcta a integer");
			    ok(ds.save() == true,"save retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'integerField1':
					    ok(param.value === -234,"request param "+param.name + " Tiene el valor -234 esperado");
				    break;
				}
			    });

			    ds.appendNew();
			    ds.set('integerField1','0');	
			    ok(ds.row().integerField1 === 0,"Asignacion de un string '0' al campo con conversion correcta a integer");
			    ok(ds.save() == true,"save retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'integerField1':
					    ok(param.value === 0,"request param "+param.name + " Tiene el valor 0 esperado");
				    break;
				}
			    });

			    ds.appendNew();
			    ds.set('integerField1',0);	
			    ok(ds.row().integerField1 === 0,"Asignacion de un numero 0 al campo con conversion correcta a integer");
			    ok(ds.save() == true,"save retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'integerField1':
					    ok(param.value === 0,"request param "+param.name + " Tiene el valor 0 esperado");
				    break;
				}
			    });
			    ds.set('integerField1',234);	
			    ok(ds.row().integerField1 === 234,"Asignacion de un numero 234 al campo con conversion correcta a integer");
			    ok(ds.save() == true,"save retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'integerField1':
					    ok(param.value === 234,"request param "+param.name + " Tiene el valor -234 esperado");
				    break;
				}
			    });


			    ds.set('integerField1',234.86);	
			    ok(ds.row().integerField1 === 234.86,"Asignacion de un numero 234.86 al campo sin conversion posible a integer");
			    ok(ds.save() == false,"save retorna false");

			    ok(ds.lastError.code == 1030,"lastError code es 1030 (fields_validation_error)");
			    ok(ds.lastError.field_errors['integerField1'] && ds.lastError.field_errors['integerField1'].code == 1021,"Error del campo integerField1 es 1021 (integer_format_error)")


			    ds.set('integerField1','234.86');	
			    ok(ds.row().integerField1 === '234.86',"Asignacion de un string '234.86' al campo sin conversion posible a integer");
			    ok(ds.save() == false,"save retorna false");

			    ok(ds.lastError.code == 1030,"lastError code es 1030 (fields_validation_error)");
			    ok(ds.lastError.field_errors['integerField1'] && ds.lastError.field_errors['integerField1'].code == 1021,"Error del campo integerField1 es 1021 (integer_format_error)")

			    ds.set('integerField1','293uu');	
			    ok(ds.row().integerField1 === '293uu',"Asignacion de un string '293uu' sin conversion posible");
			    ok(ds.save() == false,"save retorna false");
			    ok(ds.lastError.code == 1030,"lastError code es 1030 (fields_validation_error)");
			    ok(ds.lastError.field_errors['integerField1'] && ds.lastError.field_errors['integerField1'].code == 1021,"Error del campo integerField1 es 1021 (integer_format_error)")
			    ds.set('integerField1',undefined);	
			    ok(ds.row().integerField1 === undefined,"Asignacion de undefined sin conversion posible");
			    ok(ds.save() == true,"save retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'integerField1':
					    ok(param.value === undefined,"request param "+param.name + " No tiene valor");
				    break;
				}
			    });


			});
		})
	}
);

test("Validacion de campo tipo 'numeric' ",
	function(){
		expect(22);
		stop();
		loadForm("frmTest18",function(status,form,controls){
			start();
			ok(status,"Carga del formulario frmTest19");
			
			var ds = form.environment.datasources['dsTest10']
			ok(ds != undefined,"dsTest10 está presente en el formulario");
			
			var serverCall=false;
			var rows_populated_event = false;
			var ready_event = false;

			var url=""
			var requestParams = {content:[]}

 			if(ds){
			    ds.ajax = function(request){
				    url = request.url;
				    serverCall = true;
				    requestParams = JSON.parse(request.data);
				    request.success({"status":true})
			    }

			}
			stop();
			form.onReady(function(){
			    start();	
			    ok(ds.mode == "browse","dsTest10 esta en modo browse");
			    ds.appendNew();
			    ds.set('numericField1','234.87');	
			    ok(ds.row().numericField1 === 234.87,"Asignacion de un string '234.87' al campo con conversion correcta a numeric");
			    ok(ds.save() == true,"save retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'numericField1':
					    ok(param.value === 234.87,"request param "+param.name + " Tiene el valor 234.87 esperado");
				    break;
				}
			    });
			    
			    ds.appendNew();

			    ds.set('numericField1','0');	
			    ok(ds.row().numericField1 === 0,"Asignacion de un string '0' al campo con conversion correcta a numeric");
			    ok(ds.save() == true,"save retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'numericField1':
					    ok(param.value === 0,"request param "+param.name + " Tiene el valor 0 esperado");
				    break;
				}
			    });

			    ds.appendNew();
			    ds.set('numericField1',0);	
			    ok(ds.row().numericField1 === 0,"Asignacion de un numero 0 al campo con conversion correcta a numeric");
			    ok(ds.save() == true,"save retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'numericField1':
					    ok(param.value === 0,"request param "+param.name + " Tiene el valor 0 esperado");
				    break;
				}
			    });
			    ds.set('numericField1',234.56);	
			    ok(ds.row().numericField1 === 234.56,"Asignacion de un numero 234.56 al campo con conversion correcta a numeric");
			    ok(ds.save() == true,"save retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'numericField1':
					    ok(param.value === 234.56,"request param "+param.name + " Tiene el valor 234.56 esperado");
				    break;
				}
			    });


			    ds.set('numericField1','293.33uu');	
				
			    ok(ds.row().numericField1 === '293.33uu',"Asignacion de un string '293.33uu' sin conversion posible");
			    ok(ds.save() == false,"save retorna false");
			    ok(ds.lastError.code == 1030,"lastError code es 1030 (fields_validation_error)");
			    ok(ds.lastError.field_errors['numericField1'] && ds.lastError.field_errors['numericField1'].code == 1033,"Error del campo numericField1 es 1033 (numeric_format_error)")
			    ds.set('numericField1',undefined);	
			    ok(ds.row().numericField1 === undefined,"Asignacion de undefined sin conversion posible");
			    ok(ds.save() == true,"save retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'numericField1':
					    ok(param.value === undefined,"request param "+param.name + " No tiene valor");
				    break;
				}
			    });


			});
		})
	}
);

test("Validacion de campo tipo 'datetime' ",
	function(){
		expect(18)
		stop();
		loadForm("frmTest18",function(status,form,controls){
			start();
			ok(status,"Carga del formulario frmTest19");
			var ds = form.environment.datasources['dsTest10']
			ok(ds != undefined,"dsTest10 está presente en el formulario");
			
			var serverCall=false;
			var rows_populated_event = false;
			var ready_event = false;

			var url=""
			var requestParams = {content:[]}

 			if(ds){
			    ds.ajax = function(request){
				    url = request.url;
				    serverCall = true;
				    requestParams = JSON.parse(request.data);
				    request.success({"status":true})
			    }

			}
			stop();
			form.onReady(function(){
			    start();	
			    ok(ds.mode == "browse","dsTest10 esta en modo browse");
			    ds.appendNew();
			    ds.set('datetimeField1','2010-04-20');	
			    ok(ds.save() == true,"save con valor '2010-04-20' retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'datetimeField1':
					
					    ok(param.value === '2010-04-20 00:00:00',"request param "+param.name + " Tiene el valor '2010-04-20 00:00:00' esperado");
				    break;
				}
			    });
			    
			    ds.appendNew();

			    ds.set('datetimeField1','2010-04-20 14:30:23');	
			    ok(ds.save() == true,"save con valor '2010-04-20 14:30:23' retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'datetimeField1':
					
					    ok(param.value === '2010-04-20 14:30:23',"request param "+param.name + " Tiene el valor '2010-04-20 14:30:23' esperado");
				    break;
				}
			    });

			    ds.appendNew();

			    ds.set('datetimeField1','2010-04-20 24:30:23');	
			    ok(ds.save() == false,"save con valor '2010-04-20 24:30:23' retorna false");
			    ok(ds.lastError.code == 1030,"lastError code es 1030 (fields_validation_error)");
			    ok(ds.lastError.field_errors['datetimeField1'] && ds.lastError.field_errors['datetimeField1'].code == 1023,"Error del campo datetimeField1 es 1023 (datetime_format_error)")


			    ds.set('datetimeField1','0');	
			    ok(ds.save() == false,"save con valor '0' retorna false");
			    ok(ds.lastError.code == 1030,"lastError code es 1030 (fields_validation_error)");
			    ok(ds.lastError.field_errors['datetimeField1'] && ds.lastError.field_errors['datetimeField1'].code == 1023,"Error del campo datetimeField1 es 1023 (datetime_format_error)")

			    ds.set('datetimeField1','2011-02-29');	
			    ok(ds.save() == false,"save con valor '2011-02-29' retorna false");
			    ok(ds.lastError.code == 1030,"lastError code es 1030 (fields_validation_error)");
			    ok(ds.lastError.field_errors['datetimeField1'] && ds.lastError.field_errors['datetimeField1'].code == 1023,"Error del campo datetimeField1 es 1023 (datetime_format_error)")

			    ds.set('datetimeField1',undefined);	
			   
			    ok(ds.save() == true,"save con valor undefined retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'datetimeField1':
					    ok(param.value === undefined,"request param "+param.name + " No tiene valor");
				    break;
				}
			    });


			});
		})
	}
);
test("Validacion de campo tipo 'date' ",
	function(){
		expect(17)
		stop();
		loadForm("frmTest18",function(status,form,controls){
			start();
			ok(status,"Carga del formulario frmTest19");

			var ds = form.environment.datasources['dsTest10']
			ok(ds != undefined,"dsTest10 está presente en el formulario");
			
			var serverCall=false;
			var rows_populated_event = false;
			var ready_event = false;

			var url=""
			var requestParams = {content:[]}

 			if(ds){
			    ds.ajax = function(request){
				    url = request.url;
				    serverCall = true;
				    requestParams = JSON.parse(request.data);
				    request.success({"status":true})
			    }

			}
			stop();
			form.onReady(function(){
			    start();	
			    ok(ds.mode == "browse","dsTest10 esta en modo browse");
			    ds.appendNew();
			    ds.set('dateField1','2010-04-20');	
			    ok(ds.save() == true,"save con valor '2010-04-20' retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'dateField1':
					
					    ok(param.value === '2010-04-20',"request param "+param.name + " Tiene el valor '2010-04-20' esperado");
				    break;
				}
			    });
			    
			    ds.appendNew();

			    ds.set('dateField1','2010-04-20 14:30:23');	
			    ok(ds.save() == true,"save con valor '2010-04-20 14:30:23' retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'dateField1':
					
					    ok(param.value === '2010-04-20',"request param "+param.name + " Tiene el valor '2010-04-20' esperado");
				    break;
				}
			    });

			    ds.appendNew();

			    ds.set('dateField1','2010-04-20 24:30:23');	
			    ok(ds.save() == true,"save con valor '2010-04-20 24:30:23' retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'dateField1':
					
					    ok(param.value === '2010-04-20',"request param "+param.name + " Tiene el valor '2010-04-20' esperado");
				    break;
				}
			    });


			    ds.set('dateField1','0');	
			    ok(ds.save() == false,"save con valor '0' retorna false");
			    ok(ds.lastError.code == 1030,"lastError code es 1030 (fields_validation_error)");
			    ok(ds.lastError.field_errors['dateField1'] && ds.lastError.field_errors['dateField1'].code == 1032,"Error del campo datetimeField1 es 1032 (date_format_error)")

			    ds.set('dateField1','2011-02-29');	
			    ok(ds.save() == false,"save con valor '2011-02-29' retorna false");
			    ok(ds.lastError.code == 1030,"lastError code es 1030 (fields_validation_error)");
			    ok(ds.lastError.field_errors['dateField1'] && ds.lastError.field_errors['dateField1'].code == 1032,"Error del campo datetimeField1 es 1032 (date_format_error)")

			    ds.set('dateField1',undefined);	
			   
			    ok(ds.save() == true,"save con valor undefined retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'dateField1':
					    ok(param.value === undefined,"request param "+param.name + " No tiene valor");
				    break;
				}
			    });


			});
		})
	}
);

test("Validacion de campo tipo 'boolean' ",
	function(){
		expect(19)
		stop();
		loadForm("frmTest18",function(status,form,controls){
			start();
			ok(status,"Carga del formulario frmTest19");
			
			var ds = form.environment.datasources['dsTest10']
			ok(ds != undefined,"dsTest10 está presente en el formulario");
			
			var serverCall=false;
			var rows_populated_event = false;
			var ready_event = false;

			var url=""
			var requestParams = {content:[]}

 			if(ds){
			    ds.ajax = function(request){
				    url = request.url;
				    serverCall = true;
				    requestParams = JSON.parse(request.data);
				    request.success({"status":true})
			    }

			}
			stop();
			form.onReady(function(){
			    start();	
			    ok(ds.mode == "browse","dsTest10 esta en modo browse");
			    ds.appendNew();
			    ds.set('booleanField1',true);	
			    ok(ds.save() == true,"save con valor true retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'booleanField1':
					
					    ok(param.value === true,"request param "+param.name + " Tiene el valor true esperado");
				    break;
				}
			    });
			    
			    ds.appendNew();

			    ds.set('booleanField1',false);	
			    ok(ds.save() == true,"save con valor false retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'booleanField1':
					
					    ok(param.value === false,"request param "+param.name + " Tiene el valor false esperado");
				    break;
				}
			    });
				
			    ds.appendNew();
			    ds.set('booleanField1','true');	
	   		    ok(ds.row().booleanField1===true,"set de campo booleanField1 con string 'true' con conversion a true")
			    ok(ds.save() == true,"save con valor 'true' retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'booleanField1':
					
					    ok(param.value === true,"request param "+param.name + " Tiene el valor true esperado");
				    break;
				}
			    });

			    ds.appendNew();

			    ds.set('booleanField1','false');	
				ok(ds.row().booleanField1===false,"set de campo booleanField1 con string 'false' con conversion a false")
				
			    ok(ds.save() == true,"save con valor false retorna true");
				
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'booleanField1':
					
					    ok(param.value === false,"request param "+param.name + " Tiene el valor false esperado");
				    break;
				}
			    });

				ds.appendNew();
			    ds.set('booleanField1',undefined);
				ok(ds.row().booleanField1===false,'set de campo booleanField1 con undefined con conversion a false')
				
			    ok(ds.save() == true,"save con valor undefined retorna true");
				
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'booleanField1':
					
					    ok(param.value === false,"request param "+param.name + " Tiene el valor false esperado");
				    break;
				}
			    });

				
			    ds.appendNew();
			    ds.set('booleanField1','alfa');	
				
	  		    ok(ds.row().booleanField1 === true,"set de campo booleanField1 con string 'alfa' con conversion a true")
				
			    ok(ds.save() == true,"save con valor 'alfa' retorna true");
			    jQuery(requestParams.content).each(function(idx,param){
				switch(param.name){
				    case 'booleanField1':
					
					    ok(param.value === true,"request param "+param.name + " Tiene el valor true esperado");
				    break;
				}
			    });




			});
		})
	}
);
