module(...,package.seeall)

local svg={}
local function style2str(style)
	local res={}
	for k,v in pairs(style) do
		table.insert(res,k)
		table.insert(res,":");
		table.insert(res,tostring(v));
		table.insert(res,";")
	end
	return table.concat(res);
end
local function attributes2str(svgobject)
	local res = {}
	for k,v in pairs(svgobject) do
		if type(k) ~= "number" and type(v) ~= "function" then
			table.insert(res,k);
			table.insert(res,"=");
			table.insert(res,"\"");
			if k=="style" then
				table.insert(res,style2str(v));
			else
				table.insert(res,tostring(v));
			end
			table.insert(res,"\"");
			table.insert(res," ");
		end
	end
	return table.concat(res);
end
local function simple_tag2str(tagname,data)
	local res = {};
	table.insert(res,"<");
	table.insert(res,tagname);
	table.insert(res," ");
	table.insert(res,attributes2str(data))
	table.insert(res," />");
	return table.concat(res);
end
local function complex_tag2str(tagname,data)
	local res = {};
	table.insert(res,"<");
	table.insert(res,tagname);
	table.insert(res," ");
	table.insert(res,attributes2str(data))
	table.insert(res," >");

	for k,v in ipairs(data) do
		table.insert(res,v:tosvg())
		table.insert(res,"\r\n");
	end
	table.insert(res,"</");
	table.insert(res,tagname);
	table.insert(res,">");

	return table.concat(res);
end

function svg.new(self)
	self["xmlns:svg"]="http://www.w3.org/2000/svg"
 	self["xmlns"]="http://www.w3.org/2000/svg"
   	self["xmlns:xlink"]="http://www.w3.org/1999/xlink"
   	self["version"]="1.0"
 
	function self:tosvg()
		local res = {[[<?xml version="1.0" encoding="UTF-8" standalone="no"?>]],"\r\n",complex_tag2str("svg",self)};
		return table.concat(res);
	end
	return self
end
function svg.layer(self)
	function self:tosvg()
		return complex_tag2str("g",self);
	end
	return self
end
function svg.rect(self)
	function self:tosvg()
		return simple_tag2str("rect",self);
	end
	return self
end
function svg.defs(self)
	function self:tosvg()
		return complex_tag2str("defs",self);
	end
	return self
end
function svg.linearGradient(self)
	function self:tosvg()
		return complex_tag2str("linearGradient",self);
	end
	return self;
end
function svg.stop(self)
	function self:tosvg()
		return simple_tag2str("stop",self);
	end
	return self;
end
function svg.path(self)
	function self:tosvg()
		return simple_tag2str("path",self);
	end
	return self;
end
return svg