module(...,package.seeall)
local config = require (tweety_modules_prefix .. "config")
local messages = require (tweety_modules_prefix .. "messages")
local application = require (tweety_modules_prefix .. "application")
require "LuaOrm.Date"
local tweetyio = require ( tweety_modules_prefix .. "tweetyio" )
local utils = require (tweety_modules_prefix .. "utils")

-- lista los templates disponibles
function list (appClass, appSubClass, _T)
	local result = {}
	local iter, dir_obj = tweetyio.dir (config.templateDirPath .. config.pathSeparator)

	local dir_entry = iter(dir_obj)
	local fh
	local catalogEntry
	if not dir_entry then
		LogDebug("can't open path %s",config.templateDirPath);
		return false,messages.messageKey('path_access_error',{path=config.templateDirPath})
	end
	while dir_entry do
		if dir_entry ~= "." and dir_entry ~= ".." then
			fh = tweetyio.open(config.templateDirPath .. config.pathSeparator .. dir_entry .. config.pathSeparator .. config.templateFile,"rb")
			if fh then
				local content = fh:read("*a")
				local templateEntry = json.decode(content)
				templateEntry.id = dir_entry;
				if appClass then -- Carga el application.properties y compara el appClass
					local fileName = config.templateDirPath .. config.pathSeparator .. dir_entry .. config.pathSeparator .. config.mainAppFile
					local appfh = tweetyio.open(fileName,"rb")

					if appfh then
						local content = appfh:read("*a")
						tweetyio.close(appfh)
						local appData = json.decode(content)
						if  appData.catalog.appClass == appClass then
							if appSubClass and appData.catalog.appSubClass then
								if appData.catalog.appSubClass == appSubClass then
									table.insert(result,templateEntry)
								end
							else
								table.insert(result,templateEntry)
							end
						end
					end
				else
					table.insert(result,templateEntry)
				end
				tweetyio.close(fh);
			end
		end
		dir_entry = iter(dir_obj)
	end

	table.sort(result,
		function(a,b)
			if a.idx and b.idx then
				return b.idx > a.idx
			else
				if a.idx then
					return false
				end
				if b.idx then
					return true
				end
				return b.caption > a.caption
			end
		end
	)

	return true,result
end
local function readJson(filename)
	local fh;
	local txt;
	fh = tweetyio.open(filename,"rb");
	if not fh then
		return false
	end
	txt = fh:read("*a")
	tweetyio.close(fh);
	return json.decode(txt)
end

local function SaveJSON (filename, content)
	local fh;
	fh = tweetyio.open(filename,"wb");
	if not fh then
		return false
	end
	local jsonContent = json.encode(content,true)
	--LogDebug("[TEMPLATE:SaveJSON] filename: '%s' content: '%s'", filename, jsonContent)
	fh:write(jsonContent)
	tweetyio.close(fh);
	return true;
end

local function CopyFile (sourceFile, targetFile, _T)
	LogInfo("[TEMPLATE:CopyFile] _T: '%s', sourceFile: '%s' targetFile: '%s'", _T, sourceFile, targetFile)

	local fi = tweetyio.open(sourceFile,"rb");
	if not fi then
		return false
	end
	local fo = tweetyio.open(targetFile,"wb");
	if not fo then
		tweetyio.close(fi)
		return false
	end
	local content = fi:read("*a")
	content = utils.TranslateJSON(sourceFile, content, _T)
	fo:write(content);
	tweetyio.close(fo)

	return true
end

function createApplication (vccName, appName, parameters, _T)
	local templateId
	local template_path;
	if parameters.copyFrom then
		templateId = parameters.copyFrom;
		template_path = config.usersPath .. config.pathSeparator .. vccName .. config.pathSeparator .. templateId .. config.pathSeparator
	elseif parameters.templateId then
		templateId = parameters.templateId;
		template_path = config.templateDirPath .. config.pathSeparator .. templateId .. config.pathSeparator
	end
	local app_path = config.usersPath .. config.pathSeparator .. vccName .. config.pathSeparator .. appName
	local app = readJson(template_path .. config.mainAppFile);
	math.randomseed(os.time())
	if app then
		app.created_at=LuaOrm.Date():toString();
		local catalog = app.catalog;
		local res,err
		local existentApp = readJson(config.usersPath .. config.pathSeparator .. vccName .. config.pathSeparator .. appName .. config.pathSeparator .. config.mainAppFile);

		if existentApp and parameters.overwrite then
			local app = application.new{vccName=vccName}
			local status, result = app:initialize(appName);
			if status then
				status,result = app:deconfigure();
			end
			if not status then
				return status,result
			end
			res = true; -- pise todo
		else
			res,err = tweetyio.mkdir(app_path)
		end
		if res then
			if SaveJSON(app_path .. config.pathSeparator .. config.mainAppFile,app) then
				for _,form in ipairs(catalog.views) do
					CopyFile(template_path .. form.name .. config.formFileExtention,app_path .. config.pathSeparator .. form.name .. config.formFileExtention, _T)
					CopyFile(template_path .. form.name .. config.formCodeFileExtention,app_path .. config.pathSeparator .. form.name .. config.formCodeFileExtention) 
				end
				for _,res in ipairs(catalog.resources) do
					CopyFile(template_path .. res.src,app_path .. config.pathSeparator .. res.src)
				end

				for _,name in ipairs(catalog.datasources) do
					local datasourceData = readJson(template_path .. name .. config.datasourceFileExtention);
					if datasourceData and datasourceData.type == "icc-native" then
						datasourceData.description.tableName = datasourceData.name .. string.gsub(os.time() .. math.abs(math.random()*10),"%.","")
					end
					if datasourceData then 
						SaveJSON(app_path .. config.pathSeparator .. name .. config.datasourceFileExtention,datasourceData)
					end

				end
				local catalog_entry = {
					name=appName,
					version=0,
					created_at=app.created_at,
					status="untracked",
					hidden=parameters.hidden
				}
				if not SaveJSON(app_path .. config.pathSeparator .. config.catalogFile,catalog_entry) then
					return false,messages.messageKey('write_file_error',{file=app_path .. config.pathSeparator .. config.catalogFile})
				end
				catalog_entry.created_at = string.gsub(LuaOrm.Date(catalog_entry.created_at):toDifferentTimezoneString(inconcert.authUser.timezone),"%.%d*$","")
				local app = application.new{vccName=vccName}
				local status, result = app:initialize(appName);
				if status then
					status,result = app:configure();
					if not status then
						return status,result
					end
				end
				return true,catalog_entry
			else
				return false,messages.messageKey('write_file_error',{file=app_path .. config.pathSeparator .. config.mainAppFile})
			end
		else
			return false,messages.messageKey('mkdir_error',{path=app_path})
		end

	end
	return false,messages.messageKey('read_file_error',{file=template_path .. config.mainAppFile})
end
