local wsapi_request_stream = require "wsapi.request_stream"
local wsapi_response = require "wsapi.response"

module(..., package.seeall,inconcert.orbit.application.Decorate)
require "inconcert.orbit.application"
local config = require (tweety_modules_prefix .. "config")
local tweetyio =require (tweety_modules_prefix .. "tweetyio")
require "uuid"


require "ltn12"

function UploadHandler(wsapi_env)
	local request = wsapi_request_stream.new(wsapi_env)
	local response = wsapi_response.new(200, { ["Content-Type"]= "text/plain" })
	local session = tweetyio.session();

	local input_type = wsapi_env.CONTENT_TYPE
	
	
	local id = request.params["fileid"]
	local filename = request.params["fileName"]
	

	if not session.userfiles[id] then
		response.status = 500
		response:write("Invalid userfile id")
		session:release();
		return response:finish()
	end
	
	local length = tonumber(wsapi_env.CONTENT_LENGTH)
	if length <= 0 then
		response.status = 500
		response:write("Invalid file. Zero length")
		session:release();		
		return response:finish()
	end
	
	session.userfiles[id].percent = 0
	
	--LogDebug("memory used = %d Kb", collectgarbage("count"))
	
	-- Save the data right away to disk
	local tempdir = config.tempPath
	local fullFileName = config.tempPath..  config.pathSeparator .. id .. "_" .. filename
	
	local file_sink, err = ltn12.sink.file( tweetyio.open(fullFileName, "wb") )
	if not file_sink then
		local msg = LogError("Unable to save file: '%s' to disk: %s", sessionid, err or "")
		response.status = 500
		response:write(msg)
		session:release();
		return response:finish()
	end
	
	--	o Table containing decoded (name, file) and raw (headers) mime header data
	--  o String value containing a chunk of the file data
	--  o Boolean which indicates wheather the current chunk is the last one (eof)
	local readBytes = 0
	local mimeHeaders
	local chunks = 0;
	local callback = function( headers, chunk, last )
		if not mimeHeaders then
			mimeHeaders = headers
		end
		chunks = chunks+1
		readBytes = readBytes + #chunk
--		LogDebug("chunk %d",readBytes)
		file_sink(chunk)

		if last then
			file_sink(nil)
		end
		if math.fmod(chunks,3)==0 then
			local session = tweetyio.session();
			session.userfiles[id].percent= readBytes * 100 / length
			session:release();
		end	
	end
	session:release();

	request:parse_message_body( callback )

	session = tweetyio.session();
	session.userfiles[id].percent = 100
	session.userfiles[id].completed = true
	
	--LogDebug("memory used = %d Kb", collectgarbage("count"))

	LogDebug("UploadHandler, received file:%s - id:%s ", filename, id)
	LogDebug("Content type:%s \r\nSize:%s \r\n Filename:%s", mimeHeaders.headers["Content-Type"], readBytes, filename)
	
	session.userfiles[id].size = readBytes
	session.userfiles[id].filename = filename
	session.userfiles[id].fullFileName = fullFileName
	session.userfiles[id].mimeType = mimeHeaders.headers["Content-Type"];
	session:release();
	response:write(tostring(readBytes))
	-- helps the garbage collector, specially after processing huge files
	-- see this thread <http://lua-users.org/lists/lua-l/2009-07/msg00113.html> (specially the response by Ierusalimschy)
	CollectAllMemory()
	LogDebug("memory used = %d KBytes", collectgarbage("count"))
	return response:finish()
end




local lastResults = {}
local lastRequestTimes = {}

local function canMakeRequest(fileId)
    if not lastRequestTimes[fileId] then
        return true
    else
        local currentTime = os.time()
        local lastRequestTime = lastRequestTimes[fileId]
        local elapsedTime = currentTime - lastRequestTime
        
        if elapsedTime >= 5 then
            return true
        else
            return false
        end
    end
end

function GetInfoHandler(web)
    web.headers["Content-Type"] = "text/plain"
    _M.response.DontCache(web)
    
    local id = web.input['fileid'];
    
    if canMakeRequest(id) then
        lastRequestTimes[id] = os.time()
        
        local session = tweetyio.session();
        session:release();
        
        if not session.userfiles[id] then
            return "-1"
        end
        
        if session.userfiles[id].completed then
            return "-1"
        end
        
        local result = tostring(session.userfiles[id].percent or 0)
        lastResults[id] = result
        return result
    else
        return lastResults[id] or "-1"
    end
end
function _GetInfoHandler(web)
	web.headers["Content-Type"] = "text/plain"
	_M.response.DontCache(web)
	local session = tweetyio.session();
	session:release();
	local id = web.input['fileid'];
	if not session.userfiles[id] then
		return "-1"
	end
	if session.userfiles[id].completed then
		return "-1"
	end
	return tostring(session.userfiles[id].percent or 0)
end

function GetIdHandler(web)
	web.headers["Content-Type"] = "text/plain"
	_M.response.DontCache(web)
	local session = tweetyio.session();
	local newId = uuid.new()
	LogDebug("Creating id for file: %s", newId)
	if not session.userfiles then
		session.userfiles = {}
	end
	session.userfiles[newId] = {size = -1}
	session:release();
	return newId
end


function initialize(_M, prefix,appLocation)
	prefix = prefix or ""
	_M:dispatch_wsapi(UploadHandler, prefix .. "/uploader/uploadhandler")
	_M:dispatch_post(GetInfoHandler, prefix .. "/uploader/getinfohandler")
	_M:dispatch_post(GetIdHandler, prefix .. "/uploader/getidhandler")
end

