module(..., package.seeall)
--local messages = require (tweety_modules_prefix .. "messages")
if not tweety_modules_prefix then
	tweety_modules_prefix = "apps.webdesigner."
end
local messages = require (tweety_modules_prefix .. "messages")
local json = require "json"
local i18nlib = require "i18nlib"
local m_helpers = require "helpers"
local config = require (tweety_modules_prefix .. "config")
local sidereal = require ("sidereal")
local cassandra = require "lua_cassandra"

function alltrim (s) return m_helpers.string.alltrim(s) end

function ParseQueryString (queryStr, TransformValuesFn)
	return m_helpers.utils:ParseUrlQueryString(queryStr, TransformValuesFn)
end

function JSON_Decode (value)
	return m_helpers.table:JSON_Decode(value)
end

function JSON_Encode (value)
	return m_helpers.table:JSON_Encode(value)
end

function check_json_params (jsonstring)
	local status,res=pcall(json.decode,jsonstring)
	if not status then
		LogDebug("Invalid request detected json %s",tostring(jsonstring))
		return false, messages.messageKey("invalid_request")
	end
	return true, res
end

--[[
	Attempts to detect the language using the language Cookie (set by the login process) or the Accept-Language header sent by the browser
--]]
function DetectLanguage (web)
	LogInfo("[UTILS:DetectLanguage] Trying to detect language => Accept-Language: %s - Language Cookie: %s", web.vars.HTTP_ACCEPT_LANGUAGE, web.cookies.LULID)
	local acceptHeader, languageCookie = web.vars.HTTP_ACCEPT_LANGUAGE or "", web.cookies.LULID
	if languageCookie then
		return i18nlib.LCID[languageCookie], languageCookie
	else
		local langCode = string.match(acceptHeader, "^([^,]+)")
		if langCode then
			langCode = langCode:match("(%a%a)[%-.+]?")
			web:set_cookie("LULID", {value = langCode, path = "/", domain = m_cookies_domain, sameSite = "None", secure = true})
			return i18nlib.LCID[langCode], langCode
		else
			web:set_cookie("LULID", {value = "en_US", path = "/", domain = m_cookies_domain, sameSite = "None", secure = true})
			return i18nlib.LCID.en_US, "en" -- defaulting to english
		end
	end
end

function MakeTextTranslator (languageId, languageCode)
	inconcert.authUser = inconcert.authUser or {
		languageId = languageId,
		languageNumber = languageCode
	}
	local translator = inconcert.translators.webdesigner or {}

	LogInfo("[UTILS:makeTextTranslator] translator.language: '%s'", translator.language)

	return function(...)
		return translator:Text(...)
	end, i18nlib.LCID[translator.language]
end

function GetLanguageFromWebInfo (web)
	local cookies = web.vars.HTTP_COOKIE or ""
	local mytoken = cookies:match("AUTH_SID_WEBDESIGNER=(.+);")

	if not mytoken then
		mytoken = cookies:match("AUTH_SID_WEBDESIGNER=(.+)")
	end
	local authFilter = (require "authfilter")("webdesigner")
	local userFromToken = authFilter:GetUserFromToken(mytoken or "")

	LogInfo("[UTILS:GetLanguageFromWebInfo] Token: '%s'. userFromToken: '%s'.", mytoken, userFromToken)

	local webLanguageCode, webLanguageId = DetectLanguage(web)

	LogInfo("[UTILS:GetLanguageFromWebInfo] webLanguageCode: '%s'", webLanguageCode)
	LogInfo("[UTILS:GetLanguageFromWebInfo] webLanguageId: '%s'", webLanguageId)
	LogInfo("[UTILS:GetLanguageFromWebInfo] userFromToken.languageId: '%s'", userFromToken and userFromToken.languageId)

	local languageId = userFromToken and userFromToken.languageId or webLanguageId

	return languageId, webLanguageCode
end

function TranslateJSON (fileName, content, _T, translateAttributes, listChildsAttributes)
	if not _T then
		return content
	end

	translateAttributes = translateAttributes or {caption = true, description = true, title = true, textContent = true, textBoxPlaceholder = true}

	LogInfo("[UTILS:TranslateJSON] Translating: _T: '%s',\nfileName: '%s',\ncontent length: '%s',\ntranslate attrs: '%s',\nlist childs attrs: '%s'", _T, fileName, #(content or ""), json.encode(translateAttributes), json.encode(listChildsAttributes or {}))

	local function Translate (attrName, originalValue)
		if not translateAttributes[attrName] then
			return originalValue
		end
		local translatedValue = _T(originalValue)
		LogDebug("[UTILS:TranslateJSON] A traducir: '%s'\nORIGINAL : '%s'\nTRADUCIDO: '%s'", attrName, originalValue, translatedValue)
		return translatedValue
	end

	local function RecursiveTranslate (contentAttrs)
		for attrName, attrValue in pairs(contentAttrs) do
			if type(attrValue) == "string" then
				contentAttrs[attrName] = Translate(attrName, attrValue)
			elseif type(attrValue) == "table" then
				--LogDebug("[UTILS:TranslateJSON] list name: '%s'", attrName)
				contentAttrs[attrName] = RecursiveTranslate(attrValue)
			end
		end
		return contentAttrs
	end
	local decodedContent = json.decode(content)
	local translatedContent = json.encode(RecursiveTranslate(decodedContent))

	LogDebug("[UTILS:TranslateJSON] Translated content: '%s'", translatedContent)

	return translatedContent
end

--[[

]]--
function StringIsEmpty (s)
	return type(s) ~= "string" or alltrim(s) == ""
end

--[[

]]--
local m_redisClient = {}
function GetRedisClient () -- se expone a los demas luas
	local redisHost = config.redis_host
	local redisPort = config.redis_port
	local key = ("%s_%s"):format(redisHost, redisPort)
	if not m_redisClient[key] then
		m_redisClient[key] = sidereal.connect(redisHost, redisPort)
	end
	return m_redisClient[key]
end

--[[

]]--
function GetPhoneValidation(vcc)
	vcc = vcc or (inconcert and inconcert.authUser and inconcert.authUser.virtualContactCenter)
	if not vcc or type(vcc) ~= "string" then
		return { Enabled = false }
	end

	vcc = string.lower(vcc)
	local connection = cassandra:CassandraConnection("Agent")
	local validationStr = connection:GetColumnValue(vcc, "VCCs", "PhoneValidation")
	if StringIsEmpty(validationStr) then
		return { Enabled = false }
	end

	return json.decode(validationStr)
end

--[[

]]--
function GetPhoneFormat(vcc)
	local validation = GetPhoneValidation(vcc)
	if not validation.Enabled then return "original" end

	local userFormat = inconcert.authUser and inconcert.authUser.phoneFormat
	if not StringIsEmpty(userFormat) then
		return userFormat
	end

	return validation.Format or "global_plain"
end

--[[

]]--
function FormatPhone(addressTable, addressFormat)
	return m_helpers.phones.FormatPhone(addressTable, addressFormat)
end
