-------------------------------------------------------------------------------
-- Xavante configuration file.
--
-- The configuration defines the Xavante environment
-- Commented fields are optional and assume default values
--
-- Xavante defines virtualhosts for each site running with Xavante.
-- Each virtualhost can define the handlers for specific files extensions.
-- Xavante currently offers a fileHandler and a CGILuaHandler.
--
-- Xavante configuration can be redefined on the default file structure by the
-- the use of LUA_PATH, see more details in the online documentation.
--
-- Authors: Javier Guerra and Andre Carregal
-- Copyright (c) 2004-2006 Kepler Project
---
-- $Id: config.lua,v 1.4 2007/11/08 22:47:45 carregal Exp $
------------------------------------------------------------------------------
require "xavante.filehandler"
require "xavante.redirecthandler"
require "xavante.urlhandler"
require "inconcert.rewritehandler"
local wsapi_deflater = require "wsapi.deflater"
require "inconcert.loaders.debug"
require "inconcert.appMounter"
local lfs = require "lfs"
require "inconcert.appReadyFilter"
require "webHandler.path"




local Mount = inconcert.appMounter.Mount
-- Define here where Xavante HTTP documents scripts are located
local webDir = XAVANTE_WEB
webDir_one_back = webDir:match( ("(.+)%s[^%s]+$"):format(webHandler.path.dir_sep, webHandler.path.dir_sep) )
local webdesignerFilter = require("authfilter")("webdesigner")
local webdesigner = require "apps.webdesigner"

local webdesignerConf = inconcert.environment.apps.webdesigner

webdesignerFilter:InitializeConfigClient(webdesignerConf.config_server)

local function DontCache(status, headers, iterator)
	headers["Pragma"] = "no-cache"
	headers["Cache-Control"] = "no-store, no-cache, must-revalidate, post-check=0, pre-check=0"
	return status, headers, iterator
end

---
-- Filter that reenables inconcert.session for Tweety
-- TODO: this should be a temporal fix
-- TODO-2: Check again in two years. I bet this code is still here :)
--
local function MakeSessionFilter (excluded_urls)

	local SharedSessions = require "SharedSessions"

	local function MatchAny(url, rules)
		for _, rule in ipairs(rules or {}) do
			if url:match(rule) == url then
				return true
			end
		end
		return false
	end

	return function (appToFilter)

		local call_next_layer
		if type(appToFilter) == "table" and type(appToFilter["run"]) == "function" then
			call_next_layer = appToFilter.run
		else
			call_next_layer = appToFilter
		end

		local function unmap_session (...)
			inconcert.session = nil
			return ...
		end

		return function (wsapi_env)

			if MatchAny(wsapi_env.PATH_INFO, excluded_urls) then
				return call_next_layer(wsapi_env)
			end
			if inconcert and inconcert.authUser then
				local session_token = inconcert.authUser.authToken

				local session = SharedSessions.Open(session_token)
				if not session then
					session = SharedSessions.Create(session_token, 2 * 3600)	-- 2 hours
				end
				inconcert.session = session
			end

			return unmap_session( call_next_layer(wsapi_env) )
		end

	end
end

local rules = {
	["default"] = {

		{ -- Hacemos un redirect para las urls del sitio viejo con GX Portal
			match = {"^/inconcert/apps/webdesigner$"},
			with = xavante.redirecthandler,
			params = {"/inconcert/apps/webdesigner/", "redirect"}
		},	
		
		Mount{	match = "^/inconcert/apps/webdesigner[/]*.*$",
			name = "webdesigner",
			displayName = "Web Designer",
			filters = {
				MakeSessionFilter({
					"^/inconcert/apps/webdesigner[/]*.*"
				}),
				webdesignerFilter:makeFilter {
					base_domain = webdesignerConf.base_domain,
					--renderLoginFn = webdesigner.RenderLoginFormWSAPI,
					appDefinedCredentialFn=webdesigner.CustomCredentialFn,
					--users that are logged to these applications will be logged automatically for this application
					acceptCredentialsFrom = {"agent", "i6portal"},
					
					--urls that authentication is optional
					optionalUrls = {"/login/.*"
					    					},
					
					--urls that authentication is bypassed
					excludedUrls = {
						"/images/.+",
						"/styles/.+",
						"/scripts/.+%.js",
						"/clientlib/.+",
						"/lib/.+",
						"/generated/.+",
						"/themes/.+",
						"/render/.+",
						"/alive/",
						"/buildform/.+",
						"/get_chat_first_form/.+",
						"/resources/.+",
						"/[^/]+/[^/]+/resources/.+",
						"/designer/applications/[^/]+/[^/]+/resources/.+",
						"/runtime/click_to_contact_form",
						"/runtime/save_contact_form",
						"/runtime/save_postsurvey_data",
						"/runtime/save_survey_form",
						"/runtime/check_survey_validity",
						"/runtime/validate_captcha",
						"/[^/]+/applications/[^/]+/datasource/.+",
						"/[^/]+/[^/]+/datasource/.+",
						"/imgs/.+"
					},
				},				
				inconcert.appReadyFilter.makeFilter("authfilter"),
                                inconcert.appReadyFilter.makeFilter("webdesigner"),
				wsapi_deflater.makeDeflater
			}
		},
		{
			match = "^/weblibs/inconcert/",
			with = xavante.filehandler,
			params = {baseDir = "/usr/lib/inconcert/"  }
		},
		{
			match = "^/weblibs/(.*)",
			with = xavante.redirecthandler,
			params = { "/weblibs/inconcert/%1", "reparse" }
		},
		
		{ -- filehandler example
			match = "^/favicon.ico$",
			with = xavante.filehandler,
			params = {baseDir = webDir}
		},
		{ -- filehandler example
			match = ".",
			with = xavante.filehandler,
			params = {baseDir = webDir}
		}
	}
}

return rules
